using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Tao.OpenGl;
using System.Drawing.Drawing2D;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;

namespace GTG
{
	public delegate string GetStringFunction();
	public delegate void SetStringFunction(string value);
	public partial class GTGForm : Form
	{
		public GTG _gtg;
		State selectedState;
		Edge selectedEdge;
		State selectedEdgeSourceState;

		State edgeStartState;	//When creating an edge, this is the start of the edge

		UIState uiState;	//Current UI State

		Dictionary<EdgeType, string> EdgeTypeToString;
		Dictionary<string, EdgeType> StringToEdgeType;

		GetStringFunction GetStatus;
		SetStringFunction SetStatus;
		SetStringFunction SetOutput;
		//Matrix transform; //Reversing the orientation of the y-axis

		public GTGForm(GetStringFunction getStatusStringFn, SetStringFunction setStatusStringFn, SetStringFunction setOutput)
			: this(new GTG("[a-z]"), getStatusStringFn, setStatusStringFn, setOutput)
		{}
		private GTGForm(
			GTG gtg /* must never be null */,
			GetStringFunction getStatusStringFn,
			SetStringFunction setStatusStringFn,
			SetStringFunction setOutput)
		{
			InitializeComponent();
			_gtg = gtg;
			GetStatus = getStatusStringFn;
			SetStatus = setStatusStringFn;
			SetOutput = setOutput;
			InitUI();
			InitStringTables();
			InitCombos();
		}
		public static GTGForm FromFile(
			Stream file,
			GetStringFunction getStatusStringFn,
			SetStringFunction setStatusStringFn,
			SetStringFunction setOutput)
		{
			try
			{
				BinaryFormatter loader = new BinaryFormatter();
				GTG gtg = (GTG)loader.Deserialize(file);
				if (gtg == null) return null;
				GTGForm retval = new GTGForm(
					gtg,
					getStatusStringFn,
					setStatusStringFn,
					setOutput);
				return retval;
			}
			catch (Exception)
			{
				return null;
			}
		}
		private void InitCombos()
		{
 			comboEdgeType.Items.Clear();
			foreach(string key in StringToEdgeType.Keys)
			{
				comboEdgeType.Items.Add(key);
			}
		}

		private void InitStringTables()
		{
 			EdgeTypeToString = new Dictionary<EdgeType,string>(3);
			EdgeTypeToString.Add(EdgeType.Labelled, "Labelled");
			EdgeTypeToString.Add(EdgeType.Epsilon, "Epsilon");
			EdgeTypeToString.Add(EdgeType.Other, "Other");

			StringToEdgeType = new Dictionary<string,EdgeType>( EdgeTypeToString.Count );
			foreach(KeyValuePair<EdgeType, string> kv in EdgeTypeToString)
			{
				StringToEdgeType.Add(kv.Value, kv.Key);
			}
		}

		private void InitUI()
		{
			selectedEdge = null;
			selectedEdgeSourceState = null;
			selectedState = null;
			edgeStartState = null;
			uiState = UIState.Normal;
			chkState.Checked = false;
			chkEdge.Checked = false;
			txtCharSet.Text = _gtg.CharacterSet;

			UpdateGTGBars();
		}

/*		public void UpdateGTGBars2()
		{
			toolBarContainer.SuspendLayout();
			toolBarContainer.TopToolStripPanel.Controls.Remove(toolBarState);
			toolBarContainer.TopToolStripPanel.Controls.Remove(toolBarEdge);
			toolBarState.Visible = false;
			toolBarEdge.Visible = false;
			if (selectedState != null)
			{
				toolBarContainer.TopToolStripPanel.Controls.Add(toolBarState);
				toolBarState.Visible = true;

				txtStateName.Text = selectedState.Name;
				chkAccept.Checked = selectedState.IsAccept;
				chkStart.Checked = selectedState.IsStart;
			}
			if (selectedEdge != null)
			{
				toolBarContainer.TopToolStripPanel.Controls.Add(toolBarEdge);
				toolBarEdge.Visible = true;

				if(selectedEdge.Type == EdgeType.Labelled)
				{
					txtEdgeLabel.Enabled = true;
					txtEdgeLabel.Text = selectedEdge.Label;
				}
				else
					txtEdgeLabel.Enabled = false;


				EventHandler ev = new EventHandler(comboEdgeType_SelectedIndexChanged);
				comboEdgeType.SelectedIndexChanged -= ev;
				comboEdgeType.SelectedIndex =
					comboEdgeType.Items.IndexOf(EdgeTypeToString[selectedEdge.Type]);
				comboEdgeType.SelectedIndexChanged += ev;
			}
			toolBarContainer.ResumeLayout();
		}*/

		public void UpdateGTGBars()
		{
			mainToolBar.SuspendLayout();

			foreach (ToolStripItem tsi in toolBarState.Items)
				mainToolBar.Items.Remove(tsi);
			foreach (ToolStripItem tsi in toolBarEdge.Items)
				mainToolBar.Items.Remove(tsi);

			if (selectedState != null)
			{
				mainToolBar.Items.AddRange(toolBarState.Items);

				txtStateName.Text = selectedState.Name;
				chkAccept.Checked = selectedState.IsAccept;
				chkStart.Checked = selectedState.IsStart;
			}

			if (selectedEdge != null)
			{
				mainToolBar.Items.AddRange(toolBarEdge.Items);
							
				EventHandler ev = new EventHandler(comboEdgeType_SelectedIndexChanged);
				comboEdgeType.SelectedIndexChanged -= ev;
				comboEdgeType.SelectedIndex =
					comboEdgeType.Items.IndexOf(EdgeTypeToString[selectedEdge.Type]);
				comboEdgeType.SelectedIndexChanged += ev;

				mainToolBar.ResumeLayout();
				if (selectedEdge.Type == EdgeType.Labelled)
				{
					txtEdgeLabel.Enabled = true;
					txtEdgeLabel.Text = selectedEdge.Label;
					txtEdgeLabel.SelectAll();
					txtEdgeLabel.Select();
					txtEdgeLabel.Focus();
				}
				else
					txtEdgeLabel.Enabled = false;
			}
			else
			mainToolBar.ResumeLayout();
		}

		private string StatusString
		{
			get
			{
				return GetStatus();
			}
			set
			{
				SetStatus(value);
			}
		}

		private enum UIState
		{
			Normal,
			CreatingState,
			CreatingEdge,
			StateSelected,
			MovingState,
			SeekingEndState,
			EdgeSelected,
			SelfEdgeSelected
		}

		private enum SelectionType
		{
			Nothing,
			State,
			Edge,
			SelfEdge
		}

		/*private void Unsubscribe<EventClass, MethodType>(EventClass ev, MethodType method)
		{
			ev -= new EventHandler(method);
		}*/

		private void btnDelete_Click(object sender, EventArgs e)
		{
			if (uiState == UIState.StateSelected)
			{
				_gtg.RemoveState(selectedState);
				selectedState = null;
				uiState = UIState.Normal;
				UpdateGTGBars();
				drawingBoard.Invalidate();
			}
			else if (uiState == UIState.EdgeSelected)
			{
				selectedEdgeSourceState.OutEdges.Remove(selectedEdge);
				selectedEdge.Target.InEdges.Remove(selectedEdge);
				selectedEdge = null;
				selectedEdgeSourceState = null;
				uiState = UIState.Normal;
				UpdateGTGBars();
				drawingBoard.Invalidate();
			}
			else if (uiState == UIState.SelfEdgeSelected)
			{
				selectedEdgeSourceState.SelfEdge = null;
				selectedEdge = null;
				selectedEdgeSourceState = null;
				uiState = UIState.Normal;
				UpdateGTGBars();
				drawingBoard.Invalidate();
			}
		}

		private void ChangeCheckedWithoutFiringEvent(ToolStripButton btn, EventHandler ev, bool chkValue)
		{
			btn.CheckedChanged -= ev;
			btn.Checked = chkValue;
			btn.CheckedChanged += ev;
		}
		private void ChangeStateEdgeCkeckWithoutFiringEvents(bool chkStateValue, bool chkEdgeValue)
		{
			ChangeCheckedWithoutFiringEvent(
				chkState,
				new EventHandler(chkState_CheckedChanged),
				chkStateValue);

			ChangeCheckedWithoutFiringEvent(
				chkEdge,
				new EventHandler(chkEdge_CheckedChanged),
				chkEdgeValue);
		}
		/// <summary>
		/// It finds whether there edges or states that contain the specified point within there
		/// field. If there exist, it chooses the nearest state or edge (only the nearest of both)
		/// and updates the value of selectedState or selectedEdge and selectedEdgeSourceState.
		/// In this case, it returns StateSelected or EdgeSelected if any is selected, or 
		/// Normal otherwise. The values of the selectedXXX will not be changed if nothing is selected.
		/// </summary>
		/// <param name="pt"></param>
		/// <returns></returns>
		private SelectionType UpdateSelection(Point2F pt)
		{
			float stateDist;
			float edgeDist;
			SelectionType retval = SelectionType.Nothing;
			State nstate;
			Edge nedge;
			State nedgeSrc;

			if (_gtg.NearestState(pt, out nstate, out stateDist))
			{
				stateDist = (float)Math.Sqrt(stateDist);

				if (_gtg.NearestEdge(pt, out nedgeSrc, out nedge, out edgeDist))
				{
					if (stateDist <= edgeDist)
					{
						retval = SelectionType.State;
						selectedEdgeSourceState = null;
						selectedEdge = null;
						selectedState = nstate;
						return retval;
					}
					else //edge is nearer
					{
						retval = SelectionType.Edge;
						selectedEdge = nedge;
						selectedEdgeSourceState = nedgeSrc;
						selectedState = null;
						return retval;
					}
				}
				else
				{
					retval = SelectionType.State;
					selectedEdgeSourceState = null;
					selectedEdge = null;
					selectedState = nstate;
					return retval;
				}
			}

			else if (_gtg.NearestEdge(pt, out nedgeSrc, out nedge, out edgeDist))
			{
				retval = SelectionType.Edge;
				selectedEdge = nedge;
				selectedEdgeSourceState = nedgeSrc;
				selectedState = null;
				return retval;
			}

			else if(_gtg.SelectedSelfEdge(pt, out nedge)) //check self edge
			{
				retval = SelectionType.SelfEdge;
				selectedEdge = nedge;
				selectedEdgeSourceState = nedge.Source;
				selectedState = null;
				return retval;
			}

			return SelectionType.Nothing;
		}
		private void chkState_CheckedChanged(object sender, EventArgs e)
		{
		//	EventHandler ev = new EventHandler(chkState_CheckedChanged);
			switch (uiState)
			{
				case UIState.Normal:

					ChangeStateEdgeCkeckWithoutFiringEvents(true, false);
					uiState = UIState.CreatingState;

					break;
				case UIState.CreatingState:

					ChangeStateEdgeCkeckWithoutFiringEvents(false, false);
					uiState = UIState.Normal;

					break;
				case UIState.CreatingEdge:

					ChangeStateEdgeCkeckWithoutFiringEvents(true, false);
					uiState = UIState.CreatingState;

					break;
				case UIState.MovingState:

					//Impossible, but let's do it
					ChangeStateEdgeCkeckWithoutFiringEvents(true, false);
					selectedState = null;
					UpdateGTGBars();
					uiState = UIState.CreatingState;

					break;
				case UIState.StateSelected:

					//chkState must be now true
					//Unsubscribe(chkState.CheckedChanged, chkState_CheckedChanged);
					//chkState.CheckedChanged -= ev;

					//chkState.Checked = true;
			
					//selectedState = null;
					ChangeStateEdgeCkeckWithoutFiringEvents(true, false);
					selectedState = null;
					UpdateGTGBars();

					//chkState.CheckedChanged += ev;
					drawingBoard.Invalidate();
					uiState = UIState.CreatingState;

					break;
				case UIState.EdgeSelected:

					ChangeStateEdgeCkeckWithoutFiringEvents(true, false);
					selectedEdge = null;
					selectedEdgeSourceState = null;
					UpdateGTGBars();

					drawingBoard.Invalidate();
					uiState = UIState.CreatingState;

					break;
				case UIState.SelfEdgeSelected:

					ChangeStateEdgeCkeckWithoutFiringEvents(true, false);
					selectedEdge = null;
					selectedEdgeSourceState = null;
					UpdateGTGBars();

					drawingBoard.Invalidate();
					uiState = UIState.CreatingState;

					break;
				case UIState.SeekingEndState:

					//Although impossible, let's do it...
					ChangeStateEdgeCkeckWithoutFiringEvents(true, false);
					edgeStartState = null;
					//UpdateGTGBars();
					drawingBoard.Invalidate();
					uiState = UIState.CreatingState;

					break;
			}
		}

		private void chkEdge_CheckedChanged(object sender, EventArgs e)
		{
			switch (uiState)
			{
				case UIState.Normal:

					ChangeStateEdgeCkeckWithoutFiringEvents(false, true);
					uiState = UIState.CreatingEdge;

					break;
				case UIState.CreatingState:

					ChangeStateEdgeCkeckWithoutFiringEvents(false, true);
					uiState = UIState.CreatingEdge;

					break;
				case UIState.CreatingEdge:

					ChangeStateEdgeCkeckWithoutFiringEvents(false, false);
					uiState = UIState.Normal;

					break;
				case UIState.MovingState:

					//Impossible, but let's do it
					ChangeStateEdgeCkeckWithoutFiringEvents(false, true);
					selectedState = null;
					UpdateGTGBars();
					uiState = UIState.CreatingEdge;

					break;
				case UIState.StateSelected:

					//chkState must be now true
					//Unsubscribe(chkState.CheckedChanged, chkState_CheckedChanged);
					//chkState.CheckedChanged -= ev;

					//chkState.Checked = true;

					//selectedState = null;
					ChangeStateEdgeCkeckWithoutFiringEvents(false, true);
					selectedState = null;
					UpdateGTGBars();

					//chkState.CheckedChanged += ev;
					drawingBoard.Invalidate();
					uiState = UIState.CreatingEdge;

					break;
				case UIState.EdgeSelected:

					ChangeStateEdgeCkeckWithoutFiringEvents(false, true);
					selectedEdge = null;
					UpdateGTGBars();

					drawingBoard.Invalidate();
					uiState = UIState.CreatingEdge;

					break;
				case UIState.SelfEdgeSelected:

					ChangeStateEdgeCkeckWithoutFiringEvents(false, true);
					selectedEdge = null;
					selectedEdgeSourceState = null;
					UpdateGTGBars();

					drawingBoard.Invalidate();
					uiState = UIState.CreatingEdge;

					break;
				case UIState.SeekingEndState:

					//Although impossible, let's do it...
					ChangeStateEdgeCkeckWithoutFiringEvents(false, false);
//					selectedState = edge
					edgeStartState = null;
//					UpdateGTGBars();
					drawingBoard.Invalidate();
					uiState = UIState.Normal;

					break;
			}
		}

		private void txtCharSet_TextChanged(object sender, EventArgs e)
		{
			string charset = txtCharSet.Text;
			if (!GTG.ValidateCharacterSet(charset))
			{
				//status.Set("Invalid character set.");
				if (Focus())
					txtCharSet.Select();
			}
			else
			{
				if(!_gtg.RevalidateAllLabels())
					SetStatus("Although the character set is valid, Some edge labels are still invalid");
			}
		}

		private void txtStateName_TextChanged(object sender, EventArgs e)
		{
			selectedState.Name = txtStateName.Text;
			drawingBoard.Invalidate();
		}

		private void txtEdgeLabel_TextChanged(object sender, EventArgs e)
		{
			string err;
			if (!selectedEdge.ChangeLabel(_gtg.CharacterSet, txtEdgeLabel.Text, out err))
				StatusString = err;
			else
				StatusString = "";
			drawingBoard.Invalidate();
		}

		private void comboEdgeType_SelectedIndexChanged(object sender, EventArgs e)
		{
			selectedEdge.Type = StringToEdgeType[comboEdgeType.SelectedItem as string];
			UpdateGTGBars();
			drawingBoard.Invalidate();
		}

		private void drawingBoard_Paint(object sender, PaintEventArgs e)
		{
			//if (uiState != UIState.MovingState)
		//	{
				//e.Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;
				e.Graphics.SmoothingMode = SmoothingMode.HighQuality;
				//e.Graphics.InterpolationMode = InterpolationMode.HighQualityBicubic;
				//e.Graphics.CompositingQuality = CompositingQuality.HighQuality;
		//	}
			//Matrix temp = e.Graphics.Transform;
			//temp.Multiply(transform, MatrixOrder.Prepend);
			//e.Graphics.Transform = temp;

			_gtg.Draw(e.Graphics);

			if (selectedEdge != null)
			{
				//self or direct ?
				if (uiState == UIState.SelfEdgeSelected)
					selectedEdge.Source.DrawSelfEdge(
						e.Graphics,
						_gtg.CharacterSet,
						true);
				else selectedEdge.Draw(
						e.Graphics,
						true);
			}

			if (selectedState != null) selectedState.DrawState(
				 e.Graphics,
				 _gtg.CharacterSet,
				 true);
		}

		private void txtCharSet_Validating(object sender, CancelEventArgs e)
		{
			string charset = txtCharSet.Text;
			if (GTG.ValidateCharacterSet(charset))
			{
				e.Cancel = false;
			//	errProvider.SetError(txtCharSet, "Invalid character set.");
			}
			else
			{
				e.Cancel = true;
			//	errProvider.SetError(txtCharSet, "");
			}
		}

		private void txtCharSet_Validated(object sender, EventArgs e)
		{
			string charset = txtCharSet.Text;
			_gtg.CharacterSet = charset;
			drawingBoard.Invalidate();
		}

		private void txtCharSet_Leave(object sender, EventArgs e)
		{
			string charset = txtCharSet.Text;
			if (!GTG.ValidateCharacterSet(charset))
			{
				//status.Set("Invalid character set.");
				if (Focus())
				{
					txtCharSet.Focus();
					txtCharSet.Select();
				}
			}
			else
			{
				_gtg.CharacterSet = charset;
			}
		}

		private void drawingBoard_MouseDown(object sender, MouseEventArgs e)
		{
			Point2F pt = new Point2F(e.X, e.Y);
			if (e.Button != MouseButtons.Left) return;
			switch (uiState)
			{
				case UIState.Normal:

					switch (UpdateSelection(pt))
					{
						case SelectionType.Nothing: //nothing selected
							break; //do nothing
						case SelectionType.State:

							UpdateGTGBars();
							uiState = UIState.MovingState;
							drawingBoard.Invalidate();

							break;
						case SelectionType.Edge:

							UpdateGTGBars();
							uiState = UIState.EdgeSelected;
							drawingBoard.Invalidate();

							break;
						case SelectionType.SelfEdge:

							UpdateGTGBars();
							uiState = UIState.SelfEdgeSelected;
							drawingBoard.Invalidate();

							break;
					}

					break;
				case UIState.CreatingState:

					//Create and Create Strategy
					ChangeStateEdgeCkeckWithoutFiringEvents(true, false);
					selectedState = new State(pt);
					_gtg.States.Add(selectedState);
					selectedState = null;
					drawingBoard.Invalidate();
					//remain on the same state
					/* Create and Select Strategy
					ChangeStateEdgeCkeckWithoutFiringEvents(false, false);
					selectedState = new State(pt);
					_gtg.States.Add(selectedState);
					UpdateGTGBars();
					drawingBoard.Invalidate();
					uiState = UIState.StateSelected;*/
					
					break;
				case UIState.CreatingEdge:

					switch (UpdateSelection(pt))
					{
						case SelectionType.SelfEdge:
						case SelectionType.Edge:
						case SelectionType.Nothing: //nothing selected
							break; //do nothing
						case SelectionType.State:

							edgeStartState = selectedState;
							selectedState = null;
							uiState = UIState.SeekingEndState;
							//drawingBoard.Invalidate();

							break;
					}

					break;
				case UIState.MovingState:

					//Impossible
					
					break;
				case UIState.StateSelected:

					switch (UpdateSelection(pt))
					{
						case SelectionType.Nothing: //nothing selected

							selectedState = null;
							UpdateGTGBars();
							uiState = UIState.Normal;
							drawingBoard.Invalidate();

							break; 
						case SelectionType.State:

							UpdateGTGBars();
							uiState = UIState.MovingState;
							drawingBoard.Invalidate();

							break;
						case SelectionType.Edge:

							selectedState = null;
							UpdateGTGBars();
							uiState = UIState.EdgeSelected;
							drawingBoard.Invalidate();

							break;
						case SelectionType.SelfEdge:

							selectedState = null;
							UpdateGTGBars();
							uiState = UIState.SelfEdgeSelected;
							drawingBoard.Invalidate();

							break;
					}

					break;
				case UIState.EdgeSelected:

					switch (UpdateSelection(pt))
					{
						case SelectionType.Nothing: //nothing selected

							selectedEdge = null;
							selectedEdgeSourceState = null;
							UpdateGTGBars();
							uiState = UIState.Normal;
							drawingBoard.Invalidate();

							break;
						case SelectionType.State:

							selectedEdge = null;
							selectedEdgeSourceState = null;
							UpdateGTGBars();
							uiState = UIState.MovingState;
							drawingBoard.Invalidate();

							break;
						case SelectionType.Edge:

							selectedState = null;
							UpdateGTGBars();
							uiState = UIState.EdgeSelected;
							drawingBoard.Invalidate();

							break;
						case SelectionType.SelfEdge:

							selectedState = null;
							UpdateGTGBars();
							uiState = UIState.SelfEdgeSelected;
							drawingBoard.Invalidate();

							break;
					}
					
					break;
				case UIState.SelfEdgeSelected:

					switch (UpdateSelection(pt))
					{
						case SelectionType.Nothing: //nothing selected

							selectedEdge = null;
							selectedEdgeSourceState = null;
							UpdateGTGBars();
							uiState = UIState.Normal;
							drawingBoard.Invalidate();

							break;
						case SelectionType.State:

							selectedEdge = null;
							selectedEdgeSourceState = null;
							UpdateGTGBars();
							uiState = UIState.MovingState;
							drawingBoard.Invalidate();

							break;
						case SelectionType.Edge:

							selectedState = null;
							UpdateGTGBars();
							uiState = UIState.EdgeSelected;
							drawingBoard.Invalidate();

							break;
						case SelectionType.SelfEdge:

							selectedState = null;
							UpdateGTGBars();
							uiState = UIState.SelfEdgeSelected;
							drawingBoard.Invalidate();

							break;
					}

					break;
				case UIState.SeekingEndState:

					//Impossible
					break;
			}
		}

		private void drawingBoard_MouseUp(object sender, MouseEventArgs e)
		{
			Point2F pt = new Point2F(e.X, e.Y);
			State edgeDst;
			if (e.Button != MouseButtons.Left) return;
			switch (uiState)
			{
				case UIState.Normal:
					break;//do nothing
				case UIState.CreatingState:
					break;//do nothing
				case UIState.CreatingEdge:
					break;//do nothing
				case UIState.MovingState:

					selectedState.Location = pt;
					drawingBoard.Invalidate();
					uiState = UIState.StateSelected;

					break;
				case UIState.StateSelected:
					break;//do nothing
				case UIState.EdgeSelected:
					break;//do nothing
				case UIState.SelfEdgeSelected:
					break;//do nothing
				case UIState.SeekingEndState:

					switch (UpdateSelection(pt))
					{
						case SelectionType.SelfEdge:
						case SelectionType.Edge:
						case SelectionType.Nothing: //nothing selected

							edgeStartState = null;
							uiState = UIState.CreatingEdge;
							//drawingBoard.Invalidate();

							break;
						case SelectionType.State: //The other end of the edge

							edgeDst = selectedState;
							selectedState = null;
							if (edgeStartState == edgeDst) //self edge
							{
								if (edgeStartState.SelfEdge == null)
									edgeStartState.SelfEdge =
										Edge.CreateLabelledEdge(_gtg.CharacterSet, _gtg.CharacterSet, edgeStartState, edgeDst);
								//Create and select Strategy
								/*
								selectedEdge = edgeStartState.SelfEdge;
								selectedEdgeSourceState = edgeStartState;
								edgeStartState = null;
								UpdateGTGBars();
								ChangeStateEdgeCkeckWithoutFiringEvents(false, false);
								uiState = UIState.EdgeSelected;*/
								//Create and Create Strategy
								selectedEdge = null;
								selectedEdgeSourceState = null;
								edgeStartState = null;
								uiState = UIState.CreatingEdge;
								drawingBoard.Invalidate();
							}
							else
							{
								EdgeMatcher edgeCondition = new EdgeMatcher(edgeDst);
								Edge edge = edgeStartState.OutEdges.Find(edgeCondition.EndsWith);
								if (edge == null)
								{
									edge = Edge.CreateLabelledEdge(_gtg.CharacterSet, _gtg.CharacterSet, edgeStartState, edgeDst);
									edgeStartState.OutEdges.Add(edge);
									edgeDst.InEdges.Add(edge);
								}
								//Create and select Strategy
								/*selectedEdge = edge;
								selectedEdgeSourceState = edgeStartState;
								edgeStartState = null;
								UpdateGTGBars();
								ChangeStateEdgeCkeckWithoutFiringEvents(false, false);
								uiState = UIState.EdgeSelected;*/
								//Create and Create Strategy
								selectedEdge = null;
								selectedEdgeSourceState = null;
								edgeStartState = null;
								uiState = UIState.CreatingEdge;
								drawingBoard.Invalidate();
							}

							break;
					}

					break;
			}
		}

		private void drawingBoard_MouseMove(object sender, MouseEventArgs e)
		{
			Point2F pt = new Point2F(e.X, e.Y);
			
			switch (uiState)
			{
				case UIState.MovingState:

					selectedState.Location = pt;
					drawingBoard.Invalidate();
					
					break;
			}
		}

		private void chkAccept_CheckedChanged(object sender, EventArgs e)
		{
			selectedState.IsAccept = chkAccept.Checked;
			drawingBoard.Invalidate();
		}

		private void chkStart_CheckedChanged(object sender, EventArgs e)
		{
			selectedState.IsStart = chkStart.Checked;
			drawingBoard.Invalidate();
		}

		private void btnConvert_Click(object sender, EventArgs e)
		{
			if (!GTG.ValidateCharacterSet(txtCharSet.Text))
			{
				SetStatus("Fatal Error: bad character set.");
			}
			else if (!_gtg.AreAllEdgeLabelsValid())
			{
				SetStatus("Fatal Error: Illegal edge labels.");
			}
			else
			{
				SimplifiedGTG sgtg = _gtg.ToSimplifiedGTG();
				if (sgtg == null)
					SetStatus("Unknown error.");
				else
				{
					string regex;
					if (!sgtg.Process(out regex))
						SetStatus("Unknown error.");
					else
					{
						SetOutput(regex);
					}
				}
			}
		}
		internal void SetCharacterSet(string charSet)
		{
			txtCharSet.Text = charSet;
		}

		private void GTGForm_KeyDown(object sender, KeyEventArgs e)
		{
			if(e.KeyData == Keys.Delete)
			{
				switch( uiState )
				{
					case UIState.StateSelected:

						_gtg.RemoveState(selectedState);
						selectedState = null;
						UpdateGTGBars();
						uiState = UIState.Normal;
						drawingBoard.Invalidate();

						break;
					case UIState.EdgeSelected:

						selectedEdge.Source.OutEdges.Remove( selectedEdge );
						selectedEdge.Target.InEdges.Remove(selectedEdge);
						selectedEdge = null;
						selectedEdgeSourceState = null;
						UpdateGTGBars();
						uiState = UIState.Normal;
						drawingBoard.Invalidate();

						break;
					case UIState.SelfEdgeSelected:

						selectedEdge.Source.SelfEdge = null;
						selectedEdge = null;
						selectedEdgeSourceState = null;
						UpdateGTGBars();
						uiState = UIState.Normal;
						drawingBoard.Invalidate();

						break;
				}
			}
		}

		private void saveToolStripButton_Click(object sender, EventArgs e)
		{
			try
			{
				BinaryFormatter saver = new BinaryFormatter();
				DialogResult dlgResult = dlgSave.ShowDialog(this);
				if (dlgResult == DialogResult.OK)
				{
					Stream stream = dlgSave.OpenFile();
					saver.Serialize(stream, _gtg);
				}
			}
			catch (Exception)
			{
				MessageBox.Show(
					this,
					"Cannot save the GTG",
					"Error",
					MessageBoxButtons.OK,
					MessageBoxIcon.Error);					
			}
		}
	}
}