﻿using System;
using System.Windows.Forms;
using System.Drawing;

namespace Fireball.Windows.Forms.ToolList
{
    public class ToolList:ScrollableControl
    {
        private ToolListItemCollection _items;
        private ToolListRender _render;
        private bool _onOverflow;
        private int _maxHeight;

        public ToolList()
        {
            _items = new ToolListItemCollection();
            _items.AddCollectionItem += new ToolListItemCollection.CollectionChangeDelegate(_items_AddCollectionItem);
            _items.RemoveCollectionItem += new ToolListItemCollection.CollectionChangeDelegate(_items_RemoveCollectionItem);

            _render = new ToolListRenderDefault();
            _maxHeight = 0;

            this.AutoScroll = true;
        }

        void _items_RemoveCollectionItem(object sender, ToolListItemCollection.CollectionChangeEventArgs e)
        {
            for (int i = 0; i < e.ItemsIds.Length; i++)
            {
                this.Controls.RemoveByKey(e.ItemsIds[i]);
            }

            // TODO: aggiornare le posizioni degli elementi che seguono
        }
        void _items_AddCollectionItem(object sender, ToolListItemCollection.CollectionChangeEventArgs e)
        {
            int y = 0;
            if (_items.Count != 1)
            {
                y = _items[_items.Count - 2].Bottom;
            }

            for (int i = 0; i < e.ItemsIds.Length; i++)
            {
                ToolListItem item = _items[e.ItemsIds[i]];
                item.Location = new System.Drawing.Point(0, y);
                item.SetOwner(this);
                item.Width = this.InternalWidth;
                this.Controls.Add(item);

                y += item.Height;
            }
        }

        public ToolListItemCollection Items
        {
            get
            {
                return _items;
            }
        }
        public ToolListRender Render
        {
            get { return _render; }
            set { _render = value; }
        }
        public int MaxHeight
        {
            get
            {
                return _maxHeight;
            }
            set
            {
                _maxHeight = value;
            }
        }

        public void FitToContentHeight()
        {
            int h = 0;
            for (int i = 0; i < _items.Count; i++)
            {
                h += _items[i].Height;
            }
            if(_maxHeight>0)
            {
                if (h > _maxHeight)
                {
                    this.Height = _maxHeight;
                    return;
                }
            }
            this.Height = h;
        }
        public ToolListItem GetItemAt(Point pt)
        {
            for (int i = 0; i < _items.Count; i++)
            {
                if (_items[i].Bounds.Contains(pt))
                {
                    return _items[i];
                }
            }
            return null;
        }

        protected override void OnSizeChanged(EventArgs e)
        {
            for (int i = 0; i < _items.Count; i++)
            {
                _items[i].Width = InternalWidth;
            }
            this.Invalidate(true);
        }
        protected override void OnMouseMove(MouseEventArgs e)
        {
            ToolListItem tli = GetItemAt(e.Location);
            if (tli != null)
            {
                tli.SetItemState(ToolListItem.ToolItemState.Normal);
            }
        }

        internal int InternalWidth
        {
            get
            {
                if (_onOverflow)
                {
                    // TODO: sottrarre la larghezza della scrollbar
                    return this.ClientSize.Width;
                }
                else
                {
                    return this.ClientSize.Width;
                }
            }
        }

    }
}
