﻿using System;
using System.Windows.Forms;
using System.Drawing;

namespace Fireball.Windows.Forms.ToolList
{
    public class ToolListItem :Control
    {
        public enum ToolItemState
        {
            Normal=0,Hot=1,Pressed=2,Disabled=3
        }

        private ToolList _owner;
        private Image _icon;
        private ToolItemState _state;
        private bool _selected;

        public ToolListItem()
        {
            this.Left = 0;
            this.Top = 0;
            this.Height = 26;
            this.Width = 100;

            _state = ToolItemState.Normal;
        }
        public ToolListItem(string text):this()
        {
            this.Text = text;
        }

        internal void SetOwner(ToolList toolList)
        {
            _owner = toolList;
        }
        internal void SetItemState(ToolItemState state)
        {
            _state = state;
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            //this.Width = _owner.InternalWidth;

            _owner.Render.OnDrawItemBackground(new ToolListRender.ToolListDrawItemBackgroundArgs(this,e.Graphics));
            if (_icon != null)
            {
                _owner.Render.OnDrawItemIcon(new ToolListRender.ToolListDrawItemIconArgs(_icon,this,e.Graphics));
            }
            _owner.Render.OnDrawItemText(new ToolListRender.ToolListDrawItemTextArgs(this.Text, this.Font, this, e.Graphics));
        }

        public Image Icon
        {
            get
            {
                return _icon;
            }
            set
            {
                _icon = value;
            }
        }

        public ToolItemState State
        {
            get
            {
                return _state;
            }
        }

        public bool Selected
        {
            get
            {
                return _selected;
            }

        }
    }
}
