﻿using System;
using Fireball.Collections.Generic;

namespace Fireball.Windows.Forms.ToolList
{
    public class ToolListItemCollection //:IKeyedCollection<ToolListItem>
    {
        public class CollectionChangeEventArgs
        {
            public string[] ItemsIds;

            public CollectionChangeEventArgs(string[] itemsIds)
            {
                ItemsIds = itemsIds;
            }
        }

        public delegate void CollectionChangeDelegate(object sender, CollectionChangeEventArgs e);

        public event CollectionChangeDelegate AddCollectionItem;
        public event CollectionChangeDelegate RemoveCollectionItem;
        public event CollectionChangeDelegate ChangeCollectionItem;

        private KeyedCollection<ToolListItem> _coll;

        public ToolListItemCollection()
        {
            _coll = new KeyedCollection<ToolListItem>();
        }

        #region IKeyedCollection<ToolListItem> Members

        public int Count
        {
            get { return _coll.Count; }
        }
        public ToolListItem this[int index]
        {
            get
            {
                return _coll[index];
            }
            set
            {
                _coll[index] = value;
                // TODO: if (ChangeCollectionItem != null) this.ChangeCollectionItem(this, new string[] { _coll.Keys[index] });
            }
        }
        public ToolListItem this[string key]
        {
            get
            {
                return _coll[key];
            }
            set
            {
                _coll[key] = value;
                // TODO: if (ChangeCollectionItem != null) this.ChangeCollectionItem(this, new string[] { _coll.Keys[index] });
            }
        }
        public string[] Keys
        {
            get { return _coll.Keys; }
        }

        public void Add(string key, ToolListItem item)
        {
            _coll.Add(key, item);
            if (AddCollectionItem != null) this.AddCollectionItem(this, new CollectionChangeEventArgs(new string[] { key }));
        }
        public void AddRange(string[] keys, ToolListItem[] items)
        {
            _coll.AddRange(keys, items);
            if (AddCollectionItem != null) this.AddCollectionItem(this, new CollectionChangeEventArgs(keys));
        }
        public void Clear()
        {
            _coll.Clear();
        }
        public void Insert(int index, string key, ToolListItem item)
        {
            _coll.Insert(index, key, item);
            if (AddCollectionItem != null) this.AddCollectionItem(this, new CollectionChangeEventArgs(new string[] { key }));
        }
        public void RemoveAt(int index)
        {
            string key = _coll.Keys[index];
            _coll.RemoveAt(index);
            if (RemoveCollectionItem != null) this.RemoveCollectionItem(this, new CollectionChangeEventArgs(new string[] { key }));
        }
        public void Remove(string key)
        {
            _coll.Remove(key);
            if (RemoveCollectionItem != null) this.RemoveCollectionItem(this, new CollectionChangeEventArgs(new string[] { key }));
        }
        public void Remove(ToolListItem item)
        {
            string key = _coll.Keys[_coll.IndexOf(item)];
            _coll.Remove(item);
            if (RemoveCollectionItem != null) this.RemoveCollectionItem(this, new CollectionChangeEventArgs(new string[] { key }));
        }
        public bool Contains(string key)
        {
            return _coll.Contains(key);
        }
        public bool Contains(ToolListItem item)
        {
            return _coll.Contains(item);
        }
        public int IndexOf(string key)
        {
            return _coll.IndexOf(key);
        }
        public int IndexOf(ToolListItem item)
        {
            return _coll.IndexOf(item);
        }
        public ToolListItem[] GetItems()
        {
            return _coll.GetItems();
        }
        public ToolListItem[] GetItems(int startIndex)
        {
            return _coll.GetItems(startIndex);
        }
        public ToolListItem[] GetItems(int startIndex, int finalIndex)
        {
            return _coll.GetItems(startIndex,finalIndex);
        }
        public void Sort()
        {
            _coll.Sort();
        }

        #endregion
    }
}
