﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace Fireball.Windows.Forms.ToolList
{
    public abstract class ToolListRender
    {
        public class ToolListDrawItemBackgroundArgs
        {
            public ToolListItem Item;
            public Graphics Graphics;

            public ToolListDrawItemBackgroundArgs(ToolListItem item, Graphics graphics)
            {
                this.Item = item;
                this.Graphics = graphics;
            }
        }
        public class ToolListDrawItemTextArgs
        {
            public ToolListItem Item;
            public Graphics Graphics;
            public string Text;
            public Font Font;

            public ToolListDrawItemTextArgs(string text,Font font, ToolListItem item, Graphics graphics)
            {
                this.Text = text;
                this.Font = font;
                this.Item = item;
                this.Graphics = graphics;
            }
        }
        public class ToolListDrawItemIconArgs
        {
            public ToolListItem Item;
            public Graphics Graphics;
            public Image Icon;

            public ToolListDrawItemIconArgs(Image icon, ToolListItem item, Graphics graphics)
            {
                this.Icon = icon;
                this.Item = item;
                this.Graphics = graphics;
            }
        }

        public abstract void OnDrawItemBackground(ToolListDrawItemBackgroundArgs e);
        public abstract void OnDrawItemText(ToolListDrawItemTextArgs e);
        public abstract void OnDrawItemIcon(ToolListDrawItemIconArgs e);
    }

    public class ToolListRenderDefault:ToolListRender
    {
        public override void OnDrawItemBackground(ToolListDrawItemBackgroundArgs e)
        {
            Rectangle rect=new Rectangle(0,0,e.Item.Bounds.Width,e.Item.Bounds.Height);

            switch (e.Item.State)
            {
                case ToolListItem.ToolItemState.Disabled:

                    break;
                case ToolListItem.ToolItemState.Normal:
                    LinearGradientBrush lgb = new LinearGradientBrush(rect, SystemColors.Control, SystemColors.ControlLight, 80f);
                    e.Graphics.FillRectangle(lgb, rect);
                    break;
                case ToolListItem.ToolItemState.Hot:
                    lgb = new LinearGradientBrush(rect, SystemColors.Control, SystemColors.ControlLight, 80f);
                    e.Graphics.FillRectangle(lgb, rect);
                    break;
                case ToolListItem.ToolItemState.Pressed:

                    break;
            }

            
        }
        public override void OnDrawItemText(ToolListDrawItemTextArgs e)
        {
            Point pt = new Point(3, e.Item.Height / 2);
            if (e.Item.Icon != null) pt.X += 16 + 3;
            StringFormat sf=new StringFormat();
            sf.LineAlignment=StringAlignment.Center;
            e.Graphics.DrawString(e.Text, e.Font, SystemBrushes.WindowText, pt, sf);
        }
        public override void OnDrawItemIcon(ToolListDrawItemIconArgs e)
        {
            Point pt = new Point(3, (e.Item.Height - e.Icon.Height) / 2);
            e.Graphics.DrawImage(e.Icon, pt.X,pt.Y,16,16);
        }
    }
}
