﻿//Fireball.Windows.Forms.Toolbox is an Implementation of a toolbox 
//control like the whidbeytoolbox
//
//Copyright (C) 2004/2005 Sebastian Faltoni
//
//This library is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.
//
//This library is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.
//
//You should have received a copy of the GNU Lesser General Public
//License along with this library; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  US

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Text;

namespace Fireball.Windows.Forms.Toolbox
{
    public class Toolbox:Control
    {
		ToolboxCategoryCollection m_Categories = null;
		LinearGradientBrush m_BackBrush = null;

        public Toolbox()
        {
            this.SetStyle(ControlStyles.AllPaintingInWmPaint
                | ControlStyles.UserPaint
                | ControlStyles.OptimizedDoubleBuffer
                | ControlStyles.ResizeRedraw
                | ControlStyles.ContainerControl
                , true);

			m_Categories = new ToolboxCategoryCollection();
        }


		public void AddCategory(ToolboxCategory category)
		{
			m_Categories.Add(category);
		}

		public void AddItem(ToolboxCategory category, ToolboxItem item)
		{
			int index = m_Categories.IndexOf(category);
			if (index == -1)
				throw new Exception("The category on arg1 isnt on Categories Collections Add it first!");

			m_Categories[index].Items.Add(item);
		}

		public void RemoveCategory(ToolboxCategory category)
		{			
			m_Categories.Remove(category);
		}

		public void RemoveItem(ToolboxItem item)
		{
			if(item.Parent == null)
				throw new Exception("Can't Find a category that contain the item");

			int index = m_Categories.IndexOf(item.Parent);

			if (index == -1)
				throw new Exception("Can't Find a category that contain the item");
			m_Categories[index].Items.Remove(item);
		}

		protected override void OnResize(EventArgs e)
		{
			for (int i = 0; i < m_Categories.Count; i++)
			{
				ToolboxCategory current = m_Categories[i];
				ToolboxCategory prev = null;

				if (i > 0)
					prev = m_Categories[i - 1];


				current.Left = 0;
				current.Top = (prev != null ? prev.Top + prev.Height : 0);
				current.Width = this.ClientSize.Width;
			}


			Color second = ControlPaint.Light(SystemColors.ControlDark);

			Rectangle client = this.ClientRectangle;

			client.Width += 130;

            try
            {
                m_BackBrush = new LinearGradientBrush(client, SystemColors.Control, second,
                    LinearGradientMode.Horizontal);
            }
            catch
            {

            }
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			Rectangle client = this.ClientRectangle;
			client.Width--;
			client.Height--;

			e.Graphics.FillRectangle(m_BackBrush, this.ClientRectangle);
			e.Graphics.DrawRectangle(SystemPens.ControlDark, client);

			for (int i = 0; i < m_Categories.Count; i++)
			{
				ToolboxCategory current = m_Categories[i];
			}
		}
	}
}
