﻿//Fireball.Windows.Forms.Toolbox is an Implementation of a toolbox 
//control like the whidbeytoolbox
//
//Copyright (C) 2004/2005 Sebastian Faltoni
//
//This library is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.
//
//This library is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.
//
//You should have received a copy of the GNU Lesser General Public
//License along with this library; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  US


using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Text;

namespace Fireball.Windows.Forms.Toolbox
{
    public class ToolboxCategory:Control
    {
        const int TITLE_HEIGHT = 16;
        const int TOGGLE_BUTTON_SIZE = 8;
        bool m_IsExpanded = false;

        public event EventHandler Toggle;

        Panel m_Container = null;

		ToolboxItemCollection m_Items = null;


        public ToolboxCategory()
        {
            this.SetStyle(ControlStyles.AllPaintingInWmPaint
                | ControlStyles.UserPaint
                | ControlStyles.OptimizedDoubleBuffer
                | ControlStyles.ResizeRedraw
                | ControlStyles.ContainerControl
                , true);

            m_Container = new Panel();

            this.Controls.Add(m_Container);

			m_Items = new ToolboxItemCollection(this);
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            Rectangle brRect = new Rectangle(               
                new Point(0, 0), new Size(this.Width, TITLE_HEIGHT+15)
                );

            LinearGradientBrush brush = new LinearGradientBrush(
                brRect, SystemColors.Highlight, Color.FromArgb(30, SystemColors.Highlight)
                ,LinearGradientMode.Vertical);

            Rectangle title = new Rectangle(0,0,this.Width,TITLE_HEIGHT);

            e.Graphics.FillRectangle(brush, title);

            Rectangle textRect = new Rectangle(16, 0, this.Width-14, TITLE_HEIGHT);



            string DrawableText = GetTitleText(e.Graphics, this.Text, this.Width - 10);

            StringFormat flags = new StringFormat();

            flags.LineAlignment = StringAlignment.Center;

            e.Graphics.DrawString(this.Text, this.Font,
                new SolidBrush(this.ForeColor), textRect, flags);
            
            DrawToggle(e.Graphics,new Point(4,TITLE_HEIGHT/2-4));
             
        }

        Rectangle ToogleRect()
        {
            Point pt = new Point(4, TITLE_HEIGHT / 2 - 4);

            return new Rectangle(pt, 
                new Size(TOGGLE_BUTTON_SIZE, TOGGLE_BUTTON_SIZE));
        }

        string GetTitleText(Graphics gfx ,string text,int desiredWidth)
        {
            SizeF sz = gfx.MeasureString(this.Text, this.Font);

            int charWidth = (int)gfx.MeasureString("#", this.Font).Width;

            int charCount = desiredWidth / charWidth;

            if (charCount > text.Length)
                return text;
            else
                return text.Substring(0, charCount);
        }

        void DrawToggle(Graphics gfx,Point position)
        {
            if (IsExpanded)
            {
                gfx.DrawRectangle(SystemPens.WindowText, new Rectangle(position,
                    new Size(TOGGLE_BUTTON_SIZE, TOGGLE_BUTTON_SIZE)));

                DrawToogleLine(gfx, LineType.Horizontal, position);
            }
            else
            {
                gfx.DrawRectangle(SystemPens.WindowText, new Rectangle(position,
                    new Size(TOGGLE_BUTTON_SIZE, TOGGLE_BUTTON_SIZE)));

                DrawToogleLine(gfx, LineType.Horizontal, position);
                DrawToogleLine(gfx, LineType.Vertical,position);
            }
        }

        enum LineType
        {
            Horizontal = 0,
            Vertical = 1
        }

        void DrawToogleLine(Graphics gfx, LineType type, Point position)
        {
            if (type == LineType.Horizontal)
            {
                int x = position.X + 2;

                int w = TOGGLE_BUTTON_SIZE - 2;

                int y = position.Y + (TOGGLE_BUTTON_SIZE / 2);

                gfx.DrawLine(SystemPens.WindowText, x, y, x + w, y);
            }
            else
            {
                int x = position.X + (TOGGLE_BUTTON_SIZE / 2);

                int y = position.Y + 2;

                int height = position.Y + (TOGGLE_BUTTON_SIZE - 2);

                gfx.DrawLine(SystemPens.WindowText, x, y, x, height);
            }
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            if (ToogleRect().Contains(e.Location))
            {
                m_IsExpanded = !(m_IsExpanded);
                this.Invalidate();
                if (Toggle != null)
                {
                    Toggle(this, new EventArgs());
                }
            }
        }

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);
            m_Container.Top = TITLE_HEIGHT;
            m_Container.Left = 0;
            m_Container.Width = m_Container.Width;
            m_Container.Height = this.Height - m_Container.Top;
        }

        public bool IsExpanded
        {
            get
            {
                return m_IsExpanded;
            }
        }

		public ToolboxItemCollection Items
		{
			get
			{
				return m_Items;
			}
		}
    }
}
