#region License
//    Copyright (C) 2005  Sebastian Faltoni
//	  Copyright (C) 2005  Riccardo Marzi
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#endregion License

#region Using directives

using System;
using System.IO;
using System.Drawing;
using System.Text;
using System.Runtime.Serialization;
using sys = System;

#endregion

namespace Fireball.Windows.Forms.Internal
{
    public class Resourcers
    {
        private static object DecodeResource(string Resource, string SystemType)
        {
            Resource = Resource.Replace("\n", "");
            Resource = Resource.Replace("\r", "");
            Resource = Resource.Replace("\t", "");
            Resource = Resource.Replace(" ", "");

            return DecodeResourceFromBase64(Resource, SystemType);
        }

        public static Image GetDecodedImage(string resource)
        {
            return (Image)DecodeResource(resource, "System.Drawing.Image");
        }

        public static Icon GetDecodedIcon(string resource)
        {
            return (Icon)DecodeResource(resource, "System.Drawing.Icon");
        }

        public static string GetDecodedString(string resource)
        {
            return (string)DecodeResource(resource, "System.String");
        }

        private static object DecodeResourceFromBase64(string resource, string restype)
        {
            object retobj = null;
            byte[] memBytes = Convert.FromBase64String(resource);
            //IFormatter formatter = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
            MemoryStream stream = new MemoryStream(memBytes);

            if (restype == "System.String")
            {
                retobj = sys.Text.Encoding.UTF8.GetString(memBytes);
            }
            else
            {
                if (restype == "System.Drawing.Image")
                {
                    retobj = Image.FromStream(stream);
                }
                if (restype == "System.Drawing.Icon")
                {
                    retobj = new Icon(stream);
                }
               /* if (restype == "System.Windows.Forms.Cursor")
                {
                    retobj = new System.Windows.Forms.Cursor(stream);
                }
                else if (restype == "Sound")
                {
                    //TODO: altro
                    retobj = new SoundPlayer(stream);
                    return retobj;
                }*/
            }
            stream.Close();
            return retobj;
        }

        private static byte[] StringToByteArray(string str)
        {
            MemoryStream memstream;
            StreamWriter strw;

            memstream = new MemoryStream();
            strw = new StreamWriter(memstream);
            strw.Write(str);
            strw.Flush();

            byte[] buffByte;
            try
            {
                buffByte = new byte[memstream.Length];
                memstream.Position = 0;
                //long bytesRead = memstream.Read(buffByte, 0, (int)memstream.Length);
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message, ex);
            }
            return buffByte;
        }

        public static string EncodeResource(string res, string resourceType)
        {
            return CharArrayToString(EncodeResourceToBase64(res, resourceType));
        }

        private static string CharArrayToString(char[] charArray)
        {
            //string str=null;
            StringBuilder sb = new StringBuilder(charArray.Length);
            for (int n = 0; n < charArray.Length; n++)
            {
                sb.Append(charArray[n]);
                //str+=charArray[n];
            }
            return sb.ToString();
        }

        private static char[] EncodeResourceToBase64(string resource, string restype)
        {
            byte[] binData;
            if (restype == "System.String")
            {
                // se la risorsa è una stringa
                // la trasforma in un array di byte
                binData = StringToByteArray(resource);
            }
            else
            {
                // carica il file come stream
                // e lo trsforma in array di byte
                FileStream sfile;
                try
                {
                    sfile = new FileStream(resource, FileMode.Open, FileAccess.Read);
                    binData = new byte[sfile.Length];
                    //long bytesRead = sfile.Read(binData, 0, (int)sfile.Length);
                    sfile.Close();
                }
                catch (Exception ex)
                {
                    throw ex;
                }
            }

            // Converte l'input binario in output base64 UUEncoded
            // Ogni 3 byte della sequenza diventano 4 caratteri dell'array
            long arrayLen = (long)((4.0d / 3.0d) * binData.Length);
            // Se l'array non è divisibile per 4, passa al successivo multiplo di 4
            if ((arrayLen % 4) != 0)
            {
                arrayLen += 4 - (arrayLen % 4);
            }

            char[] base64CharArray = new char[arrayLen];
            try
            {
                Convert.ToBase64CharArray(binData, 0, binData.Length, base64CharArray, 0);
                return base64CharArray;
            }
            catch (ArgumentNullException ex)
            {
                throw new ArgumentNullException(ex.ParamName, ex.Message);
            }
            catch (ArgumentOutOfRangeException ex)
            {
                throw new ArgumentOutOfRangeException(ex.ParamName, ex.Message);
            }
        }
    }
}
