using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace IDE.Forms.CCW
{
    public partial class NewProject : Form
    {
        bool manuallyChosen = true; //Has the user selected from the box or manually?
        
        public string FilePath = "";
        public string FileName = "";

        public NewProject()
        {
            InitializeComponent();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            if (txtProjectName.Text=="")
            {
                MessageBox.Show("Please specify a name for your project.", "Invalid Project Name", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            try
            {
                while (txtFolderPath.Text[txtFolderPath.Text.Length - 1] == '\\')
                    txtFolderPath.Text = txtFolderPath.Text.Substring(0, txtFolderPath.Text.Length - 1);
                string folderPath = txtFolderPath.Text + "\\" + txtProjectName.Text;

                if (manuallyChosen && !Directory.Exists(folderPath))
                {
                    //DialogResult answer = MessageBox.Show("The folder you've chosen doesn't exist. Do you want to create it?",
                    //                "Folder Does Not Exist",
                    //                MessageBoxButtons.YesNoCancel,
                    //                MessageBoxIcon.Question);
                    //if (answer == DialogResult.Yes)
                        Directory.CreateDirectory(folderPath);
                    //else
                    //    return;
                }
                else if (manuallyChosen && Directory.GetFiles(folderPath).Length != 0
                      || manuallyChosen && Directory.GetDirectories(folderPath).Length != 0)
                {
                    DialogResult answer = MessageBox.Show("The folder you've chosen already exists, and it already contains some other stuff. Continue anyway?",
                                    "Folder Already Exists",
                                    MessageBoxButtons.YesNo,
                                    MessageBoxIcon.Question);
                    if (answer == DialogResult.No)
                        return;
                }
                else if (manuallyChosen && Directory.GetFiles(folderPath).Length == 0
                      || manuallyChosen && Directory.GetDirectories(folderPath).Length == 0)
                {
                    DialogResult answer = MessageBox.Show("The folder you've chosen already exists, but it's empty. Continue with this folder?",
                                    "Folder Already Exists",
                                    MessageBoxButtons.YesNo,
                                    MessageBoxIcon.Question);
                    if (answer == DialogResult.No)
                        return;
                }
                else if (!manuallyChosen && Directory.Exists(folderPath) && Directory.GetFiles(folderPath).Length != 0
                      || !manuallyChosen && Directory.Exists(folderPath) && Directory.GetDirectories(folderPath).Length != 0)
                {
                    DialogResult answer = MessageBox.Show("The folder you've chosen already contains some other stuff. Continue anyway?",
                                    "Folder Already Exists",
                                    MessageBoxButtons.YesNo,
                                    MessageBoxIcon.Question);
                    if (answer == DialogResult.No)
                        return;
                }
                else if (!manuallyChosen && !Directory.Exists(folderPath))
                    Directory.CreateDirectory(folderPath);

                //MessageBox.Show("Your files will be added to your project's folder at " + folderPath,
                //                "Successful Project Initiation",
                //                MessageBoxButtons.OK,
                //                MessageBoxIcon.Information);

                this.DialogResult = DialogResult.OK;
                FilePath = txtFolderPath.Text;
                FileName = txtProjectName.Text;
                this.Close();
            }
            catch (Exception exception)
            {
                MessageBox.Show(exception.Message);
            }
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            if (openFolder.ShowDialog() == DialogResult.OK)
            {
                manuallyChosen = false;
                txtFolderPath.Text = openFolder.SelectedPath;
            }
        }

        private void NewProject_Load(object sender, EventArgs e)
        {
            //A default project name...
            txtProjectName.Text = "CCW_" + DateTime.Now.Day + "-" + DateTime.Now.Month + "-" + DateTime.Now.Year;
            txtProjectName.Focus();
            txtProjectName.SelectAll();
        }
    }
}