﻿%option CharacterSet = [\x0-\x7Fأ-يأبتثجحخدذرزسشصضطظعغفقكلمنهويئءؤلاىآةوزظطكمنتالبييسشضصثثقففغغعههخحخحجد1233456767890ًٌَُّلإإ‘لأأٍِْلآ’]
%option Language = C#
%option ClassName = ArabicExpressionTranslator
%option FunctionName = Translate
%option ReturnType = bool
%option FileName = D:\C++ projects\Compilers\ArabicExpression\Translator
%option CompressionTechnique = Redundancy
%option EOFAction = true
%option InvalidTokenAction = false
%{
%}
Letter		[أيأبتثجحخدذرزسشصضطظعغفقكلمنهويئءؤلاىآةوزظطكمنتالبييسشضصثثقففغغعههخحخحجدًٌَُّلإإ‘لأأٍِْلآ]
//Next, we provide the macros...
WhiteSpace	(" "|[\t\r\n])+
Digit		[0-9]
Number		({Digit}+(","{Digit}+)?)|({Digit}*","{Digit}+)
Identifier	({Letter}|[_])({Digit}|{Letter})*
Pi			ط
Sin			جا
Cos			جتا				
Tan			ظا				
Cot			ظتا
Sec			قا
Csc			قتا
Log			لو

Sinh		[Ss][Ii][Nn][Hh]			
Cosh		[Cc][Oo][Ss][Hh]			
Tanh		[Tt][Aa][Nn][Hh]			
Coth		[Cc][Oo][Tt][Hh]			
Sech		[Ss][Ee][Cc][Hh]			
Csch		[Cc][Ss][Cc][Hh]				
Arcsin		[Aa][Rr][Cc][Ss][Ii][Nn]	
Arccos		[Aa][Rr][Cc][Cc][Oo][Ss]	
Arctan		[Aa][Rr][Cc][Tt][Aa][Nn]	
Arccot		[Aa][Rr][Cc][Cc][Oo][Tt]	
Arcsec		[Aa][Rr][Cc][Ss][Ee][Cc]	
Arccsc		[Aa][Rr][Cc][Cc][Ss][Cc]	
Arcsinh		[Aa][Rr][Cc][Ss][Ii][Nn][Hh]
Arccosh		[Aa][Rr][Cc][Cc][Oo][Ss][Hh]
Arctanh		[Aa][Rr][Cc][Tt][Aa][Nn][Hh]
Arccoth		[Aa][Rr][Cc][Cc][Oo][Tt][Hh]
Arcsech		[Aa][Rr][Cc][Ss][Ee][Cc][Hh]
Arccsch		[Aa][Rr][Cc][Cc][Ss][Cc][Hh]
Ln			[Ll][Nn]

%%
//Class Def
static void Main()
{
	string					fileName;
	StreamReader			sReader;
	ArabicExpressionScanner scanner;
	
	Append("Enter file path: ");
	fileName = Console.ReadLine();
	sReader = new StreamReader(fileName, true);
	//AppendLine(sReader.ReadLine());
	scanner = new ArabicExpressionScanner( sReader );
	scanner.GetToken();
	AppendLine();
}

private Hashtable IdentifierTable = new Hashtable();
private string englishExp = "";
private void Append(string str)
{
	if(englishExp == null)
		englishExp = "";
	englishExp += str;
}
public string EnglishExpression
{
	get
	{return englishExp;}
	set
	{englishExp = value;}
}
%%
 /*******************Machine Definition********************/
{WhiteSpace}	Append(Lexeme);
{Number}		{
					Append(Lexeme);
				}
"+"				{Append(Lexeme);}
"-"				{Append(Lexeme);}
"*"				{Append(Lexeme);}
"\\"			{Append('/');}
"%"				{Append(Lexeme);}
"^"				{Append(Lexeme);}
{Pi}			{Append("Pi");}
{Sin}			{Append("sin");}
{Cos}			{Append("cos");}
{Tan}			{Append("tan");}
{Cot}			{Append("cot");}
{Sec}			{Append("sec");}
{Csc}			{Append("csc");}
	{Sinh}			{Append("sinh");}
	{Cosh}			{Append("cosh");}
	{Tanh}			{Append("tanh");}
	{Coth}			{Append("coth");}
	{Sech}			{Append("sech");}
	{Csch}			{Append("csch");}
	{Arcsin}		{Append("arcsin");}
	{Arccos}		{Append("arccos");}
	{Arctan}		{Append("arctan");}
	{Arccot}		{Append("arccot");}
	{Arcsec}		{Append("arcsec");}
	{Arccsc}		{Append("arccsc");}
	{Arcsinh}		{Append("arcsinh");}
	{Arccosh}		{Append("arccosh");}
	{Arctanh}		{Append("arctanh");}
	{Arccoth}		{Append("arccoth");}
	{Arcsech}		{Append("arcsech");}
	{Arccsch}		{Append("arccsch");}
	{Ln}			{Append("ln");}
{Log}			{Append("log");}
ت				{Append("i");}
ه				{Append("e");}
س				{Append("x");}
ص				{Append("y");}
ع				{Append("z");}
{Identifier}	{
					string str = "";
					int count;
					if(!IdentifierTable.Contains( Lexeme ))
					{
						count = IdentifierTable.Count;
						str = "Var_" + count.ToString();
						IdentifierTable.Add( Lexeme, str ); 
					}
					else
						str = (string)IdentifierTable[Lexeme];
					Append( str );
				}
")"				{Append(")");}
"("				{Append("(");}
"]"				{Append("]");}
"["				{Append("[");}
,				{Append(Lexeme);}
.				{Append("InvalidToken");}
%%
//Inside namespace

%%

%%