#pragma once
template<class Slot, int Power>
class BoolArray
{
private:
	static const int BitsPerSlotMinusOne = (sizeof(Slot)<<3) - 1;
			//BitsPerSlotMinusOne = bytesPerSlot * 8 - 1
	Slot *array;
//	int length;
public:
	//count : Number of booleans to be stored.
	BoolArray(int numberOfBitsRequired, BOOL initialValue = FALSE)
	{
		//NumberOfSlots = numberOfBitsRequired / BitsPerSlot + 1
		//BitsPerSlot must be a power of 2
		//Then, NumberOfSlots = numberOfBitsRequired>>Power + 1
		int numberOfSlots = (numberOfBitsRequired>>Power) + 1;
		array = new Slot[numberOfSlots];
		if(initialValue == FALSE)
			memset(array, 0, numberOfSlots * sizeof(Slot));
		else
			memset(array, ~0, numberOfSlots * sizeof(Slot));

#ifdef	BOOLARRAY_DEBUG
		cout<<"----------------------"<<endl<<endl;
		cout<<"BoolArray::BoolArray(int, BOOL) Debug output:"<<endl<<endl;
		cout<<"Number of bits required = "<<numberOfBitsRequired<<endl;
		cout<<"Initial Value = "<<((initialValue)?"TRUE":"FALSE")<<endl;
		cout<<"sizeof(Slot) = "<<sizeof(Slot)<<" bytes"<<endl;
		cout<<"Number of bits per slot = "<<BitsPerSlotMinusOne+1<<endl;
		cout<<"Number of slots = "<<numberOfSlots<<endl;
		cout<<endl<<"BoolArray::BoolArray(int, BOOL) Debug output END"<<endl;
		cout<<"----------------------"<<endl<<endl;
#endif
	}
    BOOL operator [](int index)	const
	{
		Slot mask = index & BitsPerSlotMinusOne;
		int slotIndex = index>>Power;				//index / BitsPerSlot
		int bitIndex = index & BitsPerSlotMinusOne; //index % BitsPerSlot
		return array[slotIndex]>>bitIndex & 1;
	}
	void Set(int index, BOOL value)
	{
		int slotIndex = index>>Power;				//index / BitsPerSlot
		int bitIndex = index & BitsPerSlotMinusOne; //index % BitsPerSlot
#ifdef	BOOLARRAY_DEBUG
		cout<<"----------------------"<<endl<<endl;
		cout<<"BoolArray::Set(int index, BOOL value) Debug output:"<<endl<<endl;
		cout<<"index = "<<index<<endl;
		cout<<"slotIndex = "<<slotIndex<<endl;
		cout<<"BitIndex = "<<(index&BitsPerSlotMinusOne)<<endl;
		cout<<"value = "<<(value?"TRUE":"FALSE")<<endl;
#endif

		Slot bit = (value)?1:0;
		bit = bit<<bitIndex;
		Slot mask = 1;
		mask = mask<<bitIndex;
		mask = ~mask;

#ifdef BOOLARRAY_DEBUG
		cout<<"bit = "<<resetiosflags(ios::hex)<<(unsigned int)bit<<endl;
		cout<<"mask = "<<resetiosflags(ios::hex)<<(unsigned int)mask<<endl;
		cout<<endl<<"BoolArray::Set(int index, BOOL value) Debug output END"<<endl;
		cout<<"----------------------"<<endl<<endl;
#endif

		array[slotIndex] = (array[slotIndex] & mask) | bit;
	}
	~BoolArray()
	{
		delete[] array;
	}
};
template<class Slot, int Power>
void ShowBoolArray(const BoolArray<Slot, Power> &array, int max)
{
	for(int i=0;i<max;i++)
		cout<<"boolArray["<<i<<"] = "<<((array[i])?"TRUE":"FALSE")
			<<endl;
}