#pragma once
namespace Automata
{
template<class Slot, int Power>
class BoolArray
{
private:
	static const int BitsPerSlotMinusOne = (sizeof(Slot)<<3) - 1;
	Slot *array;
	int numberOfBits;
public:
	//count : Number of booleans to be stored.
	BoolArray(int numberOfBitsRequired, BOOL initialValue = FALSE) : array(NULL)
	{
		Resize(numberOfBitsRequired, initialValue);
	}
	BoolArray() : array(NULL)
	{
	}
	BoolArray(const BoolArray<Slot, Power> &barray)
	{
		numberOfBits = barray.numberOfBits;
		int numberOfSlots = NumberOfSlots();
		array = new Slot[numberOfSlots];
		memcpy(array, barray.array, numberOfSlots*sizeof(Slot));
	}
	BoolArray<Slot, Power>& operator =(const BoolArray<Slot, Power> &barr)
	{
		if(&barr == this)	return *this;
		if(array != NULL)
		{	delete[] array;		array = NULL;	}
		numberOfBits = barr.numberOfBits;
		int numberOfSlots = NumberOfSlots();
		array = new Slot[numberOfSlots];
		memcpy(array, barr.array, numberOfSlots*sizeof(Slot));
		return *this;
	}
	void Resize(int numberOfBitsRequired, BOOL initialValue = FALSE)
	{
		if(array!=NULL)
			delete[] array;
		numberOfBits = numberOfBitsRequired;
		int numberOfSlots = NumberOfSlots();
		array = new Slot[numberOfSlots];
		if(initialValue == FALSE)
			memset(array, 0, numberOfSlots * sizeof(Slot));
		else
			memset(array, ~0, numberOfSlots * sizeof(Slot));
	}
    BOOL operator [](int index)	const
	{
		Slot mask = index & BitsPerSlotMinusOne;
		int slotIndex = index>>Power;				//index / BitsPerSlot
		int bitIndex = index & BitsPerSlotMinusOne; //index % BitsPerSlot
		return array[slotIndex]>>bitIndex & 1;
	}
	void Set(int index, BOOL value)
	{
		int slotIndex = index>>Power;				//index / BitsPerSlot
		int bitIndex = index & BitsPerSlotMinusOne; //index % BitsPerSlot
		Slot bit = (value)?1:0;
		bit = bit<<bitIndex;
		Slot mask = 1;
		mask = mask<<bitIndex;
		mask = ~mask;
		array[slotIndex] = (array[slotIndex] & mask) | bit;
	}
	int NumberOfBitsRequired() const
	{
		return numberOfBits;
	}
	int NumberOfSlots() const
	{
		int slots = numberOfBits>>Power;
		int extraSlot = (numberOfBits&BitsPerSlotMinusOne) != 0;
		return slots + extraSlot;
	}
	int NumberOfBitsSupported() const
	{
		return NumberOfSlots()<<Power;
	}
	Slot& GetSlot(int index)
	{
		return array[index];
	}
	const Slot& GetSlot(int index) const
	{
		return array[index];
	}
	~BoolArray()
	{
		if(array != NULL)
		{	delete[] array;
			array = NULL;
		}
	}
};
};