//top cpp

#include "LineNumberer.h"

using namespace Testing;
#define LEX_CLASS LineNumberer

const int LineNumberer::startStateIndex = 0;
const UTCHAR LineNumberer::startSymbol = 0;
const UTCHAR LineNumberer::finalSymbol = 127;

const int LineNumberer::transitionTable[2][128] = 
{
{0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
,{-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1}
};

const BOOL LineNumberer::accepting[2] = 
{
FALSE
,TRUE
};

int LineNumberer::Scan()
{
	//restore everything
	lexeme.clear();
	currentState = startStateIndex;
	lastAccepting = ErrorState;
	backupLength = 0;
	fin.Backup();

	//holds the state just before error state!
	int prevState = 0;
	
	for(;;)
	{
		//read one more character
		currentChar = fin.Peek();

		if(endOfFile || currentChar == TEOF && lexeme.empty())
		{//end of file with no token in hands
		
return -1;
			break;		
		}
		else
		{
			if(currentChar == TEOF && !lexeme.empty())
			{
                prevState = currentState;
				currentState = ErrorState;
				endOfFile = TRUE;
			}
			else
			{
				fin.Advance();
				lexeme.push_back(currentChar);
				backupLength++;

				//get the next state
				prevState = currentState;
				currentState = 
					transitionTable[currentState][indexOf(currentChar)];
			}

			//is there an error?
			if(currentState != ErrorState)
			{//no error:

				//if it is an accepting state,save it
				if(accepting[currentState])
				{
					fin.ReplaceLastBackup();
					backupLength = 0;
					lastAccepting = currentState;
				}
			}
			else
			{//error state:

				if(lastAccepting == ErrorState)
				{
return -2;
					break;
				}
				else
				{//accepting
					//restore the input to the last accepting state
					fin.Restore();
					lexeme = lexeme.substr(0,lexeme.length() - backupLength);

switch(prevState){
case 1:
	{
		tout<<CodeStream().LineNo()<<_T(": ")<<Lexeme();//WriteLine(CodeStream.LineNo.ToString()+": "+Lexeme);

	}
	break;
}
					break;
				}
			}//end of error state
		}
	}
	return 0;
}

/****************************************************************************************/
//members of the code stream class
int LEX_CLASS::CodeStream::LineNo()const
{
	return lineno;
}

int LEX_CLASS::CodeStream::Column()const
{
	return column;
}

//read a char and without advancing the file pointer 
TCHAR LEX_CLASS::CodeStream::Peek()
{
	return stream.peek();
}

//read a char and advance the file pointer 
TCHAR LEX_CLASS::CodeStream::ReadChar()
{
	TCHAR ch = (TCHAR)stream.get();
	if(ch == '\n')
	{
		lineno++;
		column = 1;
	}
	else
		column++;
	position++;
	return ch;
}

//move the file stream one step ahead
void LEX_CLASS::CodeStream::Advance()
{
	//read a char but don't return it!!
	ReadChar();
}

//save current state
void LEX_CLASS::CodeStream::Backup()
{
	lines.push(lineno);
	cols.push(column);
	positions.push(position);
}

//restore most recently saved state
void LEX_CLASS::CodeStream::Restore()
{
	//restore saved state
	lineno = lines.top();
	lines.pop();
	column = cols.top();
	cols.pop();
	stream.seekg(positions.top() - position,ios_base::cur);
	position = positions.top();
	positions.pop();
}

//replace the most recently saved state with the current state
void LEX_CLASS::CodeStream::ReplaceLastBackup()
{
	//remove last saved state
	lines.pop();
	cols.pop();
	positions.pop();

	//save current state
	lines.push(lineno);
	cols.push(column);
	positions.push(position);
}
//bottom cpp
int main(int argc, _TCHAR *argv[])
{
	string inputFileName;
	tout<<argv[0]<<endl;
	tout<<_T("Enter file name: "); getline(cin, inputFileName);
	basic_ifstream<TCHAR> fin( inputFileName.c_str() );
	LineNumberer scanner(fin);
	while(scanner.Scan() != -1);
}
