#include "CompressedDFA.h"
#include "Globals.h"
#include "dfa.h"
#include <set>
#include <cmath>
#include "StringTable.h"

using namespace std;
using namespace Automata;

void CompressedDFA_Pairs_Normal::GenerateCodeCpp(
			tofstream& tout,
			tofstream& tout_h,
			Dictionary& options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			)
{
	int i,j;

	/****************************************************************************/
	//The lex.cpp file

	tout<<CppTopSection<<endl;
	tout<<_T("#include \"")<<options[OPTION_FILE_NAME]<<_T(".h\"")<<endl<<endl;
	tout<<_T("using namespace ")<<options[OPTION_NAMESPACE]<<_T(";")<<endl;
	tout<<_T("#define LEX_CLASS ")<<options[OPTION_CLASS_NAME]<<endl<<endl;

	tout<<_T("const int ")<<options[OPTION_CLASS_NAME]<<_T("::startStateIndex = ")
		<<dfa->StartStateIndex()<<_T(";")<<endl;
	tout<<_T("const UTCHAR ")<<options[OPTION_CLASS_NAME]<<_T("::startSymbol = ")
		<<(int)(dfa->Father()->StartSymbol())<<_T(";")<<endl;
	tout<<_T("const UTCHAR ")<<options[OPTION_CLASS_NAME]<<_T("::finalSymbol = ")
		<<(int)(dfa->Father()->FinalSymbol())<<_T(";")<<endl<<endl;

	//output the transition table
	for(i=0;i<dfa->NumberOfStates();i++)
	{
		int* pairsVector;
		pairsVector = TransitionListVector[i];
		if(pairsVector == NULL)
		{
			tout<<_T("//NULL");
		}
		else if(pairsVector[0] == 0)
		{
			tout<<_T("static int vec")<<i<<_T("[] = {0, ");
			for(j=0;j<dfa->Father()->NumberOfSymbols()-1;j++)
				tout<<pairsVector[j+1]<<_T(", ");
			tout<<pairsVector[j+1]<<_T("};");
		}
		else 
		{
			tout<<_T("static int vec")<<i<<_T("[] = {")<<pairsVector[0]<<_T(", ");
			for(j=0;j<pairsVector[0]-1;j++)
				tout<<pairsVector[j*2+1]<<_T(", ")<<pairsVector[j*2+2]<<_T(", ");
			tout<<pairsVector[j*2+1]<<_T(", ")<<pairsVector[j*2+2]<<_T("};");
		}
		tout<<endl;
	}
	tout<<endl;	

	//list of pointers to transitions vectors
	tout<<_T("const int * ")<<options[OPTION_CLASS_NAME]
	<<_T("::transitionTable[] = ")<<endl<<_T("{")<<endl;
	for(i = 0;i<dfa->NumberOfStates()-1;i++)
		if(TransitionListVector[i] == NULL)
			tout<<_T("NULL,")<<endl;
		else
			tout<<_T("vec")<<i<<_T(",")<<endl;
	if(TransitionListVector[i] == NULL)
		tout<<_T("NULL");
	else
		tout<<_T("vec")<<i;
	tout<<endl<<_T("};")<<endl<<endl;

	//output the accepting table
	tout<<_T("const BOOL ")<<options[OPTION_CLASS_NAME]<<_T("::accepting[")
		<<dfa->NumberOfStates()
		<<_T("] = ")
		<<endl;
	tout<<_T("{")<<endl<<(dfa->IsFinal(0)?"TRUE":"FALSE")<<endl ;
	for(i=1;i<dfa->NumberOfStates();i++)
		tout<<_T(",")<<(dfa->IsFinal(i)?"TRUE":"FALSE")<<endl ;
	tout<<_T("};")<<endl<<endl;

	//start writing the code

	//the return type of the GetNextToken function
	tout<<options[OPTION_RETURN_TYPE]<<_T(" ")
		<<options[OPTION_CLASS_NAME]<<_T("::")<<options[OPTION_FUNCTION_NAME];

	tout<<DFAPairsCpp1;

	//the EOF code
	tout<<options[ OPTION_EOF_ACTION ];

	tout<<DFAPairsCpp2
		<<DFAPairsCpp2_1;

	//the invalid token code
	tout<<options[ OPTION_INVALID_TOKEN_ACTION ];

	tout<<DFAPairsCpp3;

	//the switch statement
	tout<<_T("switch(lastAccepting){")<<endl;
	for(i=0;i<dfa->NumberOfStates();i++)
	{
		if(dfa->ActionTable()[i] != NULL)
		{
			tout<<_T("case ")<<i<<_T(":")<<endl;
			tout<<_T("\t{")<<endl;
			tout<<_T("\t")<<dfa->ActionTable()[i] <<endl;
			tout<<_T("\t}")<<endl;
			tout<<_T("\tbreak;")<<endl;
		}
	}
	tout<<_T("}")<<endl;

	tout<<DFAPairsCpp4
		<<DFAPairsCpp4_1
		<<DFAPairsCpp4_2;

	tout<<endl<<CppBottomSection;

	/****************************************************************************/
	//The lex.h file
	
	tout_h<<HeaderTopSection<<endl;
	tout_h<<DFAPairsH1;
	tout_h<<_T("namespace ")<<options[OPTION_NAMESPACE]<<endl<<_T("{")<<endl;
	tout_h<<_T("class ")<<options[OPTION_CLASS_NAME];
	if(options[OPTION_SUPER_CLASS]!=_T(""))
		tout_h<<_T(" : ");
	tout_h<<options[OPTION_SUPER_CLASS];
	tout_h<<endl;
	tout_h<<DFAPairsH2
		<<DFAPairsH2_1;
	tout_h<<options[OPTION_CLASS_NAME];
	tout_h<<DFAPairsH3;
	tout_h<<options[OPTION_RETURN_TYPE]<<_T(" ")
		<<options[OPTION_FUNCTION_NAME]<<_T("();")<<endl;
	tout_h<<ClassDefinition<<endl;
	tout_h<<_T("};")<<endl;
	tout_h<<_T("}")<<endl<<endl;
	tout_h<<_T("#endif")<<endl;	
}

void CompressedDFA_Pairs_Normal::GenerateCodeCs(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			)
{
	int i,j;

	/****************************************************************************/
	//The lex.cs file

	tout<<HeaderTopSection<<endl;
	tout<<_T("using System;")<<endl;
	tout<<_T("using System.IO;")<<endl;
	tout<<_T("using System.Collections;")<<endl<<endl;
	tout<<_T("namespace ")<<options[OPTION_NAMESPACE]<<endl<<_T("{")<<endl;
	tout<<_T("class ")<<options[OPTION_CLASS_NAME];
	if(options[OPTION_SUPER_CLASS]!=_T(""))
		tout<<_T(" : ");
	tout<<options[OPTION_SUPER_CLASS];
	tout<<endl;
	tout<<_T("{")<<endl;
	tout<<ClassDefinition<<endl;

	//calculate the max. allowed no. of array elements per line
	//--max length of an element = ceil(log10(#elements)) + 1
	//--add another 1 for the comma, another 1 for the space, another 1 for the negative
	int max = 2000/((int)ceil(log10((double)dfa->Father()->NumberOfSymbols()))+4);
	int elements;

	//output the transition table
	tout<<_T("\treadonly int[][] transitionTable = ")<<endl;
	tout<<_T("\t{")<<endl;
	for(i=0;i<dfa->NumberOfStates();i++)
	{
		int* pairsVector;
		pairsVector = TransitionListVector[i];

		elements = 0;
		if(pairsVector == NULL)
		{
			tout<<_T("null,");
		}
		else if(pairsVector[0] == 0)
		{
			tout<<_T("new int[]{0, ");
			for(j=0;j<dfa->Father()->NumberOfSymbols()-1;j++)
			{
				tout<<pairsVector[j+1]<<_T(", ");
				elements++;
				if(elements > max){tout<<endl;elements = 0;}
			}
			tout<<pairsVector[j+1]<<_T("},");
		}
		else 
		{
			tout<<_T("new int[]{")<<pairsVector[0]<<_T(", ");
			for(j=0;j<pairsVector[0]-1;j++)
			{
				tout<<pairsVector[j*2+1]<<_T(", ")<<pairsVector[j*2+2]<<_T(", ");
				elements+=2;
				if(elements > max){tout<<endl;elements = 0;}
			}
			tout<<pairsVector[j*2+1]<<_T(", ")<<pairsVector[j*2+2]<<_T("},");
		}
		tout<<endl;
	}
	tout<<endl;
	tout<<_T("\t};")<<endl<<endl;
	
	//output the accepting table
	tout<<_T("\treadonly bool[] accepting = new bool[]")<<endl;
	tout<<_T("\t{")<<endl<<(dfa->IsFinal(0)?"true":"false")<<endl ;
	for(i=1;i<dfa->NumberOfStates();i++)
		tout<<_T(",")<<(dfa->IsFinal(i)?"true":"false")<<endl ;
	tout<<_T("\t};")<<endl<<endl;

	//the start/final symbol
	tout<<_T("\tconst char startSymbol = (char)")
		<<(int)dfa->Father()->StartSymbol()<<_T(";")<<endl;
	tout<<_T("\tconst char finalSymbol = (char)")
		<<(int)dfa->Father()->FinalSymbol()<<_T(";")<<endl;

	tout<<DFAPairsCs1;

	tout<<options[OPTION_CLASS_NAME];

	tout<<DFAPairsCs2;

	tout<<options[OPTION_RETURN_TYPE]<<_T(" ")<<options[OPTION_FUNCTION_NAME];

	tout<<DFAPairsCs3;

	tout<<options [ OPTION_EOF_ACTION ];

	tout<<DFAPairsCs4
		<<DFAPairsCs4_1;

	tout<<options [ OPTION_INVALID_TOKEN_ACTION ];

	tout<<DFAPairsCs5;

	//the switch statement
	tout<<_T("switch(lastAccepting){")<<endl;
	for(i=0;i<dfa->NumberOfStates();i++)
	{
		if(dfa->ActionTable()[i] != NULL)
		{
			tout<<_T("case ")<<i<<_T(":")<<endl<<_T("\t");
			tout<<dfa->ActionTable()[i]<<endl;
			tout<<_T("\tbreak;")<<endl;
		}
	}
	tout<<_T("}")<<endl;

	tout<<DFAPairsCs6
		<<DFAPairsCs6_1
		<<DFAPairsCs6_2
		<<DFAPairsCs6_3;

	tout<<CppTopSection<<endl;
	tout<<_T("}")<<endl;
    tout<<CppBottomSection<<endl;

}

void CompressedDFA_RedundancyRemoved::GenerateCodeCpp(
			tofstream& tout,
			tofstream& tout_h,
			Dictionary& options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			)
{
	int i,j;

	//The number of the rows equals the number of states
	/****************************************************************************/
	//The lex.cpp file

	tout<<CppTopSection<<endl;
	tout<<_T("#include \"")<<options[OPTION_FILE_NAME]<<_T(".h\"")<<endl<<endl;
	tout<<_T("using namespace ")<<options[OPTION_NAMESPACE]<<_T(";")<<endl;
	tout<<_T("#define LEX_CLASS ")<<options[OPTION_CLASS_NAME]<<endl<<endl;

	tout<<_T("const int ")<<options[OPTION_CLASS_NAME]<<_T("::startStateIndex = ")
		<<dfa->StartStateIndex()<<_T(";")<<endl;
	tout<<_T("const UTCHAR ")<<options[OPTION_CLASS_NAME]<<_T("::startSymbol = ")
		<<(int)dfa->Father()->StartSymbol()<<_T(";")<<endl;
	tout<<_T("const UTCHAR ")<<options[OPTION_CLASS_NAME]<<_T("::finalSymbol = ")
		<<(int)dfa->Father()->FinalSymbol()<<_T(";")<<endl<<endl;

	//the row map
	tout<<_T("const int ")<<options[OPTION_CLASS_NAME]<<_T("::rowMap[] = {");
	for(i=0;i<dfa->NumberOfStates()-1;i++)
		tout<<RowMap[i]<<_T(", ");
	tout<<RowMap[dfa->NumberOfStates()-1]<<_T("};")<<endl;

	//the column map
	tout<<_T("const int ")<<options[OPTION_CLASS_NAME]<<_T("::columnMap[] = {");
	for(i=0;i<dfa->Father()->NumberOfSymbols()-1;i++)
		tout<<ColumnMap[i]<<_T(", ");
	tout<<ColumnMap[dfa->Father()->NumberOfSymbols()-1]<<_T("};")<<endl;

	//the transition table
	tout<<_T("const int ")<<options[OPTION_CLASS_NAME]
		<<_T("::transitionTable[")<<ReducedRowsCount<<_T("][")
		<<ReducedColumnsCount<<_T("] = ")<<endl;
	tout<<_T("{")<<endl;
	for(i=0;i<ReducedRowsCount;i++)
	{
		tout<<_T("{");
		for(j=0;j<ReducedColumnsCount;j++)
			tout<<MappedTransitionTable[i][j]<<_T(", ");
		tout<<_T("},")<<endl;
	}
	tout<<_T("};")<<endl;

	//the accepting table
	tout<<_T("const BOOL ")<<options[OPTION_CLASS_NAME]<<_T("::accepting[")
		<<dfa->NumberOfStates()
		<<_T("] = ")
		<<endl;
	tout<<_T("{")<<endl<<(dfa->IsFinal(0)?"TRUE":"FALSE")<<endl ;
	for(i=1;i<dfa->NumberOfStates();i++)
		tout<<_T(",")<<(dfa->IsFinal(i)?"TRUE":"FALSE")<<endl ;
	tout<<_T("};")<<endl<<endl;

	//start writing the code

	//the return type of the GetNextToken function
	tout<<options[OPTION_RETURN_TYPE]<<_T(" ")<<options[OPTION_CLASS_NAME]<<_T("::")
		<<options[OPTION_FUNCTION_NAME];

	tout<<DFARedundancyRemovedCpp1;

	//the EOF code
	tout<<options[ OPTION_EOF_ACTION ];

	tout<<DFARedundancyRemovedCpp2;

	//the invalid token code
	tout<<options[ OPTION_INVALID_TOKEN_ACTION ];

	tout<<DFARedundancyRemovedCpp3;

	//the switch statement
	tout<<_T("switch(lastAccepting){")<<endl;
	for(i=0;i<dfa->NumberOfStates();i++)
	{
		if(dfa->ActionTable()[i] != NULL)
		{
			tout<<_T("case ")<<i<<_T(":")<<endl;
			tout<<_T("\t{")<<endl;
			tout<<_T("\t")<<dfa->ActionTable()[i]<<endl;
			tout<<_T("\t}")<<endl;
			tout<<_T("\tbreak;")<<endl;
		}
	}
	tout<<_T("}")<<endl;

	tout<<DFARedundancyRemovedCpp4
		<<DFARedundancyRemovedCpp4_1
		<<DFARedundancyRemovedCpp4_2;

	tout<<endl<<CppBottomSection;

	/****************************************************************************/
	//The lex.h file
	
	tout_h<<HeaderTopSection<<endl;
	tout_h<<DFARedundancyRemovedH1;
	tout_h<<_T("namespace ")<<options[OPTION_NAMESPACE]<<endl<<_T("{")<<endl;
	tout_h<<_T("class ")<<options[OPTION_CLASS_NAME];
	if(options[OPTION_SUPER_CLASS]!=_T(""))
		tout_h<<_T(" : ");
	tout_h<<options[OPTION_SUPER_CLASS];
	tout_h<<endl;
	tout_h<<DFARedundancyRemovedH2;
	tout_h<<ReducedColumnsCount;
	tout_h<<DFARedundancyRemovedH3;
	tout_h<<options[OPTION_CLASS_NAME];
	tout_h<<DFARedundancyRemovedH4;
	tout_h<<options[OPTION_RETURN_TYPE]<<_T(" ")<<options[OPTION_FUNCTION_NAME]<<_T("();")
		<<endl;
	tout_h<<ClassDefinition<<endl;
	tout_h<<_T("};")<<endl;
	tout_h<<_T("}")<<endl<<endl;
	tout_h<<_T("#endif")<<endl;
}

void CompressedDFA_RedundancyRemoved::GenerateCodeCs(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			)
{
	int i,j;

	/****************************************************************************/
	//The lex.cs file

	tout<<HeaderTopSection<<endl;
	tout<<_T("using System;")<<endl;
	tout<<_T("using System.IO;")<<endl;
	tout<<_T("using System.Collections;")<<endl<<endl;
	tout<<_T("namespace ")<<options[OPTION_NAMESPACE]<<endl<<_T("{")<<endl;
	tout<<_T("class ")<<options[OPTION_CLASS_NAME];
	if(options[OPTION_SUPER_CLASS]!=_T(""))
		tout<<_T(" : ");
	tout<<options[OPTION_SUPER_CLASS];
	tout<<endl;
	tout<<_T("{")<<endl;
	tout<<ClassDefinition<<endl;

	//calculate the max. allowed no. of array elements per line
	//--max length of an element = ceil(log10(#elements)) + 1
	//--add another 1 for the comma, another 1 for the space, another 1 for the negative
	int max = 2000/((int)ceil(log10((double)dfa->Father()->NumberOfSymbols()))+4);
	int elements;

	//output the transition table
	//the row map
	tout<<_T("readonly int[] rowMap = new int[]{");
	for(i=0;i<dfa->NumberOfStates()-1;i++)
		tout<<RowMap[i]<<_T(", ");
	tout<<RowMap[dfa->NumberOfStates()-1]<<_T("};")<<endl;

	//the column map
	elements = 0;
	tout<<_T("readonly int[] columnMap = new int[]{");
	for(i=0;i<dfa->Father()->NumberOfSymbols()-1;i++)
	{
		tout<<ColumnMap[i]<<_T(", ");
		elements++;
		if(elements > max){tout<<endl;elements = 0;}
	}
	tout<<ColumnMap[dfa->Father()->NumberOfSymbols()-1]<<_T("};")<<endl;

	//the transition table
	tout<<_T("readonly int[,] transitionTable = new int[,]{")<<endl;
	for(i=0;i<ReducedRowsCount;i++)
	{
		elements = 0;
		tout<<_T("{");
		for(j=0;j<ReducedColumnsCount - 1;j++)
		{
			tout<<MappedTransitionTable[i][j]<<_T(", ");
			elements++;
			if(elements > max){tout<<endl;elements = 0;}
		}
		tout<<MappedTransitionTable[i][ReducedColumnsCount - 1]<<_T("},")<<endl;
	}
	tout<<_T("};")<<endl;
	
	//output the accepting table
	tout<<_T("\treadonly bool[] accepting = new bool[]")<<endl;
	tout<<_T("\t{")<<endl<<(dfa->IsFinal(0)?"true":"false")<<endl ;
	for(i=1;i<dfa->NumberOfStates();i++)
		tout<<_T(",")<<(dfa->IsFinal(i)?"true":"false")<<endl ;
	tout<<_T("\t};")<<endl<<endl;

	//the start/final symbol
	tout<<_T("\tconst char startSymbol = (char)")<<(int)dfa->Father()->StartSymbol()
		<<_T(";")<<endl;
	tout<<_T("\tconst char finalSymbol = (char)")<<(int)dfa->Father()->FinalSymbol()
		<<_T(";")<<endl;

	tout<<DFARedundancyRemovedCs1;

	tout<<options[OPTION_CLASS_NAME];

	tout<<DFARedundancyRemovedCs2;

	tout<<options[OPTION_RETURN_TYPE]<<_T(" ")<<options[OPTION_FUNCTION_NAME];

	tout<<DFARedundancyRemovedCs3;

	tout<<options [ OPTION_EOF_ACTION ]<<endl;

	tout<<DFARedundancyRemovedCs4;

	tout<<options [ OPTION_INVALID_TOKEN_ACTION ]<<endl;

	tout<<DFARedundancyRemovedCs5;

	//the switch statement
	tout<<_T("switch(lastAccepting){")<<endl;
	for(i=0;i<dfa->NumberOfStates();i++)
	{
		if(dfa->ActionTable()[i] != NULL)
		{
			tout<<_T("case ")<<i<<_T(":")<<endl<<_T("\t");
			tout<<dfa->ActionTable()[i]<<endl;
			tout<<_T("\tbreak;")<<endl;
		}
	}
	tout<<_T("}")<<endl;

	tout<<DFARedundancyRemovedCs6
		<<DFARedundancyRemovedCs6_1
		<<DFARedundancyRemovedCs6_2
		<<DFARedundancyRemovedCs6_3;

	tout<<CppTopSection<<endl;
	tout<<_T("}")<<endl;
    tout<<CppBottomSection<<endl;
}

int CompressedDFA_RedundancyRemoved::MemorySize() const
{
	return sizeof(int) *
		(
		dfa->Father()->NumberOfSymbols() 
		+ dfa->NumberOfStates()
		+ ReducedColumnsCount * ReducedRowsCount
		);
}

int CompressedDFA_Pairs_Normal::MemorySize() const
{
	return sizeof(int) *
		memorySize;
}