#include "stdafx.h"
#include "Set.h"
#include "SimpleRegexConventions.h"
#include ".\dfa.h"
#include "FAFactory.h"
#include "NFA.h"
#include "Set.h"
#include "LinkedList.h"
using namespace Automata;
#include <iostream>
using namespace std;
typedef LinkedList<int> StateList;
struct IntermediateState
{
	bool Accepting;
	bool ContainsCCL;
	BitSet NfaStates;
	StateList BehavioralStates;
	int *Transitions;				//externally managed.
	IntermediateState(int howManyNfaStates) : NfaStates(howManyNfaStates - 1), 
											  ContainsCCL(false),
											  Accepting(false),
											  Transitions(NULL)
	{
	}
	BOOL IsBehaviorlyEquivalentTo(const IntermediateState *rhs) const
	{
		if(Accepting != rhs->Accepting)
			return FALSE;
		if(ContainsCCL != rhs->ContainsCCL)
			return FALSE;
		if(BehavioralStates.Count() != rhs->BehavioralStates.Count())
			return FALSE;
		StateList::const_iterator iterator = BehavioralStates.Begin();
		StateList::const_iterator endIterator = BehavioralStates.End();
		while(iterator != endIterator)
		{
			if(!rhs->NfaStates.Contains(*iterator++))
				return FALSE;
		}
		return TRUE;
	}
	~IntermediateState()
	{
	}
};
typedef LinkedList<IntermediateState *> IntermediateStateList;
struct NfaInfo
{
	NFA *Nfa;
	NfaInfo(NFA *nfa) : Nfa(nfa)
	{
	}
	int StartState() const
	{
		return Nfa->startState;
	}
	int FinalState() const
	{
		return Nfa->finalState;
	}
	BOOL IsAcceptingState(int state) const
	{
		return Nfa->finalState == state;
	}
	const NondeterministicState& GetState(int stateIndex) const
	{
		return Nfa->memory->GetState(stateIndex);
	}
	int NumberOfStates() const
	{
		return Nfa->NumberOfStates();
	}
};
void eClosure(IntermediateState *hState,
			  NfaInfo *nfaInfo,
			  int state)
{
	if(hState->NfaStates.Contains(state))
		return;
	hState->NfaStates.Add(state);
	const NondeterministicState &nfaState = nfaInfo->GetState(state);
	switch( nfaState.GetType() )
	{
	/*Note: CCL Transitional states are behavioral symbol states as well. So, control must
	 *		fall to the symbol state.*/
	case NondeterministicState::CharacterClassTransition:
		hState->ContainsCCL = true;
	case NondeterministicState::SymbolTransition:
		hState->BehavioralStates.AddToBack(state);
		break;
	case NondeterministicState::EmptyTransitions:
		if( nfaInfo->IsAcceptingState(state) )
			hState->Accepting = true;
		int adj = nfaState.GetEmptyTransition1();
		if(adj != ErrorState)
			eClosure(hState, nfaInfo, adj);
		adj = nfaState.GetEmptyTransition2();
		if(adj != ErrorState)
			eClosure(hState, nfaInfo, adj);
	}	
}
/*
class DFA
	{
		friend class FAFactory;
	private:
		const FAFactory		*father;
		int					numberOfStates;
		int					startStateIndex;//Always zero
		BoolArray<int, 5>	isFinalState;	//a state qi is final if isFinalState[i] = true
		int					**transitionTable;   // transitionTable[qi][cr] = qj
	/***************************************/
	/*Helper Methods*/
	/*	void	Build(	const TCHAR *regex,
						const SimpleRegexConventions &metaConventions);
	/***************************************/
/*	public:
		DFA();
		DFA(const FAFactory *creator,
			const TCHAR *regex,
			const SimpleRegexConventions &metaConventions);

		int		NextState(int state, TCHAR inputSymbol) const;
		BOOL	IsFinal(int state) const;
		void	SetFinal(int state);
		void	RemoveFinal(int state);
		void	ClearAllFinals();
		int		StartState() const;
		int&	StartState();
		~DFA(void);
	};
*/
DFA::DFA(void) : father(NULL), numberOfStates(0), startStateIndex(0), transitionTable(NULL)
{
	
}
void DFA::MakeRejectionMachine()
{
	if(transitionTable != NULL)
		DestroyTransitionTable();
	transitionTable = NULL;
}
BOOL DFA::IsRejectionMachine() const
{
	return transitionTable == NULL;
}
DFA::DFA(const FAFactory *creator,
		 const TCHAR *regex,
		 const SimpleRegexConventions &metaConventions) : father(creator),
														transitionTable(NULL)
{
	Build(regex, metaConventions);
}
void DFA::Reset(const TCHAR *regex,
				const SimpleRegexConventions &metaConventions)
{
	if(transitionTable != NULL)
		DestroyTransitionTable();
	transitionTable = NULL;
	Build(regex, metaConventions);
}
void DFA::Build(const TCHAR *regex,
				const SimpleRegexConventions &metaConventions)
{
	int numberOfSymbols = father->NumberOfSymbols();
	NFA nfa(father, regex, metaConventions);
	if(nfa.IsRejectionMachine())
	{
/*		transitionTable = new int*;
		transitionTable[0] = new int[numberOfSymbols];
		for(int i = 0; i < numberOfSymbols; i++)
			transitionTable[0][i] = ErrorState;*/
		MakeRejectionMachine();
		return;
	}
	NfaInfo nfaInfo(&nfa);
	IntermediateState *tempState = new IntermediateState(nfa.NumberOfStates());
	IntermediateStateList states;
	IntermediateState *startState = 
					new IntermediateState(nfaInfo.NumberOfStates());
	eClosure(startState, &nfaInfo, nfaInfo.StartState());
	states.AddToBack(startState);
    
	IntermediateStateList::iterator	i = states.Begin();
	IntermediateState *currentState = NULL;
	while(i != states.End())
	{
		currentState = *i;
		currentState->Transitions = new int[numberOfSymbols];
		memset(currentState->Transitions, ErrorState, numberOfSymbols*sizeof(int));
		if(	currentState->ContainsCCL || 
			currentState->BehavioralStates.Count() >= numberOfSymbols)
		{
			FAFactory::const_iterator afterEndIterator = father->End();
			FAFactory::const_iterator symbolIterator = father->Begin();
			TCHAR currentSymbol;
			int currentSymbolIndex;
			while(symbolIterator != afterEndIterator)
			{
				currentSymbol = *symbolIterator++;
				currentSymbolIndex = father->IndexOf(currentSymbol);
				StateList::iterator bstateIterator = currentState->BehavioralStates.Begin();
				StateList::iterator bstateEnd = currentState->BehavioralStates.End();
				while(bstateIterator != bstateEnd)
				{
					const NondeterministicState &currentBState = 
						nfaInfo.GetState(*bstateIterator++);
					int nextStateIndex = currentBState.NextState(father, currentSymbol);
					if(nextStateIndex != ErrorState)
						eClosure(tempState, &nfaInfo, nextStateIndex);
				}
				int nextIStateIndex;
				//if the next state is the ErrorState(if it is nonaccepting and have no outside 
				//transitions, then set state(currentSymbol) = ErrorState
				if(!tempState->Accepting && tempState->BehavioralStates.Count() == 0)
				{
					nextIStateIndex = ErrorState;
					tempState->NfaStates.SetToEmptySet();
				}
				else
				{
					//if there is already a state that is behaviorly equivalent to tempState, 
					//then no need to create a new state. Just set it.
					IntermediateStateList::iterator iterIStates = states.Begin();
					IntermediateStateList::iterator endIStates = states.End();
					nextIStateIndex = 0;
					BOOL alreadyExists = FALSE;
					while(iterIStates != endIStates)
					{
						if(tempState->IsBehaviorlyEquivalentTo(*iterIStates++))
						{
							alreadyExists = TRUE;
							iterIStates = endIStates; //end the loop
						}
						else
						{
							nextIStateIndex++;
						}
					}
					if(!alreadyExists)
					{
						tempState->Transitions = new int[numberOfSymbols];
						states.AddToBack(tempState);
						tempState = new IntermediateState(nfaInfo.NumberOfStates());
					}
					else //if alreadyExists
					{
						//either delete it or reset it
						delete tempState;
						tempState = new IntermediateState(nfaInfo.NumberOfStates());						
					}
				}
				currentState->Transitions[currentSymbolIndex] = nextIStateIndex;
			}// end of for each symbol in E
		}//end of if |BStats| > |E|
		else
		{
			TCHAR currentSymbol;
			int currentSymbolIndex;
			StateList::iterator bstateIterator = currentState->BehavioralStates.Begin();
			StateList::iterator bstateEnd = currentState->BehavioralStates.End();
			while(bstateIterator != bstateEnd)
			{
				const NondeterministicState &currentBState = 
													nfaInfo.GetState(*bstateIterator);
				currentSymbol = currentBState.GetTransitionSymbol();
				currentSymbolIndex = father->IndexOf(currentSymbol);
				StateList::iterator bsIterator = bstateIterator++;
				while(bsIterator != bstateEnd)
				{
					const NondeterministicState &cbState = 
						nfaInfo.GetState(*bsIterator++);
					int nextStateIndex = cbState.NextState(father, currentSymbol);
					if(nextStateIndex != ErrorState)
						eClosure(tempState, &nfaInfo, nextStateIndex);
				}
				int nextIStateIndex;
				//if the next state is the ErrorState(if it is nonaccepting and have no outside 
				//transitions, then set state(currentSymbol) = ErrorState
				if(!tempState->Accepting && tempState->BehavioralStates.Count() == 0)
				{
					nextIStateIndex = ErrorState;
					tempState->NfaStates.SetToEmptySet();
				}
				else
				{
					//if there is already a state that is behaviorly equivalent to tempState, 
					//then no need to create a new state. Just set it.
					IntermediateStateList::iterator iterIStates = states.Begin();
					IntermediateStateList::iterator endIStates = states.End();
					nextIStateIndex = 0;
					BOOL alreadyExists = FALSE;
					while(iterIStates != endIStates)
					{
						if(tempState->IsBehaviorlyEquivalentTo(*iterIStates++))
						{
							alreadyExists = TRUE;
							iterIStates = endIStates; //end the loop
						}
						else
						{
							nextIStateIndex++;
						}
					}
					if(!alreadyExists)
					{
						states.AddToBack(tempState);
						tempState = new IntermediateState(nfaInfo.NumberOfStates());
					}
					else //if alreadyExists
					{
						//either delete it or reset it
						delete tempState;
						tempState = new IntermediateState(nfaInfo.NumberOfStates());						
					}
				}
				currentState->Transitions[currentSymbolIndex] = nextIStateIndex;
			}// end of for each symbol in E
		}//end of if( |BStates| < |E| and no character classes in this state )
		i++;
    }
	delete tempState;
	//Now, we must construct the transition table for the dfa
	if(transitionTable != NULL)
		DestroyTransitionTable();
	this->numberOfStates = states.Count();
	transitionTable = new int*[this->numberOfStates];
	isFinalState.Resize(this->numberOfStates, FALSE);
	IntermediateStateList::iterator iter = states.Begin();
	cout<<_T("We will now destroy intermediate states.")<<endl;
	for(int j = 0; j < this->numberOfStates; j++, iter++)
	{
		transitionTable[j] = (*iter)->Transitions;
		if((*iter)->Accepting)
			this->isFinalState.Set(j, TRUE);
		delete *iter;
	}
	cout<<endl<<_T("END OF DESTRUCTION")<<endl;
	startStateIndex = 0;
}
int DFA::NextState(int state, TCHAR inputSymbol) const
{
	return transitionTable[state][father->IndexOf(inputSymbol)];
}
BOOL DFA::IsFinal(int state) const
{
	return isFinalState[state];
}
void DFA::SetFinal(int state)
{
	isFinalState.Set(state, TRUE);
}
void DFA::RemoveFinal(int state)
{
	isFinalState.Set(state, FALSE);
}
int& DFA::StartState() 
{
	return startStateIndex;
}
DFA::~DFA(void)
{
	if(transitionTable != NULL)
		DestroyTransitionTable();
}
BOOL DFA::Match(const TCHAR *szString, int length) const
{
	if(IsRejectionMachine())
		return FALSE;
	int currentState = 0;//startState
	TCHAR currentChar;
	const TCHAR *end = szString + length;
	while(szString != end)
	{
		currentChar = *szString++;
		currentState = transitionTable[currentState][father->IndexOf(currentChar)];
		if(currentState == ErrorState)
			return FALSE;
	}
	//Now, we check currentState for acceptance...
	return isFinalState[currentState];
}
BOOL DFA::Match(const TCHAR *szString) const
{
	if(IsRejectionMachine())
		return FALSE;
	int currentState = 0;//startState
	TCHAR currentChar;
	while(*szString)
	{
		currentChar = *szString++;
		currentState = transitionTable[currentState][father->IndexOf(currentChar)];
		if(currentState == ErrorState)
			return FALSE;
	}
	//Now, we check currentState for acceptance...
	return isFinalState[currentState];
}
void DFA::DestroyTransitionTable()
{
	for(int i=0;i<numberOfStates;i++)
		if(transitionTable[i]!=NULL)
		{
			delete[] transitionTable[i];
			transitionTable[i] = NULL;
		}
	delete[] transitionTable;
}