#include "DFA.h"
#include "CompressedDFA.h"
#include "FAFactory.h"
#include "stdafx.h"
#include "Globals.h"
#include <cmath>

#include "StringTable.h"

using namespace std;
using namespace Automata;

void DFA::GenerateCodeCpp(
			tofstream& tout,
			tofstream& tout_h,
			Dictionary& options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			)
{
	int i,j;

	/****************************************************************************/
	//The lex.cpp file

	tout<<CppTopSection<<endl;
	tout<<_T("#include \"")<<options[OPTION_FILE_NAME]<<_T(".h\"")<<endl<<endl;
	tout<<_T("using namespace ")<<options[OPTION_NAMESPACE]<<_T(";")<<endl;
	tout<<_T("#define LEX_CLASS ")<<options[OPTION_CLASS_NAME]<<endl<<endl;

	tout<<_T("const int ")<<options[OPTION_CLASS_NAME]<<_T("::startStateIndex = ")<<startStateIndex<<_T(";")<<endl;
	tout<<_T("const UTCHAR ")<<options[OPTION_CLASS_NAME]<<_T("::startSymbol = ")<<(int)father->StartSymbol()<<_T(";")<<endl;
	tout<<_T("const UTCHAR ")<<options[OPTION_CLASS_NAME]<<_T("::finalSymbol = ")<<(int)father->FinalSymbol()<<_T(";")<<endl<<endl;


	//output the transition table
	tout<<_T("const int ")<<options[OPTION_CLASS_NAME]<<_T("::transitionTable[")
		<<numberOfStates
		<<_T("][")
		<<father->NumberOfSymbols()
		<<_T("] = ")
		<<endl
		<<_T("{")<<endl;
	for(i=0;i<numberOfStates;i++)
	{
		if(i!=0)tout<<_T(",");
		tout<<_T("{")<<transitionTable[i][0];
		for(j=1;j<father->NumberOfSymbols();j++)
			tout<<_T(",")<<transitionTable[i][j];
		tout<<_T("}");
		tout<<endl;
	}
	tout<<_T("};")<<endl<<endl;
	
	//output the accepting table
	tout<<_T("const BOOL ")<<options[OPTION_CLASS_NAME]<<_T("::accepting[")
		<<numberOfStates
		<<_T("] = ")
		<<endl;
	tout<<_T("{")<<endl<<(IsFinal(0)?"TRUE":"FALSE")<<endl ;
	for(i=1;i<NumberOfStates();i++)
		tout<<_T(",")<<(IsFinal(i)?"TRUE":"FALSE")<<endl ;
	tout<<_T("};")<<endl<<endl;

	//start writing the code

	//the return type of the GetNextToken function
	tout<<options[OPTION_RETURN_TYPE]<<_T(" ")<<options[OPTION_CLASS_NAME]<<_T("::")<<options[OPTION_FUNCTION_NAME];

	tout<<DFAUncompressedCpp1;

	//the EOF code
	tout<<options[ OPTION_EOF_ACTION ];

	tout<<DFAUncompressedCpp2;

	//the invalid token code
	tout<<options[ OPTION_INVALID_TOKEN_ACTION ];

	tout<<DFAUncompressedCpp3;

	//the switch statement
	tout<<_T("switch(lastAccepting){")<<endl;
	for(i=0;i<NumberOfStates();i++)
	{
		if(actionTable[i] != NULL)
		{
		tout<<_T("case ")<<i<<_T(":")<<endl;
		tout<<_T("\t{")<<endl;
		tout<<_T("\t")<<actionTable[i]<<endl;
		tout<<_T("\t}")<<endl;
		tout<<_T("\tbreak;")<<endl;
		}
	}
	tout<<_T("}")<<endl;

	tout<<DFAUncompressedCpp4
		<<DFAUncompressedCpp4_1
		<<DFAUncompressedCpp4_2;

	tout<<endl<<CppBottomSection;

	/****************************************************************************/
	//The lex.h file
	
	tout_h<<HeaderTopSection<<endl;
	tout_h<<DFAUncompressedH1;
	tout_h<<_T("namespace ")<<options[OPTION_NAMESPACE]<<endl<<_T("{")<<endl;
	tout_h<<_T("class ")<<options[OPTION_CLASS_NAME];
	if(options[OPTION_SUPER_CLASS]!=_T(""))
		tout_h<<_T(" : ");
	tout_h<<options[OPTION_SUPER_CLASS];
	tout_h<<endl;
	tout_h<<DFAUncompressedH2;
	tout_h<<father->NumberOfSymbols();
	tout_h<<DFAUncompressedH3;
	tout_h<<options[OPTION_CLASS_NAME];
	tout_h<<DFAUncompressedH4;
	tout_h<<options[OPTION_RETURN_TYPE]<<_T(" ")<<options[OPTION_FUNCTION_NAME]<<_T("();")<<endl;
	tout_h<<ClassDefinition<<endl;
	tout_h<<_T("};")<<endl;
	tout_h<<_T("}")<<endl<<endl;
	tout_h<<_T("#endif")<<endl;
}

void DFA::GenerateCodeCs(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			)
{
	int i,j;

	/****************************************************************************/
	//The lex.cs file

	tout<<HeaderTopSection<<endl;
	tout<<_T("using System;")<<endl;
	tout<<_T("using System.IO;")<<endl;
	tout<<_T("using System.Collections;")<<endl<<endl;
	tout<<_T("namespace ")<<options[OPTION_NAMESPACE]<<endl<<_T("{")<<endl;
	tout<<_T("class ")<<options[OPTION_CLASS_NAME];
	if(options[OPTION_SUPER_CLASS]!=_T(""))
		tout<<_T(" : ");
	tout<<options[OPTION_SUPER_CLASS];
	tout<<endl;
	tout<<_T("{")<<endl;
	tout<<ClassDefinition<<endl;

	//calculate the max. allowed no. of array elements per line
	//--max length of an element = ceil(log10(#elements)) + 1
	//--add another 1 for the comma, another 1 for the space, another 1 for the space
	int max = 2000/((int)ceil(log10((double)father->NumberOfSymbols()))+4);
	int elements;

	//output the transition table
	tout<<_T("\treadonly int[,] transitionTable = new int[,]")<<endl;
	tout<<_T("\t{")<<endl;
	for(i=0;i<numberOfStates;i++)
	{
		elements = 0;
		if(i!=0)tout<<_T(",");
		tout<<_T("{")<<transitionTable[i][0];
		for(j=1;j<father->NumberOfSymbols();j++)
		{
			tout<<_T(", ")<<transitionTable[i][j];
			elements++;
			if(elements > max){tout<<endl;elements = 0;}
		}
		tout<<_T("}");
		tout<<endl;
	}
	tout<<_T("\t};")<<endl<<endl;
	
	//output the accepting table
	tout<<_T("\treadonly bool[] accepting = new bool[]")<<endl;
	tout<<_T("\t{")<<endl<<(IsFinal(0)?"true":"false")<<endl ;
	for(i=1;i<NumberOfStates();i++)
		tout<<_T(",")<<(IsFinal(i)?"true":"false")<<endl ;
	tout<<_T("\t};")<<endl<<endl;

	//the start/final symbol
	tout<<_T("\tconst char startSymbol = (char)")<<(int)father->StartSymbol()<<_T(";")<<endl;
	tout<<_T("\tconst char finalSymbol = (char)")<<(int)father->FinalSymbol()<<_T(";")<<endl;

	tout<<DFAUncompressedCs1;

	tout<<options[OPTION_CLASS_NAME];

	tout<<DFAUncompressedCs2;

	tout<<options[OPTION_RETURN_TYPE]<<_T(" ")<<options[OPTION_FUNCTION_NAME];

	tout<<DFAUncompressedCs3;

	tout<<options [ OPTION_EOF_ACTION ];

	tout<<DFAUncompressedCs4;

	tout<<options [ OPTION_INVALID_TOKEN_ACTION ];

	tout<<DFAUncompressedCs5;

	//the switch statement
	tout<<_T("switch(lastAccepting){")<<endl;
	for(i=0;i<NumberOfStates();i++)
	{
		if(actionTable[i] != NULL)
		{
			tout<<_T("case ")<<i<<_T(":")<<endl<<_T("\t");
			tout<<actionTable[i]<<endl;
			tout<<_T("\tbreak;")<<endl;
		}
	}
	tout<<_T("}")<<endl;

	tout<<DFAUncompressedCs6
		<<DFAUncompressedCs6_1
		<<DFAUncompressedCs6_2
		<<DFAUncompressedCs6_3;

	tout<<CppTopSection<<endl;
	tout<<_T("}")<<endl;
    tout<<CppBottomSection<<endl;
}

void DFA::GenerateCodeJava(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			)
{
	int i,j;

	/****************************************************************************/
	//The lex.java file

	tout<<HeaderTopSection<<endl;
	tout<<_T("package ")<<options[OPTION_NAMESPACE]<<_T(";")<<endl<<endl;
	tout<<_T("import java.io.*;")<<endl;
	tout<<_T("import java.util.*;")<<endl<<endl;
	tout<<_T("public class ")<<options[OPTION_CLASS_NAME];
	if(options[OPTION_SUPER_CLASS]!=_T(""))
		tout<<_T(" : ");
	tout<<options[OPTION_SUPER_CLASS];
	tout<<endl; 
	tout<<_T("{")<<endl;
	tout<<ClassDefinition<<endl;

	//output the transition table
	tout<<_T("\tfinal int[][] transitionTable = new int[][]")<<endl;
	tout<<_T("\t{")<<endl;
	for(i=0;i<numberOfStates;i++)
	{
		if(i!=0)tout<<_T(",");
		tout<<_T("{")<<transitionTable[i][0];
		for(j=1;j<father->NumberOfSymbols();j++)
			tout<<_T(",")<<transitionTable[i][j];
		tout<<_T("}");
		tout<<endl;
	}
	tout<<_T("\t};")<<endl<<endl;
	
	//output the accepting table
	tout<<_T("\tfinal boolean[] accepting = new boolean[]")<<endl;
	tout<<_T("\t{")<<endl<<(IsFinal(0)?"true":"false")<<endl ;
	for(i=1;i<NumberOfStates();i++)
		tout<<_T(",")<<(IsFinal(i)?"true":"false")<<endl ;
	tout<<_T("\t};")<<endl<<endl;

	//the start/final symbol
	tout<<_T("\tfinal char startSymbol = (char)")<<(int)father->StartSymbol()<<_T(";")<<endl;
	tout<<_T("\tfinal char finalSymbol = (char)")<<(int)father->FinalSymbol()<<_T(";")<<endl;

	tout<<DFAUncompressedJava1;

	tout<<options[OPTION_CLASS_NAME];

	tout<<DFAUncompressedJava2;

	tout<<options[OPTION_RETURN_TYPE]<<_T(" ")<<options[OPTION_FUNCTION_NAME];

	tout<<DFAUncompressedJava3;

	tout<<options [ OPTION_EOF_ACTION ]<<endl;

	tout<<DFAUncompressedJava4;

	tout<<options [ OPTION_INVALID_TOKEN_ACTION ]<<endl;

	tout<<DFAUncompressedJava5;

	//the switch statement
	tout<<_T("switch(lastAccepting){")<<endl;
	for(i=0;i<NumberOfStates();i++)
	{
		if(actionTable[i] != NULL)
		{
			tout<<_T("case ")<<i<<_T(":")<<endl<<_T("\t");
			tout<<actionTable[i]<<endl;
			tout<<_T("\tbreak;")<<endl;
		}
	}
	tout<<_T("}")<<endl;

	tout<<DFAUncompressedJava6
		<<DFAUncompressedJava6_1
		<<DFAUncompressedJava6_2
		<<DFAUncompressedJava6_3;

	tout<<CppTopSection<<endl;
	tout<<_T("}");
	tout<<CppBottomSection<<endl;
}



