#include "stdafx.h"
#include "FAFactory.h"
#include "NFA.h"
#include "DFA.h"

#include <iostream>
#include <cstdio>
using namespace Automata;
using namespace std;

#ifdef _UNICODE
#define tout wcout
#define tin wcin
#else
#define tout cout
#define tin cin
#endif

void ReportStatus(const DFA& DFA, const TCHAR *name);


NfaVector nfa;
CodeVector code;
FAFactory charSet(0, 255);
NFA::MemoryManager memManager(&charSet);

void InitializeRegularExpressions1();
void DestroyRegularExpressions();

void InitializeRegularExpressions2();
void InitializeRegularExpressions3();

int main()
{
	InitializeRegularExpressions2();

	DFA DFA(&charSet, nfa, code);
	ReportStatus(DFA, _T("Trial"));;
	TCHAR *regex, *match;
	const int MAX = 512;
	regex = new TCHAR[MAX];
	match = new TCHAR[MAX];
	*regex = *match = NULL;
	do
	{
		tout<<_T("Enter a string to match: ");
		tin.getline(match, MAX, _T('\n'));
		if(*match == NULL) break;
		const TCHAR *result = DFA.GetMatchingAction(match);
		tout<<_T("Result: ");
		if(result == NULL)	tout<<_T("No result. It is NULL.");
		else tout<<result;
		tout<<endl;
	}while(*match!=NULL);
	delete []regex;
	delete []match;

	DestroyRegularExpressions();
}

void ReportStatus(const DFA& DFA, const TCHAR *name)
{
	tout<<endl<<endl
		<<_T("-------------------------")<<endl
		<<_T("DFA ")<<name<<" status:"<<endl
		<<_T("-------------------------")<<endl
		<<_T("IsRejectionMachine():    ")<<(DFA.IsRejectionMachine()?"TRUE":"FALSE")<<endl
		<<_T("NumberOfStates():        ")<<DFA.NumberOfStates()<<endl
		<<_T("-------------------------")<<endl
		<<endl<<endl;
}

BOOL Match(const DFA& DFA, const TCHAR *name, const TCHAR *str)
{
	if(DFA.Match(str))
	{
		tout<<_T("DFA ")<<name<<_T(" has matched \"")<<str<<_T("\"!!")<<endl;
		return TRUE;
	}
	tout<<_T("DFA ")<<name<<_T(" has not matched \"")<<str<<_T("\"!!")<<endl;
	return FALSE;
}

void InitializeRegularExpressions1()
{
	/*
	Sample Program:
	[ ]
	\t								\t
	\n								\n
	+								PLUS
	++								INC
	-								NEGATIVE
	--								DEC
	int								INTEGER
	float							FLOAT
	[0-9][0-9]*						NUMBER
	[_a-zA-Z][_a-zA-Z0-9]*          IDENTIFIER
	.*								?
	*/
	NFA *pNfa;
	//[ ] 
	pNfa = new NFA(&memManager, _T(' '));
	nfa.push_back(pNfa);
	code.push_back( _T(" ") );

	//\t \t
	pNfa = new NFA(&memManager, _T('\t'));
	nfa.push_back(pNfa);
	code.push_back( _T("\t") );
	//\n \n
	pNfa = new NFA(&memManager, _T('\n'));
	nfa.push_back(pNfa);
	code.push_back( _T("\n") );
	//+ PLUS
	pNfa = new NFA(&memManager, _T('+'));
	nfa.push_back(pNfa);
	code.push_back( _T("PLUS") );
	
	//++ INC
	NFA *pNfa2 = new NFA(&memManager, _T('+'));
	pNfa = new NFA(&memManager, _T('+'));
	pNfa->Concatenate( *pNfa2 );
	nfa.push_back(pNfa);
	code.push_back( _T("INC") );

	//- NEGATIVE
	pNfa = new NFA(&memManager, _T('-'));
	nfa.push_back(pNfa);
	code.push_back( _T("NEGATIVE") );

	//-- DEC
	pNfa2->ReallocateAndSet( _T('-'));
	pNfa = new NFA(&memManager, _T('-'));
	pNfa->Concatenate( *pNfa2 );
	nfa.push_back( pNfa );
	code.push_back( _T("DEC") );

	//int
	NFA *pNfa3 = new NFA(&memManager, _T('t'));
	pNfa2->ReallocateAndSet( _T('n') );
	pNfa = new NFA(&memManager, _T('i'));
	pNfa->Concatenate(*pNfa2);
	pNfa->Concatenate(*pNfa3);
	nfa.push_back(pNfa);
	code.push_back( _T("INT") );

	//float
	TCHAR *str = _T("float");
	TCHAR *ptr = str;
	pNfa = new NFA(&memManager, *ptr++);
	while(*ptr)
	{
		pNfa2->ReallocateAndSet( *ptr );
		pNfa->Concatenate( *pNfa2 );
		ptr++;
	}
	nfa.push_back( pNfa );
	code.push_back( _T("FLOAT") );

	//[0-9][0-9]* NUMBER
	pNfa = new NFA(&memManager);
	Set *charClass = new BitSet( charSet.NumberOfSymbols() - 1, TRUE);
	TCHAR c = _T('0');
	while(c <= _T('9'))
		charClass->Add( charSet.IndexOf( c++ ) );
	pNfa->SetCharacterClass( charClass );
	charClass = new BitSet( *((BitSet*)charClass) );
	pNfa2->SetCharacterClass( charClass );
	pNfa2->Closure();
	pNfa->Concatenate( *pNfa2 );
	nfa.push_back( pNfa );
	code.push_back( _T("NUMBER") );

	//[_a-zA-Z][_a-zA-Z0-9]* IDENTIFIER
	pNfa = new NFA(&memManager);
	charClass = new BitSet( charSet.NumberOfSymbols() - 1, TRUE);
	charClass->Add( charSet.IndexOf( _T('_') ) );
	c = _T('a');
	while(c <= _T('z'))
		charClass->Add( charSet.IndexOf( c++ ) );
	c = _T('A');
	while(c <= _T('Z'))
		charClass->Add( charSet.IndexOf( c++ ) );
	pNfa->SetCharacterClass( charClass );
	charClass = new BitSet( *((BitSet*)charClass) );
	c = _T('0');
	while(c <= _T('9'))
		charClass->Add( charSet.IndexOf( c++ ) );
	pNfa2->SetCharacterClass( charClass );
	pNfa2->Closure();
	pNfa->Concatenate( *pNfa2 );
	nfa.push_back( pNfa );
	code.push_back( _T("IDENTIFIER") );

	//.* ?
/*	pNfa = new NFA(&memManager);
	pNfa->SetAcceptAnyWord();
	nfa.push_back( pNfa );
	code.push_back( _T("?") );*/

	delete pNfa2;
	delete pNfa3;
}
void DestroyRegularExpressions()
{
	int count = nfa.size();
	for(int i=0; i<count; i++)
	{
		delete nfa[i];
	}
}
void InitializeRegularExpressions2()
{
	/*
	Sample Program:

	ada*|a*da						HAMDY_MAN
	.*								?

	*/

	// ada*|a*da	HAMDY_MAN
	NFA *pnfa = new NFA(&memManager, _T('a'));	//pnfa = a
	NFA tnfa1(&memManager, _T('d')); //tnfa1 = d
	pnfa->Concatenate(tnfa1);//pnfa = ad
	tnfa1.ReallocateAndSet(_T('a'));//tnfa1 = a
	tnfa1.Closure();//tnfa1 = a*
	pnfa->Concatenate(tnfa1);	//pnfa = ada*

	tnfa1.ReallocateAndSet(_T('a'));	//tnfa1 = a
	tnfa1.Closure(); // tnfa1 = a*
	NFA tnfa2(&memManager, _T('d')); //tnfa2 = d
	tnfa1.Concatenate(tnfa2);	// tnfa1 = a*d
	tnfa2.ReallocateAndSet(_T('a'));	//tnfa2 = a
	tnfa1.Concatenate(tnfa2);	//tnfa1 = a*da

	pnfa->Or( tnfa1 );
	if(	pnfa->Match(_T("adaaa")) && pnfa->Match(_T("aaaada")) )
		tout<<_T("The NFA OR operates Correctly.")<<endl;
	else
		tout<<_T("The NFA OR doesnot operate properly.")<<endl;
	nfa.push_back( pnfa );
	code.push_back( _T("HAMDY_MAN") );
	
	// .*	?
	/*pnfa = new NFA(&memManager);
	pnfa->SetAcceptAnyWord();

	nfa.push_back( pnfa );
	code.push_back( _T("?") );*/

}

void InitializeRegularExpressions3()
{
	/*
	Sample Program:

	m?ade?							MAD
	0[xX][0-9A-Fa-f]+				HEXA
	.*								?

	*/

	//m?ade?	MAD
	NFA *pnfa = new NFA(&memManager, _T('m'));
	pnfa->OptionalClosure();	//m?
	NFA tnfa1(&memManager, _T('a'));
	pnfa->Concatenate( tnfa1 );	//m?a
	tnfa1.ReallocateAndSet( _T('d') );
	pnfa->Concatenate( tnfa1 );	//m?ad
	tnfa1.ReallocateAndSet( _T('e') );
	tnfa1.OptionalClosure();
	pnfa->Concatenate( tnfa1 );	//m?ade?

	nfa.push_back( pnfa );
	code.push_back( _T("MAD") );

	//0[xX][0-9A-Za-z]+				HEXA
	Set *charClass = new BitSet( charSet.NumberOfSymbols() - 1, TRUE );
	pnfa = new NFA(&memManager, _T('0')); //0

	charClass->Add( charSet.IndexOf(_T('x')) );
	charClass->Add( charSet.IndexOf(_T('X')) );
	tnfa1.SetCharacterClass( charClass ); //[xX]

	pnfa->Concatenate( tnfa1 ); //0[xX]

	//Forming [0-9A-Fa-f]
	charClass = new BitSet( charSet.NumberOfSymbols() - 1, TRUE );
	int iter = charSet.IndexOf( _T('0') );
	int final = charSet.IndexOf( _T('9') );
	while( iter <= final )
		charClass->Add( iter++ );

	iter = charSet.IndexOf( _T('a') );
	final = charSet.IndexOf( _T('f') );
	while( iter <= final )
		charClass->Add( iter++ );

	iter = charSet.IndexOf( _T('A') );
	final = charSet.IndexOf( _T('F') );
	while( iter <= final )
		charClass->Add( iter++ );

	
	tnfa1.SetCharacterClass( charClass ); //[0-9a-fA-F]
	tnfa1.PlusClosure();//r+

	pnfa->Concatenate( tnfa1 ); //0[xX][0-9a-fA-F]+

	nfa.push_back( pnfa );
	code.push_back( _T("HEXA") );

	//.*	?
/*	pnfa = new NFA(&memManager);
	pnfa->SetAcceptAnyWord();
	
	nfa.push_back( pnfa );
	code.push_back( _T("?") );*/
}