Options
NameSpace = "Math.Parsing" 
ClassName = "ExpresssionParser"
Language  = CSharp

Tokens
	Number
	Identifier
	Plus
	Minus
	Multiply
	Divide
	Modulo
	Power
	Sin
	Cos
	Tan
	Cot
	Sec
	Csc
	Sinh
	Cosh
	Tanh
	Coth
	Sech
	Csch
	Arcsin
	Arccos
	Arctan
	Arccot
	Arcsec
	Arccsc
	Arcsinh
	Arccosh
	Arctanh
	Arccoth
	Arcsech
	Arccsch
	E
	Ln
	Log
	Pi
	Openpara
	Closepara
	OpenBracket
	CloseBracket 
	Comma
	EndOfFile
	InvalidToken
	
TopOfDeclaration
%%

%%

BottomOfDeclaration
%%
BottomOfDeclaration code
%%

TopOfDefinition
%%
TopOfDefinition code
%%

BottomOfDefinition
%%
BottomOfDefinition Code
%%

Grammar
///////////////////////////////////////////////////////////////////
//Exp -> Term { AddOp Term }
Exp(. SymbolTable symbols, out Node exp .) -> 
			<. _TokenType op;	Node term; .>
			Term(. symbols, out exp .) 
			{
				AddOp(. out op .)	Term(. symbols, out term .) 
				<. exp = new ExpressionNode(exp, op, temp); .>
			}
			.
///////////////////////////////////////////////////////////////////
//Term --> Factor1 {mulop Factor1}
Term(. SymbolTable symbols, out Node term .) ->	
			<. _TokenType op;	Node factor1; .>
			Factor1(. symbols, out term .)
			{
				MultOp(. out op .)	Factor1(. symbols, out factor1 .)
				<. term = new TermNode(term, op, factor1); .>
			}
			.
///////////////////////////////////////////////////////////////////
//Factor1 --> { Sign }  (UnFn | Factor2 | Fn [Factor3])  { UnFn | Fn [Factor3] }
Factor1(. SymbolsTable symbols, out Node factor1 .) -> 
			<. int tempSign, sign = 1;		Node factor1 = null, factor2 = null; .>
			{ 
				Sign(. out tempSign .) 
				<. sign *= tempSign; .> 
			}
			(
				UnFn(. symbols, out factor1 .)
				|
				IF (. LookaheadIdentifierIsVariable() .)
					Factor2(. symbols, out factor1 .)
					|
					Fn(. symbols, out factor1 .)
					(
						IF (. LookaheadIdentifierIsVariable() || LookaheadIsOpenpara() .)
						Factor3(. symbols, out factor2 .)
						<.factor1 = new TermNode(factor1, _TokenType.Multiply, factor2);.>
						|
						Eps
					)
			)
			{
				UnFn(. symbols, out factor2 .)
				<.factor1 = new TermNode(factor1, _TokenType.Multiply, factor2);.>
				|
				Fn(. symbols, out factor2 .)
				<.factor1 = new TermNode( factor1, _TokenType.Multiply, factor2);.>
				(
					IF (. LookaheadIdentifierIsVariable() || LookaheadIsOpenpara() .)
					Factor3(. symbols, out factor2 .)
					<.factor1 = new TermNode(factor1, _TokenType.Multiply, factor2);.>
					|
					Eps
				)
					
			}
			.
///////////////////////////////////////////////////////////////////
//AddOp -> Plus | Minus
AddOp(. out _TokenType op .) ->
			Plus <. op = _TokenType._Plus; .>
			|
			Minus <. op = _TokenType._Minus; .>
			.	
///////////////////////////////////////////////////////////////////
//MultOp -> Multiply | Divide | Modulo
MultOp(. out _TokenType op .) ->
			Multiply <. op = _TokenType._Multiply; .>
			|
			Divide <. op = _TokenType._Divide; .>
			|
			Modulo <. op = _TokenType._Modulo; .>
			.	
			