

using System;
using System.IO;
using System.Collections;

namespace Testing
{
class LineNumberer : IScanner, ICloneable
{
static void Main()
{
	LineNumberer ln = new LineNumberer(new StreamReader(@"j:\project\form.txt"));
	ln.Scan();
}

readonly int[] rowMap = new int[]{0, 1};
readonly int[] columnMap = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
readonly int[,] transitionTable = new int[,]{
{0, 1},
{-1, -1},
};
	readonly bool[] accepting = new bool[]
	{
false
,true
	};

	const char startSymbol = (char)0;
	const char finalSymbol = (char)127;
	const int ErrorState = -1;
	const int startStateIndex = 0;

	int indexOf(char c)
	{
		if( c >= startSymbol && c <= finalSymbol )
			return c - startSymbol ;
		else return ErrorState ;
	}

	CodeStream fin;
	char currentChar;
	int lastAccepting;//last accepting state
	int backupLength;//no of the characters read since the last accepting state
	int currentState;//the current state!
	string lexeme;//the lexeme of the current token
	bool endOfFile = false;//determines if the end of file has been reached

	public LineNumberer(StreamReader stream)
	{
		fin = new CodeStream(stream);
		lastAccepting = ErrorState;
		backupLength = 0;
		currentState = startStateIndex;
		lexeme = "";
	}

	//get the code stream
	public CodeStream CodeStream
	{
		get
		{
			return fin;
		}
	}

	//return the current lexeme
	public string Lexeme
	{
		get
		{
			return lexeme.ToString();
		}
	}

	//get the current token
	public int Scan()
	{	//restore everything
		lexeme = "";
		currentState = startStateIndex;
		lastAccepting = ErrorState;
		backupLength = 0;
		fin.Backup();
		fin.SaveCurrentPosition();

		for(;;)
		{
			//read one more character
			currentChar = fin.Peek();

			if(endOfFile || currentChar == CodeStream.TEOF && lexeme == "")
			{//end of file with no token in hands
				return -1
;
			}
			else
			{
				if(currentChar == CodeStream.TEOF && lexeme != "")
				{
					currentState = ErrorState;
					endOfFile = true;
				}
				else
				{
					fin.Advance();
					lexeme += new string(currentChar,1);
					backupLength++;

					//get the next state
					currentState = 
						transitionTable[rowMap[currentState],columnMap[indexOf(currentChar)]];
				}

				//is there an error?
				if(currentState != ErrorState)
				{//no error:

					//if it is an accepting state,save it
					if(accepting[currentState])
					{
						fin.ReplaceLastBackup();
						backupLength = 0;
						lastAccepting = currentState;
					}
				}
				else
				{//error state:

					if(lastAccepting == ErrorState)
					{
						fin.RemoveLastBackup();
						return -2
;
					}
					else
					{//accepting
						//restore the input to the last accepting state
						fin.Restore();
						lexeme = lexeme.Substring(0,lexeme.Length - backupLength);

switch(lastAccepting){
case 1:
		Console.Write(CodeStream.CurrentLineNo.ToString()+": "+Lexeme);

	break;
}

							//re-initialize the get-next-token funtion
							lexeme = "";
							currentState = startStateIndex;
							lastAccepting = ErrorState;
							backupLength = 0;
							fin.Backup();
							fin.SaveCurrentPosition();
							
						}//end of acceptance state
					}//end of error state
				}//end of the else part(it means, not yet end of file)

			}//end of the unconditional for loop
		}//end of the get-next-token function
	}//end of the lexical analyzer class

	public class CodeStream
	{
		StreamReader stream;
		Stack lines;
		Stack cols;
		Stack positions;

		int curlineno;
		int curcolumn;
		int nextlineno;
		int nextcolumn;

		int position;
		
		public const char TEOF = unchecked((char)-1);

		public CodeStream(StreamReader stream)
		{
			this.stream = stream;
			lines = new Stack();
			cols = new Stack();
			positions = new Stack();
			nextlineno = curlineno = 1;
			nextcolumn = curcolumn  = 1;
			position = 2;
		}

		public int CurrentLineNo
		{
			get
			{
				return curlineno;
			}
		}

		public int CurrentColumn
		{
			get
			{
				return curcolumn;
			}
		}

		public int NextLineNo
		{
			get
			{
				return nextlineno;
			}
		}

		public int NextColumn
		{
			get
			{
				return nextcolumn;
			}
		}

		public void SaveCurrentPosition()
		{
			curcolumn = nextcolumn;
			curlineno = nextlineno;
		}

		//Peek the next character from the stream
		public char Peek()
		{
			return unchecked((char)stream.Peek());
		}

		//read a char and advance the file pointer 
		public char ReadChar()
		{
			char ch = unchecked((char)stream.Read());
			if(ch == '\n')
			{
				nextlineno++;
				nextcolumn = 1;
			}
			else
				nextcolumn++;
			position+=2;
			return ch;
		}

		//move the file stream one step ahead
		public void Advance()
		{
			ReadChar();
		}

		//save current state
		public void Backup()
		{
			lines.Push(nextlineno);
			cols.Push(nextcolumn);
			positions.Push(position);
		}

		//restore most recently saved state
		public void Restore()
		{
			//restore saved state
			nextlineno = (int)lines.Peek();
			lines.Pop();
			nextcolumn = (int)cols.Peek();
			cols.Pop();
			stream.DiscardBufferedData();
			stream.BaseStream.Seek
				(((int)positions.Peek()) - stream.BaseStream.Position ,SeekOrigin.Current);
			position = (int)positions.Peek();
			positions.Pop();
		}

		//replace the most recently saved state with the current state
		public void ReplaceLastBackup()
		{
			//remove last saved state
			lines.Pop();
			cols.Pop();
			positions.Pop();

			//save current state
			lines.Push(nextlineno);
			cols.Push(nextcolumn);
			positions.Push(position);
		}

		//clear the last saved state, used when an invlaid token is met
		public void RemoveLastBackup()
		{
			//remove last saved state
			lines.Pop();
			cols.Pop();
			positions.Pop();
		}
	}

}

