#include "stdafx.h"
#include "Globals.h"
using namespace std;
using namespace Automata;

int errCount = 1;	
vector<tstring>		Level1ErrorsTable;
vector<tstring>		Level2ErrorsTable;


//////
TCHAR* Automata::CopyString( const TCHAR *srcString )
{
	unsigned int len = _tcsclen( srcString );
	TCHAR *retval = new TCHAR[ len + 1 ];
	return _tcscpy( retval, srcString );
}
//////
TCHAR ResolveEscapeSequence(TCHAR c)
{
	switch(c)
	{
	case _T('0'):
		return NULL;
	case _T('a'):
		return _T('\a');
	case _T('b'):
		return _T('\b');
	case _T('f'):
		return _T('\f');
	case _T('n'):
		return _T('\n');
	case _T('r'):
		return _T('\r');
	case _T('t'):
		return _T('\t');
	case _T('v'):
		return _T('\v');
	case _T('\''):
		return _T('\'');// return c;
	case _T('\"'):
		return _T('\"');// return c;
	case _T('\\'):
		return _T('\\');// return c;
	case _T('?')://Trigraphs might require us to return more
				 //than one single character.
		return _T('\?');
	default:
		return c;
	}
/*\ooo ASCII character in octal notation 
\xhh ASCII character in hexadecimal notation 
\xhhhh */
}

//////
TCHAR ParseEscapeSequence(	tstrIterator &iter,
							const tstrIterator &end)
{
	//Recall that retval < 0xFFFF if the number of octal digits 
	//in the numerical representation of the character is < 6.
	//Or if the number of decimal digit is < 5.
	int		retval = 0;
	TCHAR	c;	//Current character
	TCHAR	temp; //For holding the prefix character xX or Dd or Oo
	int		digitVal;

	//iter < end as a precondition
	temp = c = *iter++; //iter now points at the char after \c
	if( c == _T('x') || c == _T('X') ) // Possibly hex or x
	{
		if( iter == end ) //if it is the last character within the string
			return ResolveEscapeSequence( c ); //which will return c
		c = *iter++;
		//Check wether c is hexa or not
		if(! IsHexadecimalDigit(c)) //Not hexa
		{
			iter--;//So that it points to the character just after
					//the prefix char
			//return the prefix char
			return temp;
		}
		//Otherwise, it is really a hexa.
		retval = ValueOfHexadecimalDigit( c );
		for(int i = 1; i < MAX_HEXADECIMAL_DIGITS; i++)
		{
			if(iter == end)   //If reached the end
				return retval;
			c = *iter;
			digitVal = ValueOfHexadecimalDigit( c );
			if(digitVal == -1) //Not hexa
				return retval;
			else
			{
				retval = (retval << 4) + digitVal; 
				//retval = retval * 16 + digitVal;
				iter++; //Move to next character
			}
		}
		return retval;
	}
	else if( c == _T('o') || c == _T('O') ) // Possibly oct or o
	{
		if( iter == end ) //if it is the last character within the string
			return ResolveEscapeSequence( c ); //which will return c
		c = *iter++;
		//Check wether c is oct or not
		if(! IsOctalDigit(c)) //Not oct
		{
			iter--;//So that it points to the character just after
					//the prefix char
			//return the prefix char
			return temp;
		}
		//Otherwise, it is really an oct.
		retval = ValueOfOctalDigit( c );
		for(int i = 1; i < MAX_OCTAL_DIGITS; i++)
		{
			if(iter == end)   //If reached the end
				return retval; 
			c = *iter;
			digitVal = ValueOfOctalDigit( c );
			if(digitVal == -1) //Not octal digit
				return retval; //Stop here.
			else
			{
				retval = (retval << 3) + digitVal; 
				//retval = retval * 8 + digitVal;
				iter++; //Move to next character
			}
		}
		//
		if( retval >  MAX_TCHAR_VALUE)//The last char should not have
								//have been included.
		{
			iter--; //The last char is not included
			retval = (retval - digitVal) >> 3;
		}
		return retval;
	}
	else if( c == _T('D') || c == _T('d') )
	{
		if( iter == end ) //if it is the last character within the string
			return ResolveEscapeSequence( c ); //which will return c
		c = *iter++;
		//Check wether c is decimal or not
		if(! isdigit(c)) //Not Decimal
		{
			iter--;//So that it points to the character just after
					//the prefix char
			//return the prefix char
			return temp;
		}
		//Otherwise, it is really a numerical representation
		//of a unicode character.
		retval = ValueOfDecimalDigit( c );
		for(int i = 1; i < MAX_DECIMAL_DIGITS; i++)
		{
			if(iter == end)   //If reached the end
				return retval;//Return retval. 
							  //It cannot be > 0xFFFF without the 5 digits.
			c = *iter;
			digitVal = ValueOfDecimalDigit( c );
			if(digitVal == -1) //Not decimal
				return retval;
			else
			{
				retval = retval * 10 + digitVal;
				iter++; //Move to next character
			}
		}
		//
		if( retval > MAX_TCHAR_VALUE )	//The last char should not have
								//have been included.
		{
			iter--; //The last char is not included
			retval = (retval - digitVal) >> 3;
		}
		return retval;
	}
	else// a normal escape sequence rather than numerical
		return ResolveEscapeSequence( c );
}
//////
tstrIterator MatchIdentifier(
							const tstrIterator &begin, 
							const tstrIterator &end)
{
	tstrIterator iter = begin;
	TCHAR c = *iter;
	if( !isalpha(c) && c != _T('_') ) return iter;
	iter++; //Next char
	if(iter >= end) //just one char in string
		return iter;
	c = *iter;
	while( isalnum(c) || c == _T('_') )
	{
		iter++; //Look at the next char
		if( iter >= end )  //Reached the end
			return iter;
		c = *iter;	//Check next char
	}
	return iter;
}
//////
BOOL		MatchQuotedText( tstrIterator &iter,
							 const tstrIterator &end,
							 BOOL convertEscapeSequences,
							 BOOL permitEndLines,
							 tstring &text)
{
	tstrIterator tempIter;
	TCHAR c = *iter;
	while( iter < end )
	{
		c = *iter;
		switch( c )
		{
		case _T('\"')://Closing double quotes
			return TRUE;
		case _T('\\')://backslash
			if( iter + 1 >= end ) //ends with backslash ?
			{
				text += c; //append the backslash and shout!
				iter++;
				return FALSE;
			}
			if( ! convertEscapeSequences )
				text += c;//append the backslash
			iter++; //Precond: iter point just after backslash
			if(! convertEscapeSequences )
			{
				//We must append all chars consumed by the 
				//parsing of the escape sequence
				tempIter = iter;
				ParseEscapeSequence( iter, end );
				//iter points to just after the escape sequence
				//append [tempIter, iter) to text
				text.append( tempIter, iter );
				iter--; //Since it will be reincremented
			}
			else
			{
				text += ParseEscapeSequence(iter, end);
				iter--; //Since it will be reincremented
			}
			break;
		case _T('\n'):
			if(! permitEndLines)
				return FALSE;
			text += c;
			break;
		default:
			text += c;
			break;
		}

		iter++;
	}
	//return TRUE;
	return FALSE; //The string ended without " althought it should terminate with "
}