#pragma once
#include "stdafx.h"
#include <tchar.h>
#include <cctype>

extern int errCount;
extern vector<tstring>		Level1ErrorsTable;
extern vector<tstring>		Level2ErrorsTable;
using namespace std;

namespace Automata
{

	//This is necessary for Lexical Analyzer code generation
	const string DFA_CODE_GENERATION_PATH = 
		".\\DFACodeGeneration";
	const string PAIRS_NORMAL_CODE_GENERATION_PATH = 
		".\\PairsNormalCodeGeneration";
	const string REDUNDANCY_REMOVED_CODE_GENERATION_PATH = 
		".\\RedundancyRemovedCodeGeneration";


	#define DUMP_BUFFER_SIZE 1024
	inline 
	void Dump(tofstream& tout,string filename)
	{
		tifstream fin;
		TCHAR* buffer = new TCHAR[DUMP_BUFFER_SIZE];

		//read the first part of the cpp file
		fin.open(filename.c_str());
		fin.read(buffer,DUMP_BUFFER_SIZE);
		while(!fin.fail())
		{
			tout.write(buffer,DUMP_BUFFER_SIZE);
			fin.read(buffer,DUMP_BUFFER_SIZE);
		}
		tout.write(buffer,fin.gcount());
		fin.close();

		delete buffer;
	}
	
	//////////////////////////////////////////////////
	/*Global Application Constants*/
	const TCHAR		START_SYMBOL = 0;
	const TCHAR		FINAL_SYMBOL = 127;
	const tstring	OPTION_CHARACTER_SET = _T("CHARACTERSET");
	const tstring	OPTION_LANGUAGE = _T("LANGUAGE");
	const tstring	OPTION_NAMESPACE = _T("NAMESPACE");
	const tstring	OPTION_SUPER_CLASS = _T("SUPERCLASS");
	const tstring	OPTION_FUNCTION_NAME = _T("FUNCTIONNAME");
	const tstring	OPTION_RETURN_TYPE = _T("RETURNTYPE");
	const tstring	OPTION_CLASS_NAME = _T("CLASSNAME");
	const tstring	OPTION_FILE_NAME = _T("FILENAME");
	const tstring	OPTION_EOF_ACTION = _T("EOFACTION");
	const tstring	OPTION_INVALID_TOKEN_ACTION 
						= _T("INVALIDTOKENACTION");
	const tstring	OPTION_COMPRESSION_TECHNIQUE = _T("COMPRESSIONTECHNIQUE");
	const tstring	OPTION_PAIRS_THRESHOLD = _T("PAIRSTHRESHOLD");
#ifdef _UNICODE
	const int		MAX_TCHAR_VALUE = 0xFFFF;//Maximum possible value for a TCHAR.
	const int		MAX_HEXADECIMAL_DIGITS = 4;
	const int		MAX_OCTAL_DIGITS = 6;
	const int		MAX_DECIMAL_DIGITS = 5;
#else
	const int		MAX_TCHAR_VALUE = 0xFF;
	const int		MAX_HEXADECIMAL_DIGITS = 2;
	const int		MAX_OCTAL_DIGITS = 3;
	const int		MAX_DECIMAL_DIGITS = 3;
#endif
	//////////////////////////////////////////////////
	inline TCHAR LookAhead(const TCHAR* string)
	{
		return *(string+1);
	}

	TCHAR *CopyString( const TCHAR *srcString );
}

inline 
bool IsNotTSpace( int c )
{
	return !_istspace(c);
}

inline 
bool IsNotSpace( int c )
{
	return !isspace(c);
}

inline
bool IsNotWSpace( wint_t c )
{
	return !iswspace( c );
}

//Checks wether a <= x <= b or b <= x <= a.
template <class Type>
bool Between( const Type &x, const Type &a, const Type &b ) 
{
	if( x >= a && x <= b )
		return true;
	if( x >= b && x <= a )
		return true;
	return false;
}

//Checks wether c belongs to [0-9a-fA-F]
inline
bool IsHexadecimalDigit( int c )
{
	return	Between(c, (int) _T('0'), (int) _T('9')) || 
			Between(c, (int) _T('a'), (int) _T('f')) ||
			Between(c, (int) _T('A'), (int) _T('F'));
}
//Checks wether c belongs to [0-7]
inline
bool IsOctalDigit( int c )
{
	return	Between(c, (int)_T('0'), (int)_T('7'));
}
//Returns the value of the hexadecimal digit c if it belongs
//to [0-9A-Fa-f] or -1 otherwise.
inline
int ValueOfHexadecimalDigit( int c )
{
	if( Between(c, (int) _T('0'), (int) _T('9')) )
		return c - _T('0');
    if(	Between(c, (int) _T('a'), (int) _T('f')) )
		return c - _T('a') + 10;
	if( Between(c, (int) _T('A'), (int) _T('F')) )
		return c - _T('A') + 10;
	return -1;
}
//Returns the value of the octal digit c if it belongs
//to [0-7] or -1 otherwise.
inline
int ValueOfOctalDigit( int c )
{
	if( Between(c, (int) _T('0'), (int) _T('7')) )
		return c - _T('0');
    return -1;
}
//Returns the value of the decimal digit c if it belongs
//to [0-9] or -1 otherwise.
inline
int ValueOfDecimalDigit( int c )
{
	if( Between(c, (int) _T('0'), (int) _T('9')) )
		return c - _T('0');
    return -1;
}

//Returns the character corresponding to the escape sequence '\c'.
TCHAR	ResolveEscapeSequence(TCHAR c);

//Returns the escape sequence character corresponding to '\c' 
//or '\[Xx]{h}{1,4}' or '\[Oo]{oct}{1,6}' or '\[Dd]{1,5}'. If the
//value exceeds 0xFFFF, the last digit is excluded from the escape
//sequence. Eg: \O777777 = 0x3FFFF > 0xFFFF. So, It will be parsed
//as \077777 = 0x7FFF <= 0xFFFF followed by the character 7. So, 
//the function never fails (except if the preconditions are not 
//satisfied). 
//Preconditions:
//a) *iter is the character just after the backslash. 
//b) iter < end.
//Postconditions:
//a) iter points to the character just after the escape sequence.
TCHAR	ParseEscapeSequence( tstrIterator &iter,
							const tstrIterator &end);

//Matches an identifier [_a-zA-Z][_a-zA-Z0-9]+ from the string
//[begin, end) and returns an iterator for the position just
//after the last character of the identifier.
//Preconditions:
//a) begin < end. 
//b) *begin = [_a-zA-Z].
tstrIterator	MatchIdentifier(const tstrIterator &begin,
								const tstrIterator &end);
//It matches the text bracketed by double quotes "..." from the
//string [iter, end) and appends it (after ignoring the double 
//quotes to text. If convertEscapeSequences = TRUE, then escape
//sequences will be converted before appending to text. It returns 
//TRUE if the matched text ends in double quotes. Otherwise, it
//returns FALSE. If permitEndLines is FALSE, then the function 
//would also return FALSE if the text contained non-escaped 
//endlines.
//Preconditions:
//a) iter points to the character just after the first double 
//quotes.
//b) iter < end.
//Postconditions:
//a) if the function returns TRUE, *iter = _T('\"'). That is, iter
//will point to the double quotes at the end of the matched text.
BOOL MatchQuotedText(tstrIterator &iter,
					 const tstrIterator &end,
					 BOOL convertEscapeSequences,
					 BOOL permitEndLines,
					 tstring &text); //Designed mainly for
											 //use with
											//RegularExpressionTokenizer