#ifndef _OPENADDRESSHASHTABLE_H
#define _OPENADDRESSHASHTABLE_H
typedef unsigned int UINT;
typedef int BOOL;
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

template < class Type, class HashFunctionObject >
class OpenAddressHashtable
{
private:
	Type *_table;
	UINT _size;
	UINT _count;
	HashFunctionObject &hash;
	const Type _empty;
public:
	OpenAddressHashtable(UINT size,
						HashFunctionObject &hashingFunctor,
						Type emptyValue) : _size(size),
										   hash(hashingFunctor),
										   _empty(emptyValue),
										   _count(0)
	{	
		_table = new Type[_size];
		Type *begin = _table;
		Type *end = begin + _size;
		while(begin<end) *begin++ = _empty;
	}
	int Add(const Type &item)
	{
		int home;
		for(UINT i=0; i<_size; i++)
		{
			home = hash(item, i);
			if(_table[home] == _empty)
			{
				_table[home] = item;
				_count++;
				return home;
			}
		}
		return -1;
	}
	BOOL Search(Type &item) const
	{
		int home;
		for(UINT i=0; i<_size; i++)
		{
			home = hash(item, i);
			if(_table[home] == item)
			{
				item = _table[home];
				return TRUE;
			}
			if(_table[home] == _empty)
				return FALSE;
		}
		return FALSE;
	}
	UINT Count() const
	{
		return _count;
	}
	UINT Size() const
	{
		return _size;
	}
	BOOL IsFull() const
	{
		return _count>=_size;
	}
	void Clear()
	{
		Type *begin = _table;
		Type *end = begin + _size;
		while(_count>0 && begin<end)
			if( _table[begin] != _empty )
			{
				_table[begin++] = _empty;
				_count--;
			}
			else begin++;
	}
	BOOL Resize(UINT newSize, BOOL preserveOldData)
	{
		if(newSize < _count) return FALSE;
		if(newSize == _size) return TRUE;
		Type *oldTable = _table;
		UINT oldSize = _size;
		_table = new Type[newSize];
		_size = newSize;
		UINT i;
		for(i=0; i<newSize; i++)
			_table[i] = _empty;
		if(preserveOldData)
		{
			UINT itemsLeft = _count;
			_count = 0;
			Type *begin = OldTable;
			Type *end = begin + oldSize;
			while(begin<end && itemsLeft>0)
			{
				if(*begin != _empty)
				{
					Add(*begin);
					itemsLeft--;
				}
				begin++;
			}
		}
        delete[] OldTable;	
	}
	Type *GetInternalTable()
	{
		return _table;
	}
	const Type *GetInternalTable() const
	{
		return _table;
	}
	HashFunctionObject &GetHashFunction()
	{
		return _hash;
	}
	const HashFunctionObject &GetHashFunction() const
	{
		return _hash;
	}
};
template<class Type, class HashingFunction>
class DoubleHashingObject
{
private:
	UINT _m1;
	UINT _m2;
	HashingFunction &_hash;
public:
	DoubleHashingObject(	UINT size1,
							UINT size2,
							HashingFunction &f) : _m1(size1),
												  _m2(size2),
													_hash(f)
	{}
	int operator ()(const Type &item, UINT prope) const
	{
		UINT k = _hash(item);
		UINT h2 = 1 + k%_m2;
		UINT h1 = k%_m1;
		return (h1+prope*h2)%_m1;;
    }
	UINT GetSuggestedHashtableSize() const
	{
		return _m1;
	}
};
#endif