#include "stdafx.h"
#include "Globals.h"
#include "FAFactory.h"
#include "NFA.h"
#include "LexerInputReader.h"
#include "DFA.h"
#include "Set.h"
#include "RangedSet.h"
#include <fstream>
#include <iostream>
#include <string>
#include <limits>
using namespace std;
using namespace Automata;

#ifdef _UNICODE
	#define tout	wcout
	#define tin		wcin
#else	//ASCII
	#define tout	cout
	#define tin		cin
#endif

/*****Globals*****/
tstring		inputFileName;
tstring		outputFileName;

basic_ifstream<TCHAR, char_traits<TCHAR> > inputStream;
basic_ofstream<TCHAR, char_traits<TCHAR> > outputStream;

/*****Functions*****/
//It takes errorsTable and prints the errors.
void PrintStringTable(const vector<tstring> &stringTable);

//It prints the (key, value) pairs of table.
void PrintDictionary(const Dictionary &table);

//It takes an array of strings representing the command line
//arguments, and updates file names and other global variables
//correspondingly. It notifies the user of any errors it faces.
//argv: an array of strings (command line arguments).
//argc: number of entries in the array
//It returns TRUE if the command line arguments are in correct 
//format.
BOOL ParseCommandLineArguments(int argc, _TCHAR *argv[]);

//It tries to open the specified input file. If failed, it returns
//FALSE.
BOOL PrepareInput();

//Fills the option table with the default values for all options.
void InitializeOptionTable(Dictionary &optionTable);
//
void Help();


int _tmain(int argc, _TCHAR* argv[])
{
	BOOL	success;
	Dictionary optionsTable;
	BitSet	bset(	numeric_limits<TCHAR>::max()
				-	0, TRUE);
	RangedSet rangedSet( &bset, 0 );
	LexerInputReader::ErrorCode errCode;

	LexerInputReader::InitializeStaticData();
	InitializeOptionTable( optionsTable );

	inputStream.open( "LexerTest.l" );
	LexerInputReader lexer( inputStream, optionsTable );
	//Testing Parsing of escape sequences

	//Testing Parsing of character classes
	tstring ccl;// = //_T("[^\\o0-\\x0F-\\\\]");
					//_T("[\\\\]"); _T("[^]");
			//		_T("[[]");
	tout<<_T("Enter a character class:");
	getline(tin, ccl);
	tstrIterator iter = ccl.begin();
	tstrIterator end = ccl.end();
	while( iter != end )
	{
		rangedSet.Clear();
	errCode = lexer.ParseCharacterClass( iter, end, TRUE, rangedSet);
	if(iter >= end)
		tout<<_T("iter >= end")<<endl;
	else
	{
		tout<<_T("iter < end, Rest of string:")<<tstring(iter,end)<<endl;
	}
	if(errCode != LexerInputReader::NO_ERROR)
		tout<<LexerInputReader::ErrorString[ errCode ]<<endl;
	else
	{
		int min, max;
		success = rangedSet.MinimumElement(min);
		if(! success)
		{
			tout<<_T("The ranged set is empty.")<<endl;
			continue;
		}
		success = rangedSet.MaximumElement(max);
		if(! success)
		{
			tout<<_T("The ranged set is not implemented properly.")<<endl;
			continue;
		}
		tout<<_T("Start Symbol: \\d")<< min
			<<_T("		")<<(TCHAR) min<<endl;
		tout<<_T("Final Symbol: \\d")<< max
			<<_T("		")<<(TCHAR) max<<endl;	
	}
	tout<<_T("Enter a character class:");
	getline(tin, ccl);
	iter = ccl.begin();
	end = ccl.end();
	}
	return 0;
}

BOOL ParseCommandLineArguments(int argc, _TCHAR *argv[])
{
	if(argc  < 2)
	{
		tout<<_T("The syntax of the command is incorrect.\n");
		tout<<_T("Syntax: ")<<_T("lexer inputFileName [outputFileName].")
			<<endl;
		return FALSE;
	}
	if(argc > 3)
	{
		tout<<_T("The syntax of the command is incorrect.\n");
		tout<<_T("Syntax: ")<<_T("lexer inputFileName [outputFileName].")
			<<endl;
		return FALSE;
	}
	inputFileName = argv[1];
	if( argc == 3 ) outputFileName = argv[2];
	else //inputFileName.
	{
		outputFileName = argv[1];
		int dotPosition = outputFileName.find_last_of( _T('.') );
		outputFileName.erase( dotPosition );
	}
	return TRUE;
}
//////
BOOL PrepareInput()
{
	char	*fname;
	int		strLen;

	//Get ascii names
	strLen = inputFileName.size();
	fname = new char[ strLen + 1];
#ifdef _UNICODE
	wcstombs( fname, inputFileName.c_str(),  strLen );
	fname[strLen] = '\0';
#else
	strcpy(fname, inputFileName.c_str() );
#endif


	inputStream.open( fname );
	if(! inputStream.good() ) return FALSE;
	
	//Open now the output file
	delete[] fname;
//	strLen = outputFileName
//	fname = new char [
	return TRUE;
}
//////
void PrintStringTable(const vector<tstring> &stringTable)
{
	vector<tstring>::size_type i = 0, count = stringTable.size();
	tout<<_T("Number of entries: ")<<count<<endl;
	for(; i<count; i++)
		tout<<stringTable[i]<<endl;
}
//////
void PrintDictionary(const Dictionary &table)
{
	const TCHAR *formatString = _T("%-24s\t%-64s%s");
	Dictionary::const_iterator	iter,end;

	tout<<_T("Number of entries: ")<<table.size()<<endl;
	if(table.size() < 1)
		return;

	_tprintf( formatString, _T("Key"), _T("Value"), ENDLINE);
	iter = table.begin();
	end = table.end();
	for(; iter != end; iter++)
		_tprintf(	formatString,
					iter->first.c_str(),
					iter->second.c_str(),
					ENDLINE);
}
//////
void InitializeOptionTable(Dictionary &optionTable)
{
	optionTable[ OPTION_LANGUAGE ] = _T("C++");
	optionTable[ OPTION_CHARACTER_SET ] = _T("[\\d0-\\d127]");
//	optionTable[ OPTION_NAMESPACE ] = _T("MyUnsupportedUnicode");
    optionTable[ OPTION_SUPER_CLASS ] = _T("TheCppTrap");
//	optionTable[ OPTION_FUNCTION_NAME ] = _T("GreetingsCSharp");
//	optionTable[ OPTION_RETURN_TYPE ] = _T("TokenType");
	//optionTable[ OPTION_RETURN_TYPE ] = _T("EOF");
//	optionTable[ OPTION_CLASS_NAME ] = _T("UnicozeMeshCode");
//	optionTable[ OPTION_FILE_NAME ] = _T("MeshHaolac");

	options[OPTION_FILE_NAME] = _T("lex");	
	
	options[OPTION_CLASS_NAME] = _T("LexicalAnalyzer");
	options[OPTION_NAMESPACE] = _T("Compiler");
	options[OPTION_RETURN_TYPE] = _T("int");
	options[OPTION_FUNCTION_NAME] = _T("GetNextTokens");

}