#ifndef _LINKEDLIST_H
#define _LINKEDLIST_H
#include <iterator>
using namespace std;
#ifndef BOOL
#define BOOL int
#endif
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif
/**************************************/
/*Node<Type> Class*/
/**************************************/
template <class Type>
struct Node
{
	Node *Next;
	Type Data;
	Node()
	{
		Next = NULL;
		ErrorString = NULL;
	}
	Node(Type &data, Node *next): Data(data), Next(next)
	{}
    ~Node()
	{}
	void RemoveChain()
	{
		if(Next != NULL)
			Next->RemoveChain();
		delete this;
	}
};
/**************************************/
/*LinkedList<Type> Class*/
/**************************************/

template<class Type>
class LinkedList
{
private:
	Node<Type>	*head;
	Node<Type>	*tail;
	int			count;
public:
	class const_iterator : public std::iterator<input_iterator_tag,
									Type, 
									void,
									const Type *,
									const Type &>
	{
		friend class LinkedList;
	private:
		const Node<Type> *_current;
		const_iterator(const Node<Type> *node) : _current(node)
		{}
	public:
		const_iterator() : _current(NULL)
		{}
		const const_iterator& operator++() //prefix
		{
			_current = _current->Next;
			return *this;
		}
		const_iterator operator++(int) //postfix
		{
			const_iterator retval(_current);
			_current = _current->Next;
			return retval;
		}
		reference operator *() const
		{
			return _current->Data;
		}
		pointer operator ->() const
		{
			return &(_current->Data);
		}
		bool operator ==(const const_iterator& rhs) const
		{
			return _current == rhs._current;
		}
		bool operator !=(const const_iterator& rhs) const
		{
			return !(*this == rhs);
		}
	};
	class iterator : public std::iterator<forward_iterator_tag,
									Type, 
									void,
									Type *,
									Type &>
	{
		friend class LinkedList;
	private:
		Node<Type> *_current;
		iterator(Node<Type> *node) : _current(node)
		{}
	public:
		iterator() : _current(NULL)
		{}
		const iterator& operator++() //prefix
		{
			_current = _current->Next;
			return *this;
		}
		iterator operator++(int) //postfix
		{
			iterator retval(_current);
			_current = _current->Next;
			return retval;
		}
		reference operator *() const
		{
			return _current->Data;
		}
		pointer operator ->() const
		{
			return &(_current->Data);
		}
		bool operator ==(const iterator& rhs) const
		{
			return _current == rhs._current;
		}
		bool operator !=(const iterator& rhs) const
		{
			return !(*this == rhs);
		}
	};
public:
	LinkedList() : head(NULL), tail(NULL), count(0)
	{}
	void			AddToFront(Type data)
	{
		Node<Type> *node = new Node<Type>(data, head);
		head = node;
		if(tail == NULL)
			tail = node;
		count++;
	}
	void			AddToBack(Type data)
	{
		Node<Type> *node = new Node<Type>(data, NULL);
		if(tail != NULL)
			tail->Next = node;
		else //no items are present, then set head = tail = newly created node
			head = node;
		tail = node;
		count++;
	}
	void			RemoveFromFront()
	{
		if(head == tail) //there is only one item
		{
			delete head;
			head = tail = NULL;
		}
		else
		{
			Node<Type> *oldHead = head;
			head = head->Next;
			delete oldHead;
		}
		count--;
	}
	void			RemoveFromBack() /* O(n) (theta(n)) */
	{
		if(head == NULL) //No items
			return;      //Error
		if(head == tail) //Only one node
		{
			delete head;
			head = tail = NULL;
		}
		else
		{
			Node<Type> *iterator = head;
			while(iterator->Next != tail)
				iterator = iterator->Next;
			//now, iterator points to the node before tail
			delete tail;
			iterator->Next = NULL;
			tail = iterator;
		}
		count--;
	}
	const Type&		Front() const
	{
		return head->Data;
	}
	Type&			Front()
	{
		return head->Data;
	}
	const Type&		Back() const
	{
		return tail->Data;
	}
	Type&			Back()
	{
		return tail->Data;
	}
	iterator		Begin()
	{
		return iterator(head);
	}
	iterator		End()
	{
		return iterator(NULL);
	}
	const_iterator	Begin() const
	{
		return const_iterator(head);
	}
	const_iterator	End() const
	{
/*		if(tail == NULL)
			return const_iterator(NULL);
		return const_iterator(tail->Next);*/
		return const_iterator(NULL);
	}
	BOOL			IsEmpty() const
	{
		return count == 0;
	}
	void			Clear()
	{
		head->RemoveChain();
		head = tail = NULL;
		count = 0;
	}

	int				Count() const
	{
		return count;
	}
	~LinkedList()
	{
		if(head != NULL)
			head->RemoveChain();
	}
};
#endif