#include "stdafx.h"
#include "FAFactory.h"
#include "SimpleRegexConventions.h"
#include "NFA.h"

#include <iostream>
#include <cstdio>
using namespace Automata;
using namespace std;

#if UNICODE
#define tout wcout
#define tin wcin
#else
#define tout cout
#define tin cin
#endif

void ReportStatus(const NFA& nfa, const TCHAR *name);
BOOL Match(const NFA& nfa, const TCHAR *name, const TCHAR *str);
int main()
{
	SimpleRegexConventions src;
	FAFactory charSet((TCHAR)0, (TCHAR)255);
	NFA nfa(&charSet, _T("hello"), src);//6 states
	ReportStatus(nfa, _T("hello"));
	Match(nfa, _T("hello"), _T("hello"));
	Match(nfa, _T("hello"), _T("Hello"));
	Match(nfa, _T("hello"), _T("hEllo"));
	Match(nfa, _T("hello"), _T("hell"));
	Match(nfa, _T("hello"), _T("helloo"));
	TCHAR *regex, *match;
	const int MAX = 512;
	regex = new TCHAR[MAX];
	match = new TCHAR[MAX];
	*regex = *match = NULL;
	do
	{
		tout<<_T("Enter an SQL-compatible regular expression: ");
		tin.getline(regex, MAX, _T('\n'));
		if(*regex == NULL) break;
		nfa.Reset(regex, src);
		ReportStatus(nfa, regex);
		do
		{
			tout<<_T("Enter a string to match: ");
			tin.getline(match, MAX, _T('\n'));
			if(*match == NULL) break;
			Match(nfa, regex, match);
		}while(*match!=NULL);
	}while(*regex != NULL);
	delete []regex;
	delete []match;
}

void ReportStatus(const NFA& nfa, const TCHAR *name)
{
	tout<<endl<<endl
		<<_T("-------------------------")<<endl
		<<_T("NFA ")<<name<<" status:"<<endl
		<<_T("-------------------------")<<endl
		<<_T("IsRejectionMachine():    ")<<(nfa.IsRejectionMachine()?"TRUE":"FALSE")<<endl
		<<_T("NumberOfStates():        ")<<nfa.NumberOfStates()<<endl
		<<_T("NumberOfUselessStates(): ")<<nfa.NumberOfUselessStates()<<endl
		<<_T("NumberOfUselessBytes():  ")<<nfa.NumberOfUselessBytes()<<endl
		<<_T("-------------------------")<<endl
		<<endl<<endl;
}

BOOL Match(const NFA& nfa, const TCHAR *name, const TCHAR *str)
{
	if(nfa.Match(str))
	{
		tout<<_T("NFA ")<<name<<_T(" has matched \"")<<str<<_T("\"!!")<<endl;
		return TRUE;
	}
	tout<<_T("NFA ")<<name<<_T(" has not matched \"")<<str<<_T("\"!!")<<endl;
	return FALSE;
}