#include "stdafx.h"
#include "Globals.h"
#include "SimpleRegexConventions.h"
#include ".\nfa.h"

using namespace Automata;
#define Memo NFA::MemoryManager
typedef SimpleRegexConventions SRC;
//NFA Machine Number of states
const int AnySymbol = 2;
const int AnyWord = 4;
const int SingleSymbol = 2;
const int Concat = -1;
const int Or = 2;
const int Closure = 2;
const int CharClass = 2;
Memo::MemoryManager() :
						count(0),
						nextIndex(0),
						State(NULL),
						deletedSlot(-1),
						father(NULL)
{
}


Memo::MemoryManager(const FAFactory *creator) : 
						count(0),
						nextIndex(0),
						State(NULL),
						deletedSlot(-1),
						father(creator)
{}
Memo::MemoryManager(const FAFactory *creator,int numberOfStates) :	
											count(numberOfStates),
											nextIndex(0),
											deletedSlot(-1),
											father(creator)
{
	State = new NondeterministicState[count];
}
Memo::MemoryManager(const MemoryManager& mm) :  count(mm.count),
												nextIndex(mm.nextIndex),
												deletedSlot(mm.deletedSlot),
												father(mm.father)
{
	State = new NondeterministicState[count];
	memcpy(State, mm.State, count*sizeof(NondeterministicState));
}
Memo& Memo::operator =(const Memo& mm)
{
	if(this == &mm) return *this;
	count = mm.count;
	nextIndex = mm.nextIndex;
	deletedSlot = mm.deletedSlot;
	father = mm.father;
	State = new NondeterministicState[count];
	memcpy(State, mm.State, count*sizeof(NondeterministicState));
	return *this;
}
Memo::~MemoryManager()
{
	if(State != NULL)
		delete[] State;
	State = NULL;
}
void Memo::ReInitialize(const FAFactory *creator)
{
	count = 0;
	nextIndex = 0;
	if(State != NULL)
	{
		delete[] State;
		State = NULL;
	}
    deletedSlot = -1;
	father = creator;
}
void Memo::ReInitialize(const FAFactory *creator, int numberOfStates)
{
	count = numberOfStates;
	nextIndex = 0;
	if(State != NULL) delete[] State;
	State = new NondeterministicState[ numberOfStates ];
    deletedSlot = -1;
	father = creator;
}
int Memo::GetFreeSlot()
{
	if(deletedSlot == -1)
	{
		if(nextIndex == count)
			if(count == 0)
				Resize(DEFAULT_SIZE, TRUE);
			else
				Resize(count << 1, TRUE);
		return nextIndex++;
	}
	//deletedCount--;
	int topDeletedState = deletedSlot;
	deletedSlot = State[deletedSlot].GetEmptyTransition1();
	return topDeletedState;

}
void Memo::DeleteState(int state)		//updates deleted count
{
	State[state].SetEmptyTransitions(deletedSlot, ErrorState);
	deletedSlot = state;
	//deletedCount++;
}
int Memo::CreateState()
{
	return GetFreeSlot();
}
int Memo::CreateState(TCHAR symbol, int state)
{
	int newSlot = GetFreeSlot();
	State[newSlot].SetTransition(symbol, state);
	return newSlot;
}
int Memo::CreateState(Set *charClass, int state)
{
	int newSlot = GetFreeSlot();
	State[newSlot].SetTransition(charClass, state);
	return newSlot;
}
int Memo::CreateState(int emptyTransition0, int emptyTransition1)
{
	int newSlot = GetFreeSlot();
	State[newSlot].SetEmptyTransitions(emptyTransition0, emptyTransition1);
	return newSlot;
}
void Memo::GetNextState(TCHAR symbol, const Set* sourceSet, Set *clearedTargetSet) const
{
	int count = sourceSet->NumberOfElementsRequested();
	for(int i=0;i<count;i++)
		if(sourceSet->Contains(i))
		{
			int next;
			if(State[i].GetType() == NondeterministicState::CharacterClassTransition)
                next = State[i].NextState(father->IndexOf(symbol));
			else
				next = State[i].NextState(symbol);
			if(next != ErrorState)
				clearedTargetSet->Add(next);
		}
}
void Memo::Resize(int newCount, BOOL preserveOldStates)
{
	NondeterministicState *pState = new NondeterministicState[newCount];
	if(preserveOldStates)
	{
		if(State!=NULL)
		{
			//memcpy(pState, State, count*sizeof(NondeterministicState)); //copy old states
			for(int i=0; i<count; i++)
			{
				pState[i] = State[i];
				if(State[i].GetType() == NondeterministicState::CharacterClassTransition)
					State[i].SetEmptyTransitions(ErrorState, ErrorState);
			}
			delete[] State;		//Delete old States
			State = pState;		//point to new states
			count = newCount;	//set count
								//Dont touch nextIndex
		}
		else
		{
			State = pState;
			count = newCount;
			nextIndex = 0;
			deletedSlot = -1;
		}
	}
	else
	{
		if(State!=NULL)		delete[] State;
		State = pState;
		count = newCount;
		nextIndex = 0;
		deletedSlot = -1;
	}
}
void NFA::MemoryManager::_GetEmptyClosure(int state, Set *statesSet) const
{
	statesSet->Add(state);
	if(State[state].GetType() == NondeterministicState::EmptyTransitions)
	{
		int s = State[state].GetEmptyTransition1();
		if(s != ErrorState && !statesSet->Contains(s))
			_GetEmptyClosure(s, statesSet);
		s = State[state].GetEmptyTransition2();
		if(s != ErrorState && !statesSet->Contains(s))
			_GetEmptyClosure(s, statesSet);
	}
}
void NFA::MemoryManager::GetEmptyClosure(const Set* sourceSet, Set *clearedTargetSet) const
{
	//targetSet->SetToEmptySet();
	int n = sourceSet->NumberOfElementsRequested();
	for(int i=0;i<n;i++)
		if(sourceSet->Contains(i))
			_GetEmptyClosure(i, clearedTargetSet);
}
void NFA::DestroyMemoryManager()
{
	if(mustDestroyMemory && memory != NULL)
		delete memory;
}
enum State
{
	Normal,
	Escaping,
	BeginningCharacterClass,
	InsideCharacterClass,
	BeginningRejectionClass,
	InsideRejectionClass,
	AcceptingRange,
	RejectingRange
};
NFA::NFA(Memo *mem, TCHAR symbol) : memory(mem), mustDestroyMemory(false)
{
	ReallocateAndSet(symbol);
}
NFA::NFA(Memo *mem) : memory(mem), mustDestroyMemory(false)
{
	startState = finalState = ErrorState;
}
void NFA::MinimizeMemory()
{
	if(NumberOfUselessBytes()==0)
		return;
	memory->Resize(memory->NumberOfStates(), TRUE);
}
BOOL NFA::Match(const TCHAR *szString, int length) const
{
	TCHAR *string = const_cast<TCHAR *>(szString);
	TCHAR copy = string[length];
	string[length] = NULL;
	BOOL retval = Match(string);
	string[length] = copy;
	return retval;
}
BOOL NFA::Match(const TCHAR *szString) const
{
	if(finalState == ErrorState)
		return FALSE;
	int maxStateIndex = NumberOfStates() - 1;
	BitSet currentState(maxStateIndex);
	BitSet stateCopy(maxStateIndex);
	stateCopy.Add(startState);
	memory->GetEmptyClosure(&stateCopy, &currentState); //currentState has emptyClosure now.
	TCHAR currentChar;
	while((currentChar = *szString++) != NULL)
	{
		stateCopy.SetToEmptySet();
		memory->GetNextState(currentChar, &currentState, &stateCopy);//stateCopy holds the nextState
		currentState.SetToEmptySet();
		memory->GetEmptyClosure(&stateCopy, &currentState);//currentState has emptyClosure now.
		//if(currentState.IsEmpty()) return FALSE;
	}
	//Now, we check currentState for acceptance...
	return currentState.Contains(finalState);
}
void NFA::Or(const NFA &nfa)
{
	int nStart = memory->CreateState(startState, nfa.startState);
	int nfinal = memory->CreateState();
	startState = nStart;
	memory->GetState(finalState).SetEmptyTransitions(nfinal, ErrorState);
	memory->GetState(nfa.finalState).SetEmptyTransitions(nfinal, ErrorState);
	finalState = nfinal;
}
void NFA::Concatenate(const NFA &nfa)
{
	/*This will be subject to optimization. The current final state should mimic
	 *the behaviour of the nfa.startState. Then, nfa.startState is to be deleted.
	 *We can implement an avail list by providing using the empty0 value to point to
	 *the next empty state in the list and maintain a list head with the manager.
	 *This is excellent!
	 */
	//The next statement would be better implemented by memcpy().
	memory->GetState(finalState) = memory->GetState(nfa.startState);
	finalState = nfa.finalState;
    memory->DeleteState(nfa.startState);
}
void NFA::ReallocateAndSet(TCHAR symbol)
{
	finalState = memory->CreateState();
	startState = memory->CreateState(symbol, finalState);
}
//r*: 
//Accepts zero or more instances of the underlying 
//regular expression.
void NFA::Closure()
{
	int newFinal = memory->CreateState();
	int newStart = memory->CreateState(startState, newFinal);
	memory->GetState(finalState).SetEmptyTransitions(newFinal, startState);
	//startState is more efficient than newStartState because it saves depth.
	startState = newStart;
	finalState = newFinal;
}
//r+:
//Accepts one or more instances of the underlying
//regular expression.
void NFA::PlusClosure()
{
	int newFinal = memory->CreateState();
	NondeterministicState &final = memory->GetState( finalState );
	final.SetEmptyTransitions( startState, newFinal );
	finalState = newFinal;
	//Added after the discovery of the error:
	int newStart = memory->CreateState( startState, ErrorState );
	startState = newStart;
}
//r?
//Accepts the regular expression (r|epsilon)
void NFA::OptionalClosure()
{
	int newStart = memory->CreateState( startState, finalState );
	startState = newStart;
}
void NFA::SetAcceptAnySymbol()
{
	/*It seems that working with character classes will be messy. Although easier,
	 *it implies more complex efficiency issues that are totally beaten when working
	 *with character classing. For example, a|b|c|d|e can be represented as 
	 *( ( ((a+b)+c) +d) +e)
     *(((a+b)+(c+e))+e)
	 *The number of different representation increases as there are more choices.
	 *The optimal representation is the one that has minimum depth.
	 *So, we shall use character classes.
	 */
	int finalSymbolIndex = memory->GetFAFactory()->NumberOfSymbols()-1;
	BitSet *bset = new BitSet(finalSymbolIndex, FALSE);
	finalState = memory->CreateState();
	startState = memory->CreateState(bset, finalState);
}
void NFA::SetAcceptAnyWord()
{
	SetAcceptAnySymbol();
	Closure();
}
void NFA::SetAcceptEmptyWord()
{
	startState = finalState = memory->CreateState();
}
void NFA::SetCharacterClass(Set *set)
{
	finalState = memory->CreateState();
	startState = memory->CreateState(set, finalState);
}
void NFA::MakeRejectionMachine()
{
	finalState = ErrorState;
	startState = memory->CreateState();
}
void NFA::DeleteStates()
{
	memory->DeleteState(startState);
	if(finalState != startState)
		memory->DeleteState(finalState);
}
NFA::~NFA(void)
{
	if(mustDestroyMemory && memory != NULL)
		delete memory;
}