#pragma once
#include "FAFactory.h"
#include "Set.h"
#include "NondeterministicState.h"
#include "SimpleRegexConventions.h"


namespace Automata
{
class NFA
{
public:
//------------------------------------------------------------------------------------------
	class MemoryManager
	{
		//typedef vector< NondeterministicState > StateVector;
		static const int DEFAULT_SIZE = 512;
	private:
		const FAFactory	*father;
		int			nextIndex;
		int			count;
		int			deletedSlot;
		//We can also provide #OfDeletedSlots
		NondeterministicState	*State;
	private:
		void	_GetEmptyClosure(int state, Set *statesSet) const;
		int		GetFreeSlot();					//updates deleted count
	public:
		MemoryManager();	//Newly added constructor. 30-1-2006
		MemoryManager(const FAFactory *creator);
		MemoryManager(const FAFactory *creator, int numberOfStates);
		MemoryManager(const MemoryManager&);
		MemoryManager& operator =(const MemoryManager&);
		~MemoryManager();
	public:
		void		ReInitialize(	const FAFactory *creator,
									int numberOfStates );	//Deletes the previous state vector (if allocated) and reallocates the state vector for the specified number of states (if > 0).
		void		ReInitialize( const FAFactory *creator ); //Deletes the previous state vector (if allocated)
        int			CreateState();
		int			CreateState(TCHAR symbol, int state);
		int			CreateState(Set *charClass, int state);
		int			CreateState(int emptyTransition0, int emptyTransition1);
		void		DeleteState(int state);		//updates deleted count
		void		GetEmptyClosure(int state, Set *clearedStatesSet)
		{
			_GetEmptyClosure(state, clearedStatesSet);
		}
		void		GetEmptyClosure(const Set* sourceSet, Set *clearedTargetSet) const;
		void		GetNextState(TCHAR symbol, const Set* sourceSet, Set *clearedTargetSet) const;
		void		Resize(int newCount, BOOL preserveOldStates);
		const FAFactory*	GetFAFactory() const
		{ return father; }
	public:
		NondeterministicState& GetState(int index)
		{
			return State[index];
		}
		int			MaximumNumberOfStates() const
		{
			if(State == NULL)
				return 0;
			return count;
		}
		int			NumberOfStates() const
		{
			return nextIndex;
		}
		int			NumberOfStatesLeft() const		//Invalid. Doesnot count deleted states
		{
			return count - nextIndex;
		}
		void		ResetStates()
		{
			nextIndex = 0;
			deletedSlot = -1;
		}
	public:
		static int MaximumStatesSuitableFor(const TCHAR *regex,
											const TCHAR *meta );
	};
//------------------------------------------------------------------------------------------
private:
	BOOL			mustDestroyMemory;
public:
	MemoryManager	*memory;
	int				startState;
	int				finalState;
public:
	~NFA(void);
public:
	void MinimizeMemory();
	BOOL Match(const TCHAR *szString) const;
	BOOL Match(const TCHAR *szString, int length) const;
//	int MatchLongest(const TCHAR *string, Set *initialState);
//	int MatchShortest(const TCHAR *string, Set *initialState); //returns the char just after
															   //the matched string or -1 if
																//not matched.
public:
	int NumberOfUselessStates() const
	{
		return memory->NumberOfStatesLeft();
	}
	int NumberOfUselessBytes() const
	{
		return memory->NumberOfStatesLeft() * sizeof(NondeterministicState);
	}
	int NumberOfStates() const
	{
		return memory->NumberOfStates();
	}
	BOOL IsRejectionMachine() const
	{
		return finalState == EmptySet;
	}
public:
	//NFA building routines
	NFA(MemoryManager *mem,
		TCHAR symbol);
	NFA(MemoryManager *mem);
	void SetAcceptAnyWord();
	void SetAcceptAnySymbol();
	void SetAcceptEmptyWord();
	void SetCharacterClass(Set *set);
//	void SetAcceptAnySymbolExceptEndline();
	void ReallocateAndSet(TCHAR symbol);
	void Concatenate(const NFA& nfa);
	void Or(const NFA& nfa);
	void Closure(); //r*
	void PlusClosure(); //r+
	void OptionalClosure(); //r?
	void DeleteStates();
	void DestroyMemoryManager();
	void MakeRejectionMachine();
};
};