#include "stdafx.h"
#include "Globals.h"
#include "SimpleRegexConventions.h"
#include ".\nfa.h"

using namespace Automata;
#define Memo NFA::MemoryManager
typedef SimpleRegexConventions SRC;
Memo::MemoryManager(FAFactory *creator) : 
						count(0),
						nextIndex(0),
						State(NULL),
						deletedSlot(-1),
						father(creator)
{}
Memo::MemoryManager(FAFactory *creator,int numberOfStates) :	
											count(numberOfStates),
											nextIndex(0),
											deletedSlot(-1),
											father(creator)
{
	State = new NondeterministicState[count];
}
Memo::MemoryManager(const MemoryManager& mm) :  count(mm.count),
												nextIndex(mm.nextIndex),
												deletedSlot(mm.deletedSlot),
												father(mm.father)
{
	State = new NondeterministicState[count];
	memcpy(State, mm.State, count*sizeof(NondeterministicState));
}
Memo& Memo::operator =(const Memo& mm)
{
	count = mm.count;
	nextIndex = mm.nextIndex;
	deletedSlot = mm.deletedSlot;
	father = mm.father;
	State = new NondeterministicState[count];
	memcpy(State, mm.State, count*sizeof(NondeterministicState));
}
Memo::~MemoryManager()
{
	if(State != NULL)
		delete[] State;
	State = NULL;
}
int Memo::GetFreeSlot()
{
	if(deletedSlot == -1)
		return nextIndex++;
	//deletedCount--;
	int topDeletedState = deletedSlot;
	deletedSlot = State[deletedSlot].GetEmptyTransition1();
	return topDeletedState;

}
void Memo::DeleteState(int state)		//updates deleted count
{
	State[state].SetEmptyTransitions(deletedSlot, ErrorState);
	deletedSlot = state;
	//deletedCount++;
}
int Memo::CreateState()
{
	return GetFreeSlot();
}
int Memo::CreateState(TCHAR symbol, int state)
{
	int newSlot = GetFreeSlot();
	State[newSlot].SetTransition(symbol, state);
	return newSlot;
}
int Memo::CreateState(Set *charClass, int state)
{
	int newSlot = GetFreeSlot();
	State[newSlot].SetTransition(charClass, state);
	return newSlot;
}
int Memo::CreateState(int emptyTransition0, int emptyTransition1)
{
	int newSlot = GetFreeSlot();
	State[newSlot].SetEmptyTransitions(emptyTransition0, emptyTransition1);
	return newSlot;
}
void Memo::GetNextState(TCHAR symbol, const Set* sourceSet, Set *clearedTargetSet)
{
	int count = sourceSet->NumberOfElementsRequested();
	for(int i=0;i<count;i++)
		if(sourceSet->Contains(i))
		{
			int next;
			if(State[i].GetType() == NondeterministicState::CharacterClassTransition)
                next = State[i].NextState(father->IndexOf(symbol));
			else
				next = State[i].NextState(symbol);
			if(next != ErrorState)
				clearedTargetSet->Add(next);
		}

}
void Memo::Resize(int newCount, BOOL preserveOldStates)
{
	NondeterministicState *pState = new NondeterministicState[newCount];
	if(preserveOldStates)
	{
		if(State!=NULL)
		{
			memcpy(pState, State, newCount*sizeof(NondeterministicState)); //copy old states
			delete[] State;		//Delete old States
			State = pState;		//point to new states
			count = newCount;	//set count
								//Dont touch nextIndex
		}
		else
		{
			State = pState;
			count = newCount;
			nextIndex = 0;
		}
	}
	else
	{
		if(State!=NULL)		delete[] State;
		State = pState;
		count = newCount;
		nextIndex = 0;
	}
}

int Memo::MaximumStatesSuitableFor(const TCHAR *regex,
								   const SimpleRegexConventions &metaConventions)
{
	TCHAR c = *regex;
	while(c)
	{
		switch(c)
		{
			
		}
		c = *++regex;
	}
}
void NFA::MemoryManager::_GetEmptyClosure(int state, Set *statesSet)
{
	statesSet->Add(state);
	if(State[state].GetType() == NondeterministicState::SymbolTransition)
		return;
	int s = State[state].GetEmptyTransition1();
	if(s != ErrorState && !statesSet->Contains(s))
		_GetEmptyClosure(s, statesSet);
	s = State[state].GetEmptyTransition2();
	if(s != ErrorState && !statesSet->Contains(s))
		_GetEmptyClosure(s, statesSet);
}
void NFA::MemoryManager::GetEmptyClosure(const Set* sourceSet, Set *clearedTargetSet)
{
	//targetSet->SetToEmptySet();
	int n = sourceSet->NumberOfElementsRequested();
	for(int i=0;i<n;i++)
		if(sourceSet->Contains(i))
			_GetEmptyClosure(i, clearedTargetSet);
}
void NFA::DestroyMemoryManager()
{
	if(mustDestroyMemory && memory != NULL)
		delete memory;
}
NFA::NFA(FAFactory *creator,
		const TCHAR *regex,
		const SimpleRegexConventions &metaConventions)
{
	Reset(creator, regex, metaConventions);
}
enum State
{
	Normal,
	Escaping,
	BeginningCharacterClass,
	InsideCharacterClass,
	BeginningRejectionClass,
	InsideRejectionClass,
	AcceptingRange,
	RejectingRange
};
void NFA::Reset(	FAFactory *creator,
					const TCHAR *regex,
					const SRC &metaConventions)
{
	mustDestroyMemory = true;
	int expectedNumberOfStates = Memo::MaximumStatesSuitableFor(regex, metaConventions);
	if(expectedNumberOfStates < 1)
	{
		memory = new Memo(creator, 1);
		MakeRejectionMachine();
	}
	memory = new Memo(creator, expectedNumberOfStates);
    SetAcceptEmptyWord();
	NFA nfa(memory);
	TCHAR *meta = new TCHAR[SimpleRegexConventions::NumberOfMetaCharacters];
	metaConventions.GetMetaCharacter((TCHAR**)&meta);
	State state = Normal;
	State stateBeforeEsc;
	TCHAR c = *regex;
	TCHAR lookAhead = NULL;
	Set *charClass = NULL;
	TCHAR storedChar = EmptySet;
	int maxSymbolIndex = creator->NumberOfSymbols()-1;
	int symbolIndex;
	int beginIndex;
	while(c != NULL)
	{
		switch(state)
		{
		case Normal:
			switch(c)
			{
		//In SQL Server, ] creates a rejecting machine
			case meta[SRC::AnyWord]:
				nfa.SetAcceptAnyWord();
				Concatenate(nfa);
				break;
			case meta[SRC::AnySingleCharacter]:
				nfa.SetAcceptAnySymbol();
				Concatenate(nfa);
				break;
			case meta[SRC::Escape]:
				state = Escaping;
				stateBeforeEsc = Normal;
				break;
			case meta[SRC::LeftBracket]:
				state = BeginningCharacterClass;
				break;
			default:
				nfa.ReallocateAndSet(c);
				Concatenate(nfa);
			}
			break;
		case BeginningCharacterClass:
			switch(c)
			{
			case meta[SRC::RightBracket]: //[]: Stop, convert this machine to Rejection
				memory->Resize(1, FALSE);
				MakeRejectionMachine();
				break;
			case meta[SRC::NotOneOfThose]: //^:
				if(LookAhead(regex) == meta[SRC::RightBracket])//[^]
				{
					regex++;//regex -> "].......". So, we consumed the ']' iteration.
					nfa.ReallocateAndSet(meta[SRC::NotOneOfThose]);
					Concatenate(nfa);
					state = Normal;
					break;
				}
				charClass = new BitSet(maxSymbolIndex, FALSE);
				state = BeginningRejectionClass;
				break;
			case meta[SRC::Escape]:
				state = Escaping;
				stateBeforeEsc = BeginningCharacterClass;	//charClass not created yet!!
				break;
			default:	// {], ^}` : ["symbol"
				lookAhead = LookAhead(regex);
				if(lookAhead == meta[SRC::RightBracket])
				{
					regex++;
					nfa.ReallocateAndSet(c);
					Concatenate(nfa);
					state = Normal;
				}
				else if(lookAhead == meta[SRC::RangeBetween])// [symb-
				{
					regex++;
					//check if the actual next char is >= c
					//if ok, create charClass and set the range true and go in state:
					//InsideCharacterClass
					//if not, set state = BeginningCharClass
					lookAhead = LookAhead(regex);
					if(lookAhead == NULL) //Just break and it will be converted to rejector
						break;
					regex++;
					if(lookAhead == meta[SRC::Escape])//[symb-\ <--
					{
						lookAhead = LookAhead(regex);
						if(lookAhead == NULL)
							break;
						regex++;
					}
					if(lookAhead>=c)//[c-lookAhead <--
					{
						charClass = new BitSet(maxSymbolIndex, TRUE);
                        while(c<=lookAhead)
							charClass->Add(creator->IndexOf(c++));
						state = InsideCharacterClass;
					}
					//otherwise, the range is invalid and we should stay in the same state
					//else{state = BeginningCharacterClass;}
				}
				else	//[s <-- s or [(InvalidRanges)* s <-- s
				{
					charClass = new BitSet(maxSymbolIndex, TRUE);
					charClass->Add(creator->IndexOf(c));
					state = InsideCharacterClass;
				}	
			}
			break;
		case InsideCharacterClass:	//[s+s <--
			switch(c)
			{
				//case '-': Normal. Treat it as other
			case meta[SRC::RightBracket]: //[s+] <--
				nfa.SetCharacterClass(charClass);
				Concatenate(nfa);
				state = Normal;
				charClass = NULL;//The NondeterministicState will delete it.
				break;
			case meta[SRC::Escape]: //[s+\ <---
				state = Escaping;
				stateBeforeEsc = InsideCharacterClass;
				break;
			default: //[s+su <---
				if(LookAhead(regex) == meta[SRC::RangeBetween]) //[s+s-
				{
					regex++;
					storedChar = c;
					state = AcceptingRange;
				}
				else	//[s+ss
				{
					//remain on the same state
					charClass->Add(creator->IndexOf(c));
				}
			}
			break;
		case AcceptingRange: //[s-u <--- , [ssss...-u <--- , [ssss...-\ <---
			switch(c)
			{
			case meta[SRC::Escape]:	//[sss-\ <----
					//Same storedChar
					stateBeforeEsc = AcceptingRange;
					state = Escaping;
					break;
			default: //[sss-u
				int symbolIndex = creator->IndexOf(c);
				int beginIndex = creator->IndexOf(storedChar);
				for(int i = beginIndex; i<=symbolIndex; i++)
					charClass->Add(i);
				state = InsideCharacterClass;
			}
			break;
		case BeginningRejectionClass:
			switch(c)
			{
				//case ']': [^] already handled and the bitclass is now allocated
			case meta[SRC::Escape]://[^\ <--
				stateBeforeEsc = BeginningRejectionClass;
				state = Escaping;
				break;
			default: //[^anyThing
				if(LookAhead(regex) == meta[SRC::RangeBetween])//[^any-
				{
					regex++;
					storedChar = c;
					state = RejectingRange;
				}
				else	//[^any any
				{
                    charClass->Remove(creator->IndexOf(c));
					state = InsideRejectionClass;
				}
			}
		case InsideRejectionClass:
			switch(c)
			{
			case meta[SRC::RightBracket]://[^ssss]
				nfa.SetCharacterClass(charClass);
				Concatenate(nfa);
				charClass = NULL;
				state = Normal;
				break;
			case meta[SRC::Escape]://[^ssss\ <---
				stateBeforeEsc = InsideCharacterClass;
				state = Escaping;
				break;
			default:
				if(LookAhead(regex) == meta[SRC::RangeBetween])
				{
					regex++;
					storedChar = c;
					state = RejectingRange;
				}
				else
				{
					//remain on the same state...
					charClass->Remove(creator->IndexOf(c));
				}	
			}
			break;
		case RejectingRange:
			switch(c)
			{
			case meta[SRC::Escape]:
				stateBeforeEsc = RejectingRange;
				state = Escaping;
				//keep storedChar
				break;
			default:
				int symbolIndex = creator->IndexOf(c);
				int beginIndex = creator->IndexOf(storedChar);
				for(int i = beginIndex; i<=symbolIndex; i++)
					charClass->Remove(i);
				state = InsideRejectionClass;
			}
			break;
		case Escaping:
			switch(stateBeforeEsc)
			{
			case Normal:
				nfa.ReallocateAndSet(c);
				Concatenate(nfa);
				state = Normal;
				break;
			case BeginningCharacterClass: //[\u <---
				lookAhead = LookAhead(regex);
				if(lookAhead == meta[SRC::RightBracket])//[\u]
				{
					regex++;
					nfa.ReallocateAndSet(c);
					Concatenate(nfa);
					state = Normal;
				}
				else
				{
					if(lookAhead == meta[SRC::RangeBetween])//[\u-
					{
						regex++;
						//check if the next actual char is less than c
						lookAhead = LookAhead(regex);
						if(lookAhead == NULL)
							break;
						regex++;
						if(lookAhead == meta[SRC::Escape])
						{
							lookAhead = LookAhead(regex);
							if(lookAhead == NULL)
								break;
							regex++;
						}
						if(lookAhead>=c)
						{
							charClass = new BitSet(maxSymbolIndex, TRUE);
							while(c<=lookAhead)
								charClass->Add(creator->IndexOf(c++));
							state = InsideCharacterClass;
							break;								
						}
						//else stay at BeginningCharacterClass
						else 
							state = BeginningCharacterClass;
					}
					charClass = new BitSet(maxSymbolIndex, TRUE);
					charClass->Add(creator->IndexOf(c));
					state = InsideCharacterClass;
				}
				break;
			case InsideCharacterClass:
				if(LookAhead(regex) == meta[SRC::RangeBetween])
				{
					regex++;
					storedChar = c;
					state = AcceptingRange;
				}
				else
				{
					charClass->Add(creator->IndexOf(c));
					state = InsideCharacterClass;
				}
				break;
			case AcceptingRange:
				//storedChar = c;
				symbolIndex = creator->IndexOf(c);
				beginIndex = creator->IndexOf(storedChar);
				for(int i = beginIndex; i<=symbolIndex; i++)
					charClass->Add(i);
				state = InsideCharacterClass;
				break;
			case BeginningRejectionClass://[^\_
				if(LookAhead(regex) == meta[SRC::RangeBetween])
				{
					regex++;
					storedChar = c;
					state = RejectingRange;
				}
				else
				{
					charClass->Remove(creator->IndexOf(c));
					state = InsideRejectionClass;
				}
				break;
			case InsideRejectionClass:
				if(LookAhead(regex) == meta[SRC::RangeBetween]) //[^sss\s -
				{
					regex++;
					state = RejectingRange;
					storedChar = c;
				}
				else			//[^sss\s
				{
					charClass->Remove(creator->IndexOf(c));
					state = InsideRejectionClass;
				}
				break;
			case RejectingRange:
				symbolIndex = creator->IndexOf(c);
				beginIndex = creator->IndexOf(storedChar);
				for(int i = beginIndex; i<=symbolIndex; i++)
					charClass->Remove(i);
				state = InsideRejectionClass;
				break;
			}
			break;	//Break Escaping
		}
		c = *++regex;
	}
	delete[] meta;
	if(state != Normal)
	{
		if(charClass != NULL)
			delete charClass;
        memory->Resize(1, FALSE);
		MakeRejectionMachine();
	}
}
void NFA::Reset(const TCHAR *regex, const SRC &metaConventions)
{
	FAFactory *creator = memory->GetFAFactory();
	DestroyMemoryManager();
	Reset(creator, regex, metaConventions);
}
NFA::NFA(Memo *mem, TCHAR symbol) : memory(mem), mustDestroyMemory(false)
{
	ReallocateAndSet(symbol);
}
NFA::NFA(Memo *mem) : memory(mem), mustDestroyMemory(false)
{
	startState = finalState = ErrorState;
}

/*NFA::NFA(const FAFactory *creator,
		 const TCHAR *substring) : 	father(creator), mustDestroyMemory(TRUE)
{
	SimpleRegexConventions rgx();
	memory = new NFA::MemoryManager();
	TCHAR *cpy = new TCHAR[1 + strlen(substring) + 1];
	cpy[0] = _T('%');
	_tcscpy(cpy+1, substring);
	BuildNFA(cpy, rgx);
	delete[] cpy;
}*/
void NFA::MinimizeMemory()
{
	if(NumberOfUselessBytes==0)
		return;
	memory->Resize(memory->NumberOfStates(), TRUE);
}
BOOL NFA::Match(const TCHAR *szString, int length)
{
	TCHAR *string = const_cast<TCHAR *>(szString);
	TCHAR copy = string[length];
	string[length] = NULL;
	Match(string);
	string[length] = copy;
}
BOOL NFA::Match(const TCHAR *szString)
{
	if(finalState == ErrorState)
		return FALSE;
	BitSet currentState(NumberOfStates());
	BitSet stateCopy(NumberOfStates());
	stateCopy.Add(startState);
	memory->GetEmptyClosure(&stateCopy, &currentState); //currentState has emptyClosure now.
	TCHAR currentChar;
	while((currentChar = *szString++) != NULL)
	{
		stateCopy.SetToEmptySet();
		memory->GetNextState(currentChar, &currentState, &stateCopy);//stateCopy holds the nextState
		currentState.SetToEmptySet();
		memory->GetEmptyClosure(&stateCopy, &currentState);//currentState has emptyClosure now.
		//if(currentState.IsEmpty()) return FALSE;
	}
	//Now, we check currentState for acceptance...
	return currentState.Contains(finalState);
}
void NFA::Or(const NFA &nfa)
{
	int nStart = memory->CreateState(startState, nfa.startState);
	int nfinal = memory->CreateState();
	startState = nStart;
	memory->GetState(finalState).SetEmptyTransitions(nfinal, ErrorState);
	memory->GetState(nfa.finalState).SetEmptyTransitions(nfinal, ErrorState);
	finalState = nfinal;
}
void NFA::Concatenate(const NFA &nfa)
{
	/*This will be subject to optimization. The current final state should mimic
	 *the behaviour of the nfa.startState. Then, nfa.startState is to be deleted.
	 *We can implement an avail list by providing using the empty0 value to point to
	 *the next empty state in the list and maintain a list head with the manager.
	 *This is excellent!
	 */
	//The next statement would be better implemented by memcpy().
	memory->GetState(finalState) = memory->GetState(nfa.startState);
	finalState = nfa.finalState;
    memory->DeleteState(nfa.startState);
}
void NFA::ReallocateAndSet(TCHAR symbol)
{
	finalState = memory->CreateState();
	startState = memory->CreateState(symbol, finalState);
}
void NFA::Closure()
{
	int newFinal = memory->CreateState();
	int newStart = memory->CreateState(startState, newFinal);
	memory->GetState(finalState).SetEmptyTransitions(newFinal, startState);
	//startState is more efficient than newStartState because it saves depth.
	startState = newStart;
	finalState = newFinal;
}
void NFA::SetAcceptAnySymbol()
{
	/*It seems that working with character classes will be messy. Although easier,
	 *it implies more complex efficiency issues that are totally beaten when working
	 *with character classing. For example, a|b|c|d|e can be represented as 
	 *( ( ((a+b)+c) +d) +e)
     *(((a+b)+(c+e))+e)
	 *The number of different representation increases as there are more choices.
	 *The optimal representation is the one that has minimum depth.
	 *So, we shall use character classes.
	 */
	int finalSymbolIndex = memory->GetFAFactory()->NumberOfSymbols()-1;
	BitSet *bset = new BitSet(finalSymbolIndex, FALSE);
	finalState = memory->CreateState();
	startState = memory->CreateState(bset, finalState);
}
void NFA::SetAcceptAnyWord()
{
	SetAcceptAnySymbol();
	Closure();
}
void NFA::SetAcceptEmptyWord()
{
	startState = finalState = memory->CreateState();
}
void NFA::SetCharacterClass(Set *set)
{
	finalState = memory->CreateState();
	startState = memory->CreateState(set, finalState);
}
void NFA::MakeRejectionMachine()
{
	finalState = ErrorState;
	startState = memory->CreateState();
}
void NFA::DeleteStates()
{
	memory->DeleteState(startState);
	if(finalState != startState)
		memory->DeleteState(finalState);
}
NFA::~NFA(void)
{
	if(mustDestroyMemory && memory != NULL)
		delete memory;
}