#pragma once

namespace Automata
{

struct Transition
{
	union
	{
		TCHAR Symbol;
		Set *CharacterClass;
	};
	int State;
	void Set(TCHAR c, int s)
	{
		Symbol = c; 
		State = s;
	}
	void Set(Automata::Set *set, int s)
	{
		CharacterClass = set;
		State = s;
	}
};

struct NondeterministicState
{

public:
	enum StateType
	{
		EmptyTransitions,
		SymbolTransition,
		CharacterClassTransition
	};
private :
	StateType type;
	union
	{
		int emptyTransition[2];
		Transition transition;
	};
public:
	NondeterministicState(void);
	NondeterministicState(int emptyState0, int emptyState1);
	NondeterministicState(TCHAR symbol, int state);
	NondeterministicState(Set *charClass, int state);
	int GetEmptyTransition1() const;
	int GetEmptyTransition2() const;
	void SetEmptyTransitions(int state0, int state1);
	void GetTransition(Transition *transition) const;
	void GetTransition(TCHAR *symbol, int *state) const;
	void GetTransition(Set **characterClass, int *state) const;
	TCHAR GetTransitionSymbol() const;
//	void SetTransition(const Transition *transition);
	void SetTransition(TCHAR symbol, int state);
	void SetTransition(Set *characterClass, int state);
	StateType GetType() const;
	int NextState(TCHAR symbol) const;
	int NextState(int symbolIndex) const;
	int NextState(const FAFactory *father, TCHAR symbol) const;
	~NondeterministicState(void);
};
};