#pragma once
#include "Set.h"
namespace Automata
{
//Represents a Set with range capabilities. That is, instead of 
//being restricted to elements [0, max], it can be restricted to
//any range subset from the set of integers. Eg: there can be a 
//ranged set whose elements are anyway between -5 and 120.
//Note: The RangedSet class is not responsible for deallocating
//the wrapped Set. In addition, it doesnot check set for being
//pointing to NULL or not.
class RangedSet
{
protected:
	Set		*_set;
	int		_min;
public:
	//Constructs a RangedSet with range 
	//[min, min+set->NumberOfElementsRequired()].
	RangedSet(Set *set, int min); //We could solve the NULL pointer
								  //by accepting a reference to
								  //object and then taking its
								  //memory address.

	 //Returns the minimum value allowed in the range.
	int		RangeMinimum() const
	{ return _min; }

	//Returns the maximum value allowed in the range.
	int		RangeMaximum() const;

	//Checks wether x belongs to the set.
	BOOL	Contains(int x) const;

	//Checks wether x is within the range of the set. If the
	//the function returns TRUE, x does not	necessarily belong
	//to the set.
	BOOL	WithinRange(int x) const;

	//It assigns min the value of the smallest element present in 
	//the set. This is not necessarily the RangeMinimum(). It
	//returns TRUE if the set is not empty. Otherwise, it returns
	//FALSE.
	BOOL	MinimumElement(int &min) const;
		
	//It assigns max the value of the greatest element present in 
	//the set. This is not necessarily the RangeMaximum(). It
	//returns TRUE if the set is not empty. Otherwise, it returns
	//FALSE.
	BOOL	MaximumElement(int &max) const;

	//Adds the element x to the set. Note x must be within the
	//allowed range of the set. The function returns TRUE if x
	//is within the set range.
	BOOL	Add(int x);

	//Adds all elements in the range [a, b] or [b, a] according
	//to which of them is smaller. Note that a and b must
	//within the range of the set. Otherwise, nothing will be
	//added to the set, and the function will return FALSE.
	BOOL	AddRange(int a, int b);
	
	//Adds all the elements in the range to the set.
	void	AddAllElements();

	//Removes the element x from the set. Note x must be within 
	//the allowed range of the set. The function returns TRUE if x
	//is within the set range.
	BOOL	Remove(int x);

	//Removes all elements in the range [a, b] or [b, a] according
	//to which of them is smaller. Note that a and b must
	//within the range of the set. Otherwise, nothing will be
	//removed from the set, and the function will return FALSE.
	BOOL	RemoveRange(int a, int b);
	
	//Removes all the elements from the set.
	void	Clear();

	~RangedSet(void);
};
}