#ifndef _LEXELLANT_H_
#define _LEXELLANT_H_

#include <iostream>
#include <TCHAR.h>
#include <fstream>
#include <string>
#include <locale>
#include <cstdio>
#include <stack>
using namespace std;

#define tofstream	basic_ofstream<TCHAR>
#define tifstream	basic_ifstream<TCHAR>
#define tstring		basic_string<TCHAR>

#if TCHAR == wchar_t
typedef unsigned short UTCHAR;
#else
typedef unsigned char UTCHAR;
#endif

#define TEOF		char_traits<TCHAR>::eof()
#define ErrorState	-1

typedef int BOOL;
#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif

/*********************************************************************************/
//utf8Codecvt

template<typename internT = wchar_t, typename externT = char>
class UTF8_UTF32 : public std::codecvt<internT, externT, mbstate_t>
{
	using std::codecvt<internT, externT, mbstate_t>::error;
	using std::codecvt<internT, externT, mbstate_t>::noconv;
	using std::codecvt<internT, externT, mbstate_t>::ok;
	using std::codecvt<internT, externT, mbstate_t>::partial;
protected:
	typename std::codecvt<internT, externT, mbstate_t>::result
	do_out(mbstate_t&,
	const internT* from, const internT* from_end,
	const internT*& from_next,
	externT* to, externT* to_end,
	externT*& to_next) const
	{
		from_next = from;
		to_next = to;
		while(from_next < from_end) {
			if(*from_next < 0x80) {
				if(to_next + 1 > to_end) break;
				*to_next++ = static_cast<externT>(*from_next++);
			}
			else if(*from_next < 0x00000800) {
				if(to_next + 2 > to_end) break;
				*to_next++ = static_cast<externT>(0xC0 | (*from_next >> 6));
				*to_next++ = static_cast<externT>(0x80 | (*from_next & 0x3F));
				from_next++;
			}
			else if(*from_next < 0x00010000) {
				if(*from_next >= 0xD800 && *from_next < 0xE000) return error;
				if(to_next + 3 > to_end) break;
				*to_next++ = static_cast<externT>(0xE0 | ( *from_next >> 12));
				*to_next++ = static_cast<externT>(0x80 | ((*from_next >> 6) & 0x3F));
				*to_next++ = static_cast<externT>(0x80 | ( *from_next & 0x3F));
				from_next++;
			}
			else if(*from_next < 0x00110000) {
				if(to_next + 4 > to_end) break;
				*to_next++ = static_cast<externT>(0xF0 | ( *from_next >> 18));
				*to_next++ = static_cast<externT>(0x80 | ((*from_next >> 12) & 0x3F));
				*to_next++ = static_cast<externT>(0x80 | ((*from_next >> 6) & 0x3F));
				*to_next++ = static_cast<externT>(0x80 | ( *from_next & 0x3F));
				from_next++;
				}
			else {
				return error;
			}
		}
		return (from_next == from_end) ? ok : partial;
	}

typename std::codecvt<internT, externT, mbstate_t>::result
do_unshift(mbstate_t&,
externT* to, externT*,
externT*& to_next) const
{
to_next = to;
return noconv;
}
typename std::codecvt<internT, externT, mbstate_t>::result
do_in(mbstate_t&,
const externT* from, const externT* from_end,
const externT*& from_next,
internT* to, internT* to_end,
internT*& to_next) const
{
from_next = from;
to_next = to;
while(to_next < to_end && from_next < from_end) {
if((*from_next & 0x80) == 0x00) {
*to_next++ = *from_next++;
}
else if((*from_next & 0xE0) == 0xC0) {
if(from_next + 2 > from_end) break;
*to_next = (*from_next++ & 0x1F) << 6;
if((*from_next & 0xC0) != 0x80) return error;
*to_next |= *from_next++ & 0x3F;
if(*to_next < 0x80) return error;
to_next++;
}
else if((*from_next & 0xF0) == 0xE0) {
if(from_next + 3 > from_end) break;
*to_next = (*from_next++ & 0x0F) << 12;
if((*from_next & 0xC0) != 0x80) return error;
*to_next |= (*from_next++ & 0x3F) << 6;
if((*from_next & 0xC0) != 0x80) return error;
*to_next |= *from_next++ & 0x3F;
if(*to_next < 0x00000800) return error;
if(*to_next >= 0xD800 && *to_next < 0xE000) return error;
to_next++;
}
else if((*from_next & 0xF8) == 0xF0) {
if(from_next + 4 > from_end) break;
*to_next = (*from_next++ & 0x07) << 18;
if((*from_next & 0xC0) != 0x80) return error;
*to_next |= (*from_next++ & 0x3F) << 12;
if((*from_next & 0xC0) != 0x80) return error;
*to_next |= (*from_next++ & 0x3F) << 6;
if((*from_next & 0xC0) != 0x80) return error;
*to_next |= *from_next++ & 0x3F;
if(*to_next < 0x00010000 || *to_next >= 0x00110000) return error;
to_next++;
}

else {
return error;
}
}
return (from_next == from_end) ? ok : partial;
}
int do_encoding() const throw() { return 0; }
bool do_always_noconv() const throw() { return false; }
int do_length(mbstate_t&, const externT* from,
const externT* end, size_t max) const
{
int len = 0;
while(from < end && static_cast<size_t>(len) < max) {
if((*from & 0x80) == 0x00) {
from++;
}
else if((*from & 0xE0) == 0xC0) {
if(from + 2 > end || (from[1] & 0xC0) != 0x80) break;
from += 2;
}
else if((*from & 0xF0) == 0xE0) {
if(from + 3 > end ||
(from[1] & 0xC0) != 0x80 ||
(from[2] & 0xC0) != 0x80) break;
from += 3;
}
else if((*from & 0xF8) == 0xF0) {
if(from + 4 > end ||
(from[1] & 0xC0) != 0x80 ||
(from[2] & 0xC0) != 0x80 ||
(from[3] & 0xC0) != 0x80) break;
from += 4;
}
else {
break;
}
len++;
}
return len;
}
int do_max_length() const throw() { return 4; }
};

typedef UTF8_UTF32<wchar_t, char> Utf8Codecvt;

/*********************************************************************************/
//StreamReader
class StreamReader
{
public:
	// Text file encoding type enum
	enum EncodingType {FileNotFound ,ANSI,UTF16_LE,UTF16_BE,UTF32_LE,UTF32_BE,UTF_8};
private:
	BOOL			_isValid;
	FILE			*_file;
	EncodingType	_encodingType;//The current encoding type

	Utf8Codecvt		utf8Codec;
public:
	// Check file encoding type
	static EncodingType CheckFileEncodingType(const wchar_t* const filename)
	{
		FILE* _file=_wfopen(filename,L"rb");
		if(_file == NULL)
			return FileNotFound;
		// Read first 4 byte for testing
		unsigned char buf[4];
		size_t nRead=fread((void*)buf,sizeof(unsigned char),4,_file);
		// Close file
		fclose(_file);

		// Testing
		if(nRead<2)
			return ANSI;
		// Testting tocken
		unsigned char utf32_le[]={0xFF,0xFE,0x00,0x00};
		if(memcmp(buf,&utf32_le,4)==0)
			return UTF32_LE;

		unsigned char utf32_be[]={0x00,0x00,0xFE,0xFF};
		if(memcmp(buf,&utf32_be,4)==0)
			return UTF32_BE;

		unsigned char utf_8[]={0xEF,0xBB,0xBF};
		if(memcmp(buf,&utf_8,3)==0)
			return UTF_8;

		unsigned char utf16_le[]={0xFF,0xFE};
		if(memcmp(buf,&utf16_le,2)==0)
			return UTF16_LE;

		unsigned char utf16_be[]={0xFE,0xFF};
		if(memcmp(buf,&utf16_be,2)==0)
			return UTF16_BE;
		// Else
		return ANSI;
	}
	static EncodingType CheckFileEncodingType(const char* const filename)
	{
		FILE* _file=fopen(filename,"rb");
		if(_file == NULL)
			return FileNotFound;
		// Read first 4 byte for testing
		unsigned char buf[4];
		size_t nRead=fread((void*)buf,sizeof(unsigned char),4,_file);
		// Close file
		fclose(_file);

		// Testing
		if(nRead<2)
			return ANSI;
		// Testting tocken
		unsigned char utf32_le[]={0xFF,0xFE,0x00,0x00};
		if(memcmp(buf,&utf32_le,4)==0)
			return UTF32_LE;

		unsigned char utf32_be[]={0x00,0x00,0xFE,0xFF};
		if(memcmp(buf,&utf32_be,4)==0)
			return UTF32_BE;

		unsigned char utf_8[]={0xEF,0xBB,0xBF};
		if(memcmp(buf,&utf_8,3)==0)
			return UTF_8;

		unsigned char utf16_le[]={0xFF,0xFE};
		if(memcmp(buf,&utf16_le,2)==0)
			return UTF16_LE;

		unsigned char utf16_be[]={0xFE,0xFF};
		if(memcmp(buf,&utf16_be,2)==0)
			return UTF16_BE;
		// Else
		return ANSI;
	}
public:
	StreamReader(void)
	{
		_file = NULL;
		_isValid = FALSE;
		_encodingType = ANSI;
	}
	~StreamReader(void);
	
	StreamReader(const wchar_t* filename, wchar_t *mode=L"rb")
	{
		_isValid = Open(filename,mode);
	}
	StreamReader(const char *filename, char *mode="rb")
	{
		_isValid = Open(filename,mode);	
	}
public:
	// Open the file
	BOOL Open(const wchar_t* const filename,  wchar_t *mode=L"rb")
	{
		// Testing for file type
		_encodingType = CheckFileEncodingType(filename);
		_file=_wfopen(filename,mode);
		if(_file==NULL)
			return FALSE;
		// Write Encoding tag
		if(wcschr(mode,L'w')!=NULL)
		{
			Close();
			return FALSE;
		}
		// Seek  file pos
		if(wcschr(mode,L'r')!=NULL)
			SkipEncodingTag(_encodingType);
		return TRUE;
	}
	BOOL Open(const char* const filename, char *mode="rb")
	{
		// Testing for file type
		_encodingType = CheckFileEncodingType(filename);
			_file=fopen(filename,mode);
		if(_file==NULL)
			return FALSE;
		// Write Encoding tag
		if(strchr(mode,'w')!=NULL)
		{
			Close();
			return FALSE;
		}
		// Seek  file pos
		if(strchr(mode,'r')!=NULL)
			SkipEncodingTag(_encodingType);
		
		return TRUE;
	}
	
	//Closes the file if open
	void Close()
	{
		if( _file != NULL )
		{
			fclose(_file);
			_file = NULL;
		}
	}
public:
	// Read a line from a MBCS file 
	// Return value:if end of file,return value is false,else is true
//	void ReadLine(tstring& s, TCHAR eof, TCHAR eol=_T('\n'));
	void ReadLine( tstring &line )
	{
		line.clear();
		tstring::iterator iter;
		TCHAR	tc;
		wchar_t wc;
		char	c;
		switch(_encodingType)
		{
			case ANSI:
			{
				while( (c = fgetc(_file)) != '\n' && c != EOF )
				{
					line.append(1, c);
				}
			}
			break;
			case UTF_8:
			{
				while( (tc = get()) != _T('\n') && tc != char_traits<TCHAR>::eof() )
				{
					line.append(1, tc);
				}
			}
			break;
			case UTF16_BE:
			{
				while( (wc = fgetwc(_file)) != 0x0A00 && wc != char_traits<wchar_t>::eof())
				{
					//reverse bytes
					wc = ((wc&0x00FF)<<8) | ((wc&0xFF00)>>8);
					line.append(1, wc);
				}
			}
			break;
			case UTF16_LE:
			{
				while( (wc = fgetwc(_file)) != L'\n' && wc != char_traits<wchar_t>::eof())
				{
					line.append(1, wc);
				}
			}
			break;
			case UTF32_BE:
			{
				// TODO:will be implement later
			}
			break;
			case UTF32_LE:
			{
				// TODO:will be implement later
			}
			break;
			default:
			{
			}
		}
		iter = line.end();
		while(line.begin() != iter && iscntrl(*(iter-1)))
		{
			iter--;
		}
		line.erase(iter, line.end());
	}

	// skip file encoding prefix
	int SkipEncodingTag(EncodingType type)
	{
		switch(_encodingType)
		{
		case UTF_8:
			return fseek(_file,3,SEEK_SET);
		case UTF16_BE:
		case UTF16_LE:
			return fseek(_file,2,SEEK_SET);
		case UTF32_BE:
		case UTF32_LE:
			return fseek(_file,4,SEEK_SET);
		}
	}
	
	// Get current open file's encoding type
	EncodingType GetEncodingType(void)
	{
		return _encodingType;
	}

	// Check file is valid to read
	BOOL IsValid(void)
	{
		return _isValid;
	}

	// Write the encoding type tag at beginning of the file
	void WriteEncodingTag(EncodingType type)
	{
		switch(type)
		{
		case UTF_8:
			{
				unsigned char utf_8[]={0xEF,0xBB,0xBF};
				fwrite(utf_8,sizeof(unsigned char),3,_file);
			}			
			break;
		case UTF16_BE:
			{
				unsigned char utf16_be[]={0xFE,0xFF};
				fwrite(utf16_be,sizeof(unsigned char),2,_file);
			}
			break;
		case UTF16_LE:
			{
				unsigned char utf16_le[]={0xFF,0xFE};
				fwrite(utf16_le,sizeof(unsigned char),2,_file);
			}			
			break;
		case UTF32_BE:
			{
				unsigned char utf32_be[]={0x00,0x00,0xFE,0xFF};
				fwrite(utf32_be,sizeof(unsigned char),4,_file);
			}
		case UTF32_LE:
			{
				unsigned char utf32_le[]={0xFF,0xFE,0x00,0x00};
				fwrite(utf32_le,sizeof(unsigned char),4,_file);
			}		
			break;
		default:; // ANSI, do nothing
		}
	}

	//Returns the next character from the file and advances the read
	//pointer.
	int get()
	{
		wchar_t wc;
		switch(_encodingType)
		{
		case ANSI:
			return fgetc(_file);
		case UTF_8:
			mbstate_t	state;
			wchar_t		theChar;
			UTF8_UTF32<wchar_t, char>::_Byte	c[4];
			UTF8_UTF32<wchar_t, char>::_Byte	*dummy1, *endByteSeq;
			wchar_t		*dummy2;
			codecvt_base::result result;
			endByteSeq = c;
			while( ! eof() ) //while not end of file
			{
				*endByteSeq = fgetc( _file ); //get char
				endByteSeq++;
				result = utf8Codec.in(	state,
									c,
									endByteSeq,
									dummy1,
									&theChar,
									&theChar+1,
									dummy2 );
				if( result == codecvt_base::ok || result == codecvt_base::noconv ) 
					return theChar;
				else if(result == codecvt_base::error)
					throw exception("Invalid byte sequence in utf8 file.");
			}
			return char_traits< TCHAR >::eof();
		case UTF16_BE:
			{
				wc = fgetwc(_file); //not very good implementation
				wc = (wc&0xFF00)>>8 | (wc&0x00FF)<<8;
				return wc;
			}
		case UTF16_LE: //not very good implementation
			{
				return fgetwc(_file);
			}
		default:
			return fgetc(_file);
		}
	}

	//Returns the physical position of the read pointer of the file.
	long tellg()
	{
		return ftell( _file );
	}

	//Moves the physical position of the read pointer of the file an
	//offset from the specified origin. The function returns zero if
	//the seek was succeful. Otherwise, a nonzero value is returned.
	//origin can be: 
	//a) SEEK_SET : Beginning of the file
	//b) SEEK_CUR : Current position
	//c) SEEK_END : End of the file
	int seekg( long offset, int origin )
	{
		return fseek( _file, offset, origin );
	}

	//Moves the physical position of the read pointer of the file to
	//the specified absolute position. The function returns zero if
	//the seek was succeful. Otherwise, a nonzero value is returned.
	int seekg( long absolutePosition )
	{
		return fseek( _file, absolutePosition, SEEK_SET );
	}

	//Returns the next character from the file but doesnot consume it.
	int peek()
	{
		int		retval;
		long	pos;
		pos = tellg(); //save position
		retval = get(); //read next char
		seekg( pos, SEEK_SET ); //restore
        return retval;
	}

	//Returns TRUE if we reached end of file.
	BOOL eof()
	{
		return feof( _file ) || tellg() == size();
	}
	//Returns the size of the file in bytes.
	long size()
	{
		long pos = tellg();
		long length;

		seekg(0, SEEK_END);
		length = tellg();
		seekg(pos, SEEK_SET);
		return length;
	}
};

#endif