#include "StdAfx.h"
#include "Set.h"
#include <limits>
using namespace std;
using namespace Automata;

Set::~Set(void)
{
}
BitSet::BitSet() : bits()
{}
BitSet::BitSet(int maxElement, BOOL initiallyEmpty) : bits(maxElement + 1, !initiallyEmpty)
{}
BitSet::BitSet(const BitSet& set) : bits(set.bits)
{}
BitSet& BitSet::operator =(const BitSet &set)
{
	if(&set == this) return *this;
	bits = set.bits;
	return *this;
}
BOOL BitSet::Contains(int element) const
{
//	if(element == -1)	return TRUE;//Although it is mathematically not TRUE;
	return bits[element];
}
void BitSet::Add(int element)
{
	if(element == EmptySet)		return;
	return bits.Set(element, TRUE);
}
void BitSet::Remove(int element)
{
	if(element == EmptySet)		return;
	return bits.Set(element, FALSE);
}
BOOL BitSet::IsEmpty() const
{
	int slotCount = bits.NumberOfSlots();
	for(int i=0; i<slotCount; i++)
		if(bits.GetSlot(i) != 0)	return FALSE;
	return TRUE;
}
int BitSet::NumberOfElementsRequested() const
{
	return bits.NumberOfBitsRequired();
}
int BitSet::MaximumNumberOfElements() const
{
	return bits.NumberOfBitsSupported();
}
int BitSet::MaximumElement() const
{
	return  bits.NumberOfBitsRequired()-1;
}
BOOL BitSet::IsSubsetOf(const BitSet &B) const
{
	//a subset b
	//a = this, b = B
	//n1 subset n2. So, n1 <= n2 and every e in n1 must be in n2
	//Or, if n1 > n2, then every e in n1 must be in n2
	int n1 = bits.NumberOfSlots();
	int n2 = B.bits.NumberOfSlots();
	int minimum = (n1<=n2)?n1:n2;
	//A subset B if and only if slot(B`,i).slot(A,i) = 0 for every i.
	SlotType bdash;
	SlotType a;
	for(int i=0;i<minimum;i++)
	{
		bdash = ~ B.bits.GetSlot(i);
		a = bits.GetSlot(i);
		if( (bdash & a) != 0)
			return FALSE;
	}
	if (n1 <= n2)
		return TRUE;
	for(int i=minimum; i<n1; i++)
		if( bits.GetSlot(i) != 0)
			return FALSE;
	return TRUE;	
}
BOOL BitSet::IsSubsetOf(const Set &B) const
{
//	const BitSet *pset = dynamic_cast<const BitSet *>(&B);
	const BitSet *pset = (const BitSet *)&B;
	if(pset != NULL)
		return IsSubsetOf(*pset);
	return FALSE;	
}
BOOL BitSet::IsProperSubsetOf(const Set &B) const
{
	if(IsEmpty())
		return FALSE;
	return IsSubsetOf(B);
}
BOOL BitSet::IsProperSubsetOf(const BitSet &B) const
{
	if(IsEmpty())
		return FALSE;
	return IsSubsetOf(B);
}
void BitSet::Complement()
{
	int nslots = bits.NumberOfSlots();
	for(int i=0;i<nslots;i++)
		bits.GetSlot(i) = ~bits.GetSlot(i);
}
void BitSet::SetToEmptySet()
{
	int nslots = bits.NumberOfSlots();
	SlotType z = 0;
	for(int i=0;i<nslots;i++)
		bits.GetSlot(i) = z;
}
void BitSet::AddAllElements()
{
	int nslots = bits.NumberOfSlots();
	SlotType id = ~0;
	for(int i=0;i<nslots;i++)
		bits.GetSlot(i) = id;
}
void BitSet::SetUnion(const Set &B)
{
//	const BitSet *pset = dynamic_cast<const BitSet *>(&B);
	const BitSet *pset = (const BitSet *)&B;
	if(pset != NULL)
		SetUnion(*pset);
}
void BitSet::SetUnion(const BitSet &B)
{
	//n1 must be == n2
	int nslots = bits.NumberOfSlots();
	for(int i=0; i<nslots; i++)
		bits.GetSlot(i) |= B.bits.GetSlot(i);
}
void BitSet::SetIntersection(const Set &B)
{
//	const BitSet *pset = dynamic_cast<const BitSet *>(&B);
	const BitSet *pset = (const BitSet *)&B;
	if(pset != NULL)
		SetIntersection(*pset);
}
void BitSet::SetIntersection(const BitSet &B)
{
	//n1 must be == n2
	int nslots = bits.NumberOfSlots();
	for(int i=0; i<nslots; i++)
		bits.GetSlot(i) &= B.bits.GetSlot(i);
}
void BitSet::SetDifference(const Set &B)
{
//	const BitSet *pset = dynamic_cast<const BitSet *>(&B);
	const BitSet *pset = (const BitSet *)&B;
	if(pset != NULL)
		SetDifference(*pset);
}
void BitSet::SetDifference(const BitSet &B)
{
	//n1 must be == n2
	//A-B = AB`
	int nslots = bits.NumberOfSlots();
	for(int i=0; i<nslots; i++)
		bits.GetSlot(i) &= ~B.bits.GetSlot(i);

}
//////
void Set::RemoveRange(int a, int b)
{
	int n = min(a, b);
	int x = max(a, b);
	while(n <= x)
		Remove( n++ );
}
//////
void Set::AddRange(int a, int b)
{
	int n = min(a, b);
	int x = max(a, b);
	while(n <= x)
		Add( n++ );
}
//////
int Set::Minimum() const
{
	int maxIndex = MaximumElement();
	for(int i=0; i <= maxIndex; i++)
		if( Contains( i ) ) return i;
	return -1;
}
//////
int Set::Maximum() const
{
	int maxIndex = MaximumElement();
	for(int i=maxIndex; i >= 0; i--)
		if( Contains( i ) ) return i;
	return -1;
}