#pragma once
namespace Automata
{
class Set
{
public:
	virtual BOOL Contains(int element) const = 0;
	virtual void Add(int element) = 0;
	virtual void Remove(int element) = 0;
	virtual BOOL IsEmpty() const = 0;
	virtual int NumberOfElementsRequested() const = 0;
	virtual int MaximumNumberOfElements() const = 0;
	virtual int MaximumElement() const = 0;
	virtual BOOL IsSubsetOf(const Set &set) const = 0;
	virtual BOOL IsProperSubsetOf(const Set &set) const = 0;
	virtual void Complement() = 0;
	virtual void SetToEmptySet() = 0;
	virtual void AddAllElements() = 0;
	virtual void SetUnion(const Set &set) = 0;
	virtual void SetIntersection(const Set &set) = 0;
	virtual void SetDifference(const Set &set) = 0;

	//Adds the elements in the range [a, b] or [b, a] (if b < a).
	virtual void AddRange(int a, int b); 

	//Removes the elements in the range [a, b] or [b, a] (if b < a).
	virtual void RemoveRange(int a, int b);

	//Returns the minimum element in the set or -1 if the set is
	//empty.
	virtual int Minimum() const;

	//Returns the maximum element in the set or -1 if the set is
	//empty
	virtual int Maximum() const;
	void Clear()
	{ SetToEmptySet(); }
	virtual ~Set(void) = 0;
};
/*------------------------------------------------------------------------------------------
*BitSet
*///----------------------------------------------------------------------------------------
class BitSet : public Set
{
	static const int Power = 5;
	typedef int SlotType;
	BoolArray<SlotType, Power> bits;
public:
	BitSet();
	BitSet(int maxElement, BOOL initiallyEmpty = TRUE);
	BitSet& operator =(const BitSet &set);
	BitSet(const BitSet& set);
	BOOL Contains(int element) const;
	void Add(int element);
	void Remove(int element);
	BOOL IsEmpty() const;
	int NumberOfElementsRequested() const;
	int MaximumNumberOfElements() const;
	int MaximumElement() const;
	BOOL IsSubsetOf(const Set &B)const;
	BOOL IsSubsetOf(const BitSet &B)const;
	BOOL IsProperSubsetOf(const Set &B) const;
	BOOL IsProperSubsetOf(const BitSet &B) const;
	void Complement();
	void SetToEmptySet();
	void AddAllElements();
	void SetUnion(const Set &B);
	void SetUnion(const BitSet &B);
	void SetIntersection(const Set &B);
	void SetIntersection(const BitSet &B);
	void SetDifference(const Set &B);
	void SetDifference(const BitSet &B);

};
}