#					include		"stdafx.h"
#include <tchar.h>
#include <cstdio>
#include "BoolArray.h"
using namespace Automata;
using namespace std;
namespace Automata
{
class Set
{

public:
	virtual BOOL Contains(int element) const = 0;
	virtual void Add(int element) = 0;
	virtual void Remove(int element) = 0;
	virtual BOOL IsEmpty() const = 0;
	virtual int NumberOfElementsRequested() const = 0;
	virtual int MaximumNumberOfElements() const = 0;
	virtual int MaximumElement() const = 0;
	virtual BOOL IsSubsetOf(const Set &set) const = 0;
	virtual BOOL IsProperSubsetOf(const Set &set) const = 0;
	virtual void Complement() = 0;
	virtual void SetToEmptySet() = 0;
	virtual void AddAllElements() = 0;
	virtual void SetUnion(const Set &set) = 0;
	virtual void SetIntersection(const Set &set) = 0;
	virtual void SetDifference(const Set &set) = 0;
	virtual ~Set(void) = 0;
};
/*------------------------------------------------------------------------------------------
*BitSet
*///----------------------------------------------------------------------------------------
class BitSet : public Set
{
	static const int Power = 5;
	typedef int SlotType;
	BoolArray<SlotType, Power> bits;
public:
	BitSet();
	BitSet(int maxElement, BOOL initiallyEmpty = TRUE);
	BitSet& operator =(const BitSet &set);
	BitSet(const BitSet& set);
	BOOL Contains(int element) const;
	void Add(int element);
	void Remove(int element);
	BOOL IsEmpty() const;
	int NumberOfElementsRequested() const;
	int MaximumNumberOfElements() const;
	int MaximumElement() const;
	BOOL IsSubsetOf(const Set &B)const;
	BOOL IsSubsetOf(const BitSet &B)const;
	BOOL IsProperSubsetOf(const Set &B) const;
	BOOL IsProperSubsetOf(const BitSet &B) const;
	void Complement();
	void SetToEmptySet();
	void AddAllElements();
	void SetUnion(const Set &B);
	void SetUnion(const BitSet &B);
	void SetIntersection(const Set &B);
	void SetIntersection(const BitSet &B);
	void SetDifference(const Set &B);
	void SetDifference(const BitSet &B);
};
};
using namespace Automata;
void ShowSet(const Set &set, char *name)
{
	cout<<endl<<name<<" = { ";
	int max = set.MaximumElement();
	int i=0;
	for(;i<=max; i++)
		if(set.Contains(i))
		{
			cout<<i;
			break;//just max is enough
		}
	for(i++;i<=max;i++)
		if(set.Contains(i))		cout<<", "<<i;
	cout<<"}"<<endl<<endl;
}
const char *Format(BOOL value)
{
	return value?"TRUE":"FALSE";
}
void main()
{
	BitSet A(10);
	printf("A.NumberOfElements() = %d\r\n", A.NumberOfElementsRequested());
	printf("A.MaximumNumberOfElements() = %d\r\n", A.MaximumNumberOfElements());
	printf("A.MaximumElement() = %d\r\n", A.MaximumElement());
	ShowSet(A, "A");
	printf("Complement!\r\n");
	A.Complement();
	ShowSet(A, "A");
	printf("Complement!\r\n");
	A.Complement();
	ShowSet(A, "A");

	printf("AddAllElements!\r\n");
	A.AddAllElements();
	ShowSet(A, "A");

	int item;
	do
	{
		printf("Enter an item to remove:");	cin>>item;
		A.Remove(item);
		ShowSet(A, "A");
	}while(item!=-1);

	Set *B = new BitSet(A);
	ShowSet(*B, "B");
	do
	{
		printf("Enter an item to Add:");	cin>>item;
		B->Add(item);
		ShowSet(*B, "B");
		printf("A.IsSubsetOf(*B) : %s\r\n", Format(A.IsSubsetOf(*B)));
		printf("B->IsSubsetOf(A) : %s\r\n", Format(B->IsSubsetOf(A)));
		printf("A.IsProperSubsetOf(*B) : %s\r\n", Format(A.IsProperSubsetOf(*B)));
		printf("B->IsProperSubsetOf(A) : %s\r\n", Format(B->IsProperSubsetOf(A)));
	}while(item!=-1);

	
	printf("\r\nIf C = A, then C.SetIntersection(*B): \r\n");
	BitSet C = A;
	C.SetIntersection(*B);
	ShowSet(C, "C");


	printf("\r\nIf C = A, then C.SetUnion(*B): \r\n");
	C = A;
	C.SetUnion(*B);
	ShowSet(C, "C");


	printf("\r\nIf C = A, then C.SetDifference(*B): \r\n");
	C = A;
	C.SetDifference(*B);
	ShowSet(C, "C");

	printf("\r\nIf C = *B, then C->SetDifference(A): \r\n");
	C = *(BitSet*)B;
	C.SetDifference(A);
	ShowSet(C, "C");
	
		
	int x;
	int y;
	cin>>x>>y;
	int arr[2] = {x, y};
	for(int i=0;i<2;i++)
		cout<<arr[i]<<endl;

//	if( 5 > 4 and 5 == 5) cout<<"Trigraphs work!!"<<endl;

}