// Testbed.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#define BOOLARRAY_DEBUG
using namespace std;
typedef int BOOL;
const int FALSE = 0;
const int TRUE = 1;
template<class Slot, int Power>
class BoolArray
{
private:
	static const int BitsPerSlotMinusOne = (sizeof(Slot)<<3) - 1;
	Slot *array;
	int numberOfBits;
public:
	//count : Number of booleans to be stored.
	BoolArray(int numberOfBitsRequired, BOOL initialValue = FALSE) : array(NULL)
	{
		Resize(numberOfBitsRequired, initialValue);
	}
	BoolArray() : array(NULL)
	{
	}
	BoolArray(const BoolArray<Slot, Power> &barray)
	{
		numberOfBits = barray.numberOfBits;
		int numberOfSlots = NumberOfSlots();
		array = new Slot[numberOfSlots];
		memcpy(array, barray.array, numberOfSlots*sizeof(Slot));
	}
	void Resize(int numberOfBitsRequired, BOOL initialValue = FALSE)
	{
		if(array!=NULL)
			delete[] array;
		numberOfBits = numberOfBitsRequired;
		int numberOfSlots = NumberOfSlots();
		array = new Slot[numberOfSlots];
		if(initialValue == FALSE)
			memset(array, 0, numberOfSlots * sizeof(Slot));
		else
			memset(array, ~0, numberOfSlots * sizeof(Slot));
	}
    BOOL operator [](int index)	const
	{
		Slot mask = index & BitsPerSlotMinusOne;
		int slotIndex = index>>Power;				//index / BitsPerSlot
		int bitIndex = index & BitsPerSlotMinusOne; //index % BitsPerSlot
		return array[slotIndex]>>bitIndex & 1;
	}
	void Set(int index, BOOL value)
	{
		int slotIndex = index>>Power;				//index / BitsPerSlot
		int bitIndex = index & BitsPerSlotMinusOne; //index % BitsPerSlot
		Slot bit = (value)?1:0;
		bit = bit<<bitIndex;
		Slot mask = 1;
		mask = mask<<bitIndex;
		mask = ~mask;
		array[slotIndex] = (array[slotIndex] & mask) | bit;
	}
	int NumberOfBitsRequired()
	{
		return numberOfBits;
	}
	int NumberOfSlots()
	{
		int slots = numberOfBits>>Power;
		int extraSlot = (numberOfBits&BitsPerSlotMinusOne) != 0;
		return slots + extraSlot;
	}
	int NumberOfBitsSupported()
	{
		return NumberOfSlots()<<Power;
	}
	Slot& GetSlot(int index)
	{
		return array[index];
	}
	const Slot& GetSlot(int index) const
	{
		return array[index];
	}
	~BoolArray()
	{
		if(array != NULL)
		{	delete[] array;
			array = NULL;
		}
	}
};
template<class Slot, int Power>
void ShowBoolArray(const BoolArray<Slot, Power> &array, int max, const char *name)
{
	for(int i=0;i<max;i++)
		cout<<name<<"["<<i<<"] = "<<((array[i])?"TRUE":"FALSE")
			<<endl;
}
int _tmain(int argc, _TCHAR* argv[])
{
	const int max = 31;
	BoolArray<char, 3> boolArray(max, TRUE);
	//cout<<endl<<endl<<endl<<"Before filling:"<<endl;
	//ShowBoolArray(boolArray, max, "boolArray");
	BoolArray<char, 3> copy1 = boolArray;
	cout<<endl<<endl<<endl;
	//ShowBoolArray(copy1, max, "copy1");
	for(int i=0;i<max;i++)
		boolArray.Set(i, i&1);
	//cout<<endl<<endl<<endl<<"After filling:"<<endl;
	//ShowBoolArray(boolArray, max, "boolArray");
	cout<<endl<<endl<<endl;
	//ShowBoolArray(copy1, max, "copy1");
	BoolArray<char, 3> copy2 = boolArray;
	cout<<endl<<endl<<endl;
	cout<<"Before Complementing: "<<endl;
	cout<<endl<<endl<<endl;
	ShowBoolArray(copy2, max, "copy2");
	cout<<endl<<endl<<endl;
	
	int slots = copy2.NumberOfSlots();
	for(int i=0; i<slots; i++)
		copy2.GetSlot(i) ^= ~0;

	cout<<"After Complementing: "<<endl;
	cout<<endl<<endl<<endl;
	ShowBoolArray(copy2, max, "copy2");
	cout<<endl<<endl<<endl;


	return 0;
}

