///////////////////////////////////////////////////////////////////////////////
// You can read,modify,delete or reuse this code in any where you liked to.
// But tou SHOULD NEVER DELETE the following copyright infomation.
// Copyright(C) 2005-2006 cokkiy.org cokkiy@gmail.com
////////////////////////////////////////////////////////////////////////////////

//********************************************************
//
//        ANSI/UNT8/UTF16-Big Endian/UTF16-Little Endian
//            File read and write in line mode
//
//            Copyright(c) 2005 cokkiy@gmail.com
//
//  Please attention:The Ansi version function DOSE NOT test any more,
//  If you want use this file in ANSI program,please test it yourself
//********************************************************



#pragma once
#include "stdafx.h"
#include <cstdio>
#include <string>
using namespace std;
#include <tchar.h>
#include <locale>
#include "utf8Codecvt.h"
// define ReadLine
#ifdef _UNICODE
#define ReadLine ReadLineW
#else
#define ReadLine ReadLineA
#endif

// Define WriteLine
#ifdef _UNICODE
#define WriteLine WriteLineW
#else
#define WriteLine WriteLineA
#endif

// Define check file encoding type
#ifdef _UNICODE
#define CheckFileEncodingType CheckFileEncodingTypeW
#else
#define CheckFileEncodingType CheckFileEncodingTypeA
#endif

// CTextFileIO class declaration
class CTextFileIO
{
public:
	CTextFileIO(void);
public:
	~CTextFileIO(void);
public:
	CTextFileIO(const wchar_t* filename,  wchar_t *mode=L"rb");
	CTextFileIO(const char *filename,char *mode="rb");
public:
	// Open the file
	BOOL OpenW(const wchar_t* const filename,  wchar_t *mode=L"rb");
	BOOL OpenA(const char* const filename, char *mode="rb");
private:
	// Internal FILE object 
	FILE* _file;

public:
	// Read a line from a MBCS file 
	// Return value:if end of file,return value is false,else is true
	BOOL _ReadLine(string& s, int eol='\n', int eof=EOF);
	// Read aline from a UNICODE file
	// Return value:if end of file,return value is false,else is true
	BOOL _ReadLine(wstring& s, wint_t eol=L'\n', wint_t eof=WEOF);

	// ANSI version of read line
	inline BOOL ReadLineA(std::string &s);
	inline BOOL ReadLineA(char *s);
	// Unicode version of read line
	BOOL ReadLineW(std::wstring &ws);
	inline BOOL ReadLineW(wchar_t *ws);

public:
	// To write an ANSI zero-terminated string (please consider using STL instead)
	BOOL _WriteLine(const char* const s, int ret = '\r',int newline='\n', size_t length = -1);

	// To write a Unicode zero-terminated string (please consider using STL instead)
	BOOL _WriteLine(const wchar_t* const s, wint_t =L'\r', wint_t newline = L'\n', size_t length = -1);

	// To write an ANSI STL string
	BOOL _WriteLine(const std::string& s, int ret = '\r',int newline='\n')
	{
		return _WriteLine(s.c_str(), ret,newline, s.size());
	};

	// To write a Unicode STL string
	BOOL _WriteLine(const std::wstring& s, wint_t ret=L'\r', wint_t newline = L'\n')
	{
		return _WriteLine(s.c_str(), ret, newline, s.size());
	};
	// ANSI version of write line
	BOOL WriteLineA(const char* const s);
	BOOL WriteLineA(const std::string &s)
	{
		return WriteLineA(s.c_str());
	}
	// Unicode version of write line
	BOOL WriteLineW(const wchar_t* const ws);
	BOOL WriteLineW(const std::wstring &ws)
	{
		return WriteLineW(ws.c_str());
	};

public:
	// Text file encoding type enum
	enum EncodingType {ANSI,UTF16_LE,UTF16_BE,UTF32_LE,UTF32_BE,UTF_8};
private:
	// File encoding type
	EncodingType encodingType;
public:
	// Check file encoding type
	static EncodingType CheckFileEncodingTypeW(const wchar_t* const filename);
	static EncodingType CheckFileEncodingTypeA(const char* const filename);
public:
	// Get current open file's encoding type
	CTextFileIO::EncodingType GetEncodingType(void)
	{
		return encodingType;
	}
	// Omit file encoding tag
	int OmitEncodingTag(EncodingType type);
	// Indicate file can be read or not
	BOOL _IsValidate;
public:

	// Check file is validate to read
	BOOL IsValidate(void)
	{
		return _IsValidate;
	}

	// Write the encoding type tag an beginner of file
	void WriteEncodingTag(EncodingType type);

	//Returns the next character from the file and advances the read
	//pointer.
	int get()
	{
		Utf8Codecvt	codec;
		mbstate_t	state;
        TCHAR		theChar;
		UTF8_UTF32<wchar_t, char>::_Byte	c[4];
		UTF8_UTF32<wchar_t, char>::_Byte	*dummy1, *endByteSeq;
		TCHAR		*dummy2;
		codecvt_base::result result;
		endByteSeq = c;
		while( ! eof() ) //while not end of file
		{
			*endByteSeq = fgetc( _file ); //get char
			endByteSeq++;
			result = codec.in(	state,
								c,
								endByteSeq,
								dummy1,
								&theChar,
								&theChar+1,
								dummy2 );
			if( result == codecvt_base::ok || result == codecvt_base::noconv ) 
				return theChar;
			else if(result == codecvt_base::error)
				throw exception("Invalid byte sequence in utf8 file.");
		}
		return char_traits< TCHAR >::eof();
	}

	//Returns the physical position of the read pointer of the file.
	long tellg()
	{
		return ftell( _file );
	}
	//Moves the physical position of the read pointer of the file an
	//offset from the specified origin. The function returns zero if
	//the seek was succeful. Otherwise, a nonzero value is returned.
	//origin can be: 
	//a) SEEK_SET : Beginning of the file
	//b) SEEK_CUR : Current position
	//c) SEEK_END : End of the file
	int seekg( long offset, int origin )
	{
		return fseek( _file, offset, origin );
	}
	//Moves the physical position of the read pointer of the file to
	//the specified absolute position. The function returns zero if
	//the seek was succeful. Otherwise, a nonzero value is returned.
	int seekg( long absolutePosition )
	{
		return fseek( _file, absolutePosition, SEEK_SET );
	}
	//Returns the next character from the file but doesnot consume it.
	int peek()
	{
		int		retval;
		long	pos;
		pos = tellg(); //save position
		retval = get(); //read next char
		seekg( pos, SEEK_SET ); //restore
        return retval;
	}
	//Returns TRUE if we reached end of file.
	BOOL eof()
	{
		return feof( _file ) || tellg() == size();
	}
	//Returns the size of the file in bytes.
	long size()
	{
		long pos = tellg();
		long length;

		seekg(0, SEEK_END);
		length = tellg();
		seekg(pos, SEEK_SET);
		return length;
	}
};
//typedef CTextFileIO InputStream;
