#pragma once
#include "FAFactory.h"
#include "DFA.h"

using namespace std;

/*
Very Important Note (by hatem)
===================
The compressed DFA is useless if it's original DFA has been deallocated.
*/
namespace Automata
{
	class CompressedDFA
	{
		friend class FAFactory;

	protected:
		//Data
		DFA *dfa; 
	
		//Methods
		virtual void Build(){}

	public:
		CompressedDFA() : dfa(NULL)
		{}
		CompressedDFA(DFA* originalDFA) : dfa(originalDFA)
		{
			Build();
		}
        DFA* OriginalDFA()
		{
			return dfa;
		}
		virtual int	 MemorySize() const = 0;
		virtual void GenerateCodeCpp(
			tofstream& tout,
			tofstream& tout_h,
			Dictionary& options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			) = 0;
		virtual void GenerateCodeCs(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			) = 0;
		virtual void GenerateCodeJava(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			) = 0;
		virtual int NextState(int currentState, TCHAR inputSymbol) = 0;
		virtual void Reset() = 0;
	};

	class CompressedDFA_Pairs_Normal : public CompressedDFA
	{
		friend class DFA;

		int threshold;
		int memorySize;

	public:
		CompressedDFA_Pairs_Normal()
		{}
		CompressedDFA_Pairs_Normal(DFA* originalDFA):CompressedDFA(originalDFA)
		{} 

		//Data
		int **TransitionListVector;

		//Methods
		void GenerateCodeCpp(
			tofstream& tout,
			tofstream& tout_h,
			Dictionary& options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			);
		void GenerateCodeCs(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			);
		void GenerateCodeJava(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			){}
		
		int MemorySize() const;

		int NextState(int currentState, TCHAR inputSymbol)
		{
			//if( TransitionListVector[currentState] == NULL )
			//	  return -1;
			//if( TransitionListVector[currentState][0] == 0 )
			//	  return TransitionListVector[currentState][dfa->father->IndexOf(inputSymbol) + 1];
			//for( int symbolPair=0; symbolPair<TransitionListVector[currentState][0]; symbolPair++ )
			//	  if( TransitionListVector[currentState][symbolPair*2 + 1] == inputSymbol )
			//		  return TransitionListVector[currentState][symbolPair*2 + 2];
			//return -1;
			return -1;
		}

		void Build()
		{}

		void Reset()
		{}
	};

	class CompressedDFA_RedundancyRemoved : public CompressedDFA
	{
	public:
		CompressedDFA_RedundancyRemoved()
		{}
		CompressedDFA_RedundancyRemoved(DFA* originalDFA):CompressedDFA(originalDFA)
		{} 

		//Data
		int *RowMap, *ColumnMap; //Horizontal and vertical indexers
		int **MappedTransitionTable;
		int ReducedColumnsCount, ReducedRowsCount;

		//Methods
		void GenerateCodeCpp(
			tofstream& tout,
			tofstream& tout_h,
			Dictionary& options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			);
		void GenerateCodeCs(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			);
		void GenerateCodeJava(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			){}

		int MemorySize() const;
		
		int NextState(int currentState, TCHAR inputSymbol)
		{
			//return MappedTransitionTable[ RowMap[currentState] ][ ColumnMap[dfa->father->IndexOf(inputSymbol)] ];
			return -1; //Dummy, for compilation
		}
		
		void Build()
		{}
		
		void Reset()
		{}
	};

	class CompressedDFA_Pairs_Default : public CompressedDFA
	{
		int threshold;

	public:
		CompressedDFA_Pairs_Default() : CompressedDFA()
		{
			
		}
		CompressedDFA_Pairs_Default(DFA* originalDFA) : CompressedDFA(originalDFA)
		{
            
		}

		//Data
		int **TransitionListVector;

		//Methods
		int MemorySize() const
		{
			return 0;
		}

		void GenerateCodeCpp(
			tofstream& tout,
			tofstream& tout_h,
			Dictionary& options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			){}
		void GenerateCodeCs(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			){}
		void GenerateCodeJava(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			){}
		
		int NextState(int currentState, TCHAR inputSymbol)
		{
			//if( TransitionListVector[currentState] == NULL )
			//	  return -1;
			//if( TransitionListVector[currentState][0] == 0 )
			//	  return TransitionListVector[currentState][dfa->father->IndexOf(inputSymbol) + 1];
			//for( int symbolPair=0; symbolPair<TransitionListVector[currentState][0]; symbolPair++ )
			//	  if( TransitionListVector[currentState][symbolPair*2 + 2] == inputSymbol )
			//		  return TransitionListVector[currentState][symbolPair*2 + 3];
			//return TransitionListVector[currentState][1]; //Return the default state
			return -1; //Dummy, for compilation
		}
		
		void Build()
		{}
		
		void Reset()
		{}
	};
	class CompressedDFA_Aho : public CompressedDFA
	{
	public:
		CompressedDFA_Aho() : CompressedDFA()
		{
			
		}
		CompressedDFA_Aho(DFA* originalDFA) :  CompressedDFA(originalDFA)
		{
			
		} 

		//Data


		//Methods
		int MemorySize() const
		{
			return 0;
		}

		void GenerateCodeCpp(
			tofstream& tout,
			tofstream& tout_h,
			Dictionary& options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			){}
		void GenerateCodeCs(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			){}
		void GenerateCodeJava(
			tofstream& tout,
			Dictionary options,
			const tstring& HeaderTopSection,
			const tstring& ClassDefinition,
			const tstring& CppTopSection,
			const tstring& CppBottomSection
			){}
		
		int NextState(int currentState, TCHAR inputSymbol)
		{
			return 0;
		}
		
		void Build()
		{}
		
		void Reset()
		{}
	};
}