#pragma once
template<typename internT = wchar_t, typename externT = char>
class UTF8_UTF32 : public std::codecvt<internT, externT, mbstate_t>
{
	using std::codecvt<internT, externT, mbstate_t>::error;
	using std::codecvt<internT, externT, mbstate_t>::noconv;
	using std::codecvt<internT, externT, mbstate_t>::ok;
	using std::codecvt<internT, externT, mbstate_t>::partial;
protected:
	typename std::codecvt<internT, externT, mbstate_t>::result
	do_out(mbstate_t&,
	const internT* from, const internT* from_end,
	const internT*& from_next,
	externT* to, externT* to_end,
	externT*& to_next) const
	{
		from_next = from;
		to_next = to;
		while(from_next < from_end) {
			if(*from_next < 0x80) {
				if(to_next + 1 > to_end) break;
				*to_next++ = static_cast<externT>(*from_next++);
			}
			else if(*from_next < 0x00000800) {
				if(to_next + 2 > to_end) break;
				*to_next++ = static_cast<externT>(0xC0 | (*from_next >> 6));
				*to_next++ = static_cast<externT>(0x80 | (*from_next & 0x3F));
				from_next++;
			}
			else if(*from_next < 0x00010000) {
				if(*from_next >= 0xD800 && *from_next < 0xE000) return error;
				if(to_next + 3 > to_end) break;
				*to_next++ = static_cast<externT>(0xE0 | ( *from_next >> 12));
				*to_next++ = static_cast<externT>(0x80 | ((*from_next >> 6) & 0x3F));
				*to_next++ = static_cast<externT>(0x80 | ( *from_next & 0x3F));
				from_next++;
			}
			else if(*from_next < 0x00110000) {
				if(to_next + 4 > to_end) break;
				*to_next++ = static_cast<externT>(0xF0 | ( *from_next >> 18));
				*to_next++ = static_cast<externT>(0x80 | ((*from_next >> 12) & 0x3F));
				*to_next++ = static_cast<externT>(0x80 | ((*from_next >> 6) & 0x3F));
				*to_next++ = static_cast<externT>(0x80 | ( *from_next & 0x3F));
				from_next++;
				}
			else {
				return error;
			}
		}
		return (from_next == from_end) ? ok : partial;
	}

typename std::codecvt<internT, externT, mbstate_t>::result
do_unshift(mbstate_t&,
externT* to, externT*,
externT*& to_next) const
{
to_next = to;
return noconv;
}
typename std::codecvt<internT, externT, mbstate_t>::result
do_in(mbstate_t&,
const externT* from, const externT* from_end,
const externT*& from_next,
internT* to, internT* to_end,
internT*& to_next) const
{
from_next = from;
to_next = to;
while(to_next < to_end && from_next < from_end) {
if((*from_next & 0x80) == 0x00) {
*to_next++ = *from_next++;
}
else if((*from_next & 0xE0) == 0xC0) {
if(from_next + 2 > from_end) break;
*to_next = (*from_next++ & 0x1F) << 6;
if((*from_next & 0xC0) != 0x80) return error;
*to_next |= *from_next++ & 0x3F;
if(*to_next < 0x80) return error;
to_next++;
}
else if((*from_next & 0xF0) == 0xE0) {
if(from_next + 3 > from_end) break;
*to_next = (*from_next++ & 0x0F) << 12;
if((*from_next & 0xC0) != 0x80) return error;
*to_next |= (*from_next++ & 0x3F) << 6;
if((*from_next & 0xC0) != 0x80) return error;
*to_next |= *from_next++ & 0x3F;
if(*to_next < 0x00000800) return error;
if(*to_next >= 0xD800 && *to_next < 0xE000) return error;
to_next++;
}
else if((*from_next & 0xF8) == 0xF0) {
if(from_next + 4 > from_end) break;
*to_next = (*from_next++ & 0x07) << 18;
if((*from_next & 0xC0) != 0x80) return error;
*to_next |= (*from_next++ & 0x3F) << 12;
if((*from_next & 0xC0) != 0x80) return error;
*to_next |= (*from_next++ & 0x3F) << 6;
if((*from_next & 0xC0) != 0x80) return error;
*to_next |= *from_next++ & 0x3F;
if(*to_next < 0x00010000 || *to_next >= 0x00110000) return error;
to_next++;
}

else {
return error;
}
}
return (from_next == from_end) ? ok : partial;
}
int do_encoding() const throw() { return 0; }
bool do_always_noconv() const throw() { return false; }
int do_length(mbstate_t&, const externT* from,
const externT* end, size_t max) const
{
int len = 0;
while(from < end && static_cast<size_t>(len) < max) {
if((*from & 0x80) == 0x00) {
from++;
}
else if((*from & 0xE0) == 0xC0) {
if(from + 2 > end || (from[1] & 0xC0) != 0x80) break;
from += 2;
}
else if((*from & 0xF0) == 0xE0) {
if(from + 3 > end ||
(from[1] & 0xC0) != 0x80 ||
(from[2] & 0xC0) != 0x80) break;
from += 3;
}
else if((*from & 0xF8) == 0xF0) {
if(from + 4 > end ||
(from[1] & 0xC0) != 0x80 ||
(from[2] & 0xC0) != 0x80 ||
(from[3] & 0xC0) != 0x80) break;
from += 4;
}
else {
break;
}
len++;
}
return len;
}
int do_max_length() const throw() { return 4; }
};

typedef UTF8_UTF32<wchar_t, char> Utf8Codecvt;
