#include "stdafx.h"
#include "ProductionRule.h"
#include "GenericCFGNode.h"
#include "ParserGen.h"
#include "Globals.h"
#include "BitSet_Min.h"
#include<fstream>
using namespace RDParserGenerator;


TerminalEntry::TerminalEntry() {}
TerminalEntry::TerminalEntry(unsigned int NI,TokenType T)
{
	NameIndex = NI;
	Type = T;
}

NonTerminalEntry::NonTerminalEntry() {}
NonTerminalEntry::NonTerminalEntry(unsigned int NI)
{
	NameIndex = NI;
}



ParserGenerator::ParserGenerator(string fileName)
{
	this->scanner = new Scanner(fileName);
	ResumeOnError = TRUE;
	HasErrors = FALSE;
	nextTerminalIndex = TerminalBase;
	nextNonTerminalIndex = NonTerminalBase;
	nextCodeIndex = CodeBase;
}

ParserGenerator::~ParserGenerator()
{
	delete scanner;
}
void ParserGenerator::Parse()
{
	TokensNames[0] = _T("_EOF"); //I use here a dummy 0 because I am not interested in mapping here
	
	TerminalEntry TE1(nextTerminalIndex,Eps);
	Terminals["Eps"]=TE1;
	nextTerminalIndex++;

	/*TokensNames[nextTerminalIndex] = _T("_Char");
	TerminalEntry TE2(nextTerminalIndex,Character);
	Terminals["Char"]=TE2;
	nextTerminalIndex++;

	TokensNames[nextTerminalIndex] = _T("_String");
	TerminalEntry TE3(nextTerminalIndex,String);
	Terminals["String"]=TE3;
	nextTerminalIndex++;*/

	token = scanner->GetToken();
	if(token.Type == Options)
	{
		match(Options);
		OptionsSet();
	}
	if(token.Type == Tokens)
	{
		match(Tokens);
		TokensSet();
	}
	while(token.Type == DeclTop || token.Type == DeclBottom || token.Type == DefTop || token.Type == DefBottom)
	{
		switch(token.Type)
		{
		case DeclTop:
			match(DeclTop);
			TopOfDeclaration();
			break;
		case DeclBottom:
			match(DeclBottom);
			BottomOfDeclaration();
			break;
		case DefTop:
			match(DefTop);
			TopOfDefinition();
			break;	
		case DefBottom:
			match(DefBottom);
			BottomOfDefinition();
			break;
		}
	}
	if(token.Type == Grammar)
	{
		match(Grammar);
		while(token.Type!=EndOfFile)
			Productions();
	}
	//and EOF to Terminals
	TerminalEntry TE(nextTerminalIndex,EndOfFile);
	nextTerminalIndex++;
	Terminals["EOF"]=TE;
	
	//check Nonterminals without rules
	for(map<tstring,NonTerminalEntry>::iterator NTIter = NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++)
	{
		if((*NTIter).second.Rule==NULL)
		{
			cout<<"Input file contains error(s)\n";
			exit(1);
		}
	}

	CalculateFirstSetS();
	CalculateFollowSetS();
	PrintRules(Grammer);
	cout<<"==================================================\n";
	//PrintFirstSet();
	cout<<"==================================================\n";
	//PrintFollowSets();
	GenerateLL1Table();
	//PrintLL1Table();
	//RemoveLeftRecursion();
	LeftFactor();
	//PrintRules(LRR_Grammer);
	
}

void ParserGenerator::TokensSet()
{

	while(token.Type == Identifier)
	{
		//add as a terminal
		//unsigned int Index = TerminalBase+nextTerminalIndex; //Why Hamdy? We initialize nextTerminalIndex with TerminalBase 
		TerminalEntry TE(nextTerminalIndex,token.Type);
		Terminals[token.Lexeme]=TE;
		//TerminalsInverse[Index]=&Terminals[token.Lexeme];
		TokensNames[nextTerminalIndex] = _T("_") + token.Lexeme;
		nextTerminalIndex++;
		match(Identifier);
	}

}

void ParserGenerator::OptionsSet()
{
	if(token.Type == NameSpace)
	{
		match(NameSpace);
		match(Equal);
		StrNameSpace = token.Lexeme;//match(String)
		match(String);
	}
	if(token.Type == ClassName)
	{
		match(ClassName);
		match(Equal);
		StrClassName = token.Lexeme;//match string
		match(String);
	}
	if(token.Type == Language)
	{
		match(Language);
		match(Equal);
		ParserLanguage = token.Type;
		if(token.Type==Java||token.Type==CPlusPlus||token.Type==CSharp)
			match(token.Type);
		else
			match(String);
	}
}

void ParserGenerator::TopOfDeclaration()
{
	if (token.Type = String)
		StrDecTop += token.Lexeme;
	match(String);
}

void ParserGenerator::TopOfDefinition()
{
	if (token.Type = String)
		StrDefTop += token.Lexeme;
	match(String);
}

void ParserGenerator::BottomOfDeclaration()
{
	if (token.Type = String)
		StrDecBottom += token.Lexeme;
	match(String);
}

void ParserGenerator::BottomOfDefinition()
{
	if (token.Type = String)
		StrDefBottom += token.Lexeme;
	match(String);
}



void ParserGenerator::match(TokenType expected)
{
	if (token.Type == expected)
		token = scanner->GetToken();
	else
	{
		cout<<"Input file contains error(s)\n";
		exit(1);
		//perform error action
		/*tstring msg; //construct the msg
		onError(msg);
		HasErrors = TRUE;
		if (ResumeOnError)
			token = scanner->GetToken();*/
	}
}
void ParserGenerator::Productions()
{
	// hamdy, start from here
	/*ProductionRule * Rule = new ProductionRule();
	if(token.Type==Identifier)
	{
		if(Temp_NonTerminals.find(token.Lexeme)==NonTerminals.end())
		{	
			Rule->LHS = nextNonTerminalIndex;
			NonTerminalEntry NTE(nextNonTerminalIndex);
			NonTerminals[token.Lexeme]=NTE;
			nextNonTerminalIndex++;			
		}
		else
		{
			Rule->LHS = NonTerminals[token.Lexeme].NameIndex;
		}		
	}
	match(Identifier);
	match(Production);
	//Rule->RHS = new OredNode();
	Expression(&Rule->RHS);
	match(Dot);
	Grammer.push_back(Rule);*/
	// hamdy, start from here
	ProductionRule * Rule = new ProductionRule();
	map<tstring,NonTerminalEntry>::iterator NTiter;
	bool check1=false;
	bool check2=false;
	if(token.Type==Identifier)
	{
		if(Terminals.find(token.Lexeme)!=Terminals.end())
		{
			cout<<"Input file contains error(s)\n";
			exit(1);
		}
		if((NTiter=NonTerminals.find(token.Lexeme))==NonTerminals.end())
		{	
			Rule->LHS = nextNonTerminalIndex;
			NonTerminalEntry NTE(nextNonTerminalIndex);
			NTE.Rule=Rule;
			NonTerminals[token.Lexeme]=NTE;
			nextNonTerminalIndex++;
			check1=true;

		}
		else
		{
			ProductionRule* debugnode= (*NTiter).second.Rule;
			Rule->LHS = NonTerminals[token.Lexeme].NameIndex;
			if((*NTiter).second.Rule==NULL)
			{
				(*NTiter).second.Rule=Rule;
				check2= true;
			}	
		}	
	}
 	match(Identifier);
	/*if(token.Type==Attributes)
	{

		match(Attributes);
	}*/
	match(Production);
	//Rule->RHS = new OredNode();
	
	if(check1||check2)
	{
		Expression(&Rule->RHS);
		Grammer.push_back(Rule);
	}
	else 
	{
		GenericCFGNode* RuleNode;
		Expression(&RuleNode);
		if((*NTiter).second.Rule->RHS->NType ==_Ored&&RuleNode->NType==_Ored)
		{
			for(list<GenericCFGNode*>::iterator Iter= ((OredNode*)RuleNode)->Children.begin();Iter!=((OredNode*)RuleNode)->Children.end();Iter++)
			{
				((OredNode*)(*NTiter).second.Rule->RHS)->Children.push_back(*Iter);
			}
			((OredNode*)RuleNode)->Children.clear();
		//	delete RuleNode;
		
		}
		else if((*NTiter).second.Rule->RHS->NType ==_Ored)
		{
			((OredNode*)(*NTiter).second.Rule->RHS)->Children.push_back(RuleNode);
		}
		else if(RuleNode->NType==_Ored)
		{
			((OredNode*)RuleNode)->Children.push_back((*NTiter).second.Rule->RHS);
			(*NTiter).second.Rule->RHS=RuleNode;
		}
		else
		{
			GenericCFGNode* TempNode = (*NTiter).second.Rule->RHS;
			(*NTiter).second.Rule->RHS= new OredNode();
			((OredNode*)(*NTiter).second.Rule->RHS)->Children.push_back(TempNode);
			((OredNode*)(*NTiter).second.Rule->RHS)->Children.push_back(RuleNode);
		}
	}
		
	match(Dot);
	
	
}
void ParserGenerator::Expression(GenericCFGNode** node)
{
	//GenericCFGNode*  node;
	//node1 = &node;
	if(token.Type==If||token.Type==Weak||token.Type==OpenBracket||token.Type==OpenCurly||token.Type==OpenSquareBracket||token.Type==Eps||token.Type==Any||token.Type==Sync||token.Type==SemAction||token.Type==Identifier||token.Type==String||token.Type==Character)
	{
		//AndedNode* ChildNode = new AndedNode();
		GenericCFGNode* ChildNode;
		Term(&ChildNode);
		if(token.Type!=Or)
		{
			*node = ChildNode;
			return;
		}
		*node = new OredNode();
		((OredNode*)*node)->Children.push_back(ChildNode);
	}
	while(token.Type==Or)
	{
		match(Or);
		GenericCFGNode* Child ;//= new AndedNode();
		Term(&Child);
		((OredNode*)*node)->Children.push_back(Child);
	}
}
void ParserGenerator::Term(GenericCFGNode** node)
{
	if(token.Type==Eps)
	{
		match(Eps);
		*node = new TerminalNode();
		((TerminalNode*)*node)->TerminalType = Eps;
		((TerminalNode*)*node)->NameIndex = 0;
	}
	else
	{
		bool ResolverExist=false;
		if(token.Type==If)
		{
			match(If);
			match(OpenBracket);
			*node = new AndedNode();
			((AndedNode*)*node)->ResolverString=token.Lexeme;
			match(BulkOfCode);
			match(CloseBracket);
			ResolverExist= true;
		}
		if(token.Type==Weak||token.Type==OpenBracket||token.Type==OpenCurly||token.Type==OpenSquareBracket||token.Type==Any||token.Type==Sync||token.Type==SemAction||token.Type==Identifier||token.Type==String||token.Type==Character)
		{
			GenericCFGNode* child;
			Factor(&child);
			if(!ResolverExist)
			{
				if(!(token.Type==Weak||token.Type==OpenBracket||token.Type==OpenCurly||token.Type==OpenSquareBracket||token.Type==Any||token.Type==Sync||token.Type==SemAction||token.Type==Identifier||token.Type==String||token.Type==Character))
				{
					*node = child;
					return;
				}
				*node = new AndedNode();
			}
			((AndedNode*)*node)->Children.push_back(child);
		}
		while(token.Type==Weak||token.Type==OpenBracket||token.Type==OpenCurly||token.Type==OpenSquareBracket||token.Type==Any||token.Type==Sync||token.Type==SemAction||token.Type==Identifier||token.Type==String||token.Type==Character)
		{
			GenericCFGNode* child;
			Factor(&child);
			((AndedNode*)*node)->Children.push_back(child);
		}
	}

}
void ParserGenerator::Factor(GenericCFGNode** node)
{
	//GenericCFGNode* node ;
	//node1=&node; 
	/*
	if(token.Type==OpenBracket)
	{
		//*node = new BracketNode();
		match(OpenBracket);
		//GenericCFGNode* child;//=new OredNode();
		Expression(node);
		//((BracketNode*)node)->Children.push_back(child);
		match(CloseBracket);
	}
	else if(token.Type==OpenSquareBracket)
	{
		*node = new OptionNode();
		match(OpenSquareBracket);
		GenericCFGNode* child;//=new OredNode();
		Expression(&child);
		((OptionNode*)*node)->Children.push_back(child);
		match(CloseSquareBracket);
	}
	else if(token.Type==OpenCurly)
	{
		*node = new ClosureNode();
		match(OpenCurly);
		GenericCFGNode* child;//OredNode* child=new OredNode();
		Expression(&child);
		((ClosureNode*)*node)->Children.push_back(child);
		match(CloseCurly);
	}
	else*/
	if(token.Type==SemAction)
	{
		*node = new SemActionNode();
       // match(OpenActions);
		((SemActionNode*)*node)->SemAction = token.Lexeme;
		((SemActionNode*)*node)->NameIndex = nextCodeIndex;
		nextCodeIndex++;
		//match(BulkOfCode);
		//match(CloseActions);
		match(SemAction);
	}
	/*else if(token.Type==Any)
	{
		//is there an action for First and FollowSet
		*node = new AnyNode();
		match(Any);
	}*/
	else if(token.Type==Sync)
	{
		*node = new SyncNode();
		match(Sync);
	}
	else
	{	
		bool IsWeak = false;
		if(token.Type==Weak)
		{			
			IsWeak = true;
			match(Weak);
		}
		/*if(token.Type==String||token.Type==Character)
		{
			*node = new TerminalNode();
			((TerminalNode*)*node)->IsWeak = IsWeak;
			((TerminalNode*)*node)->TerminalType=token.Type; 
			if(Terminals.find(token.Lexeme)==Terminals.end())
			{
				((TerminalNode*)*node)->NameIndex = nextTerminalIndex;
				TerminalEntry TE(nextTerminalIndex,token.Type);
				Terminals[token.Lexeme] = TE;
				nextTerminalIndex++;
			}
			else
			{
				((TerminalNode*)*node)->NameIndex = Terminals[token.Lexeme].NameIndex;

			}
			if(token.Type==Character)
				match(Character);
			else
				match(String);		
		}
		else */
		if(token.Type==Identifier)
		{
			if(Terminals.find(token.Lexeme)!=Terminals.end())
			{
				*node = new TerminalNode();
				((TerminalNode*)*node)->IsWeak=IsWeak;
				((TerminalNode*)*node)->TerminalType=token.Type;
				((TerminalNode*)*node)->NameIndex = Terminals[token.Lexeme].NameIndex;
			}
			else
			{
				*node = new NonTerminalNode();
				((NonTerminalNode*)*node)->IsWeak=IsWeak;
				((TerminalNode*)*node)->TerminalType=token.Type;
				if(NonTerminals.find(token.Lexeme)==NonTerminals.end())
				{
					((NonTerminalNode*)*node)->NameIndex = nextNonTerminalIndex;
					NonTerminalEntry TE(nextNonTerminalIndex);
					TE.Rule=NULL;
					NonTerminals[token.Lexeme] = TE;
					nextNonTerminalIndex++;

				}
				else
				{
					((NonTerminalNode*)*node)->NameIndex = NonTerminals[token.Lexeme].NameIndex;

				}

			}
			match(Identifier);
			/*
			if(token.Type==Attributes)
			{
				//match(OpenAttributes);
				switch((*node)->NType)
				{
				case _Terminal:
					((TerminalNode*)*node)->Attributes = token.Lexeme;
					break;
				case _NonTerminal:
					((NonTerminalNode*)*node)->Attributes = token.Lexeme;
		            break;
				}
				//match(CloseAttributes);
				match(Attributes);
		    }*/

		}
		
	}
	
}

tstring ParserGenerator::GetNonTerminalName( unsigned int NameIndex )
{
	for( map<tstring,NonTerminalEntry>::iterator iter = NonTerminals.begin(); iter != NonTerminals.end(); iter++)
	{
		if ( iter->second.NameIndex == NameIndex )
			return iter->first;
	}
	return _T("Undefined");
}

void ParserGenerator::onError(tstring errorMsg)
{
	//write to file for example
}
void ParserGenerator::CalculateFirstSetS()
{
	//Initialize FirstSet
	map<tstring,TerminalEntry>::iterator TIter;
	for(TIter=Terminals.begin();TIter!=Terminals.end();TIter++)
	{
		(*TIter).second.FirstSet = new BitSet_Min(nextTerminalIndex);
		(*TIter).second.FirstSet->Add((*TIter).second.NameIndex-TerminalBase);
	}
	map<tstring,NonTerminalEntry>::iterator NTIter;
	for(NTIter=NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++)
	{
		(*NTIter).second.FirstSet = new BitSet_Min(nextTerminalIndex);
		//(*NTIter).second.FollowSet = new BitSet_Min(nextTerminalIndex);
	}
	//FirstSet Implementation
	bool Change= true;
	while(Change)
	{
		Change = false;
		for(list<ProductionRule*>::iterator PIter= Grammer.begin();PIter!=Grammer.end();PIter++)
		{
			//for debugging
			GenericCFGNode* DebugNode = (*PIter)->RHS; 
			//not implemented yet
			bool Changed=false;
			BitSet_Min firstset;
			BitSet_Min fs = (*PIter)->RHS->FirstSet(this);
			for(map<tstring,NonTerminalEntry>::iterator iter=NonTerminals.begin();iter!=NonTerminals.end();iter++)
			{
				if((*iter).second.NameIndex==(*PIter)->LHS)
				{
					firstset = *(*iter).second.FirstSet;
					fs.SetUnion(firstset);
					//if(!(fs.IsSubsetOf(firstset)&&firstset.IsSubsetOf(fs)))//remember for optimization
					if(!fs.Equals(firstset))
						Changed= true;
					//delete (*iter).second.FirstSet;
					(*iter).second.FirstSet->SetUnion(fs);
					Change |= Changed;

					break;
				}
			}
			
			
			
		}
	}
}

void ParserGenerator::CalculateFollowSetS()
{
	//Initialize FollowSet
	for(map<tstring,NonTerminalEntry>::iterator NTIter=NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++)
	{
		if((*NTIter).second.NameIndex==NonTerminalBase)
		{
			(*NTIter).second.FollowSet = new BitSet_Min(nextTerminalIndex);
			(*NTIter).second.FollowSet->Add(nextTerminalIndex-1);
		}
		else
			(*NTIter).second.FollowSet = new BitSet_Min(nextTerminalIndex);
	}
	//FollowSet Calculation
	
	bool Change= true;
	while(Change)
	{
		Change = false;
		for(list<ProductionRule*>::iterator PIter= Grammer.begin();PIter!=Grammer.end();PIter++)
		{
			//for debug
			GenericCFGNode* debugnode= (*PIter)->RHS;
			///////////////
			Change|=(*PIter)->RHS->FollowSet(this);
			BitSet_Min Bs = (*PIter)->RHS->GetNonTerminals(this);
			int cursor=-1;


			if(!Bs .IsEmpty())
			{
				BitSet_Min followset;
				for(map<tstring,NonTerminalEntry>::iterator NTIter=NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++)
				{
					if((*NTIter).second.NameIndex==(*PIter)->LHS)
					{
						followset = *((*NTIter).second.FollowSet);
					}
				}
				while((cursor=Bs.NextMemberFrom(cursor+1))!=-1)
				{
					for(map<tstring,NonTerminalEntry>::iterator NTiter=NonTerminals.begin();NTiter!=NonTerminals.end();NTiter++)
					{
						if((*NTiter).second.NameIndex==cursor+NonTerminalBase)
						{
							//Shakek feeha remember for debuging
							(*NTiter).second.FollowSet->SetUnion(followset);
							
						}
					}

				}
			}
		}
	}

	
	
}
void ParserGenerator::PrintFirstSet()
{
	for(map<tstring,NonTerminalEntry>::iterator NTiter=NonTerminals.begin();NTiter!=NonTerminals.end();NTiter++)
	{
		cout<<"first set of ["<<(*NTiter).first.c_str()<<"] is {   ";
		int cursor= -1;
		while((cursor =(*NTiter).second.FirstSet->NextMemberFrom(cursor+1))!=-1)
		{
			for(map<tstring,TerminalEntry>::iterator Titer=Terminals.begin();Titer!=Terminals.end();Titer++)
			{
				if((*Titer).second.NameIndex==cursor+TerminalBase)
				{
					cout<<(*Titer).first.c_str();
				}
			}
			if((*NTiter).second.FirstSet->NextMemberFrom(cursor+1)!=-1)
				cout<<" , ";
			
		}
		cout<<"   }\n";
	}
}
void ParserGenerator::PrintFollowSets()
{
	for(map<tstring,NonTerminalEntry>::iterator NTiter=NonTerminals.begin();NTiter!=NonTerminals.end();NTiter++)
	{
		cout<<"follow set of ["<<(*NTiter).first.c_str()<<"] is {   ";
		int cursor= -1;
		while((cursor =(*NTiter).second.FollowSet->NextMemberFrom(cursor+1))!=-1)
		{
			for(map<tstring,TerminalEntry>::iterator Titer=Terminals.begin();Titer!=Terminals.end();Titer++)
			{
				if((*Titer).second.NameIndex==cursor+TerminalBase)
				{
					cout<<(*Titer).first.c_str();
				}
			}
			if((*NTiter).second.FollowSet->NextMemberFrom(cursor+1)!=-1)
				cout<<" , ";
			
		}
		cout<<"   }\n";
	}
}
void ParserGenerator::PrintRules(list<ProductionRule*> G)
{
	cout<<"Production Rules"<<endl;
	cout<<"==================================================\n";
	for(list<ProductionRule*>::iterator PIter= G.begin();PIter!=G.end();PIter++)
	{
		for(map<tstring,NonTerminalEntry>::iterator iter=NonTerminals.begin();iter!=NonTerminals.end();iter++)
		{
			if((*iter).second.NameIndex==(*PIter)->LHS)
			{
				cout<<(*iter).first.c_str();
			}
		}
		cout<<"( "<<(*PIter)->Attributes.c_str()<<" )"<<endl;
		cout<<"{ ";
		cout<<(*PIter)->SemAction.c_str()<<endl<<"  ";
		(*PIter)->RHS->Print(this);
        cout<<endl<<"}";
		cout<<endl<<endl;
	}		

}
void ParserGenerator::PrintLL1Table()
{
	/*cout<<"======================================="<<endl;
	cout<<"LL1 Table"<<endl;
	cout<<"==============================="<<endl;
	for(unsigned int i=0;i<nextNonTerminalIndex-NonTerminalBase;i++)
	{
		cout<<"rules for ( ";
		for(map<tstring,NonTerminalEntry>::iterator iter=NonTerminals.begin();iter!=NonTerminals.end();iter++)
		{
			if((*iter).second.NameIndex==i+NonTerminalBase)
			{
				cout<<(*iter).first.c_str();
			}
		}
		cout<<")"<<endl;
		cout<<"---------------------"<<endl;
		for(unsigned int j=1;j<nextTerminalIndex-TerminalBase;j++)
		{
			for(map<tstring,TerminalEntry>::iterator Titer=Terminals.begin();Titer!=Terminals.end();Titer++)
			{
				if((*Titer).second.NameIndex==j+TerminalBase)
				{
					cout<<(*Titer).first.c_str()<<": ";
				}
			}
			if(LL1Table[i][j-1]!=NULL)
				LL1Table[i][j-1]->Print(this);
			else 
				cout<<"NULL";
			cout<<" ."<<endl;
	

		}
		cout<<endl;

	}*/
	cout<<"======================================="<<endl;
	cout<<"LL1 Table"<<endl;
	cout<<"==============================="<<endl;
	for(unsigned int i=0;i<nextNonTerminalIndex-NonTerminalBase;i++)
	{
		cout<<"rules for ( ";
		for(map<tstring,NonTerminalEntry>::iterator iter=NonTerminals.begin();iter!=NonTerminals.end();iter++)
		{
			if((*iter).second.NameIndex==i+NonTerminalBase)
			{
				cout<<(*iter).first.c_str();
			}
		}
		cout<<")"<<endl;
		cout<<"---------------------"<<endl;
		for(unsigned int j=0;j<nextTerminalIndex-TerminalBase-1;j++)
		{
			if(j!=0)
			{
				for(map<tstring,TerminalEntry>::iterator Titer=Terminals.begin();Titer!=Terminals.end();Titer++)
				{
					if((*Titer).second.NameIndex==j+TerminalBase)
					{
						cout<<(*Titer).first.c_str()<<": ";
					}
				}
			}
			else
				cout<<"EOF: ";
			if(LL1Table[i][j]!=NULL)
			{
				//LL1Table[i][j]->Print(this);
				for(list<int>::iterator Iter=LL1Table[i][j]->begin();Iter!=LL1Table[i][j]->end();Iter++)
				{
					if(*Iter>=TerminalBase&&*Iter<NonTerminalBase)
					{
						for(map<tstring,TerminalEntry>::iterator Titer=Terminals.begin();Titer!=Terminals.end();Titer++)
						{
							if((*Titer).second.NameIndex==*Iter)
							{
								cout<<(*Titer).first.c_str()<<" ";
							}
						}

					}
					else if(*Iter>=NonTerminalBase&&*Iter<CodeBase)
					{
						for(map<tstring,NonTerminalEntry>::iterator NTiter=NonTerminals.begin();NTiter!=NonTerminals.end();NTiter++)
						{
							if((*NTiter).second.NameIndex==*Iter)
							{
								cout<<(*NTiter).first.c_str()<<" ";
							}
						}
					}
					else if(*Iter>=CodeBase)
					{
						//cout<<" CODE ";
						cout<<"( FnName= "<<Codes[*Iter].FunctionName.c_str()<<"  Code = "<<Codes[*Iter].Code.c_str()<<")";
					}
				}
			}
			else 
				cout<<"NULL";
			cout<<" ."<<endl;
	

		}
		cout<<endl;

	}
}


void ParserGenerator::GenerateLL1Table()
{
	/*
	LL1Table = new GenericCFGNode**[nextNonTerminalIndex-NonTerminalBase];
	map<tstring,NonTerminalEntry>::iterator NTIter;
	int i=0;
	for(NTIter=NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++,i++)
	{
		LL1Table[(*NTIter).second.NameIndex-NonTerminalBase] = new GenericCFGNode*[nextTerminalIndex-TerminalBase-1];
		if((*NTIter).second.Rule->RHS->NType!=_Ored)
		{
			for(unsigned int k=1;k<nextTerminalIndex-TerminalBase;k++)
			{
				if((*NTIter).second.FirstSet->Contains(k))
					LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k-1]=(*NTIter).second.Rule->RHS;
				else
					if((*NTIter).second.FirstSet->Contains(0)&&(*NTIter).second.FollowSet->Contains(k))
					{	
						LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k-1]=(*NTIter).second.Rule->RHS;
					}
					else
						LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k-1]=NULL;
			}
		}
		else
		{
				
			for(unsigned int k=1;k<nextTerminalIndex-TerminalBase;k++)
			{
				bool b=false;
				for(list<GenericCFGNode*>::iterator Iter= ((OredNode*)(*NTIter).second.Rule->RHS)->Children.begin();Iter!=((OredNode*)(*NTIter).second.Rule->RHS)->Children.end();Iter++)
				{
					if((*Iter)->FirstSet(this).Contains(k))
					{
						LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k-1]=*Iter;//->.second.Rule->RHS;
						b=true;
						break;//remember get first rule if more than one
					}
					else
						if((*Iter)->FirstSet(this).Contains(0)&&(*NTIter).second.FollowSet->Contains(k))
						{	
							LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k-1]=*Iter;
							b=true;
							break;//remember get first rule if more than one
						}							
				}
				if(!b)
					LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k-1]=NULL;
					
			}
		}
	}*/
	
	//for(int i=0
	LL1Table = new list<int>**[nextNonTerminalIndex-NonTerminalBase];
	map<tstring,NonTerminalEntry>::iterator NTIter;
	int i=0;
	for(NTIter=NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++,i++)
	{
		LL1Table[(*NTIter).second.NameIndex-NonTerminalBase] = new list<int>*[nextTerminalIndex-TerminalBase-1];
		int Fn_count=0;
		tstring NT_Name = NTIter->first;
		if((*NTIter).second.Rule->RHS->NType!=_Ored)
		{
			for(unsigned int k=0;k<nextTerminalIndex-TerminalBase-1;k++)
			{
				if(k==0)
				{
					if((*NTIter).second.FirstSet->Contains(nextTerminalIndex-TerminalBase-1))
						LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k]=(*NTIter).second.Rule->RHS->GetIndeces(this,NT_Name,&Fn_count);
					else
						if((*NTIter).second.FirstSet->Contains(0)&&(*NTIter).second.FollowSet->Contains(nextTerminalIndex-TerminalBase-1))
						{	
							LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k]=(*NTIter).second.Rule->RHS->GetIndeces(this,NT_Name,&Fn_count);
						}
						else
							LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k]=NULL;
		
				}
				else
				{
					if((*NTIter).second.FirstSet->Contains(k))
						LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k]=(*NTIter).second.Rule->RHS->GetIndeces(this,NT_Name,&Fn_count);
					else
						if((*NTIter).second.FirstSet->Contains(0)&&(*NTIter).second.FollowSet->Contains(k))
						{	
							LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k]=(*NTIter).second.Rule->RHS->GetIndeces(this,NT_Name,&Fn_count);
						}
						else
							LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k]=NULL;
			
				}
			}
		}
		else
		{
			//Fn_count=0;
			for(unsigned int k=0;k<nextTerminalIndex-TerminalBase-1;k++)
			{
				bool b=false;
                for(list<GenericCFGNode*>::iterator Iter= ((OredNode*)(*NTIter).second.Rule->RHS)->Children.begin();Iter!=((OredNode*)(*NTIter).second.Rule->RHS)->Children.end();Iter++)
				{
					if(k==0)
					{
						if((*Iter)->FirstSet(this).Contains(nextTerminalIndex-TerminalBase-1))
						{
							LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][0]=(*Iter)->GetIndeces(this,NT_Name,&Fn_count);//->.second.Rule->RHS;
							b=true;
							break;//remember get first rule if more than one
						}
						else
							if((*Iter)->FirstSet(this).Contains(0)&&(*NTIter).second.FollowSet->Contains(nextTerminalIndex-TerminalBase-1))
							{	
								LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][0]=(*Iter)->GetIndeces(this,NT_Name,&Fn_count);
								b=true;
								break;//remember get first rule if more than one
							}	
					}
					else
					{
						if((*Iter)->FirstSet(this).Contains(k))
						{
							LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k]=(*Iter)->GetIndeces(this,NT_Name,&Fn_count);//->.second.Rule->RHS;
							b=true;
							break;//remember get first rule if more than one
						}
						else
							if((*Iter)->FirstSet(this).Contains(0)&&(*NTIter).second.FollowSet->Contains(k))
							{	
								LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k]=(*Iter)->GetIndeces(this,NT_Name,&Fn_count);
								b=true;
								break;//remember get first rule if more than one
							}	
					}
				}
				if(!b)
					LL1Table[(*NTIter).second.NameIndex-NonTerminalBase][k]=NULL;
					
			}
		}
	}
}
CodeEntry::CodeEntry(tstring C,tstring fn)
{
	Code=C;
	FunctionName=fn ;
}
CodeEntry::CodeEntry()
{

}
void ParserGenerator::RemoveLeftRecursion()
{
	map<tstring,NonTerminalEntry> Temp_NonTerminals;
	for(map<tstring,NonTerminalEntry>::iterator NTIter = NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++)
	{
		ProductionRule* Rule = new ProductionRule();
		Rule->Attributes = (*NTIter).second.Rule->Attributes;
		Rule->LHS= (*NTIter).second.NameIndex;
		Rule->SemAction = (*NTIter).second.Rule->SemAction;
		GenericCFGNode* mynode = (*NTIter).second.Rule->RHS->Copy();
		for(map<tstring,NonTerminalEntry>::iterator NTIterJ = NonTerminals.begin();NTIterJ!=NTIter;NTIterJ++)
		{
			GenericCFGNode* temp =  mynode->ReplaceIfPossible(this,NTIterJ);
			mynode->Remove();
			delete mynode;
			mynode= temp;
		}

		
		Rule->RHS =mynode;


		
		mynode->Print(this);cout<<endl;

		//Immediate Left Recursion If possible
		ProductionRule* R= ILRemoval(Rule,&Temp_NonTerminals);

		//remmeber here u destroy the current
		(*NTIter).second.Rule->RHS->Remove();
		delete (*NTIter).second.Rule->RHS;
		(*NTIter).second.Rule->RHS = R->RHS->Copy();

		//////////////////////////////////

		PrintRules(LRR_Grammer);
		
	}
	for(map<tstring,NonTerminalEntry>::iterator NTIter = Temp_NonTerminals.begin();NTIter!=Temp_NonTerminals.end();NTIter++)
	{
		NonTerminals[(*NTIter).first] = (*NTIter).second;
	}
	

}
ProductionRule* ParserGenerator::ILRemoval(ProductionRule* Rule,map<tstring,NonTerminalEntry>* Temp_NonTerminals)
{
	
	ProductionRule*  R1 = new ProductionRule();
	R1->Attributes = Rule->Attributes;
	R1->LHS = Rule->LHS;
	R1->SemAction = Rule->SemAction;
	R1->RHS = new OredNode();


	NonTerminalEntry NTE(this->nextNonTerminalIndex);
	nextNonTerminalIndex++;
	ProductionRule* R2 = new ProductionRule();
	R2->Attributes = Rule->Attributes;
	R2->LHS = NTE.NameIndex;
	R2->RHS = new OredNode();
	

	if(Rule->RHS->NType==_Ored)
	{
		for(list<GenericCFGNode*>::iterator iter = ((OredNode*)Rule->RHS)->Children.begin();iter!=((OredNode*)Rule->RHS)->Children.end();iter++)
		{
			NonTerminalNode* NTN= new NonTerminalNode();
			
			NTN->NameIndex = NTE.NameIndex;
			if((*iter)->NType==_Anded)
			{
				AndedNode* chnode= (AndedNode*)(*iter)->Copy();
				list<GenericCFGNode*>::iterator Anditer;
				for(Anditer= ((AndedNode*)chnode)->Children.begin();Anditer!=((AndedNode*)chnode)->Children.end()&&((*Anditer)->NType==_SemAction||(*Anditer)->NType==_SyncNode);Anditer++);
				if(Anditer!=((AndedNode*)chnode)->Children.end()&&(*Anditer)->NType == _NonTerminal)
				{
					
					if(((NonTerminalNode*)*Anditer)->NameIndex == Rule->LHS)
					{
						NTN->Attributes = ((NonTerminalNode*)*Anditer)->Attributes;
						NTN->IsWeak = ((NonTerminalNode*)*Anditer)->IsWeak;
						NTN->TerminalType =((NonTerminalNode*)*Anditer)->TerminalType;
						(*Anditer)->Remove();
						chnode->Children.remove(*Anditer);
						chnode->Children.push_back(NTN);
						((OredNode*)R2->RHS)->Children.push_back(chnode);
						
					}
					else
					{
						chnode->Children.push_back(NTN);
						((OredNode*)R1->RHS)->Children.push_back(chnode);

					}
				}
				else
				{
					chnode->Children.push_back(NTN->Copy());
					((OredNode*)R1->RHS)->Children.push_back(chnode);
				}
			}
			else if((*iter)->NType==_NonTerminal&& ((NonTerminalNode*)*iter)->NameIndex==Rule->LHS)
			{
				AndedNode* chnode = new AndedNode();
				chnode->Children.push_back((*iter)->Copy());
				chnode->Children.push_back(NTN);
				((OredNode*)R2->RHS)->Children.push_back(chnode);
			}
			else
			{
				AndedNode* chnode = new AndedNode();
				chnode->Children.push_back((*iter)->Copy());
				chnode->Children.push_back(NTN);
				((OredNode*)R1->RHS)->Children.push_back(chnode);
			}
			
		}
	}
	else if(Rule->RHS->NType==_Anded)
	{
		NonTerminalNode* NTN= new NonTerminalNode();
		NTN->NameIndex = NTE.NameIndex;
		
		AndedNode* chnode= (AndedNode*)Rule->RHS->Copy();
		list<GenericCFGNode*>::iterator Anditer;
		for(Anditer= ((AndedNode*)chnode)->Children.begin();Anditer!=((AndedNode*)chnode)->Children.end()&&((*Anditer)->NType==_SemAction||(*Anditer)->NType==_SyncNode);Anditer++);
		if(Anditer!=((AndedNode*)chnode)->Children.end()&&(*Anditer)->NType == _NonTerminal)
		{
					
			if(((NonTerminalNode*)*Anditer)->NameIndex == Rule->LHS)
			{
		    	(*Anditer)->Remove();
				chnode->Children.remove(*Anditer);
				chnode->Children.push_back(NTN);
				((OredNode*)R2->RHS)->Children.push_back(chnode);
			}
			else
			{
				chnode->Children.push_back(NTN);
				((OredNode*)R1->RHS)->Children.push_back(chnode);
			}
		}
		else
		{
			chnode->Children.push_back(NTN);
			((OredNode*)R1->RHS)->Children.push_back(chnode);
		 }
			
	}
	else if(Rule->RHS->NType == _NonTerminal&&((NonTerminalNode*)Rule->RHS)->NameIndex==Rule->LHS)
	{	
		NonTerminalNode* NTN= new NonTerminalNode();
		NTN->NameIndex = NTE.NameIndex;
		AndedNode* chnode = new AndedNode();
		chnode->Children.push_back(Rule->RHS->Copy());
		chnode->Children.push_back(NTN);
		((OredNode*)R2->RHS)->Children.push_back(chnode);
	}
	else
	{
		NonTerminalNode* NTN= new NonTerminalNode();
		NTN->NameIndex = NTE.NameIndex;
		AndedNode* chnode = new AndedNode();
		chnode->Children.push_back(Rule->RHS->Copy());
		chnode->Children.push_back(NTN);
		((OredNode*)R1->RHS)->Children.push_back(chnode);
	}






	TerminalNode*  TN= new TerminalNode();
	TN->TerminalType = Eps;
	TN->NameIndex = 0;
	((OredNode*)R2->RHS)->Children.push_back(TN);

	if(((OredNode*)R2->RHS)->Children._Mysize==1)
	{
		LRR_Grammer.push_back(Rule);
		return Rule;
		R1->RHS->Remove();
		delete R1->RHS;
		delete R1;

		R2->RHS->Remove();
		delete R2->RHS;
		delete R2;
	}
	else
	{
		if(((OredNode*)R1->RHS)->Children.size()==1)
		{
			R1->RHS =*((OredNode*)R1->RHS)->Children.begin();
		}
		LRR_Grammer.push_back(R1);
		LRR_Grammer.push_back(R2);
		tstring name = GetNonTerminalName(Rule->LHS);
		name+="_1";
        (*Temp_NonTerminals)[name] = NTE;
		return R1;

	}
}


tstring ParserGenerator::GetTerminalName(unsigned int NameIndex)
{
	for( map<tstring,TerminalEntry>::iterator iter = Terminals.begin(); iter != Terminals.end(); iter++)
	{
		if ( iter->second.NameIndex == NameIndex )
			return iter->first;
	}

}
void ParserGenerator::PrintFile(string filename)
{
	ofstream of;
	of.open(filename.c_str());
	//PrintOptions
	of<<"Options"<<endl<<endl;
	
	
	if(StrNameSpace!="")
	{
		of<<" NameSpace = "<<"\""<<StrNameSpace<<"\"";
	}
	if(StrClassName!="")
	{
		of<<"ClassName = "<<'\"'<<StrClassName<<"\"";
	}
	of<<"Language = ";
	switch(ParserLanguage)
	{
	case Java:
		of<<"JAVA";
		break;
	case CPlusPlus:
		of<<"CPlusPlus";
	case CSharp:
		of<<"CSharp";

	}
	
	of<<endl<<endl;
	
	//PrintTokens
	of<<"Tokens"<<endl<<endl;

	for(map<tstring,TerminalEntry>::iterator Titer= Terminals.begin();Titer!=Terminals.end();Titer++)
	{
		of<<(*Titer).first<<endl;
	}
	of<<endl;
	

	//PrintTODec
	
	of<<"TopOfDeclaration"<<endl;
	of<<"%%"<<endl;
	of<<StrDecTop<<endl;
	of<<"%%"<<endl;
	of<<endl;

	//Print TODef
	
	of<<"TopOfDefinition"<<endl;
	of<<"%%"<<endl;
	of<<StrDefTop<<endl;
	of<<"%%"<<endl;
	of<<endl;

	//Print BODec
	
	of<<"BottomOfDeclaration"<<endl;
	of<<"%%"<<endl;
	of<<StrDecBottom<<endl;
	of<<"%%"<<endl;
	of<<endl;
	//Print BODef
	
	of<<"BottomOfDefinition"<<endl;
	of<<"%%"<<endl;
	of<<StrDefBottom<<endl;
	of<<"%%"<<endl;
	of<<endl;

	//Grammer
	of<<"Grammar"<<endl<<endl;
	//PrintRulesByNonTerminals
	for(map<tstring,NonTerminalEntry>::iterator NTIter = NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++)
	{
		if((*NTIter).second.NameIndex==NonTerminalBase)
		{
			of<<(*NTIter).first;
			if((*NTIter).second.Rule->Attributes!="")
			{
				of<<"(. "<<(*NTIter).second.Rule->Attributes<<" .)";
			}
			of<<"->";
			(*NTIter).second.Rule->RHS->PrintFile(&of,this);
			of<<"."<<endl<<endl;
		}
	}
	for(map<tstring,NonTerminalEntry>::iterator NTIter = NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++)
	{
		if((*NTIter).second.NameIndex!=NonTerminalBase)
		{
			of<<(*NTIter).first;
			if((*NTIter).second.Rule->Attributes!="")
			{
				of<<(*NTIter).second.Rule->Attributes;
			}
			of<<"->";
			(*NTIter).second.Rule->RHS->PrintFile(&of,this);
			of<<"."<<endl<<endl;
		}
	}
	/*for(list<ProductionRule*>::iterator iter = LRR_Grammer.begin();iter!=LRR_Grammer.end();iter++)
	{
		of<<GetNonTerminalName((*iter)->LHS);
		if((*iter)->Attributes!="")
			of<<" ( "<<(*iter)->Attributes<<" )   ->";
		of<<" -> ";
		(*iter)->RHS->PrintFile(&of,this);
		of<<" ."<<endl<<endl;
	}
	of.close();*/
}




factor ParserGenerator::GetMaxFactor(vector<IntVector> input)
{
	factor result;

	int n = input.size();
	int x, y; // x and y holds the two vectors (indexes) that have the longest common factor
	int max = 0;
	int tempLen, bound, k;

	for(int i=0; i < n; i++)
	{
		for(int j = 0; j < n; j++)
		{
			if ( i == j ) //we don't compare a vector with itself
				continue;
			tempLen = 0;
			bound = GetMin( input[i].size(), input[j].size() );
			for( k = 0; k < bound; k++)
			{
				if ( input[i][k] == input[j][k] )
					tempLen++;
				else
					break;
			}
			if ( tempLen > max )
			{
				max = tempLen;
				x = i;
				y = j;
			}
		}
	}

	if ( max > 0 )
	{
		for(int i = 0; i < max; i++)
			result.FactorList.push_back( input[x][i] );

		result.ListIDs.push_back( x );
		result.ListIDs.push_back( y );

		bool isCandidate;

		for(int i = 0; i < n; i++)
		{
			if ( i == x || i == y )
				continue;
			isCandidate = true;
			if ( input[i].size() >= max )
			{
				for( k = 0; k < max; k++)
				{
					if ( input[i][k] != input[x][k] )
					{
						isCandidate = false;
						break;
					}
				}
				if ( isCandidate )
					result.ListIDs.push_back( i );
			}
		}
	}

	return result;
}
int ParserGenerator::GetMin(int x, int y)
{
	return ( x < y )? x : y;
}
tstring ParserGenerator::ToString(int num)
{
	if(num==0)
		return "0";

	tstring s = "";
	int quotient = num/10;
	int rem = num%10;
	s=(char)(rem+(int)'0')+s;
	while(quotient!=0)
	{		
		rem = quotient %10;
		quotient = quotient/10;
		s=(char)(rem+(int)'0')+s;   		
	}
	return s;

}
void ParserGenerator::LeftFactor()
{
	bool changed  = true;
	vector<unsigned int> Indeces;// = new vector<int>(nextNonTerminalIndex-NonTerminalBase);
	for(unsigned int i=0 ;i<nextNonTerminalIndex-NonTerminalBase;i++)
	{
		Indeces.push_back(1);
	}
	while(changed)
	{
		changed = false;
		for(map<tstring,NonTerminalEntry>::iterator NTIter = NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++)
		{
			vector<IntVector> IVV = GetIVV((*NTIter).second.Rule->RHS);
			factor f = GetMaxFactor(IVV);
			if(f.FactorList.size()>0)
			{
				ProductionRule* R1 = new ProductionRule();
				R1->LHS = (*NTIter).second.Rule->LHS;
				R1->Attributes = (*NTIter).second.Rule->Attributes;
				R1->SemAction = (*NTIter).second.Rule->SemAction;
				
				int  index = 0;
				if((*NTIter).second.Rule->RHS->NType==_Ored)
				{
					R1->RHS = new OredNode();

					ProductionRule* R2 = new ProductionRule();
					R2->LHS = nextNonTerminalIndex;
					R2->RHS = new OredNode();

					NonTerminalEntry NTE(nextNonTerminalIndex);
					Indeces.push_back(1);
					nextNonTerminalIndex++;
					int i=0;
					for(list<GenericCFGNode*>::iterator chIter =((OredNode*)(*NTIter).second.Rule->RHS)->Children.begin();chIter!=((OredNode*)(*NTIter).second.Rule->RHS)->Children.end();chIter++)
					{
						if(i<f.ListIDs.size()&&index==f.ListIDs[i])
						{
							
							if((*chIter)->NType==_Anded)
							{
								int count = f.FactorList.size();
								AndedNode* FactoredNode;
								if(i==0)
									FactoredNode= new AndedNode();
								AndedNode* AndNode = (AndedNode*)*chIter;
								list<GenericCFGNode*>::iterator AndIter;
								for(AndIter = AndNode->Children.begin();(count!=0)&&AndIter!=AndNode->Children.end();AndIter++)
								{
									
									if(i==0 && count>0)
									{
										FactoredNode->Children.push_back((*AndIter)->Copy());										
										
									}
									if((*AndIter)->NType!=Sync)
											count--;
								}
								if(i==0)
								{
									NonTerminalNode* NTnode = new NonTerminalNode();
									NTnode->NameIndex = NTE.NameIndex;
									FactoredNode->Children.push_back(NTnode);
									((OredNode*)R1->RHS)->Children.push_back(FactoredNode);
								}
								if(AndIter!=AndNode->Children.end())
								{
									AndedNode* R2Node = new AndedNode();
									for(;AndIter!=AndNode->Children.end();AndIter++)
									{
										R2Node->Children.push_back((*AndIter)->Copy());
									}
									if(R2Node->Children._Mysize==1)
									{
										((OredNode*)R2->RHS)->Children.push_back((*R2Node->Children.begin())->Copy());
										R2Node->Remove();
										delete R2Node;
									}
									else
									{	
										((OredNode*)R2->RHS)->Children.push_back(R2Node);
									}
								}
								else
								{
									TerminalNode* TN= new TerminalNode();
									TN->NameIndex=0;
									TN->TerminalType = Eps;
									((OredNode*)R2->RHS)->Children.push_back(TN);

								}
							}
							else
							{
								if(i==0)
								{
									((OredNode*)R1->RHS)->Children.push_back((*chIter)->Copy());
								}
								TerminalNode* TN= new TerminalNode();
								TN->NameIndex=0;
								TN->TerminalType = Eps;
								((OredNode*)R2->RHS)->Children.push_back(TN);
                                								
							}
							i++;
						}
						else
						{
							((OredNode*)R1->RHS)->Children.push_back((*chIter)->Copy());		

						}	
						index++;
					}
					if(((OredNode*)R1->RHS)->Children._Mysize==1)
					{
						GenericCFGNode * tempnode = (*((OredNode*)R1->RHS)->Children.begin())->Copy();
						R1->RHS->Remove();
						delete R1->RHS;
						R1->RHS = tempnode;

					}
					if(((OredNode*)R2->RHS)->Children._Mysize==1)
					{
						GenericCFGNode * tempnode = (*((OredNode*)R2->RHS)->Children.begin())->Copy();
						R2->RHS->Remove();
						delete R2->RHS;
						R2->RHS = tempnode;

					}
					

					(*NTIter).second.Rule->RHS->Remove();
					delete (*NTIter).second.Rule->RHS;
					(*NTIter).second.Rule = R1;
					NTE.Rule = R2;
					
					NonTerminals[(*NTIter).first+"_"+ToString(Indeces[(*NTIter).second.NameIndex-NonTerminalBase])] =NTE;
					Indeces[(*NTIter).second.NameIndex-NonTerminalBase]++;
					//LRR_Grammer.push_back(R1);
					//LRR_Grammer.push_back(R2);
	
				}
				else
				{
					//R1->RHS = (*NTIter).second.Rule->RHS->Copy();
					//LRR_Grammer.push_back(R1);
					cout<<"Error In factor function";
				}
				changed=true;
			}
			
		}
		PrintByNonTerminals();
	}
	
	//PrintByNonTerminals();

	
}
void ParserGenerator::PrintByNonTerminals()
{
	cout<<"=======================================================\n";
	//PrintRulesByNonTerminals
	for(map<tstring,NonTerminalEntry>::iterator NTIter = NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++)
	{
		cout<<(*NTIter).first;
		if((*NTIter).second.Rule->Attributes!="")
		{
			cout<<(*NTIter).second.Rule->Attributes;
		}
		cout<<"->";
		(*NTIter).second.Rule->RHS->Print(this);
		cout<<"."<<endl<<endl;
	}
}
/*
GenericCFGNode* ParserGenerator::GenerateNode(IntVector IV)
{
	if(IV.size<=1)
	{

	}
	else
	{
		AndedNode * node = new AndedNode();

	}
}*/
vector<IntVector> ParserGenerator::GetIVV(GenericCFGNode* node)
{
	vector<IntVector> ReturnedIVV;
	if(node->NType == _Ored)
	{
		for(list<GenericCFGNode*>::iterator chIter = ((OredNode*)node)->Children.begin();chIter!=((OredNode*)node)->Children.end();chIter++)
		{
			IntVector chIV;
			
			if((*chIter)->NType==_Anded)
			{
				AndedNode* AndNode = (AndedNode*)*chIter;
				for(list<GenericCFGNode*>::iterator AndIter =AndNode->Children.begin();AndIter!=  AndNode->Children.end();AndIter++)
				{
					if((*AndIter)->NType==_Terminal)
					{
						chIV.push_back(((TerminalNode*)*AndIter)->NameIndex);
					}
					else if ((*AndIter)->NType==_NonTerminal)
					{
						chIV.push_back(((NonTerminalNode*)*AndIter)->NameIndex);

					}
					else if((*AndIter)->NType== _SemAction)
					{
						chIV.push_back(((SemActionNode*)*AndIter)->NameIndex);
					}
				}
			}
			else if((*chIter)->NType==_Terminal)
			{
				chIV.push_back(((TerminalNode*)*chIter)->NameIndex);
			}
			else if ((*chIter)->NType==_NonTerminal)
			{
				chIV.push_back(((NonTerminalNode*)*chIter)->NameIndex);
			}
			else if((*chIter)->NType== _SemAction)
			{
				chIV.push_back(((SemActionNode*)*chIter)->NameIndex);
			} 



			ReturnedIVV.push_back(chIV);
		}
	}
	else if (node->NType == _Anded)
	{
		IntVector chIV;
		AndedNode* AndNode = (AndedNode*)node;
		for(list<GenericCFGNode*>::iterator AndIter =AndNode->Children.begin();AndIter!=  AndNode->Children.end();AndIter++)
		{
			if((*AndIter)->NType==_Terminal)
			{
				chIV.push_back(((TerminalNode*)*AndIter)->NameIndex);
			}
			else if ((*AndIter)->NType==_NonTerminal)
			{
				chIV.push_back(((NonTerminalNode*)*AndIter)->NameIndex);
			}
			else if((*AndIter)->NType== _SemAction)
			{
				chIV.push_back(((SemActionNode*)*AndIter)->NameIndex);
			}
		}
		ReturnedIVV.push_back(chIV);
		
	}
	else if(node->NType==_Terminal)
	{
		IntVector chIV;
		chIV.push_back(((TerminalNode*)node)->NameIndex);
		ReturnedIVV.push_back(chIV);
	}
	else if (node->NType==_NonTerminal)
	{
		IntVector chIV;
		chIV.push_back(((NonTerminalNode*)node)->NameIndex);
		ReturnedIVV.push_back(chIV);
	}
	else if(node->NType== _SemAction)
	{
		IntVector chIV;
		chIV.push_back(((SemActionNode*)node)->NameIndex);
		ReturnedIVV.push_back(chIV);
	}

	return ReturnedIVV;

}
