
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;


namespace AMAP
{
	public class Parser
	{
		public Parser( IScanner lexicalAnalyzer )
		{
			scanner = lexicalAnalyzer; 
		} 

		public void program()
		{
			Tree.Root = new Node( NodeType.Program );
            if ( currentToken.TokenType == _TokenType._start )
			{
				Node startNode = new Node( NodeType.Start );
				Tree.Root.Children.Add( startNode );
				generator.GenerateCodeForNode( startNode );
			} 
			match( _TokenType._start );
			statementpart(Tree.Root);
			match( _TokenType._finish );
			if ( currentToken.TokenType == _TokenType._finish )
			{
				Node finishNode = new Node( NodeType.Finish );
				Tree.Root.Children.Add( finishNode );
				generator.GenerateCodeForNode( finishNode );
			}
		}
		public void statementpart( Node node )
		{
			Node n = new Node( NodeType.StatementPart );
			node.Children.Add( n );
			compoundstatement( n );
		}
		public void compoundstatement( Node node )
		{
			Node n = new Node( NodeType.CompoundStatement );
			node.Children.Add( n ); 
			statementsequence( n );
		}
		public void statementsequence( Node node )
		{
			Node n = new Node( NodeType.StatementSequence );
			node.Children.Add( n ); 
			statement( n );
			while ( currentToken.TokenType == _TokenType._typeface || currentToken.TokenType == _TokenType._slice || currentToken.TokenType == _TokenType._learn || currentToken.TokenType == _TokenType._get || currentToken.TokenType == _TokenType._recognize )
			{
							statement( n );
			}
		}
		public void statement( Node node )
		{
			Node n = new Node(); 
			if ( currentToken.TokenType == _TokenType._typeface )
			{
			string typefacesNum; 
			match( _TokenType._typeface );
			match( _TokenType._openBracket );
			typefacesNum = currentToken.Lexeme; 
			match( _TokenType._digit );
			match( _TokenType._closeBracket );
			Tree.TypeFacesNumber = Convert.ToInt32( typefacesNum );
			n.Type = NodeType.TypeFace; 
			}
			else if ( currentToken.TokenType == _TokenType._slice )
			{
			string sliceNum; 
			match( _TokenType._slice );
			match( _TokenType._openBracket );
			sliceNum = currentToken.Lexeme; 
			match( _TokenType._digit );
			sliceNum += currentToken.Lexeme; 
			match( _TokenType._digit );
			match( _TokenType._closeBracket );
			Tree.SliceNumber = Convert.ToInt32( sliceNum );
			n.Type = NodeType.Slice; 
			}
			else if ( currentToken.TokenType == _TokenType._learn )
			{
			match( _TokenType._learn );
			n.Type = NodeType.Learn; 
			}
			else if ( currentToken.TokenType == _TokenType._get )
			{
			match( _TokenType._get );
			n.Type = NodeType.Get; 
			}
			else if ( currentToken.TokenType == _TokenType._recognize )
			{
			match( _TokenType._recognize );
			n.Type = NodeType.Recognize; 
			}
			else
				SyntaxError(currentToken);
			node.Children.Add( n );
			generator.GenerateCodeForNode( n ); 
		}

		public void Parse() 
		{
			currentToken = scanner.GetNextToken();
			program();
			if ( currentToken.TokenType != _TokenType._EOF )
				SyntaxError(currentToken);
		}
		protected void match( _TokenType expected )
		{
			if ( currentToken.TokenType == expected )
				currentToken = scanner.GetNextToken();
			else
				ErrorHandler( expected, currentToken );
		}

		protected void ErrorHandler( _TokenType expected, Token found )
		{
			 // Write your own error handling here 
		}

		protected void SyntaxError( Token errorAtToken )
		{
			 // Write your own error handling here 
		}

		Token currentToken;
		IScanner scanner;

		public SyntaxTree Tree;
		ICodeGenerator generator;

		public void BeginParsing(ICodeGenerator codeGenerator) 
		{
			Tree = new SyntaxTree();
			generator = codeGenerator;
			generator.Tree = this.Tree;
			Parse();
		}

	}


	public class ParserClient
	{
		public static void CompileToAssembly()
		{
			OpenFileDialog dlg = new OpenFileDialog();
			dlg.Title = "Select the source file";
			Parser p;
			if ( dlg.ShowDialog() == DialogResult.OK )
			{
				Scanner s = new Scanner( dlg.FileName );
				p = new Parser(s);
				SaveFileDialog sDlg = new SaveFileDialog();
				sDlg.Title = "Select the output file";
				if ( sDlg.ShowDialog() == DialogResult.OK )
				{
					ICodeGenerator gen = new AssemblyGenerator(sDlg.FileName);
					p.BeginParsing(gen);
					System.GC.Collect();
				}
			}
		}

		public static void CompileToC()
		{
			OpenFileDialog dlg = new OpenFileDialog();
			dlg.Title = "Select the source file";
			Parser p;
			if ( dlg.ShowDialog() == DialogResult.OK )
			{
				Scanner s = new Scanner( dlg.FileName );
				p = new Parser(s);
				SaveFileDialog sDlg = new SaveFileDialog();
				sDlg.Title = "Select the output file";
				if ( sDlg.ShowDialog() == DialogResult.OK )
				{
					ICodeGenerator gen = new CGenerator(sDlg.FileName);
					p.BeginParsing(gen);
					System.GC.Collect();
				}
			}
		}
	}
	public interface ICodeGenerator
	{
		void GenerateCodeForNode( Node node );
		SyntaxTree Tree
		{
			set;
			get;
		}
	}
	public class SyntaxTree
	{
		public Node Root;
		public int TypeFacesNumber;
		public int SliceNumber;
	}

	public class Node
	{
		public Node()
		{
		}
		public Node( NodeType nodeType )
		{
			Type = nodeType;
		}
		public NodeType Type;
		public ArrayList Children = new ArrayList();
	}
	public enum NodeType 
	{
		Program, Start, StatementPart, Finish, CompoundStatement,
		StatementSequence, TypeFace, Slice, Learn, Get, Recognize
	}

	public class AssemblyGenerator : ICodeGenerator
	{
		SyntaxTree Tree;
		StreamWriter stream;
		public AssemblyGenerator(string fileName)
		{
			stream = new StreamWriter( fileName, false);
			stream.AutoFlush = true;
		}
		~AssemblyGenerator()
		{

		}

		SyntaxTree ICodeGenerator.Tree
		{
			set
			{
				Tree = value;
			}
			get
			{
				return Tree;
			}
		}

		void  ICodeGenerator.GenerateCodeForNode( Node node )
		{
			switch( node.Type )
			{
				case NodeType.Start:
					stream.WriteLine("Bgn");
					break;
				case NodeType.Finish:
					stream.WriteLine("End");
					break;
				case NodeType.Learn:
					string sslNum = Tree.SliceNumber.ToString();
					sslNum = (sslNum.Length == 1)? "0" + sslNum : sslNum;
					stream.WriteLine("SSL " + sslNum);
					for(int i=0; i < Tree.TypeFacesNumber; i++)
					{
						stream.WriteLine("INP");
						stream.WriteLine("LDA");
					}
					break;
				case NodeType.Get:
					stream.WriteLine("INP");
					break;
				case NodeType.Recognize:
					stream.WriteLine("MFC"); //clear the match register
					stream.WriteLine("CMP");
					stream.WriteLine("OUT");
					break;
			}
			stream.Flush();
		}

	}


	public class CGenerator : ICodeGenerator
	{
		SyntaxTree Tree;
		StreamWriter stream;
		public CGenerator(string fileName)
		{
			stream = new StreamWriter( fileName, false);
			stream.AutoFlush = true;
		}
		~CGenerator()
		{

		}

		SyntaxTree ICodeGenerator.Tree
		{
			set
			{
				Tree = value;
			}
			get
			{
				return Tree;
			}
		}

		void  ICodeGenerator.GenerateCodeForNode( Node node )
		{
			switch( node.Type )
			{
				case NodeType.Start:
					stream.WriteLine("outportb(control_port_address, 1)");
					break;
				case NodeType.Finish:
					stream.WriteLine("outportb(control_port_address, 14); //End"); //End
					break;
				case NodeType.Learn:
					for(int i=0; i < Tree.TypeFacesNumber; i++)
					{
						stream.WriteLine("outportb(control_port_address, 9); //INP"); //INP
						stream.WriteLine("outportb(ij_output_port_address, input_pattern_to_AMAP); //INP"); //continuing INP
						stream.WriteLine("outportb(control_port_address, 3); //LDA"); //LDA
					}
					break;
				case NodeType.Get:
					stream.WriteLine("outportb(control_port_address, 9); //INP"); //INP
					stream.WriteLine("outportb(ij_output_port_address, input_pattern_to_AMAP); //INP"); //continuing INP
					break;
				case NodeType.Recognize:
					stream.WriteLine("outportb(control_port_address, 13); //MFC"); //MFC
					stream.WriteLine("outportb(control_port_address, 7); //CMP"); //CMP
					stream.WriteLine("outportb(control_port_address, 11); //Out"); //out
					break;
				case NodeType.Slice:
					int val = Tree.SliceNumber;
					val = (val << 4);
					val = (val | 5);
					stream.WriteLine("outportb(control_port_address, " + val.ToString() + "); //SSL"); //SSL
					break;
			}
			stream.Flush();
		}

	}


	public class Scanner : IScanner 
	{
		public Scanner(string fileName)
		{
			inputFile = new FileStream(fileName, FileMode.Open, FileAccess.Read);
			maxCharToRead = 1024;
			currentCharPosition = 0;
			buffer = new byte[maxCharToRead];
			bufferSize = inputFile.Read(buffer, 0, maxCharToRead);
			CurrentLineNum = 1;
		}
		public Token GetNextToken()
		{
			Token currentToken = new Token();
			currentToken.Lexeme = "";
			States state = States.Start;
			bool save;
			while (state != States.Done)
			{
				char c = getNextCharachter();
				save = true;
				switch(state)
				{
					case States.Start:
						if (char.IsDigit(c))
						{
							//currentToken.Lexeme = c.ToString();
							currentToken.TokenType = _TokenType._digit;
							state = States.Done;
						}
						else if(char.IsLetter(c))
							state = States.InID;
						else if (c == '(')
						{
							//currentToken.Lexeme = c.ToString();
							currentToken.TokenType = _TokenType._openBracket;
							state = States.Done;
						}
						else if (c == ')')
						{
							//currentToken.Lexeme = c.ToString();
							currentToken.TokenType = _TokenType._closeBracket;
							state = States.Done;
						}
						else if ((c == ' ') || (c == '\t') || (c == '\n') || c == '\r')
						{
							save = false;
							if (c == '\n')
								CurrentLineNum++;
						}
						else
						{
							state = States.Done;
							switch (c)
							{
								case (char)0:
									save = false;
									currentToken.TokenType = _TokenType._EOF;
									break;
								default:
									currentToken.TokenType = _TokenType._Error;
									break;
							}
						}
						break;
					case States.InID:
						if(!char.IsLetter(c))
						{
							if (c != 0)
								ungetNextCharachter();
							save = false;
							state = States.Done;
							currentToken.TokenType = _TokenType._id;
						}
						break;
					case States.Done: 
					default: //should never happen
						state = States.Done;
						currentToken.TokenType = _TokenType._Error;
						break;
				}
				if(save)
					currentToken.Lexeme += c;
				if (state == States.Done && currentToken.TokenType == _TokenType._id)
					currentToken.TokenType = reservedLookUp(currentToken.Lexeme);
			}
			//currentToken.LineNum = CurrentLineNum;
			return currentToken;
		}
		protected char getNextCharachter()
		{
			if (currentCharPosition < bufferSize)
				return (char)buffer[currentCharPosition++];
			else if (inputFile.Position < inputFile.Length)
			{
				bufferSize = inputFile.Read(buffer, 0, maxCharToRead);
				currentCharPosition = 1;
				return (char)buffer[0];
			}
			else 
				return Convert.ToChar(0); //EOF
		}
		protected void ungetNextCharachter()
		{
			currentCharPosition--;
		}
		protected _TokenType reservedLookUp(string lexeme)
		{
			for(int i=0; i < reservedWords.Length; i++)
				if (reservedWords[i].Lexeme == lexeme)
					return (_TokenType)reservedWords[i].TokenType;
			return _TokenType._id;
		}
		protected FileStream inputFile;
		protected int currentCharPosition, bufferSize, maxCharToRead;
		protected byte[] buffer;
		protected int CurrentLineNum;
		protected Token[] reservedWords = new Token[] {new Token(_TokenType._start,"start"), 
													   new Token(_TokenType._finish,"finish"), 
			                                           new Token(_TokenType._typeface, "typeface"),  new Token(_TokenType._slice, "slice"),
													   new Token(_TokenType._learn, "learn"), new Token(_TokenType._get, "get"),
			                                           new Token(_TokenType._recognize, "recognize")
		                                              };
			~Scanner()
		{
			inputFile.Close();
		}
	}

	

	public enum States
	{
		Start, InID, Done
	}


	public enum _TokenType
	{
		_EOF, _Char, _String, _start, _finish, _typeface, _slice, _learn, _get, _recognize, _digit, _openBracket, _closeBracket, _Error, _id
	}
	public interface IScanner
	{
		Token GetNextToken();
	}
	public class Token
	{
		public _TokenType TokenType;
		public string Lexeme;

		public Token(){}
		public Token( _TokenType type, string lexeme)
		{
			this.TokenType = type;
			this.Lexeme = lexeme;
		}
	}

}

