#include "stdafx.h"
#include "CPlusPlusGenerator.h"
#include "ProductionRule.h"

using namespace RDParserGenerator;

CPlusPlusGenerator::CPlusPlusGenerator( char* decFileName, char* defFileName )
{
	DeclarationFile.open( decFileName );
	DefinitionFile.open( defFileName );
	this->decFileName = decFileName;
}

void CPlusPlusGenerator::GenerateCode( ParserGenerator* parser )
{
	Parser = parser;
	tstring startSymbol;
	DefinitionFile << "#include \"" << decFileName << "\"\n";
	DefinitionFile << "using namespace " << parser->StrNameSpace << ";\n";

	DeclarationFile << "#ifndef _" << parser->StrClassName << endl;
	DeclarationFile << "#define _" << parser->StrClassName << endl;

	DeclarationFile << parser->StrDecTop << endl << endl;
	DeclarationFile << "namespace " << parser->StrNameSpace << endl << '{' << endl;

	//write the classes to be used by the parser class

	//write the _TokenType enumerator
	DeclarationFile << "\tenum _TokenType\n\t{\n\t\t";
	//write the enum members
	for( map<unsigned int, tstring>::iterator iter = parser->TokensNames.begin(); iter != parser->TokensNames.end(); )
	{
		DeclarationFile << iter->second;
		iter++; // I have moved it here due to the next test	
		if ( iter != parser->TokensNames.end() )
			DeclarationFile << ", ";
	}

	 //close the enum
	DeclarationFile << "\n\t};\n";

	//write  the Token class
	DeclarationFile << "\tstruct Token\n\t{\n\t\t_TokenType TokenType;\n\t};" << endl;
	
	//end of helper classes
	
	DeclarationFile << "\ttemplate<class IScanner>\n\tclass " << parser->StrClassName << endl << '\t' << '{' << endl; 
	
	DefinitionFile << parser->StrDefTop << endl;

    //writing the constructor
	//for the class template to work the constructor definition must be written in the .h file
	DeclarationFile << "\tpublic:\n"
		<< "\t\t" << parser->StrClassName << "( IScanner lexicalAnalyzer )\n\t\t{\n\t\t\tscanner = lexicalAnalyzer; \n\t\t}\n";

//	DefinitionFile << "template<class IScanner>\n" << parser->StrClassName << "<IScanner>::" << parser->StrClassName << "( IScanner lexicalAnalyzer )\n{\n\tscanner = lexicalAnalyzer; \n}\n" << endl;

	ProductionRule* firstProd = parser->Grammer.front();
	startSymbol = parser->GetNonTerminalName( firstProd->LHS ) + _T("(") + firstProd->Attributes + _T(")");
	for( list<ProductionRule*>::iterator iter = parser->Grammer.begin(); iter != parser->Grammer.end(); iter++)
	{
		//note *iter is ProductionRule*
		DeclarationFile << "\t\tvoid " << parser->GetNonTerminalName( (*iter)->LHS )
			           << '(' << (*iter)->Attributes << ");\n";

		DefinitionFile << "template<class IScanner>\nvoid " << parser->StrClassName << "<IScanner>::" << parser->GetNonTerminalName( (*iter)->LHS )
			           << '(' << (*iter)->Attributes << ")\n{\n";

		(*iter)->RHS->GenerateCode( this );
		
		DefinitionFile << '}' << endl;
	}

	//writing the Parse function
	DeclarationFile << "\t\tvoid Parse();\n";

	DefinitionFile << "template<class IScanner>\nvoid " << parser->StrClassName << "<IScanner>::Parse()" << "\n\{\n\tcurrentToken = scanner.GetNextToken();\n\t" << startSymbol << ";\n"
		<< "\tif ( currentToken.TokenType != _EOF )\n"
		<< "\t\tSyntaxError(currentToken);"<< "\n}" << endl;	


	//writing the match function
	DeclarationFile << "\tprotected:\n" 
		<< "\t\tvoid match( _TokenType expected );\n";

	DefinitionFile << "template<class IScanner>\nvoid " << parser->StrClassName << "<IScanner>::match( _TokenType expected )\n{\n\tif ( currentToken.TokenType == expected )\n\t\tcurrentToken = scanner.GetNextToken();\n\telse\n\t\t\tErrorHandler( expected, currentToken );\n}\n";
	//writing the ErrorHandler code
	DeclarationFile << "\t\tvoid ErrorHandler( _TokenType expected, Token found );\n";
	DefinitionFile << "template<class IScanner>\nvoid " << parser->StrClassName << "<IScanner>::ErrorHandler( _TokenType expected, Token found )\n{\n\t // Write your own error handling here \n}\n";

	//write the SyntaxError function
	DeclarationFile << "\t\t" 
		<< "void SyntaxError( Token errorAtToken );\n";

	DefinitionFile << "template<class IScanner>\nvoid " << parser->StrClassName << "<IScanner>::SyntaxError( Token errorAtToken )\n{\n\t // Write your own error handling here \n}\n" << endl;

	DefinitionFile << parser->StrDefBottom << endl;

	//writing the data members
	DeclarationFile << "\t\t" << "Token currentToken;" << endl
				   << "\t\t" << "IScanner scanner;" << endl;

	DeclarationFile << "\t};" << endl << endl; //end class
    
	DeclarationFile << endl << '}'; //end namespace

	DeclarationFile << endl << parser->StrDecBottom ;

	DeclarationFile << endl << "#endif" ;
	
	DeclarationFile.close();
	DefinitionFile.close();
}

void CPlusPlusGenerator::AndGenerator( AndedNode* node )
{
	if( node->ResolverString.length() > 0 )
		DefinitionFile << "if ( " << node->ResolverString << " )\n\t{\n";
	for( list<GenericCFGNode*>::iterator iter = node->Children.begin(); iter != node->Children.end(); iter++)
	{
		(*iter)->GenerateCode(this);
	}
	if( node->ResolverString.length() > 0 )
		DefinitionFile << "\t}\n";
}

void CPlusPlusGenerator::ClosureGenerator( ClosureNode* node )
{
	DefinitionFile << "\twhile (";
	BitSet_Min firstSet = node->FirstSet( Parser );
	int index = 0; //we begin from zero because we need to skip Eps if found
	
	while ( (index = firstSet.NextMemberFrom( index+1 )) != -1 )
	{
		DefinitionFile << " currentToken.TokenType == " << Parser->TokensNames[index];
		if ( firstSet.NextMemberFrom( index+1 ) != -1 )
			DefinitionFile << " ||";
	}
	DefinitionFile << " )\n\t{\n\t\t";
	for( list<GenericCFGNode*>::iterator iter = node->Children.begin(); iter != node->Children.end(); iter++)
	{
		(*iter)->GenerateCode(this);
	}
	DefinitionFile << "\t\t}\n";
}

void CPlusPlusGenerator::NonTerminalGenerator( NonTerminalNode* node )
{
	DefinitionFile << '\t' << Parser->GetNonTerminalName( node->NameIndex ) <<  '(' << node->Attributes << ");" << endl ;
}

void CPlusPlusGenerator::OptionalGenerator( OptionNode* node )
{
	DefinitionFile << "\tif (";
	BitSet_Min firstSet = node->FirstSet( Parser );
	int index = 0; //we begin from zero because we need to skip Eps if found
	
	while ( (index = firstSet.NextMemberFrom( index+1 )) != -1 )
	{
		DefinitionFile << " currentToken.TokenType == " << Parser->TokensNames[index];
		if ( firstSet.NextMemberFrom( index+1 ) != -1 )
			DefinitionFile << " ||";
	}
	DefinitionFile << " )\n\t{\n";
	for( list<GenericCFGNode*>::iterator iter = node->Children.begin(); iter != node->Children.end(); iter++)
	{
		(*iter)->GenerateCode(this);
	}
	DefinitionFile << "\t}\n";
}

void CPlusPlusGenerator::OrGenerator( OredNode* node )
{
	BitSet_Min firstSet = node->FirstSet( Parser );
	bool containsEps = ( firstSet.NextMemberFrom(0) == 0 )? 1 : 0;
	int index;
	bool firstIf = 1;
	for( list<GenericCFGNode*>::iterator iter = node->Children.begin(); iter != node->Children.end();iter++)
	{
		firstSet = (*iter)->FirstSet( Parser );
		if ( firstSet.NextMemberFrom(1) == -1 )
			continue;
		if ( firstIf )
		{
			DefinitionFile << "\tif (";
			firstIf = 0;
		}
		else
			DefinitionFile << "\telse if (";

		index = 0;
		while ( (index = firstSet.NextMemberFrom( index+1 )) != -1 )
		{
			DefinitionFile << " currentToken.TokenType == " << Parser->TokensNames[index];
			if ( firstSet.NextMemberFrom( index+1 ) != -1 )
				DefinitionFile << " ||";
		}
	    DefinitionFile << " )\n\t{\n";
		(*iter)->GenerateCode(this);
		DefinitionFile << "\t}\n";
	}
	if ( !containsEps ) //so it is not optional
		DefinitionFile << "\telse\n\t\tSyntaxError(currentToken);\n";
}

void CPlusPlusGenerator::SemActionGenerator( SemActionNode* node )
{
	DefinitionFile << '\t' << node->SemAction << endl;
}

void CPlusPlusGenerator::TerminalGenerator( TerminalNode* node )
{
	DefinitionFile << "\tmatch( " <<  Parser->TokensNames[node->NameIndex-TerminalBase] << " );" << endl ;
}