#pragma once
#ifndef _GenericCFGNode_h
#define _GenericCFGNode_h
#include "Globals.h"
#include "stdafx.h"
#include "BitSet_Min.h"
#include "Scanner.h"
//#include "ICodeGenerator.h" //it includes ParserGen.h
#include "ParserGen.h"
#include <list>
using namespace std;

//using namespace RDParserGenerator;
namespace RDParserGenerator
{
	
	class ParserGenerator;
	class ICodeGenerator;

struct Factor
{
	list<unsigned int> FactorList;
	vector<int> ListIDs;
};


//static int GetMin(unsigned int x,unsigned int y);

class GenericCFGNode
{
public:
	NodeType NType;
	virtual BitSet_Min FirstSet(ParserGenerator* P);
	virtual bool FollowSet(ParserGenerator* P);
	virtual BitSet_Min GetNonTerminals(ParserGenerator* P);
	void virtual GenerateCode( ICodeGenerator* gen ) = 0;
	void virtual Print(ParserGenerator* P)=0;
	tstring virtual check_LL1(ParserGenerator* P)=0;
	virtual GenericCFGNode*   Copy()=0;
	void virtual Remove()=0;
	virtual bool LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)=0;
	virtual bool LeftFactor(ParserGenerator* P,GenericCFGNode** RHS_Node )=0;
	static Factor GetMaxFactor( vector< vector<unsigned int> > Input );
	void virtual PrintFile(ofstream* of,ParserGenerator* P,bool CancelBracket)=0;
};
class OredNode:public GenericCFGNode
{
public:
	list<GenericCFGNode*> Children;
	OredNode();
	virtual BitSet_Min FirstSet(ParserGenerator* P);
	virtual bool FollowSet(ParserGenerator* P);
	virtual BitSet_Min GetNonTerminals(ParserGenerator* P);
	tstring virtual check_LL1(ParserGenerator* P);
	void GenerateCode ( ICodeGenerator* gen );
	void virtual Print(ParserGenerator* P);
//	tstring virtual check_LL1(ParserGenerator* P);
	virtual GenericCFGNode*   Copy();
	void virtual Remove();
	virtual bool LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs);
	virtual bool LeftFactor(ParserGenerator* P,GenericCFGNode** RHS_Node );
	bool SubLeftFactor(ParserGenerator* P,list<GenericCFGNode*>*ChList,GenericCFGNode** RetNode);
	void virtual PrintFile(ofstream* of,ParserGenerator* P,bool CancelBracket);
};


class ClosureNode:public OredNode
{
public:
	ClosureNode();
	virtual BitSet_Min FirstSet(ParserGenerator* P);
	virtual bool FollowSet(ParserGenerator* P);
	virtual BitSet_Min GetNonTerminals(ParserGenerator* P);
	void GenerateCode ( ICodeGenerator* gen );
	void virtual Print(ParserGenerator* P);
	tstring virtual check_LL1(ParserGenerator* P);
	virtual GenericCFGNode*   Copy();
	void virtual Remove();
	virtual bool LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs);
	virtual bool LeftFactor(ParserGenerator* P,GenericCFGNode** RHS_Node );
	void virtual PrintFile(ofstream* of,ParserGenerator* P,bool CancelBracket);
	
};

class OptionNode:public OredNode
{
public:
	
	OptionNode();
	virtual BitSet_Min FirstSet(ParserGenerator* P);
	virtual bool FollowSet(ParserGenerator* P);
	virtual BitSet_Min GetNonTerminals(ParserGenerator* P);
	void GenerateCode ( ICodeGenerator* gen );
	void virtual Print(ParserGenerator* P);
	tstring virtual check_LL1(ParserGenerator* P);
	virtual GenericCFGNode*   Copy();
	void virtual Remove();
	virtual bool LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs);
	virtual bool LeftFactor(ParserGenerator* P,GenericCFGNode** RHS_Node );
	void virtual PrintFile(ofstream* of,ParserGenerator* P,bool CancelBracket);
	
};
/*
class BracketNode:public OredNode
{
public:	
	BracketNode()
	{
		this->NType = _Brackets;
	}
	virtual BitSet_Min FirstSet(ParserGenerator* P)
	{		
		BitSet_Min BS=(*Children.begin())->FirstSet( P);
		return BS;
	}
};*/
class AndedNode:public OredNode
{
public:
	tstring ResolverString;
	AndedNode();
	virtual BitSet_Min FirstSet(ParserGenerator* P);
	virtual bool FollowSet(ParserGenerator* P);
	virtual BitSet_Min GetNonTerminals(ParserGenerator* P);
	void GenerateCode ( ICodeGenerator* gen );
	void virtual Print(ParserGenerator* P);
	tstring virtual check_LL1(ParserGenerator* P);
	virtual GenericCFGNode*   Copy();
	void virtual Remove();
	virtual bool LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs);
	virtual bool LeftFactor(ParserGenerator* P,GenericCFGNode** RHS_Node );
	void virtual PrintFile(ofstream* of,ParserGenerator* P,bool CancelBracket);
};

class SemActionNode:public GenericCFGNode
{
public:
	tstring SemAction;
	SemActionNode();
	virtual BitSet_Min FirstSet(ParserGenerator* P);
	virtual BitSet_Min GetNonTerminals(ParserGenerator* P);

	void GenerateCode ( ICodeGenerator* gen );
	void virtual Print(ParserGenerator* P);
	tstring virtual check_LL1(ParserGenerator* P);
	virtual GenericCFGNode*   Copy();
	void virtual Remove();
	virtual bool LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs);
	virtual bool LeftFactor(ParserGenerator* P,GenericCFGNode** RHS_Node );
	void virtual PrintFile(ofstream* of,ParserGenerator* P,bool CancelBracket);
};


class TerminalNode:public GenericCFGNode
{
public:
	int NameIndex;
	tstring Attributes;
	RDParserGenerator::TokenType TerminalType;
	bool IsWeak;
	TerminalNode();
	virtual BitSet_Min FirstSet(ParserGenerator* P);
	virtual BitSet_Min GetNonTerminals(ParserGenerator* P);
	void GenerateCode ( ICodeGenerator* gen );
	void virtual Print(ParserGenerator* P);
	tstring virtual check_LL1(ParserGenerator* P);
	virtual GenericCFGNode*   Copy();
	void virtual Remove();
	virtual bool LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs);
	virtual bool LeftFactor(ParserGenerator* P,GenericCFGNode** RHS_Node );
	void virtual PrintFile(ofstream* of,ParserGenerator* P,bool CancelBracket);
};
class NonTerminalNode :public TerminalNode
{
public:
	NonTerminalNode();
	virtual BitSet_Min FirstSet(ParserGenerator* P);
	virtual BitSet_Min GetNonTerminals(ParserGenerator* P);
	void GenerateCode ( ICodeGenerator* gen );
	void virtual Print(ParserGenerator* P);
	tstring virtual check_LL1(ParserGenerator* P);
	virtual GenericCFGNode*   Copy();
	void virtual Remove();
	virtual bool LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs);
	virtual bool LeftFactor(ParserGenerator* P,GenericCFGNode** RHS_Node );
	void virtual PrintFile(ofstream* of,ParserGenerator* P,bool CancelBracket);
};
class AnyNode:public GenericCFGNode//feeha shak
{
public:
	AnyNode();
	virtual BitSet_Min FirstSet(ParserGenerator* P);
	virtual BitSet_Min GetNonTerminals(ParserGenerator* P);
	void GenerateCode ( ICodeGenerator* gen );
	void virtual Print(ParserGenerator* P);
	tstring virtual check_LL1(ParserGenerator* P);
	virtual GenericCFGNode*   Copy();
	void virtual Remove();
	virtual bool LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs);
	virtual bool LeftFactor(ParserGenerator* P,GenericCFGNode** RHS_Node );
	void virtual PrintFile(ofstream* of,ParserGenerator* P,bool CancelBracket);
};
class SyncNode:public GenericCFGNode
{
public:
	SyncNode();
	virtual BitSet_Min FirstSet(ParserGenerator* P);
	virtual BitSet_Min GetNonTerminals(ParserGenerator* P);
	void GenerateCode ( ICodeGenerator* gen );
	void virtual Print(ParserGenerator* P);
	tstring virtual check_LL1(ParserGenerator* P);
	virtual GenericCFGNode*   Copy();
	void virtual Remove();
	virtual bool LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs);
	virtual bool LeftFactor(ParserGenerator* P,GenericCFGNode** RHS_Node );
	void virtual PrintFile(ofstream* of,ParserGenerator* P,bool CancelBracket);
};

}
#endif