#include "stdafx.h"
#include "CPlusPlusGenerator.h"
#include "ProductionRule.h"

using namespace RDParserGenerator;

CPlusPlusGenerator::CPlusPlusGenerator( char* decFileName, char* defFileName )
{
	DeclarationFile.open( decFileName );
	DefinitionFile.open( defFileName );
	this->decFileName = decFileName;
}

void CPlusPlusGenerator::GenerateCode( ParserGenerator* parser )
{
	Parser = parser;

	DeclarationFile << "#ifndef _" << parser->StrClassName << endl;
	DeclarationFile << "#define _" << parser->StrClassName << endl;
	DeclarationFile << parser->StrDecTop << endl << endl;

	DefinitionFile << parser->StrDefTop << endl;
	DeclarationFile << "#include <stack>\n";
	DefinitionFile << "#include \"" << decFileName << "\"\n";
	DefinitionFile << "using namespace " << parser->StrNameSpace << ";\n";

	DeclarationFile << "namespace " << parser->StrNameSpace << endl << '{' << endl;
	//write the helper classes

	//write the _TokenType enumerator
	DeclarationFile << "\tenum _TokenType\n\t{\n\t\t";
	//write the enum members
	for( map<unsigned int, tstring>::iterator iter = parser->TokensNames.begin(); iter != parser->TokensNames.end(); )
	{
		DeclarationFile << iter->second;
		iter++; // I have moved it here due to the next test	
		if ( iter != parser->TokensNames.end() )
			DeclarationFile << ", ";
	}

	//close the enum
	DeclarationFile << endl << "\t};" << endl;

	//write the Token struct
	DeclarationFile << "\tstruct Token\n\t{\n\t\tint TokenType;\n\t};" << endl;
	//write the LL1Entry struct
	DeclarationFile << "\tstruct LL1Entry\n\t{\n\t\tint* Production;\n\t\tint Length;\n\t};" << endl;

	//end of helper classes

	DeclarationFile << "\ttemplate<class IScanner>\n\tclass " << parser->StrClassName << endl << '\t' << '{' << endl; 
	
	//writing the constructor
	//for the class template to work the constructor definition must be written in the .h file
	DeclarationFile << "\tpublic:\n"
		<< "\t\t" << parser->StrClassName << "( IScanner lexicalAnalyzer ): _TerminalBase(" << TerminalBase << "), _NonTerminalBase(" << NonTerminalBase << "), _CodeBase(" << CodeBase << ")\n\t\t{\n\t\t\tscanner = lexicalAnalyzer; \n\t\t}\n";
	

	DefinitionFile << parser->StrDefBottom << endl;


	//writing the Parse function
	DeclarationFile << "\t\tvoid Parse();\n";
	DefinitionFile << "template<class IScanner>\nvoid " << parser->StrClassName << "<IScanner>::Parse()"
		        << "\n{\n\tcurrentToken = scanner.GetNextToken();\n\tInitializeParser();\n";
	this->generateLL1Controller();	
	DefinitionFile << "\n}" << endl; //end the parse function
	//	<< "\t\t\tif ( currentToken.TokenType != (int)_TokenType._EOF )\n"
	//	<< "\t\t\t\tSyntaxError(currentToken);"<< "\n\t\t}" << endl;	

	this->initializeParser(); //write the _fillL1Table()

	//write the SyntaxError function
	DeclarationFile << "\t\t" 
		<< "void SyntaxError( Token errorAtToken );\n";
	
	DefinitionFile << "template<class IScanner>\nvoid " << parser->StrClassName << "<IScanner>::SyntaxError( Token errorAtToken )\n{\n\t // Write your own error handling here \n}\n" << endl;

	DeclarationFile << "\n\tprotected:\n";
	DeclarationFile << "\t\tbool IsTerminal(int id);\n";
	DefinitionFile << "template<class IScanner>\nbool " << parser->StrClassName << "<IScanner>::" << "IsTerminal(int id)\n{\n\treturn ( id >= _TerminalBase && id < _NonTerminalBase );\n}\n";
	
	DeclarationFile << "\t\tbool IsNonTerminal(int id);\n";
	DefinitionFile << "template<class IScanner>\nbool " << parser->StrClassName << "<IScanner>::" << "IsNonTerminal(int id)\n{\n\treturn ( id >= _NonTerminalBase && id < _CodeBase );\n}\n";

	DeclarationFile << "\t\tbool IsCode(int id);\n";
	DefinitionFile << "template<class IScanner>\nbool " << parser->StrClassName << "<IScanner>::IsCode(int id)\n{\n\treturn ( id >= _CodeBase );\n}\n";

	//writing the data members
	DeclarationFile << '\t' << '\t' << "Token currentToken;" << endl
				   << '\t' << '\t' << "IScanner scanner;" << endl;
	DeclarationFile << "\t\tconst int _TerminalBase;\n"; 
	DeclarationFile << "\t\tconst int _NonTerminalBase;\n";
	DeclarationFile << "\t\tconst int _CodeBase;\n";
	DeclarationFile << "\t\ttypedef void (" << parser->StrClassName << "::* VoidFunction)();\n";
	DeclarationFile << "\t\tVoidFunction* voidFunctions;\n";
	DeclarationFile << "\t\t" << "LL1Entry*** LL1Table;" << endl;
	DeclarationFile << "\t\t" << "std::stack<int> _stack;" << endl;

	this->generateUserFunctions();

	DeclarationFile << "\t};" << endl << endl; //end class
    
	DeclarationFile << endl << '}'; //end namespace

	DeclarationFile << endl << parser->StrDecBottom ;

	DeclarationFile << endl << "#endif" ;
	
	DeclarationFile.close();
	DefinitionFile.close();
}

void CPlusPlusGenerator::generateLL1Controller()
{
	DefinitionFile << "\t" << "_stack.push( (int)_TokenType._EOF );\n";
	DefinitionFile << "\t" << "_stack.push( _NonTerminalBase );\n";
	DefinitionFile << "\t" << "while ( stack.top() != (int)_TokenType._EOF )\n\t{\n\t\t"
		           << "int top = (int)stack.Peek();" << "\n\t\t"
				   << "if ( IsTerminal( top ) && top == (int)currentToken.TokenType )" << "\n\t\t{\n\t\t\t"
				   << "stack.pop();" << "\n\t\t\t"
				   << "currentToken = scanner.GetNextToken();" << "\n\t\t}\n\t\t"
				   << "else if ( IsNonTerminal( top ) )" << "\n\t\t{\n\t\t\t"
				   << "LL1Entry* entry = LL1Table[top - _NonTerminalBase][(int)currentToken.TokenType - _TerminalBase];" << "\n\t\t\t"
				   << "if ( entry == 0 )" << "\n\t\t\t\t" 
	               << "SyntaxError( currentToken );" << "\n\t\t\t"
				   << "else //generate" << "\n\t\t\t{\n\t\t\t\t"
				   << "stack.pop();" << "\n\t\t\t\t" 
				   << "for(int i = entry->Length-1; i >= 0; i--)" << "\n\t\t\t\t\t"
				   << "stack.push( entry->Production[i] );" << "\n\t\t\t}" << "\n\t\t}\n\t\t" 
				   << "else if ( IsCode( top ) )" << "\n\t\t{\n\t\t\t"
				   << "stack.pop();" << "\n\t\t\t"
				   << "(this->*this->voidFunctions[top - _CodeBase])();" << "\n\t\t}"
				   << "\n\t}\n\t" //end while
				   << "if ( !( stack.top() == (int)_TokenType._EOF && currentToken.TokenType == (int)_TokenType._EOF ) )" << "\n\t\t"
				   << "SyntaxError( currentToken );" << endl;
}

void CPlusPlusGenerator::initializeParser()
{
	DeclarationFile << "\n\t\tvoid InitializeParser();\n";
	DefinitionFile << "template<class IScanner>\nvoid " << Parser->StrClassName << "<IScanner>::InitializeParser()\n{\n";
	int rowsNo = Parser->nextNonTerminalIndex - NonTerminalBase;
	int colsNo = Parser->nextTerminalIndex - TerminalBase - 1;
	DefinitionFile << "\t//Initialize the LL1 table\n";
	DefinitionFile << "\tLL1Table = new LL1Entry**[" << rowsNo << "];\n";
	DefinitionFile << "\tfor(int i=0; i < " << rowsNo << "; i++)\n\t{\n";
	DefinitionFile << "\t\tLL1Table = new LL1Entry*[" << colsNo << "];\n";
	DefinitionFile << "\t}\n";
	int len;
	for(int i = 0; i < rowsNo; i++)
	{
		for(int j = 0; j < colsNo; j++)
		{
			list<int>* lis = Parser->LL1Table[i][j];
			if ( lis == 0 )
				DefinitionFile << "\tLL1Table[" << i << "][" << j << "] = 0;\n";
			else if ( lis->size() == 1 && *(lis->begin()) == 0 )
			{
				DefinitionFile << "\tLL1Table[" << i << "][" << j << "] = new LL1Entry[1];\n";
				DefinitionFile << "\tLL1Table[" << i << "][" << j << "]->Length = 0;\n";
			}
			else
			{
				DefinitionFile << "\tint temp" << i << j << "[] = {";
				len = 0;
				for( list<int>::iterator iter = lis->begin(); iter != lis->end() ; )
				{
					DefinitionFile << *iter;
					iter++;
					len++;
					if ( iter != lis->end() )
						DefinitionFile << ", ";
				}
				DefinitionFile << "};\n";
				DefinitionFile << "\tLL1Table[" << i << "][" << j << "]->Production = temp" << i << j << ";\n";
				DefinitionFile << "\tLL1Table[" << i << "][" << j << "]->Length = " << len << ";\n";
			}
		}
	}

	DefinitionFile << "\n\t//Initialize the memeber function pointers\n";
	DefinitionFile << "\tvoidFunctions = new VoidFunction[" << Parser->Codes.size() << "];\n";
	int i = 0;
	for( map<unsigned int,CodeEntry>::iterator iter = Parser->Codes.begin(); iter != Parser->Codes.end(); iter++ )
	{
		DefinitionFile << "\n\tvoidFunctions[" << i++ << "] = &" << Parser->StrClassName << "::" << iter->second.FunctionName << ";";
	}

	DefinitionFile << "\n}\n";
}

void CPlusPlusGenerator::generateUserFunctions()
{
	DefinitionFile << "\n\t\t//Grammer embedded code\n";
	for( map<unsigned int,CodeEntry>::iterator iter = Parser->Codes.begin(); iter != Parser->Codes.end(); iter++ )
	{
		DeclarationFile << "\n\t\tvoid " << iter->second.FunctionName << "();\n";
		DefinitionFile << "template<class IScanner>\nvoid " << Parser->StrClassName << "<IScanner>::" << iter->second.FunctionName << "()\n{\n\t"
			           << iter->second.Code << "\n}\n";
	}
}