#include "stdafx.h"
#include "CSharpGenerator.h"
#include "ProductionRule.h"

using namespace RDParserGenerator;

CSharpGenerator::CSharpGenerator( char* fileName )
{
	DefinitionFile.open( fileName );
}

void CSharpGenerator::GenerateCode( ParserGenerator* parser )
{
	if ( !DefinitionFile.is_open() )
		DefinitionFile.open( parser->StrClassName.c_str() );

	Parser = parser;

	DefinitionFile << parser->StrDecTop << endl << endl;
	DefinitionFile << "namespace " << parser->StrNameSpace << endl << '{' << endl;
	DefinitionFile << parser->StrDefTop << endl;
	DefinitionFile << '\t' << "public class " << parser->StrClassName << endl << '\t' << '{' << endl; 
	DefinitionFile << parser->StrDefBottom << endl;

    //writing the constructor
    DefinitionFile << '\t' << '\t' 
		<< "public " << parser->StrClassName << "( IScanner lexicalAnalyzer )\n\t\t{\n\t\t\tscanner = lexicalAnalyzer; \n\t\t} \n" << endl;

	//writing the Parse function
	DefinitionFile << "\n\t\t"
		<< "public void Parse() " << "\n\t\t{\n\t\t\tcurrentToken = scanner.GetNextToken();\n\t\t\tInitializeParser();\n";
	this->generateLL1Controller();	
	DefinitionFile << "\n\t\t}" << endl; //end the parse function
	//	<< "\t\t\tif ( currentToken.TokenType != (int)_TokenType._EOF )\n"
	//	<< "\t\t\t\tSyntaxError(currentToken);"<< "\n\t\t}" << endl;	

	this->initializeParser(); //write the _fillL1Table()

	//write the SyntaxError function
	DefinitionFile << '\t' << '\t' 
		<< "protected void SyntaxError( Token errorAtToken )\n\t\t{\n\t\t\t // Write your own error handling here \n\t\t}\n" << endl;

	DefinitionFile << "\t\t" << "protected bool IsTerminal(int id)\n\t\t{\n\t\t\treturn ( id >= _TerminalBase && id < _NonTerminalBase );\n\t\t}\n";
	
	DefinitionFile << "\t\t" << "protected bool IsNonTerminal(int id)\n\t\t{\n\t\t\treturn ( id >= _NonTerminalBase && id < _CodeBase );\n\t\t}\n";

	DefinitionFile << "\t\t" << "protected bool IsCode(int id)\n\t\t{\n\t\t\treturn ( id >= _CodeBase );\n\t\t}\n";

	//writing the data members
	DefinitionFile << '\t' << '\t' << "Token currentToken;" << endl
				   << '\t' << '\t' << "IScanner scanner;" << endl;
	DefinitionFile << "\t\tconst int _TerminalBase = " << TerminalBase << ";\n"; 
	DefinitionFile << "\t\tconst int _NonTerminalBase = " << NonTerminalBase << ";\n";
	DefinitionFile << "\t\tconst int _CodeBase = " << CodeBase << ";\n";
	DefinitionFile << "\t\tdelegate void VoidFunction();\n";
	DefinitionFile << "\t\tVoidFunction[] voidFunctions;\n";
	DefinitionFile << "\t\t" << "int[,][] LL1Table = new int[" << parser->nextNonTerminalIndex-NonTerminalBase 
		           << ", " << parser->nextTerminalIndex-TerminalBase-1 << "][];" << endl;
	DefinitionFile << "\t\t" << "System.Collections.Stack stack = new System.Collections.Stack();" << endl;

	this->generateUserFunctions();

	DefinitionFile << "\t}" << endl << endl; //end class
 
	//write the _TokenType enumerator
	DefinitionFile << "\tpublic enum _TokenType\n\t{\n\t\t";
	//write the enum members
	for( map<unsigned int, tstring>::iterator iter = parser->TokensNames.begin(); iter != parser->TokensNames.end(); )
	{
		DefinitionFile << iter->second;
		iter++; // I have moved it here due to the next test	
		if ( iter != parser->TokensNames.end() )
			DefinitionFile << ", ";
	}

	//close the enum
	DefinitionFile << endl << "\t}" << endl;

	//write the IScanner interface and the Token class
	DefinitionFile << '\t' << "public interface IScanner\n\t{\n\t\tToken GetNextToken();\n\t}" << endl;
	DefinitionFile << '\t' << "public class Token\n\t{\n\t\tpublic _TokenType TokenType;\n\t}" << endl;

	DefinitionFile << endl << '}'; //end namespace

	DefinitionFile << endl << parser->StrDecBottom ;

	DefinitionFile.close();
}

void CSharpGenerator::generateLL1Controller()
{
	DefinitionFile << "\t\t\t" << "stack.Push(_TokenType._EOF);\n";
	DefinitionFile << "\t\t\t" << "stack.Push( _NonTerminalBase );\n";
	DefinitionFile << "\t\t\t" << "while ( ((int)stack.Peek() != (int)_TokenType._EOF) && (currentToken.TokenType !=  _TokenType._EOF) )\n\t\t\t{\n\t\t\t\t"
		           << "int top = (int)stack.Peek();" << "\n\t\t\t\t"
				   << "if ( IsTerminal( top ) && top == (int)currentToken.TokenType )" << "\n\t\t\t\t{\n\t\t\t\t\t"
				   << "stack.Pop();" << "\n\t\t\t\t\t"
				   << "currentToken = scanner.GetNextToken();" << "\n\t\t\t\t}\n\t\t\t\t"
				   << "else if ( IsNonTerminal( top ) )" << "\n\t\t\t\t{\n\t\t\t\t\t"
				   << "int[] production = LL1Table[top - _NonTerminalBase, (int)currentToken.TokenType - _TerminalBase];" << "\n\t\t\t\t\t"
				   << "if ( production == null )" << "\n\t\t\t\t\t\t" 
	               << "SyntaxError( currentToken );" << "\n\t\t\t\t\t"
				   << "else //generate" << "\n\t\t\t\t\t{\n\t\t\t\t\t\t"
				   << "stack.Pop();" << "\n\t\t\t\t\t\t" 
				   << "for(int i = production.Length-1; i >= 0; i--)" << "\n\t\t\t\t\t\t\t"
				   << "stack.Push( production[i] );" << "\n\t\t\t\t\t}" << "\n\t\t\t\t}\n\t\t\t\t" 
				   << "else if ( IsCode( top ) )" << "\n\t\t\t\t{\n\t\t\t\t\t"
				   << "stack.Pop();" << "\n\t\t\t\t\t"
				   << "voidFunctions[top - _CodeBase]();" << "\n\t\t\t\t}"
				   << "\n\t\t\t}\n\t\t\t" //end while
				   << "if ( !( (int)stack.Peek() == (int)_TokenType._EOF && currentToken.TokenType == _TokenType._EOF ) )" << "\n\t\t\t\t"
				   << "SyntaxError( currentToken );" << endl;
}

void CSharpGenerator::initializeParser()
{
	DefinitionFile << "\n\t\tvoid InitializeParser()\n\t\t{\n";
	int rowsNo = Parser->nextNonTerminalIndex - NonTerminalBase;
	int colsNo = Parser->nextTerminalIndex - TerminalBase - 1;
	DefinitionFile << "\t\t\t//Initialize the LL1 table\n";
	for(int i = 0; i < rowsNo; i++)
	{
		for(int j = 0; j < colsNo; j++)
		{
			DefinitionFile << "\t\t\tLL1Table[" << i << ", " << j << "] = ";
			list<int>* lis = Parser->LL1Table[i][j];
			if ( lis == 0 )
				DefinitionFile << "null;\n";
			else if ( lis->size() == 1 && *(lis->begin()) == 0 )
				DefinitionFile << "new int[] {};\n";
			else
			{
				DefinitionFile << "new int[] {";
				for( list<int>::iterator iter = lis->begin(); iter != lis->end() ; )
				{
					DefinitionFile << *iter;
					iter++;
					if ( iter != lis->end() )
						DefinitionFile << ", ";
				}
				DefinitionFile << "};\n";
			}
		}
	}

	DefinitionFile << "\n\t\t\t//Initialize the delegates\n";
	DefinitionFile << "\t\t\tvoidFunctions = new VoidFunction[" << Parser->Codes.size() << "];\n";
	int i = 0;
	for( map<unsigned int,CodeEntry>::iterator iter = Parser->Codes.begin(); iter != Parser->Codes.end(); iter++ )
	{
		DefinitionFile << "\n\t\t\tvoidFunctions[" << i++ << "] = new VoidFunction(" << iter->second.FunctionName << ");";
	}

	DefinitionFile << "\n\t\t}\n";
}

void CSharpGenerator::generateUserFunctions()
{
	DefinitionFile << "\n\t\t//Grammer embedded code\n";

	for( map<unsigned int,CodeEntry>::iterator iter = Parser->Codes.begin(); iter != Parser->Codes.end(); iter++ )
	{
		DefinitionFile << "\n\t\tprotected void " << iter->second.FunctionName << "()\n\t\t{\n\t\t\t"
			           << iter->second.Code << "\n\t\t}\n";
	}
}