#include "stdafx.h"
#include "GenericCFGNode.h"
#include "ParserGen.h"
#include "ICodeGenerator.h"
#include "ProductionRule.h"
using namespace RDParserGenerator;


//GenericCFGNode class
BitSet_Min GenericCFGNode::FirstSet(ParserGenerator* P)
{
	BitSet_Min BS(P->nextTerminalIndex);
	return BS;
}
bool GenericCFGNode::FollowSet(ParserGenerator* P)
{
	//Unreachable return
	return false;
}
BitSet_Min GenericCFGNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextTerminalIndex);
	return BS;
}

//OredNode class
OredNode::OredNode()
{
	this->NType = _Ored;
}
BitSet_Min OredNode::FirstSet(ParserGenerator* P)
{
	BitSet_Min BS(P->nextTerminalIndex); //the terminal size
	list<GenericCFGNode*>::iterator Ch_iter;
	for(Ch_iter=Children.begin();Ch_iter!=Children.end();Ch_iter++)
	{
		BitSet_Min CH_BS = (*Ch_iter)->FirstSet( P);
		BS.SetUnion(CH_BS);
	}
	return BS;
}
bool OredNode::FollowSet(ParserGenerator* P)
{
	bool Changed= false;
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		Changed|=(*Iter)->FollowSet(P);
	}
	//unreachable logically
	return Changed; 
}
BitSet_Min OredNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		BS.SetUnion((*Iter)->GetNonTerminals(P));
	}
	return BS;
}


//ClosureNode class
ClosureNode::ClosureNode()
{
	this->NType = _Closure;
}
BitSet_Min ClosureNode::FirstSet(ParserGenerator* P)
{		
	BitSet_Min BS=(*Children.begin())->FirstSet( P);
	BS.Add(0);//for Epsilon
	return BS;
}
bool ClosureNode::FollowSet(ParserGenerator* P)
{
	//I think this is to be reformulated ya hamdy -- MSaber
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		return (*Iter)->FollowSet(P);
	}
	//unreachable logically
	return false; 
}
BitSet_Min ClosureNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex);
	return (*this->Children.begin())->GetNonTerminals(P);
}


//OptionNode class
OptionNode::OptionNode()
{
	this->NType = _Optional;
}
BitSet_Min OptionNode::FirstSet(ParserGenerator* P)
{		
	BitSet_Min BS=(*Children.begin())->FirstSet( P);
	BS.Add(0);//for Epsilon
	return BS;
}
bool OptionNode::FollowSet(ParserGenerator* P)
{
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		return (*Iter)->FollowSet(P);
	}
	//unreachable logically
	return false; 
}
BitSet_Min OptionNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex);
	return (*this->Children.begin())->GetNonTerminals(P);
}
/*
class BracketNode:public OredNode
{
public:	
	BracketNode()
	{
		this->NType = _Brackets;
	}
	virtual BitSet_Min FirstSet(ParserGenerator* P)
	{		
		BitSet_Min BS=(*Children.begin())->FirstSet( P);
		return BS;
	}
};*/

//AndedNode class
AndedNode::AndedNode()
{
	this->NType=_Anded;
}
BitSet_Min AndedNode::FirstSet(ParserGenerator* P)
{
	BitSet_Min BS(P->nextTerminalIndex); //the terminal size
	list<GenericCFGNode*>::iterator Ch_iter;
	for(Ch_iter=Children.begin();Ch_iter!=Children.end();Ch_iter++)
	{
		BitSet_Min CH_BS = (*Ch_iter)->FirstSet( P);
		BS.SetUnion(CH_BS);
		if(!CH_BS.Contains(0))
		{
			BS.Remove(0);
			break;
		}
	}
	return BS;
}
bool AndedNode::FollowSet(ParserGenerator* P)
{
	bool Changed= false;
		
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		Changed|=(*Iter)->FollowSet(P);
		BitSet_Min NTs = (*Iter)->GetNonTerminals(P);
		//bool EpsInAll= true;
		if(!NTs.IsEmpty())
		{
			BitSet_Min FirstSetOfNextTerms(P->nextTerminalIndex);
			list<GenericCFGNode*>::iterator ChIter= Iter;
			ChIter++;
			bool GetOut=false;
			for(;ChIter!=Children.end();ChIter++)
			{
				BitSet_Min fs = (*ChIter)->FirstSet(P);
				if(!fs.Contains(0))
				{
					GetOut=true;
				}
				fs.Remove(0);
				//BitSet_Min fSet = *(*NTiter).second.FollowSet;
				FirstSetOfNextTerms.SetUnion(fs);
				//if(fSet(*NTiter).second.FollowSet->IsSubsetOf())
				if(GetOut)
				{
	//				EpsInAll=false;
					break;
				}
			}
			int start=-1;
			while((start=NTs.NextMemberFrom(start+1))!=-1)
			{
                //BitSet_Min firstset;
				int NIndex = start+NonTerminalBase;
				for(map<tstring,NonTerminalEntry>::iterator NTiter=P->NonTerminals.begin();NTiter!=P->NonTerminals.end();NTiter++)
				{
					if((*NTiter).second.NameIndex==NIndex)
					{
						//Shakek feeha remember for debuging
						//bool GetOut=false;
						BitSet_Min fs = *(*NTiter).second.FollowSet;
                           (*NTiter).second.FollowSet->SetUnion(FirstSetOfNextTerms);
						if(!fs.Equals(*(*NTiter).second.FollowSet))
							Changed = true;
						/*if(EpsInAll)
						{
							//Make an action
						}*/
					}
				}
			}
		}
		//GetNonTerminals(Child)
		//Add to NonTerminals first of elli ba3diha
	}
	//unreachable logically
	return Changed; 
}
BitSet_Min AndedNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	/*list<GenericCFGNode*>::iterator Iter= Children.end();
	Iter--;
	for(;Iter!=Children.begin();Iter--)*/
	for(list<GenericCFGNode*>::reverse_iterator Iter= Children.rbegin();Iter!=Children.rend();Iter++)
	{
		BS.SetUnion((*Iter)->GetNonTerminals(P));
		BitSet_Min fs = (*Iter)->FirstSet(P); 
		if(!fs.Contains(0))
			break;
	}		
	return BS;
}



//SemActionNode class
SemActionNode::SemActionNode()
{
	this->NType = _SemAction;
	this->SemAction="";
}
BitSet_Min SemActionNode::FirstSet(ParserGenerator* P)
{
	BitSet_Min BS(P->nextTerminalIndex);//ter,minal size
	BS.Add(0);
	return BS;
}
BitSet_Min SemActionNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	return BS;
}




//TerminalNode class
TerminalNode::TerminalNode()
{
	this->NType = _Terminal;		
}
BitSet_Min TerminalNode::FirstSet(ParserGenerator* P)
{
	//return its set
	BitSet_Min BS(P->nextTerminalIndex);
	BS.Add(NameIndex);
	return BS;
}
BitSet_Min TerminalNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	return BS;
}




//NonTerminalNode class
NonTerminalNode::NonTerminalNode()
{
	this->NType = _NonTerminal;		
}
BitSet_Min NonTerminalNode::FirstSet(ParserGenerator* P)
{
	//return its set/
	//BitSet_Min BS(P->nextTerminalIndex); //remember for optimization
	for(map<tstring,NonTerminalEntry>::iterator iter=P->NonTerminals.begin();iter!=P->NonTerminals.end();iter++)
	{
		if((*iter).second.NameIndex==NameIndex)
		{
			/*firstset = *(*iter).second.FirstSet;
			break;*/
			return *(*iter).second.FirstSet;
		}
	}
	//return BS;
}
BitSet_Min NonTerminalNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	BS.Add(this->NameIndex-NonTerminalBase);
	return BS;
}



//AnyNode class
AnyNode::AnyNode()
{
	this->NType = _AnyNode;
}
BitSet_Min AnyNode::FirstSet(ParserGenerator* P)
{
	BitSet_Min BS(P->nextTerminalIndex);
	BS.AddAllElements();
	//return its set
	return BS; //Shakek feeeeeha
}
BitSet_Min AnyNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	return BS;
}


//SyncNode class
SyncNode::SyncNode()
{
	this->NType = _SyncNode;
}
BitSet_Min SyncNode::FirstSet(ParserGenerator* P)
{
	//return its set
	BitSet_Min BS(P->nextTerminalIndex);//terminal size
	BS.Add(0);
	return BS;
}
BitSet_Min SyncNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	return BS;
}


void OredNode::Print(ParserGenerator* P)
{
	cout<<"Ored ( ";
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		(*Iter)->Print(P);
		list<GenericCFGNode*>::iterator tempIter =Iter;
		tempIter++;
		if(tempIter!=Children.end())
			cout<<" , ";
	}
	cout<<" ) ";
}
void AndedNode::Print(ParserGenerator* P)
{
	cout<<"Anded ( ";
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		(*Iter)->Print(P);
		list<GenericCFGNode*>::iterator tempIter =Iter;
		tempIter++;
		if(tempIter!=Children.end())
			cout<<" , ";
	}
	cout<<" ) ";
}
void ClosureNode::Print(ParserGenerator* P)
{
	cout<<"Closure ( ";
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		(*Iter)->Print(P);
		list<GenericCFGNode*>::iterator tempIter =Iter;
		tempIter++;
		if(tempIter!=Children.end())
			cout<<" , ";
	}
	cout<<" ) ";
}
void OptionNode::Print(ParserGenerator* P)
{
	cout<<"Option ( ";
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		(*Iter)->Print(P);
		list<GenericCFGNode*>::iterator tempIter =Iter;
		tempIter++;
		if(tempIter!=Children.end())
			cout<<" , ";
	}
	cout<<" ) ";
}
void TerminalNode::Print(ParserGenerator* P)
{
	for(map<tstring,TerminalEntry>::iterator iter=P->Terminals.begin();iter!=P->Terminals.end();iter++)
	{
		if((*iter).second.NameIndex==NameIndex)
		{
			/*firstset = *(*iter).second.FirstSet;
			break;*/
			cout<<'\''<<(*iter).first.c_str()<<'\'';
		}
	}
}
void NonTerminalNode::Print(ParserGenerator* P)
{
	for(map<tstring,NonTerminalEntry>::iterator iter=P->NonTerminals.begin();iter!=P->NonTerminals.end();iter++)
	{
		if((*iter).second.NameIndex==NameIndex)
		{
			/*firstset = *(*iter).second.FirstSet;
			break;*/
			cout<<(*iter).first.c_str();
		}
	}

}
void SyncNode::Print(ParserGenerator* P)
{
	cout<<"Sync";
}
void SemActionNode::Print(ParserGenerator* P)
{
	cout<<this->SemAction.c_str();
}
void AnyNode::Print(ParserGenerator* P)
{
	cout<<"ANY";
}
/*
list<int>* AndedNode::GetIndeces()
{
	list<int> l;
	return l;

}*/
list<int>*GenericCFGNode::GetIndeces(ParserGenerator*P,tstring NTName,int* Fn_count)
{
	list<int>* l;
	return l;
}
list<int>* TerminalNode::GetIndeces(ParserGenerator*P,tstring NTName,int* Fn_count)
{
	list<int>* l= new list<int>;
	if(NameIndex!=0)
	{
		l->push_back(NameIndex);
	}
	return l;
}
list<int>* NonTerminalNode::GetIndeces(ParserGenerator*P,tstring NTName,int* Fn_count)
{
	list<int>* l= new list<int>;
	l->push_back(NameIndex);
	return l;
}
list<int>* AndedNode::GetIndeces(ParserGenerator*P,tstring NTName,int* Fn_count)
{
	list<int>* l= new list<int>;
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		if((*Iter)->NType==_Terminal)
		{
			if(((TerminalNode*)*Iter)->NameIndex!=0)
				l->push_back(((TerminalNode*)*Iter)->NameIndex);
		}
		else if((*Iter)->NType==_NonTerminal)
			l->push_back(((NonTerminalNode*)*Iter)->NameIndex);
		else
			if((*Iter)->NType==_SemAction)
			{
				unsigned int Ind_Debug = ((SemActionNode*)*Iter)->NameIndex;
				//const char* Debug_Code=P->Codes[Ind_Debug].Code.c_str();
				l->push_back(((SemActionNode*)*Iter)->NameIndex);
					//Generate FuctionName
				if(P->Codes.find(((SemActionNode*)*Iter)->NameIndex)==P->Codes.end())
				{
					tstring fn_Name=NTName+'_';
					if(*(Fn_count)==0)
					{
						fn_Name+='0';
					}
					else
					{
						tstring temp="";
						unsigned int x = *(Fn_count);
						while(x!=0)
						{
							int i=x%10+(int)'0';
							char c = (char)i;
							temp=c+temp;
							x=x/10;
						}
						fn_Name+=temp;
					}
	
					//FunctionNameGenerated
					CodeEntry CE(((SemActionNode*)*Iter)->SemAction,fn_Name);
					P->Codes[((SemActionNode*)*Iter)->NameIndex]=CE;
					*Fn_count=*Fn_count+1;
				}
				//cout<<"hello";
			}
			
	}
	return l;
}
list<int>* SemActionNode::GetIndeces(ParserGenerator*P,tstring NTName,int* Fn_count)
{
	list<int>* l= new list<int>;
	l->push_back(NameIndex);
	if(P->Codes.find(NameIndex)==P->Codes.end())
	{
	//Generate FuctionName
	tstring fn_Name=NTName+'_';
	if(*(Fn_count)==0)
	{
		fn_Name+='0';
	}
	else
	{
		tstring temp="";
		int x = *(Fn_count);
		while(x!=0)
		{
			int i=x%10+(int)'0';
			char c = (char)i;
			temp=c+temp;
			x=x/10;
		}
		fn_Name+=temp;
	}

	//FunctionNameGenerated
	CodeEntry CE(this->SemAction,fn_Name);
	P->Codes[NameIndex]=CE;
	*Fn_count=*Fn_count+1;
	}

	return l;
}
tstring OredNode::check_LL1(ParserGenerator* P)
{
	tstring s="";
	
	list<GenericCFGNode*>::iterator Iter= Children.begin();
	BitSet_Min FS = (*Iter)->FirstSet(P);
	Iter++;
	for(;Iter!=Children.end();Iter++)
	{
		FS.SetIntersection((*Iter)->FirstSet(P));	
	}
	FS.Remove(0);
	if(!FS.IsEmpty())
	{
		s+="several alternatives start with ";
		int cursor = FS.NextMemberFrom(0);
		s+=P->GetTerminalName(TerminalBase+cursor);
        while((cursor=FS.NextMemberFrom(cursor+1))!=-1)
		{
			s+=",";
			s+=P->GetTerminalName(TerminalBase+cursor);
		}
		
	}
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		tstring s1 =(*Iter)->check_LL1(P);
		if(s1!="")
		{
			if(s!="")
				s+=" ; ";
			s+=s1;
		}
	}

	return s;

}
tstring AndedNode::check_LL1(ParserGenerator* P)
{
	tstring s="";
	BitSet_Min BS(P->nextTerminalIndex);
	//BitSet FS2(P->nextTerminalIndex);
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		if((*Iter)->FirstSet(P).Contains(0))
		{
			BitSet_Min  FS = (*Iter)->FirstSet(P);
			list<GenericCFGNode*>::iterator Iter1=Iter;
			Iter1++;
			
			while(Iter1!=Children.end()&&(*Iter1)->FirstSet(P).Contains(0))
			{
				BitSet_Min  FS1 = (*Iter1)->FirstSet(P);
				FS1.SetIntersection(FS);
				BS.SetUnion(FS1);
				Iter1++;
			}
			if(Iter1!=Children.end())
			{
				BitSet_Min  FS1 = (*Iter1)->FirstSet(P);
				FS1.SetIntersection(FS);
				BS.SetUnion(FS1);				
			}
		}
		
	}
	BS.Remove(0);
	if(!BS.IsEmpty())
	{
		int cursor = BS.NextMemberFrom(0);
		s+="several alternatives start with ";
		s+=P->GetTerminalName(TerminalBase+cursor);
		while((cursor=BS.NextMemberFrom(cursor+1))!=-1)
		{
			s+=",";
			s+=P->GetTerminalName(TerminalBase+cursor);
		}
	}
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		s+=(*Iter)->check_LL1(P);
	}
	return s;

}
tstring OptionNode::check_LL1(ParserGenerator *P)
{
	tstring s="";
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		if((*Iter)->FirstSet(P).Contains(0))
		{
            s+="contents of [...] must not be deletable"; 
		}
	} 

	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		s+=(*Iter)->check_LL1(P);
	}
	return s;
	
}
tstring ClosureNode::check_LL1(ParserGenerator *P)
{
	tstring s="";
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		if((*Iter)->FirstSet(P).Contains(0))
		{
			s+="contents of {...} must not be deletable"; 
		}
	} 

	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		s+=(*Iter)->check_LL1(P);
	}
	return s;
}
tstring TerminalNode::check_LL1(ParserGenerator * P)
{
	return "";
}
tstring NonTerminalNode::check_LL1(ParserGenerator * P)
{
	return "";
}
tstring SemActionNode::check_LL1(ParserGenerator *P)
{
	return "";
}
tstring SyncNode::check_LL1(ParserGenerator*P)
{
	return "";
}
tstring AnyNode::check_LL1(ParserGenerator*P)
{
	return "";
}
bool OredNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	bool LeadToTerminal=false;
	for(list<GenericCFGNode*>::iterator ChIter = Children.begin();ChIter!=Children.end();ChIter++)
	{
		LeadToTerminal |=(*ChIter)->LeadToTerminal(P,LHSs);
	}
	return LeadToTerminal;
}
bool AndedNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	bool LeadToTerminal=true;
	for(list<GenericCFGNode*>::iterator ChIter = Children.begin();ChIter!=Children.end();ChIter++)
	{
		LeadToTerminal &=(*ChIter)->LeadToTerminal(P,LHSs);
	}
	return LeadToTerminal;
}
bool TerminalNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	return true;
}
bool NonTerminalNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	for(list<unsigned int>::iterator Liter = LHSs.begin();Liter!=LHSs.end();Liter++)
	{
		if( NameIndex == (*Liter))
		{
			return false;
		}
	}
	for(map<tstring,NonTerminalEntry>::iterator NTIter=P->NonTerminals.begin();NTIter!=P->NonTerminals.end();NTIter++)
	{
		if((*NTIter).second.NameIndex==NameIndex)
		{
			if((*NTIter).second.Rule!=NULL)
			{
				LHSs.push_back(NameIndex);
				return (*NTIter).second.Rule->RHS->LeadToTerminal(P,LHSs);
			}
			else
				return false;
			break;
		}
	}
	return false;
}
bool SyncNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	return true;
}
bool AnyNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	return true;
}
bool SemActionNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	return true;
}
bool ClosureNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	return true;
}
bool OptionNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	return true;
}