#include "stdafx.h"
#include "JavaGenerator.h"
#include "ProductionRule.h"

using namespace RDParserGenerator;

JavaGenerator::JavaGenerator( char* fileName )
{
	DefinitionFile.open( fileName );
}

void JavaGenerator::GenerateCode( ParserGenerator* parser )
{
	if ( !DefinitionFile.is_open() )
		DefinitionFile.open( parser->StrClassName.c_str() );

	Parser = parser;

	DefinitionFile << parser->StrDecTop << endl << endl;
	DefinitionFile << "package " << parser->StrNameSpace << ';' << endl;
	DefinitionFile << "public class " << parser->StrClassName << endl  << '{' << endl; 
	DefinitionFile << parser->StrDefTop << endl;

    //writing the constructor
    DefinitionFile << '\t'
		<< "public " << parser->StrClassName << "( IScanner lexicalAnalyzer )\n\t{\n\t\tscanner = lexicalAnalyzer; \n\t}\n" << endl;

	//writing the Parse function
	DefinitionFile << "\n\t"
		<< "public void Parse() " << "\n\t{\n\t\tcurrentToken = scanner.GetNextToken();\n\t\tInitializeParser();\n";
	this->generateLL1Controller();	
	DefinitionFile << "\n\t}" << endl; //end the parse function

	this->initializeParser(); //write the _fillL1Table()

	//write the SyntaxError function
	DefinitionFile << '\t' 
		<< "protected void SyntaxError( Token errorAtToken )\n\t{\n\t\t // Write your own error handling here \n\t}\n" << endl;

	DefinitionFile << '\t' << "protected boolean IsTerminal(int id)\n\t{\n\t\treturn ( id >= _TerminalBase && id < _NonTerminalBase );\n\t}\n";
	
	DefinitionFile << '\t' << "protected boolean IsNonTerminal(int id)\n\t{\n\t\treturn ( id >= _NonTerminalBase && id < _CodeBase );\n\t}\n";

	DefinitionFile << '\t' << "protected boolean IsCode(int id)\n\t{\n\t\treturn ( id >= _CodeBase );\n\t}\n";

	//writing the data members
	DefinitionFile << '\t' << "Token currentToken;" << endl
				   << '\t' << "IScanner scanner;" << endl;
	DefinitionFile << "\tfinal int _TerminalBase = " << TerminalBase << ";\n"; 
	DefinitionFile << "\tfinal int _NonTerminalBase = " << NonTerminalBase << ";\n";
	DefinitionFile << "\tfinal int _CodeBase = " << CodeBase << ";\n";
	//DefinitionFile << "\t\tdelegate void VoidFunction();\n";
	//DefinitionFile << "\t\tVoidFunction[] voidFunctions;\n";
	DefinitionFile << '\t' << "int LL1Table[][][] = new int[" << parser->nextNonTerminalIndex-NonTerminalBase 
		           << "][" << parser->nextTerminalIndex-TerminalBase-1 << "][];" << endl;
	DefinitionFile << "\t" << "java.util.Stack stack = new java.util.Stack();" << endl;

	//write the  enumerator as a set of integers
	//write the enum members
	int enumMember = 0;
	for( map<unsigned int, tstring>::iterator iter = parser->TokensNames.begin(); iter != parser->TokensNames.end(); iter++)
	{
		DefinitionFile << "\tpublic final int " << iter->second << " = " << enumMember++ << ";\n";
	}

	this->generateUserFunctions();
	this->writeCallFunctionNumber();

	DefinitionFile << parser->StrDefBottom << endl;

	DefinitionFile << "}" << endl << endl; //end class

	//write the IScanner interface and the Token class
	DefinitionFile << "interface IScanner\n{\n\tToken GetNextToken();\n}" << endl;
	DefinitionFile << "class Token\n{\n\tpublic int TokenType;\n}" << endl;

	DefinitionFile << parser->StrDecBottom << endl;

	DefinitionFile.close();
}

void JavaGenerator::generateLL1Controller()
{
	DefinitionFile << "\t\t" << "stack.push( new Integer( _EOF ) );\n";
	DefinitionFile << "\t\t" << "stack.push( new Integer( _NonTerminalBase ) );\n";
	DefinitionFile << "\t\t" << "while ( ((Integer)stack.peek()).intValue() != _EOF && ( (int)currentToken.TokenType != _EOF)  )\n\t\t{\n\t\t\t"
		           << "int top = ((Integer)stack.peek()).intValue();" << "\n\t\t\t"
				   << "if ( IsTerminal( top ) && top == (int)currentToken.TokenType )" << "\n\t\t\t{\n\t\t\t\t"
				   << "stack.pop();" << "\n\t\t\t\t"
				   << "currentToken = scanner.GetNextToken();" << "\n\t\t\t}\n\t\t\t"
				   << "else if ( IsNonTerminal( top ) )" << "\n\t\t\t{\n\t\t\t\t"
				   << "int production[] = LL1Table[top - _NonTerminalBase][(int)currentToken.TokenType - _TerminalBase];" << "\n\t\t\t\t"
				   << "if ( production == null )" << "\n\t\t\t\t\t" 
	               << "SyntaxError( currentToken );" << "\n\t\t\t\t"
				   << "else //generate" << "\n\t\t\t\t{\n\t\t\t\t\t"
				   << "stack.pop();" << "\n\t\t\t\t\t" 
				   << "for(int i = production.length-1; i >= 0; i--)" << "\n\t\t\t\t\t\t"
				   << "stack.push( new Integer( production[i] ) );" << "\n\t\t\t\t}" << "\n\t\t\t}\n\t\t\t" 
				   << "else if ( IsCode( top ) )" << "\n\t\t\t{\n\t\t\t\t"
				   << "stack.pop();" << "\n\t\t\t\t"
				   << "callFunctionNumber(top - _CodeBase);" << "\n\t\t\t}"
				   << "\n\t\t}\n\t\t" //end while
				   << "if ( !( ((Integer)stack.peek()).intValue() == _EOF && currentToken.TokenType == _EOF ) )" << "\n\t\t\t"
				   << "SyntaxError( currentToken );" << endl;
}

void JavaGenerator::initializeParser()
{
	DefinitionFile << "\n\tvoid InitializeParser()\n\t{\n";
	DefinitionFile << "\t\t" << "int emptyArr[] = {};\n";
	int rowsNo = Parser->nextNonTerminalIndex - NonTerminalBase;
	int colsNo = Parser->nextTerminalIndex - TerminalBase - 1;
	DefinitionFile << "\t\t//Initialize the LL1 table\n";
	for(int i = 0; i < rowsNo; i++)
	{
		for(int j = 0; j < colsNo; j++)
		{
			list<int>* lis = Parser->LL1Table[i][j];
			if ( lis == 0 )
				DefinitionFile << "\t\tLL1Table[" << i << "][" << j << "] = null;\n";
			else if ( lis->size() == 1 && *(lis->begin()) == 0 )
				DefinitionFile << "\t\tLL1Table[" << i << "][" << j << "] = emptyArr;\n";
			else
			{
				DefinitionFile << "\t\tint temp" << i << j << "[] = {";
				for( list<int>::iterator iter = lis->begin(); iter != lis->end() ; )
				{
					DefinitionFile << *iter;
					iter++;
					if ( iter != lis->end() )
						DefinitionFile << ", ";
				}
				DefinitionFile << "};\n";
				DefinitionFile << "\t\tLL1Table[" << i << "][" << j << "] = temp" << i << j << ";\n";
			}
		}
	}

	DefinitionFile << "\n\t}\n";
}

void JavaGenerator::writeCallFunctionNumber()
{
	DefinitionFile << "\n\tprotected void callFunctionNumber( int functionID )\n\t{\n"
		           << "\t\tswitch( functionID )\n\t\t{\n";
	int i = 0;
	for( map<unsigned int,CodeEntry>::iterator iter = Parser->Codes.begin(); iter != Parser->Codes.end(); iter++ )
	{
		DefinitionFile << "\t\t\tcase " << i++ << ":\n\t\t\t\t" << iter->second.FunctionName << "();\n"
		               << "\t\t\t\tbreak;\n";
	}
	DefinitionFile << "\t\t}\n"; //close the switch statement
	DefinitionFile << "\t}\n"; //close the function

}

void JavaGenerator::generateUserFunctions()
{
	DefinitionFile << "\n\t//Grammer embedded code\n";

	for( map<unsigned int,CodeEntry>::iterator iter = Parser->Codes.begin(); iter != Parser->Codes.end(); iter++ )
	{
		DefinitionFile << "\n\tprotected void " << iter->second.FunctionName << "()\n\t{\n\t\t"
			           << iter->second.Code << "\n\t}\n";
	}
}