

namespace MyLangCompiler
{



/**/
//

	public class Parser
	{

		public Parser( IScanner lexicalAnalyzer )
		{
			scanner = lexicalAnalyzer; 
		} 


		public void Parse() 
		{
			currentToken = scanner.GetNextToken();
			InitializeParser();
			stack.Push(_TokenType._EOF);
			stack.Push( _NonTerminalBase );
			while ( ((int)stack.Peek() != (int)_TokenType._EOF) && (currentToken.TokenType !=  _TokenType._EOF) )
			{
				int top = (int)stack.Peek();
				if ( IsTerminal( top ) && top == (int)currentToken.TokenType )
				{
					stack.Pop();
					currentToken = scanner.GetNextToken();
				}
				else if ( IsNonTerminal( top ) )
				{
					int[] production = LL1Table[top - _NonTerminalBase, (int)currentToken.TokenType - _TerminalBase];
					if ( production == null )
						SyntaxError( currentToken );
					else //generate
					{
						stack.Pop();
						for(int i = production.Length-1; i >= 0; i--)
							stack.Push( production[i] );
					}
				}
				else if ( IsCode( top ) )
				{
					stack.Pop();
					voidFunctions[top - _CodeBase]();
				}
			}
			if ( !( (int)stack.Peek() == (int)_TokenType._EOF && currentToken.TokenType == _TokenType._EOF ) )
				SyntaxError( currentToken );

		}

		void InitializeParser()
		{
			//Initialize the LL1 table
			LL1Table[0, 0] = null;
			LL1Table[0, 1] = new int[] {32001, 32002, 5};
			LL1Table[0, 2] = new int[] {32001, 32002, 5};
			LL1Table[0, 3] = null;
			LL1Table[0, 4] = null;
			LL1Table[0, 5] = null;
			LL1Table[1, 0] = null;
			LL1Table[1, 1] = new int[] {1};
			LL1Table[1, 2] = new int[] {64004, 2, 64005};
			LL1Table[1, 3] = null;
			LL1Table[1, 4] = null;
			LL1Table[1, 5] = null;
			LL1Table[2, 0] = null;
			LL1Table[2, 1] = null;
			LL1Table[2, 2] = null;
			LL1Table[2, 3] = new int[] {64000, 3, 64001, 32003};
			LL1Table[2, 4] = null;
			LL1Table[2, 5] = null;
			LL1Table[3, 0] = null;
			LL1Table[3, 1] = null;
			LL1Table[3, 2] = null;
			LL1Table[3, 3] = null;
			LL1Table[3, 4] = new int[] {64002, 4, 64003, 3, 32003};
			LL1Table[3, 5] = new int[] {};

			//Initialize the delegates
			voidFunctions = new VoidFunction[6];

			voidFunctions[0] = new VoidFunction(VarList_0);
			voidFunctions[1] = new VoidFunction(VarList_1);
			voidFunctions[2] = new VoidFunction(Var_0);
			voidFunctions[3] = new VoidFunction(Var_1);
			voidFunctions[4] = new VoidFunction(DataType_0);
			voidFunctions[5] = new VoidFunction(DataType_1);
		}
		protected void SyntaxError( Token errorAtToken )
		{
			 // Write your own error handling here 
		}

		protected bool IsTerminal(int id)
		{
			return ( id >= _TerminalBase && id < _NonTerminalBase );
		}
		protected bool IsNonTerminal(int id)
		{
			return ( id >= _NonTerminalBase && id < _CodeBase );
		}
		protected bool IsCode(int id)
		{
			return ( id >= _CodeBase );
		}
		Token currentToken;
		IScanner scanner;
		const int _TerminalBase = 0;
		const int _NonTerminalBase = 32000;
		const int _CodeBase = 64000;
		delegate void VoidFunction();
		VoidFunction[] voidFunctions;
		int[,][] LL1Table = new int[4, 6][];
		System.Collections.Stack stack = new System.Collections.Stack();

		//Grammer embedded code

		protected void VarList_0()
		{
			 MessageBox.Show("ident will detected"); 
		}

		protected void VarList_1()
		{
			 MessageBox.Show("ident is detected"); 
		}

		protected void Var_0()
		{
			 MessageBox.Show("Comma will be detected"); 
		}

		protected void Var_1()
		{
			 MessageBox.Show("comma is detected"); 
		}

		protected void DataType_0()
		{
			 MessageBox.Show("Float will be detected"); 
		}

		protected void DataType_1()
		{
			 MessageBox.Show("Float is detected"); 
		}
	}

	public enum _TokenType
	{
		_EOF, _int, _float, _identifier, _comma, _real
	}
	public interface IScanner
	{
		Token GetNextToken();
	}
	public class Token
	{
		public _TokenType TokenType;
	}

}
