



namespace AMAP
{


	public class Parser
	{


		public Parser( IScanner lexicalAnalyzer )
		{
			scanner = lexicalAnalyzer; 
		} 


		public void Parse() 
		{
			currentToken = scanner.GetNextToken();
			InitializeParser();
			stack.Push( _NonTerminalBase );
			while ( (int)stack.Peek() != (int)_TokenType._EOF )
			{
				int top = (int)stack.Peek();
				if ( IsTerminal( top ) && top == (int)currentToken.TokenType )
				{
					stack.Pop();
					currentToken = scanner.GetNextToken();
				}
				else if ( IsNonTerminal( top ) )
				{
					int[] production = LL1Table[top - _NonTerminalBase, (int)currentToken.TokenType - _TerminalBase];
					if ( production == null )
						SyntaxError( currentToken );
					else //generate
					{
						stack.Pop();
						for(int i = production.Length-1; i >= 0; i--)
							stack.Push( production[i] );
					}
				}
				else if ( IsCode( top ) )
				{
					stack.Pop();
					voidFunctions[top - _CodeBase]();
				}
			}
			if ( !( (int)stack.Peek() == (int)_TokenType._EOF && currentToken.TokenType == (int)_TokenType._EOF ) )
				SyntaxError( currentToken );

		}

		void InitializeParser()
		{
			//Initialize the LL1 table
			LL1Table[0, 0] = null;
			LL1Table[0, 1] = new int[] {1, 32001, 2};
			LL1Table[0, 2] = null;
			LL1Table[0, 3] = null;
			LL1Table[0, 4] = null;
			LL1Table[0, 5] = null;
			LL1Table[0, 6] = null;
			LL1Table[0, 7] = null;
			LL1Table[0, 8] = null;
			LL1Table[0, 9] = null;
			LL1Table[0, 10] = null;
			LL1Table[1, 0] = null;
			LL1Table[1, 1] = null;
			LL1Table[1, 2] = null;
			LL1Table[1, 3] = new int[] {32002};
			LL1Table[1, 4] = new int[] {32002};
			LL1Table[1, 5] = new int[] {32002};
			LL1Table[1, 6] = new int[] {32002};
			LL1Table[1, 7] = new int[] {32002};
			LL1Table[1, 8] = null;
			LL1Table[1, 9] = null;
			LL1Table[1, 10] = null;
			LL1Table[2, 0] = null;
			LL1Table[2, 1] = null;
			LL1Table[2, 2] = null;
			LL1Table[2, 3] = new int[] {32003};
			LL1Table[2, 4] = new int[] {32003};
			LL1Table[2, 5] = new int[] {32003};
			LL1Table[2, 6] = new int[] {32003};
			LL1Table[2, 7] = new int[] {32003};
			LL1Table[2, 8] = null;
			LL1Table[2, 9] = null;
			LL1Table[2, 10] = null;
			LL1Table[3, 0] = null;
			LL1Table[3, 1] = null;
			LL1Table[3, 2] = null;
			LL1Table[3, 3] = new int[] {32004, 32005};
			LL1Table[3, 4] = new int[] {32004, 32005};
			LL1Table[3, 5] = new int[] {32004, 32005};
			LL1Table[3, 6] = new int[] {32004, 32005};
			LL1Table[3, 7] = new int[] {32004, 32005};
			LL1Table[3, 8] = null;
			LL1Table[3, 9] = null;
			LL1Table[3, 10] = null;
			LL1Table[4, 0] = null;
			LL1Table[4, 1] = null;
			LL1Table[4, 2] = null;
			LL1Table[4, 3] = new int[] {3, 9, 8, 10};
			LL1Table[4, 4] = new int[] {4, 9, 8, 8, 10};
			LL1Table[4, 5] = new int[] {5};
			LL1Table[4, 6] = new int[] {6};
			LL1Table[4, 7] = new int[] {7};
			LL1Table[4, 8] = null;
			LL1Table[4, 9] = null;
			LL1Table[4, 10] = null;
			LL1Table[5, 0] = null;
			LL1Table[5, 1] = null;
			LL1Table[5, 2] = new int[] {};
			LL1Table[5, 3] = new int[] {32004, 32005};
			LL1Table[5, 4] = new int[] {32004, 32005};
			LL1Table[5, 5] = new int[] {32004, 32005};
			LL1Table[5, 6] = new int[] {32004, 32005};
			LL1Table[5, 7] = new int[] {32004, 32005};
			LL1Table[5, 8] = null;
			LL1Table[5, 9] = null;
			LL1Table[5, 10] = null;

			//Initialize the delegates
			voidFunctions = new VoidFunction[0];

		}
		protected void SyntaxError( Token errorAtToken )
		{
			 // Write your own error handling here 
		}

		protected bool IsTerminal(int id)
		{
			return ( id >= _TerminalBase && id < _NonTerminalBase );
		}
		protected bool IsNonTerminal(int id)
		{
			return ( id >= _NonTerminalBase && id < _CodeBase );
		}
		protected bool IsCode(int id)
		{
			return ( id >= _CodeBase );
		}
		Token currentToken;
		IScanner scanner;
		const int _TerminalBase = 0;
		const int _NonTerminalBase = 32000;
		const int _CodeBase = 64000;
		delegate void VoidFunction();
		VoidFunction[] voidFunctions;
		int[,][] LL1Table = new int[6, 11][];
		System.Collections.Stack stack = new System.Collections.Stack();

		//Grammer embedded code
	}

	public enum _TokenType
	{
		_EOF, _start, _finish, _typeface, _slice, _learn, _get, _recognize, _digit, _openBracket, _closeBracket
	}
	public interface IScanner
	{
		Token GetNextToken();
	}
	public class Token
	{
		public int TokenType;
	}

}

