#include "stdafx.h"
#include "CSharpGenerator.h"
#include "ProductionRule.h"

using namespace RDParserGenerator;

CSharpGenerator::CSharpGenerator( char* fileName )
{
	DefinitionFile.open( fileName );
}

void CSharpGenerator::GenerateCode( ParserGenerator* parser )
{
	if ( !DefinitionFile.is_open() )
		DefinitionFile.open( parser->StrClassName.c_str() );

	Parser = parser;
	tstring startSymbol;

	DefinitionFile << parser->StrDecTop << endl << endl;
	DefinitionFile << "namespace " << parser->StrNameSpace << endl << '{' << endl;
	
	DefinitionFile << '\t' << "public class " << parser->StrClassName << endl << '\t' << '{' << endl; 
	
	

    //writing the constructor
    DefinitionFile << '\t' << '\t' 
		<< "public " << parser->StrClassName << "( IScanner lexicalAnalyzer )\n\t\t{\n\t\t\tscanner = lexicalAnalyzer; \n\t\t} \n" << endl;

	ProductionRule* firstProd = parser->Grammer.front();
	startSymbol = parser->GetNonTerminalName( firstProd->LHS ) + _T("(") + firstProd->Attributes + _T(")");
	for( list<ProductionRule*>::iterator iter = parser->Grammer.begin(); iter != parser->Grammer.end(); iter++)
	{
		//note *iter is ProductionRule*
		DefinitionFile << "\t\tpublic void " << parser->GetNonTerminalName( (*iter)->LHS )
			           << '(' << (*iter)->Attributes << ")\n\t\t{\n";

		(*iter)->RHS->GenerateCode( this );
		
		DefinitionFile << "\t\t}" << endl;
	}

	//writing the Parse function
	DefinitionFile << "\n\t\t"
		<< "public void Parse() " << "\n\t\t{\n\t\t\tcurrentToken = scanner.GetNextToken();\n\t\t\t" << startSymbol << ";\n"
		<< "\t\t\tif ( currentToken.TokenType != _TokenType._EOF )\n"
		<< "\t\t\t\tSyntaxError(currentToken);"<< "\n\t\t}" << endl;	


	//writing the match function
	DefinitionFile << '\t' << '\t' 
		<< "protected void match( _TokenType expected )\n\t\t{\n\t\t\tif ( currentToken.TokenType == expected )\n\t\t\t\tcurrentToken = scanner.GetNextToken();\n\t\t\telse\n\t\t\t\tErrorHandler( expected, currentToken );\n\t\t}\n" << endl;
	//writing the ErrorHandler code
	DefinitionFile << '\t' << '\t' 
		<< "protected void ErrorHandler( _TokenType expected, Token found )\n\t\t{\n\t\t\t // Write your own error handling here \n\t\t}\n" << endl;

	//write the SyntaxError function
	DefinitionFile << '\t' << '\t' 
		<< "protected void SyntaxError( Token errorAtToken )\n\t\t{\n\t\t\t // Write your own error handling here \n\t\t}\n" << endl;

	//writing the data members
	DefinitionFile << '\t' << '\t' << "Token currentToken;" << endl
				   << '\t' << '\t' << "IScanner scanner;" << endl;

	DefinitionFile << parser->StrDefBottom << endl;

	DefinitionFile << "\t}" << endl << endl; //end class

	DefinitionFile << parser->StrDefTop << endl;
    
	//write the _TokenType enumerator
	DefinitionFile << "\tpublic enum _TokenType\n\t{\n\t\t";
	 //write the enum members
	for( map<unsigned int, tstring>::iterator iter = parser->TokensNames.begin(); iter != parser->TokensNames.end(); )
	{
		DefinitionFile << iter->second;
		iter++; // I have moved it here due to the next test	
		if ( iter != parser->TokensNames.end() )
			DefinitionFile << ", ";
	}

	 //close the enum
	DefinitionFile << endl << "\t}" << endl;

	//write the IScanner interface and the Token class
	DefinitionFile << '\t' << "public interface IScanner\n\t{\n\t\tToken GetNextToken();\n\t}" << endl;
	DefinitionFile << '\t' << "public class Token\n\t{\n\t\tpublic _TokenType TokenType;\n\t\tpublic string Lexeme;\n\n\t\tpublic Token(){}\n\t\tpublic Token( _TokenType type, string lexeme)\n\t\t{\n\t\t\tthis.TokenType = type;\n\t\t\tthis.Lexeme = lexeme;\n\t\t}\n\t}" << endl;

	DefinitionFile << endl << '}'; //end namespace

	DefinitionFile << endl << parser->StrDecBottom ;

	DefinitionFile.close();
}

void CSharpGenerator::AndGenerator( AndedNode* node )
{
	if( node->ResolverString.length() > 0 )
		DefinitionFile << "if ( " << node->ResolverString << " )\n\t{\n";
	for( list<GenericCFGNode*>::iterator iter = node->Children.begin(); iter != node->Children.end(); iter++)
	{
		(*iter)->GenerateCode(this);
	}
	if( node->ResolverString.length() > 0 )
		DefinitionFile << "\t}\n";
}

void CSharpGenerator::ClosureGenerator( ClosureNode* node )
{
	DefinitionFile << "\t\t\twhile (";
	BitSet_Min firstSet = node->FirstSet( Parser );
	int index = 0; //we begin from zero because we need to skip Eps if found
	
	while ( (index = firstSet.NextMemberFrom( index+1 )) != -1 )
	{
		DefinitionFile << " currentToken.TokenType == _TokenType." << Parser->TokensNames[index];
		if ( firstSet.NextMemberFrom( index+1 ) != -1 )
			DefinitionFile << " ||";
	}
	DefinitionFile << " )\n\t\t\t{\n\t\t\t\t";
	for( list<GenericCFGNode*>::iterator iter = node->Children.begin(); iter != node->Children.end(); iter++)
	{
		(*iter)->GenerateCode(this);
	}
	DefinitionFile << "\t\t\t}\n";
}

void CSharpGenerator::NonTerminalGenerator( NonTerminalNode* node )
{
	DefinitionFile << "\t\t\t" << Parser->GetNonTerminalName( node->NameIndex ) <<  '(' << node->Attributes << ");" << endl ;
}

void CSharpGenerator::OptionalGenerator( OptionNode* node )
{
	DefinitionFile << "\t\t\tif (";
	BitSet_Min firstSet = node->FirstSet( Parser );
	int index = 0; //we begin from zero because we need to skip Eps if found
	
	while ( (index = firstSet.NextMemberFrom( index+1 )) != -1 )
	{
		DefinitionFile << " currentToken.TokenType == _TokenType." << Parser->TokensNames[index];
		if ( firstSet.NextMemberFrom( index+1 ) != -1 )
			DefinitionFile << " ||";
	}
	DefinitionFile << " )\n\t\t\t{\n";
	for( list<GenericCFGNode*>::iterator iter = node->Children.begin(); iter != node->Children.end(); iter++)
	{
		(*iter)->GenerateCode(this);
	}
	DefinitionFile << "\t\t\t}\n";
}

void CSharpGenerator::OrGenerator( OredNode* node )
{
	BitSet_Min firstSet = node->FirstSet( Parser );
	bool containsEps = ( firstSet.NextMemberFrom(0) == 0 )? 1 : 0;
	int index;
	bool firstIf = 1;
	for( list<GenericCFGNode*>::iterator iter = node->Children.begin(); iter != node->Children.end();iter++)
	{
		firstSet = (*iter)->FirstSet( Parser );
		if ( firstSet.NextMemberFrom(1) == -1 )
			continue;
		if ( firstIf )
		{
			DefinitionFile << "\t\t\tif (";
			firstIf = 0;
		}
		else
			DefinitionFile << "\t\t\telse if (";

		index = 0;
		while ( (index = firstSet.NextMemberFrom( index+1 )) != -1 )
		{
			DefinitionFile << " currentToken.TokenType == _TokenType." << Parser->TokensNames[index];
			if ( firstSet.NextMemberFrom( index+1 ) != -1 )
				DefinitionFile << " ||";
		}
	    DefinitionFile << " )\n\t\t\t{\n";
		(*iter)->GenerateCode(this);
		DefinitionFile << "\t\t\t}\n";
	}
	if ( !containsEps ) //so it is not optional
		DefinitionFile << "\t\t\telse\n\t\t\t\tSyntaxError(currentToken);\n";
}

void CSharpGenerator::SemActionGenerator( SemActionNode* node )
{
	DefinitionFile << "\t\t\t" << node->SemAction << endl;
}

void CSharpGenerator::TerminalGenerator( TerminalNode* node )
{
	DefinitionFile << "\t\t\tmatch( _TokenType." <<  Parser->TokensNames[node->NameIndex-TerminalBase] << " );" << endl ;
}