#include "stdafx.h"
#include "GenericCFGNode.h"
#include "ProductionRule.h"
#include "ParserGen.h"
#include "ICodeGenerator.h"
using namespace RDParserGenerator;


//GenericCFGNode class
BitSet_Min GenericCFGNode::FirstSet(ParserGenerator* P)
{
	BitSet_Min BS(P->nextTerminalIndex);
	return BS;
}
bool GenericCFGNode::FollowSet(ParserGenerator* P)
{
	//Unreachable return
	return false;
}
BitSet_Min GenericCFGNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextTerminalIndex);
	return BS;
}

//OredNode class
OredNode::OredNode()
{
	this->NType = _Ored;
}
BitSet_Min OredNode::FirstSet(ParserGenerator* P)
{
	BitSet_Min BS(P->nextTerminalIndex); //the terminal size
	list<GenericCFGNode*>::iterator Ch_iter;
	for(Ch_iter=Children.begin();Ch_iter!=Children.end();Ch_iter++)
	{
		BitSet_Min CH_BS = (*Ch_iter)->FirstSet( P);
		BS.SetUnion(CH_BS);
	}
	return BS;
}
bool OredNode::FollowSet(ParserGenerator* P)
{
	bool Changed= false;
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		Changed|=(*Iter)->FollowSet(P);
	}
	//unreachable logically
	return Changed; 
}
BitSet_Min OredNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		BS.SetUnion((*Iter)->GetNonTerminals(P));
	}
	return BS;
}
void OredNode::GenerateCode ( ICodeGenerator* gen )
{
	gen->OrGenerator(this);
}

//ClosureNode class
ClosureNode::ClosureNode()
{
	this->NType = _Closure;
}
BitSet_Min ClosureNode::FirstSet(ParserGenerator* P)
{		
	BitSet_Min BS=(*Children.begin())->FirstSet( P);
	BS.Add(0);//for Epsilon
	return BS;
}
bool ClosureNode::FollowSet(ParserGenerator* P)
{
	//I think this is to be reformulated ya hamdy -- MSaber
	BitSet_Min NTs = this->GetNonTerminals(P);
	BitSet_Min FS = this->FirstSet(P);
	FS.Remove(0);
	int cursor=0;
	bool Changed = false;
	while((cursor = NTs.NextMemberFrom(cursor))!=-1)
	{
		int NIndex = cursor+NonTerminalBase;
		for(map<tstring,NonTerminalEntry>::iterator NTiter=P->NonTerminals.begin();NTiter!=P->NonTerminals.end();NTiter++)
		{
			if((*NTiter).second.NameIndex==NIndex)
			{
				BitSet_Min fs = *(*NTiter).second.FollowSet;
                (*NTiter).second.FollowSet->SetUnion(FS);
				if(!fs.Equals(*(*NTiter).second.FollowSet))
					Changed = true;
			
			}
		}
		cursor++;
	}
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		bool b=(*Iter)->FollowSet(P);
		return Changed|b;
	}
	//unreachable logically
	return false; 
}
BitSet_Min ClosureNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex);
	return (*this->Children.begin())->GetNonTerminals(P);
}
void ClosureNode::GenerateCode ( ICodeGenerator* gen )
{
	gen->ClosureGenerator( this );
}

//OptionNode class
OptionNode::OptionNode()
{
	this->NType = _Optional;
}
BitSet_Min OptionNode::FirstSet(ParserGenerator* P)
{		
	BitSet_Min BS=(*Children.begin())->FirstSet( P);
	BS.Add(0);//for Epsilon
	return BS;
}
bool OptionNode::FollowSet(ParserGenerator* P)
{
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		return (*Iter)->FollowSet(P);
	}
	//unreachable logically
	return false; 
}
BitSet_Min OptionNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex);
	return (*this->Children.begin())->GetNonTerminals(P);
}
void OptionNode::GenerateCode ( ICodeGenerator* gen )
{
	gen->OptionalGenerator( this );
}
/*
class BracketNode:public OredNode
{
public:	
	BracketNode()
	{
		this->NType = _Brackets;
	}
	virtual BitSet_Min FirstSet(ParserGenerator* P)
	{		
		BitSet_Min BS=(*Children.begin())->FirstSet( P);
		return BS;
	}
};*/

//AndedNode class
AndedNode::AndedNode()
{
	this->NType=_Anded;
}
BitSet_Min AndedNode::FirstSet(ParserGenerator* P)
{
	BitSet_Min BS(P->nextTerminalIndex); //the terminal size
	list<GenericCFGNode*>::iterator Ch_iter;
	for(Ch_iter=Children.begin();Ch_iter!=Children.end();Ch_iter++)
	{
		BitSet_Min CH_BS = (*Ch_iter)->FirstSet( P);
		BS.SetUnion(CH_BS);
		if(!CH_BS.Contains(0))
		{
			BS.Remove(0);
			break;
		}
	}
	return BS;
}
bool AndedNode::FollowSet(ParserGenerator* P)
{
	bool Changed= false;
		
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		Changed|=(*Iter)->FollowSet(P);
		BitSet_Min NTs = (*Iter)->GetNonTerminals(P);
		//bool EpsInAll= true;
		if(!NTs.IsEmpty())
		{
			BitSet_Min FirstSetOfNextTerms(P->nextTerminalIndex);
			list<GenericCFGNode*>::iterator ChIter= Iter;
			ChIter++;
			bool GetOut=false;
			for(;ChIter!=Children.end();ChIter++)
			{
				BitSet_Min fs = (*ChIter)->FirstSet(P);
				if(!fs.Contains(0))
				{
					GetOut=true;
				}
				fs.Remove(0);
				//BitSet_Min fSet = *(*NTiter).second.FollowSet;
				FirstSetOfNextTerms.SetUnion(fs);
				//if(fSet(*NTiter).second.FollowSet->IsSubsetOf())
				if(GetOut)
				{
	//				EpsInAll=false;
					break;
				}
			}
			int start=-1;
			while((start=NTs.NextMemberFrom(start+1))!=-1)
			{
                //BitSet_Min firstset;
				int NIndex = start+NonTerminalBase;
				for(map<tstring,NonTerminalEntry>::iterator NTiter=P->NonTerminals.begin();NTiter!=P->NonTerminals.end();NTiter++)
				{
					if((*NTiter).second.NameIndex==NIndex)
					{
						//Shakek feeha remember for debuging
						//bool GetOut=false;
						BitSet_Min fs = *(*NTiter).second.FollowSet;
                           (*NTiter).second.FollowSet->SetUnion(FirstSetOfNextTerms);
						if(!fs.Equals(*(*NTiter).second.FollowSet))
							Changed = true;
						/*if(EpsInAll)
						{
							//Make an action
						}*/
					}
				}
			}
		}
		//GetNonTerminals(Child)
		//Add to NonTerminals first of elli ba3diha
	}
	//unreachable logically
	return Changed; 
}
BitSet_Min AndedNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	/*list<GenericCFGNode*>::iterator Iter= Children.end();
	Iter--;
	for(;Iter!=Children.begin();Iter--)*/
	for(list<GenericCFGNode*>::reverse_iterator Iter= Children.rbegin();Iter!=Children.rend();Iter++)
	{
		BS.SetUnion((*Iter)->GetNonTerminals(P));
		BitSet_Min fs = (*Iter)->FirstSet(P); 
		if(!fs.Contains(0))
			break;
	}		
	return BS;
}
void AndedNode::GenerateCode ( ICodeGenerator* gen )
{
	gen->AndGenerator( this );
}


//SemActionNode class
SemActionNode::SemActionNode()
{
	this->NType = _SemAction;
}
BitSet_Min SemActionNode::FirstSet(ParserGenerator* P)
{
	BitSet_Min BS(P->nextTerminalIndex);//ter,minal size
	BS.Add(0);
	return BS;
}
BitSet_Min SemActionNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	return BS;
}

void SemActionNode::GenerateCode ( ICodeGenerator* gen )
{
	gen->SemActionGenerator( this );
}


//TerminalNode class
TerminalNode::TerminalNode()
{
	this->NType = _Terminal;		
}
BitSet_Min TerminalNode::FirstSet(ParserGenerator* P)
{
	//return its set
	BitSet_Min BS(P->nextTerminalIndex);
	BS.Add(NameIndex);
	return BS;
}
BitSet_Min TerminalNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	return BS;
}

void TerminalNode::GenerateCode ( ICodeGenerator* gen )
{
	gen->TerminalGenerator( this );
}


//NonTerminalNode class
NonTerminalNode::NonTerminalNode()
{
	this->NType = _NonTerminal;		
}
BitSet_Min NonTerminalNode::FirstSet(ParserGenerator* P)
{
	//return its set/
	//BitSet_Min BS(P->nextTerminalIndex); //remember for optimization
	for(map<tstring,NonTerminalEntry>::iterator iter=P->NonTerminals.begin();iter!=P->NonTerminals.end();iter++)
	{
		if((*iter).second.NameIndex==NameIndex)
		{
			/*firstset = *(*iter).second.FirstSet;
			break;*/
			return *(*iter).second.FirstSet;
		}
	}
	//return BS;
}
BitSet_Min NonTerminalNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	BS.Add(this->NameIndex-NonTerminalBase);
	return BS;
}

void NonTerminalNode::GenerateCode ( ICodeGenerator* gen )
{
	gen->NonTerminalGenerator( this );
}

//AnyNode class
AnyNode::AnyNode()
{
	this->NType = _AnyNode;
}
BitSet_Min AnyNode::FirstSet(ParserGenerator* P)
{
	BitSet_Min BS(P->nextTerminalIndex);
	BS.AddAllElements();
	//return its set
	return BS; //Shakek feeeeeha
}
BitSet_Min AnyNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	return BS;
}

void AnyNode::GenerateCode ( ICodeGenerator* gen )
{

}

//SyncNode class
SyncNode::SyncNode()
{
	this->NType = _SyncNode;
}
BitSet_Min SyncNode::FirstSet(ParserGenerator* P)
{
	//return its set
	BitSet_Min BS(P->nextTerminalIndex);//terminal size
	BS.Add(0);
	return BS;
}
BitSet_Min SyncNode::GetNonTerminals(ParserGenerator* P)
{
	BitSet_Min BS(P->nextNonTerminalIndex-NonTerminalBase);
	return BS;
}

void SyncNode::GenerateCode ( ICodeGenerator* gen )
{

}
void OredNode::Print(ParserGenerator* P)
{

}
void AndedNode::Print(ParserGenerator* P)
{

}
void ClosureNode::Print(ParserGenerator* P)
{
}
void OptionNode::Print(ParserGenerator* P)
{
}
void TerminalNode::Print(ParserGenerator* P)
{
}
void NonTerminalNode::Print(ParserGenerator* P)
{

}
void SyncNode::Print(ParserGenerator* P)
{
}
void SemActionNode::Print(ParserGenerator* P)
{
}
void AnyNode::Print(ParserGenerator* P)
{
}
tstring OredNode::check_LL1(ParserGenerator* P)
{
	tstring s="";
	BitSet_Min FS(P->nextTerminalIndex-TerminalBase);
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		BitSet_Min TempSet = (*Iter)->FirstSet(P);
		for(list<GenericCFGNode*>::iterator ChIter= Children.begin();ChIter!=Children.end();ChIter++)
		{
			BitSet_Min CHFS = (*ChIter)->FirstSet(P);
			CHFS.SetIntersection(TempSet);
			if(ChIter!=Iter)
			{				
				FS.SetUnion(CHFS);
			}
		}
	}
	/*list<GenericCFGNode*>::iterator Iter= Children.begin();
	BitSet_Min FS = (*Iter)->FirstSet(P);
	Iter++;
	for(;Iter!=Children.end();Iter++)
	{
		FS.SetIntersection((*Iter)->FirstSet(P));	
	}*/
	//FS.Remove(0)
	if(!FS.IsEmpty())
	{
		s+="several alternatives start with ";
		int cursor = FS.NextMemberFrom(0);
		s+=P->GetTerminalName(TerminalBase+cursor);
        while((cursor=FS.NextMemberFrom(cursor+1))!=-1)
		{
			s+=",";
			s+=P->GetTerminalName(TerminalBase+cursor);
		}
		
	}
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		tstring s1 =(*Iter)->check_LL1(P);
		if(s1!="")
		{
			if(s!="")
				s+=" ; ";
			s+=s1;
		}
	}

	return s;

}
tstring AndedNode::check_LL1(ParserGenerator* P)
{
	tstring s="";
	BitSet_Min BS(P->nextTerminalIndex);
	//BitSet FS2(P->nextTerminalIndex);
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		if((*Iter)->FirstSet(P).Contains(0))
		{
			BitSet_Min  FS = (*Iter)->FirstSet(P);
			list<GenericCFGNode*>::iterator Iter1=Iter;
			Iter1++;
			
			while(Iter1!=Children.end()&&(*Iter1)->FirstSet(P).Contains(0))
			{
				BitSet_Min  FS1 = (*Iter1)->FirstSet(P);
				FS1.SetIntersection(FS);
				BS.SetUnion(FS1);
				Iter1++;
			}
			if(Iter1!=Children.end())
			{
				BitSet_Min  FS1 = (*Iter1)->FirstSet(P);
				FS1.SetIntersection(FS);
				BS.SetUnion(FS1);				
			}
		}
		
	}
	BS.Remove(0);
	if(!BS.IsEmpty())
	{
		int cursor = BS.NextMemberFrom(0);
		s+="several alternatives start with ";
		s+=P->GetTerminalName(TerminalBase+cursor);
		while((cursor=BS.NextMemberFrom(cursor+1))!=-1)
		{
			s+=",";
			s+=P->GetTerminalName(TerminalBase+cursor);
		}
	}
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		s+=(*Iter)->check_LL1(P);
	}
	return s;

}
tstring OptionNode::check_LL1(ParserGenerator *P)
{
	tstring s="";
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		if((*Iter)->FirstSet(P).Contains(0))
		{
            s+="contents of [...] must not be deletable"; 
		}
	} 

	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		s+=(*Iter)->check_LL1(P);
	}
	return s;
	
}
tstring ClosureNode::check_LL1(ParserGenerator *P)
{
	tstring s="";
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		if((*Iter)->FirstSet(P).Contains(0))
		{
			s+="contents of {...} must not be deletable"; 
		}
	} 

	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		s+=(*Iter)->check_LL1(P);
	}
	return s;
}
tstring TerminalNode::check_LL1(ParserGenerator * P)
{
	return "";
}
tstring NonTerminalNode::check_LL1(ParserGenerator * P)
{
	return "";
}
tstring SemActionNode::check_LL1(ParserGenerator *P)
{
	return "";
}
tstring SyncNode::check_LL1(ParserGenerator*P)
{
	return "";
}
tstring AnyNode::check_LL1(ParserGenerator*P)
{
	return "";
}
GenericCFGNode* OredNode::Copy()
{
	OredNode * NodeCopy = new OredNode();
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		GenericCFGNode* ch = (*Iter)->Copy();
		NodeCopy->Children.push_back(ch);
	}
	return NodeCopy;
}
GenericCFGNode* AndedNode::Copy()
{
	AndedNode * NodeCopy = new AndedNode();
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		GenericCFGNode* ch = (*Iter)->Copy();
		NodeCopy->Children.push_back(ch);
	}
	return NodeCopy;
}
GenericCFGNode* TerminalNode::Copy()
{
	TerminalNode*  NodeCopy= new TerminalNode();
	NodeCopy->Attributes = Attributes;
	NodeCopy->IsWeak = IsWeak;
	NodeCopy->NameIndex = NameIndex;
	//NodeCopy->NType = NType;
	NodeCopy->TerminalType = TerminalType;

	return NodeCopy;

}
GenericCFGNode* NonTerminalNode::Copy()
{
	NonTerminalNode* NodeCopy = new NonTerminalNode();
	NodeCopy->Attributes = Attributes;
	NodeCopy->IsWeak = IsWeak;
	NodeCopy->NameIndex = NameIndex;
	//NodeCopy->NType = NType;
	NodeCopy->TerminalType = TerminalType;

	return NodeCopy;
}
GenericCFGNode* SemActionNode::Copy()
{
	SemActionNode* NodeCopy = new SemActionNode();
	//NodeCopy->NameIndex = NameIndex;
	NodeCopy->SemAction = SemAction;
	return NodeCopy; 
}
GenericCFGNode* SyncNode::Copy()
{
	return  new SyncNode();
}
GenericCFGNode* AnyNode::Copy()
{
	return new AnyNode();
}
GenericCFGNode* ClosureNode::Copy()
{
	ClosureNode* NodeCopy = new ClosureNode();
	NodeCopy->Children.push_back((*Children.begin())->Copy());
	return NodeCopy;

}

GenericCFGNode* OptionNode::Copy()
{
	OptionNode* NodeCopy = new OptionNode();
	NodeCopy->Children.push_back((*Children.begin())->Copy());
	return NodeCopy;

}
void OredNode::Remove()
{
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		(*Iter)->Remove();
		delete (*Iter);
	}
	Children.clear();

}
void AndedNode::Remove()
{
	for(list<GenericCFGNode*>::iterator Iter= Children.begin();Iter!=Children.end();Iter++)
	{
		(*Iter)->Remove();
		delete (*Iter);
	}
	Children.clear();

}
void NonTerminalNode::Remove()
{

}
void TerminalNode::Remove()
{

}
void  SyncNode::Remove()
{

}
void SemActionNode::Remove()
{

}
void AnyNode::Remove()
{

}
void ClosureNode::Remove()
{
	(*Children.begin())->Remove();
	delete *Children.begin(); 
	Children.clear();
}
void OptionNode::Remove()
{
	(*Children.begin())->Remove();
	delete *Children.begin(); 
	Children.clear();
}
bool OredNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	bool LeadToTerminal=false;
	for(list<GenericCFGNode*>::iterator ChIter = Children.begin();ChIter!=Children.end();ChIter++)
	{
		LeadToTerminal |=(*ChIter)->LeadToTerminal(P,LHSs);
	}
	return LeadToTerminal;
}
bool AndedNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	bool LeadToTerminal=true;
	for(list<GenericCFGNode*>::iterator ChIter = Children.begin();ChIter!=Children.end();ChIter++)
	{
		LeadToTerminal &=(*ChIter)->LeadToTerminal(P,LHSs);
	}
	return LeadToTerminal;
}
bool ClosureNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	return true;
}
bool OptionNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	return true;
}
bool TerminalNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	return true;
}
bool NonTerminalNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	for(list<unsigned int>::iterator Liter = LHSs.begin();Liter!=LHSs.end();Liter++)
	{
		if( NameIndex == (*Liter))
		{
			return false;
		}
	}
	for(map<tstring,NonTerminalEntry>::iterator NTIter=P->NonTerminals.begin();NTIter!=P->NonTerminals.end();NTIter++)
	{
		if((*NTIter).second.NameIndex==NameIndex)
		{
			if((*NTIter).second.Rule!=NULL)
			{
				LHSs.push_back(NameIndex);
				return (*NTIter).second.Rule->RHS->LeadToTerminal(P,LHSs);
			}
			else
				return false;
			break;
		}
	}
	return false;
}
bool SyncNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	return true;
}
bool AnyNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	return true;
}
bool SemActionNode::LeadToTerminal(ParserGenerator* P,list<unsigned int> LHSs)
{
	return true;
}