#include "stdafx.h"
#include "JavaGenerator.h"
#include "ProductionRule.h"

using namespace RDParserGenerator;

JavaGenerator::JavaGenerator( char* fileName )
{
	DefinitionFile.open( fileName );
}

void JavaGenerator::GenerateCode( ParserGenerator* parser )
{
	if ( !DefinitionFile.is_open() )
		DefinitionFile.open( parser->StrClassName.c_str() );

	Parser = parser;
	tstring startSymbol;

	DefinitionFile << parser->StrDecTop << endl << endl;
	DefinitionFile << "package " << parser->StrNameSpace << ";\n";
	DefinitionFile << parser->StrDefTop << endl;
	DefinitionFile << "public class " << parser->StrClassName << "\n{\n"; 
	
	DefinitionFile << parser->StrDefBottom << endl;

    //writing the constructor
    DefinitionFile << '\t'
		<< "public " << parser->StrClassName << "( IScanner lexicalAnalyzer )\n\t{\n\t\tscanner = lexicalAnalyzer; \n\t}\n";

	ProductionRule* firstProd = parser->Grammer.front();
	startSymbol = parser->GetNonTerminalName( firstProd->LHS ) + _T("(") + firstProd->Attributes + _T(")");
	for( list<ProductionRule*>::iterator iter = parser->Grammer.begin(); iter != parser->Grammer.end(); iter++)
	{
		//note *iter is ProductionRule*
		DefinitionFile << "\tpublic void " << parser->GetNonTerminalName( (*iter)->LHS )
			           << '(' << (*iter)->Attributes << ")\n\t{\n";

		(*iter)->RHS->GenerateCode( this );
		
		DefinitionFile << "\t}" << endl;
	}

	//writing the Parse function
	DefinitionFile << "\n\t"
		<< "public void Parse() " << "\n\t{\n\t\tcurrentToken = scanner.GetNextToken();\n\t\t" << startSymbol << ";\n"
		<< "\t\tif ( currentToken.TokenType != _EOF )\n"
		<< "\t\tSyntaxError(currentToken);"<< "\n\t}" << endl;	


	//writing the match function
	DefinitionFile << '\t' 
		<< "protected void match( int expected )\n\t{\n\t\tif ( currentToken.TokenType == expected )\n\t\t\tcurrentToken = scanner.GetNextToken();\n\t\telse\n\t\t\tErrorHandler( expected, currentToken );\n\t}\n" << endl;
	//writing the ErrorHandler code
	DefinitionFile << '\t'
		<< "protected void ErrorHandler( int expected, Token found )\n\t{\n\t\t // Write your own error handling here \n\t}\n" << endl;

	//write the SyntaxError function
	DefinitionFile << '\t' 
		<< "protected void SyntaxError( Token errorAtToken )\n\t{\n\t\t // Write your own error handling here \n\t}\n" << endl;

	//writing the data members
	DefinitionFile  << '\t' << "protected Token currentToken;" << endl
				    << '\t' << "protected IScanner scanner;" << endl;
    
	//write the  enumerator as a set of integers
	//write the enum members
	int enumMember = 0;
	for( map<unsigned int, tstring>::iterator iter = parser->TokensNames.begin(); iter != parser->TokensNames.end(); iter++)
	{
		DefinitionFile << "\tpublic final int " << iter->second << " = " << enumMember++ << ";\n";
	}

	DefinitionFile << "\n}" << endl; //end the class

	//write the IScanner interface and the Token class
	DefinitionFile << "interface IScanner\n{\n\tToken GetNextToken();\n}" << endl;
	DefinitionFile << "class Token\n{\n\tpublic int TokenType;\n}" << endl;

	DefinitionFile << endl << parser->StrDecBottom ;

	DefinitionFile.close();
}

void JavaGenerator::AndGenerator( AndedNode* node )
{
	if( node->ResolverString.length() > 0 )
		DefinitionFile << "if ( " << node->ResolverString << " )\n\t{\n";
	for( list<GenericCFGNode*>::iterator iter = node->Children.begin(); iter != node->Children.end(); iter++)
	{
		(*iter)->GenerateCode(this);
	}
	if( node->ResolverString.length() > 0 )
		DefinitionFile << "\t}\n";
}

void JavaGenerator::ClosureGenerator( ClosureNode* node )
{
	DefinitionFile << "\t\twhile (";
	BitSet_Min firstSet = node->FirstSet( Parser );
	int index = 0; //we begin from zero because we need to skip Eps if found
	
	while ( (index = firstSet.NextMemberFrom( index+1 )) != -1 )
	{
		DefinitionFile << " currentToken.TokenType == " << Parser->TokensNames[index];
		if ( firstSet.NextMemberFrom( index+1 ) != -1 )
			DefinitionFile << " ||";
	}
	DefinitionFile << " )\n\t\t{\n\t\t\t";
	for( list<GenericCFGNode*>::iterator iter = node->Children.begin(); iter != node->Children.end(); iter++)
	{
		(*iter)->GenerateCode(this);
	}
	DefinitionFile << "\t\t}\n";
}

void JavaGenerator::NonTerminalGenerator( NonTerminalNode* node )
{
	DefinitionFile << "\t\t" << Parser->GetNonTerminalName( node->NameIndex ) <<  '(' << node->Attributes << ");" << endl ;
}

void JavaGenerator::OptionalGenerator( OptionNode* node )
{
	DefinitionFile << "\t\tif (";
	BitSet_Min firstSet = node->FirstSet( Parser );
	int index = 0; //we begin from zero because we need to skip Eps if found
	
	while ( (index = firstSet.NextMemberFrom( index+1 )) != -1 )
	{
		DefinitionFile << " currentToken.TokenType == " << Parser->TokensNames[index];
		if ( firstSet.NextMemberFrom( index+1 ) != -1 )
			DefinitionFile << " ||";
	}
	DefinitionFile << " )\n\t\t{\n";
	for( list<GenericCFGNode*>::iterator iter = node->Children.begin(); iter != node->Children.end(); iter++)
	{
		(*iter)->GenerateCode(this);
	}
	DefinitionFile << "\t\t}\n";
}

void JavaGenerator::OrGenerator( OredNode* node )
{
	BitSet_Min firstSet = node->FirstSet( Parser );
	bool containsEps = ( firstSet.NextMemberFrom(0) == 0 )? 1 : 0;
	int index;
	bool firstIf = 1;
	for( list<GenericCFGNode*>::iterator iter = node->Children.begin(); iter != node->Children.end();iter++)
	{
		firstSet = (*iter)->FirstSet( Parser );
		if ( firstSet.NextMemberFrom(1) == -1 )
			continue;
		if ( firstIf )
		{
			DefinitionFile << "\t\tif (";
			firstIf = 0;
		}
		else
			DefinitionFile << "\t\telse if (";

		index = 0;
		while ( (index = firstSet.NextMemberFrom( index+1 )) != -1 )
		{
			DefinitionFile << " currentToken.TokenType == " << Parser->TokensNames[index];
			if ( firstSet.NextMemberFrom( index+1 ) != -1 )
				DefinitionFile << " ||";
		}
	    DefinitionFile << " )\n\t\t{\n";
		(*iter)->GenerateCode(this);
		DefinitionFile << "\t\t}\n";
	}
	if ( !containsEps ) //so it is not optional
		DefinitionFile << "\t\telse\n\t\t\tSyntaxError(currentToken);\n";
}

void JavaGenerator::SemActionGenerator( SemActionNode* node )
{
	DefinitionFile << "\t\t" << node->SemAction;
}

void JavaGenerator::TerminalGenerator( TerminalNode* node )
{
	DefinitionFile << "\t\tmatch( " <<  Parser->TokensNames[node->NameIndex-TerminalBase] << " );" << endl ;
}