#include "stdafx.h"
#include "ProductionRule.h"
#include "GenericCFGNode.h"
#include "ParserGen.h"
#include "Globals.h"
#include "BitSet_Min.h"
using namespace RDParserGenerator;
location::location(int l,int c)
{
	lin_No = l;
	Col_No = c;
}

TerminalEntry::TerminalEntry() {}
TerminalEntry::TerminalEntry(unsigned int NI,TokenType T,bool u)
{
	NameIndex = NI;
	Type = T;
	used = u;
	
}

NonTerminalEntry::NonTerminalEntry() {}
NonTerminalEntry::NonTerminalEntry(unsigned int NI,bool defined)
{
	NameIndex = NI;
	Defined = defined;
}



ParserGenerator::ParserGenerator(string fileName)
{
	this->scanner = new Scanner(fileName);
	ResumeOnError = TRUE;
	HasErrors = FALSE;
	nextTerminalIndex = TerminalBase;
	nextNonTerminalIndex = NonTerminalBase;
	NumberOfErrors=0;
	StartSymbolFound =false;
	Braces=0;
	Brackets = 0;
	SquareBrakets=0;
	filename = fileName;
	//ErrorsFile.open("Errors.txt");
}

ParserGenerator::~ParserGenerator()
{
	//ErrorsFile.close();
	delete scanner;
}
void ParserGenerator::Parse()
{
	TokensNames[0] = _T("_EOF"); //I use here a dummy 0 because I am not interested in mapping here
	
	TerminalEntry TE1(nextTerminalIndex,Eps);
	Terminals["@%"]=TE1;
	nextTerminalIndex++;

	/*TokensNames[nextTerminalIndex] = _T("_Char");
	TerminalEntry TE2(nextTerminalIndex,Character);
	Terminals["Char"]=TE2;
	nextTerminalIndex++;

	TokensNames[nextTerminalIndex] = _T("_String");
	TerminalEntry TE3(nextTerminalIndex,String);
	Terminals["String"]=TE3;
	nextTerminalIndex++;*/

	token = scanner->GetToken();
	la1 = scanner->GetToken();
	la2 = scanner->GetToken();

	if(token.Type == Options)
	{
		match(Options);
		OptionsSet();
	}
	if(token.Type == Tokens)
	{
		match(Tokens);
		TokensSet();
	}
	while(token.Type == DeclTop || token.Type == DeclBottom || token.Type == DefTop || token.Type == DefBottom)
	{
		switch(token.Type)
		{
		case DeclTop:
			match(DeclTop);
			TopOfDeclaration();
			break;
		case DeclBottom:
			match(DeclBottom);
			BottomOfDeclaration();
			break;
		case DefTop:
			match(DefTop);
			TopOfDefinition();
			break;	
		case DefBottom:
			match(DefBottom);
			BottomOfDefinition();
			break;
		}
	}
	//if(token.Type == Grammar)
	//{
	match(Grammar,"Error: Missing Keyword grammar",false);
		while(token.Type!=EndOfFile)
			Productions();
	//}
	
	//and EOF to Terminals
	TerminalEntry TE(nextTerminalIndex,EndOfFile);
	nextTerminalIndex++;
	Terminals["^&)"]=TE;
	CalculateFirstSetS();
	CheckSemErrors();
	PrintErrors();
	if(HasErrors==FALSE)
	{
		
		CalculateFollowSetS();
		PrintRules();
		PrintFirstSet();
		PrintFollowSets();
	}
}
void ParserGenerator::PrintErrors()
{
	//ofstream ErrorsFile;
	//ErrorsFile.open("temp\\errors.txt");
	int errorNo = 1;
	for(map<int,LineErrorEntry>::iterator EIter=FileErrors.begin() ;EIter!=FileErrors.end();EIter++)
	{
		for(map<int,StringList>::iterator LIter =(*EIter).second.begin();LIter!=(*EIter).second.end();LIter++)
		{
			for(list<tstring>::iterator SIter  = (*LIter).second.begin();SIter!=(*LIter).second.end();SIter++)
			{
				//ErrorsFile<<"Line "<<(*EIter).first<<" Column "<<(*LIter).first<<" : "<<(*SIter).c_str()<<endl;
				cerr << " \t" << errorNo++ << "\t" << (*SIter).c_str()<<"\t"<<filename<<"\t"<<(*EIter).first<<"\t"<<(*LIter).first<<endl;
			}
		}
	}
	//ErrorsFile.close();
}
void ParserGenerator::CheckSemErrors()
{
	//Adding Sematic Errors
	for(map<tstring,NonTerminalEntry>::iterator NTIter=NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++)
	{
		if((*NTIter).second.Defined==false)
		{
			/*(*NTIter).second.FollowSet = new BitSet_Min(nextTerminalIndex);
			(*NTIter).second.FollowSet->Add(nextTerminalIndex-1);*/
			HasErrors = TRUE;
			for(vector<location>::iterator iter = (*NTIter).second.locations.begin();iter!=(*NTIter).second.locations.end();iter++)
			{
				tstring msg = "Error: NonTerminal "+(*NTIter).first+" undefined";
				FileErrors[(*iter).lin_No][(*iter).Col_No].push_back(msg);
			}
		}
		if((*NTIter).second.reachable==false&&(*NTIter).second.Defined==true)
		{
			list<location>::iterator Piter = (*NTIter).second.Rule->locations.begin();
			for(;Piter!=(*NTIter).second.Rule->locations.end();Piter++)
			{
				//HasErrors = TRUE;
				tstring msg = "Warning: Unreachable Production for "+(*NTIter).first;
				FileErrors[(*Piter).lin_No][(*Piter).Col_No].push_back(msg);
			}
		}

		
	}
	
	
	for(map<tstring,TerminalEntry>::iterator TIter=Terminals.begin();TIter!=Terminals.end();TIter++)
	{
		vector<location>::iterator iter = (*TIter).second.locations.begin();
		tstring debug = (*TIter).first;
		if((*TIter).second.Type == EndOfFile||(*TIter).second.Type==Eps||(*TIter).second.Type==Any)
		{
			continue;
		}
		if((*TIter).second.used==false)
		{
			//HasErrors = TRUE;
			tstring msg = "Warning: Terminal "+(*TIter).first+" defined but not used";
			FileErrors[(*iter).lin_No][(*iter).Col_No].push_back(msg);
			
		}
		
		iter++;
		for(;iter!=(*TIter).second.locations.end();iter++)
		{
			HasErrors = TRUE;
			tstring msg = "Error: Terminal "+(*TIter).first+" redefinition";
			FileErrors[(*iter).lin_No][(*iter).Col_No].push_back(msg);
		}
		
	}
	
	//int x=(4)*((((5))));

	for(list<ProductionRule*>::iterator PIter= Grammer.begin();PIter!=Grammer.end();PIter++)
	{
		tstring NTName;
		map<tstring,NonTerminalEntry>::iterator iter;
		for(iter=NonTerminals.begin();iter!=NonTerminals.end();iter++)
		{
			if((*iter).second.NameIndex==(*PIter)->LHS)
			{
				NTName= (*iter).first;
				break;
			}
		}
		tstring s = (*PIter)->RHS->check_LL1(this);
		if(s!="")
		{	
			//s = "LL1 warning in "+NTName+ " : "+s;
			s = "Warning: LL1 Conflict in "+NTName+ " ( "+s+" ) ";
			
			location l =*(*iter).second.Rule->locations.begin(); 
			//HasErrors = TRUE;

			FileErrors[l.lin_No][l.Col_No].push_back(s);
		}

        list<unsigned int> L;
		L.push_back((*PIter)->LHS);
		if((*PIter)->RHS->LeadToTerminal(this,L)==false)
		{
			list<location>::iterator Piter = (*PIter)->locations.begin();
			for(;Piter!=(*PIter)->locations.end();Piter++)
			{
				HasErrors = TRUE;
				tstring msg = "Error: Production for "+GetNonTerminalName((*PIter)->LHS)+" does not lead to terminal" ;
				FileErrors[(*Piter).lin_No][(*Piter).Col_No].push_back(msg);
				break;
			}
		}
	   
	}

}
tstring ParserGenerator::GetTerminalName(unsigned int NameIndex)
{
	for( map<tstring,TerminalEntry>::iterator iter = Terminals.begin(); iter != Terminals.end(); iter++)
	{
		if ( iter->second.NameIndex == NameIndex )
			return iter->first;
	}

}
void ParserGenerator::TokensSet()
{

	bool Cond =true;
	while(token.Type == Identifier||token.Type==String||Cond)
	{
		Cond = false;
		while(token.Type==ClassName||token.Type==NameSpace||token.Type==If||token.Type==Options||token.Type==Eps)
		{
			match(Identifier,"Error: "+token.Lexeme+" is a Reserved keyword can not be used as a token ,can be used as\""+token.Lexeme+"\"");
		}
		//add as a terminal
		//unsigned int Index = TerminalBase+nextTerminalIndex; //Why Hamdy? We initialize nextTerminalIndex with TerminalBase 
		map<tstring,TerminalEntry>::iterator TIter;
		if((TIter = Terminals.find(token.Lexeme))!=Terminals.end())
		{
			location l(token.LineNo,token.ColNo);
			(*TIter).second.locations.push_back(l);
			match(token.Type);
			continue;
		}
		TerminalEntry TE(nextTerminalIndex,token.Type);
		location loc(token.LineNo,token.ColNo);
		TE.locations.push_back(loc);
		Terminals[token.Lexeme]=TE;
		//TerminalsInverse[Index]=&Terminals[token.Lexeme];
		TokensNames[nextTerminalIndex] = _T("_") + token.Lexeme;
		nextTerminalIndex++;
		match(token.Type);
		while(token.Type==ClassName||token.Type==NameSpace||token.Type==If||token.Type==Options||token.Type==Eps)
		{
			match(Identifier,"Error: "+token.Lexeme+" is a Reserved keyword can not be used as a token ,can be used as\""+token.Lexeme+"\"");
		}
	}
}

void ParserGenerator::OptionsSet()
{
	if(token.Type == NameSpace)
	{
		match(NameSpace);
		match(Equal,"Error: Missing =",false);
		StrNameSpace = token.Lexeme;//match(String)
		match(String,"Error: Missing namespace name",false);
	}
	if(token.Type == ClassName)
	{
		match(ClassName);
		match(Equal,"Error: Missing =",false);
		StrClassName = token.Lexeme;//match string
		match(String,"Error: Missing ClassName",false);
	}
	if(token.Type == Language)
	{
		match(Language);
		match(Equal,"Error: Missing =",false);
		ParserLanguage = token.Type;
		if(token.Type==CSharp||token.Type==Java||token.Type==CPlusPlus)
		{
			match(token.Type);
		}
		else 
		{
			if(token.Type==Identifier||token.Type==String)
				match(token.Type,"Error: Invalid Language Type:valid types are (CSharp,JAVA,CPlusPlus)");
			else
				match(token.Type,"Error: missing language Type",false);
		}
	}
}

void ParserGenerator::TopOfDeclaration()
{
	if (token.Type == BulkOfCode)
		StrDecTop += token.Lexeme;
	match(BulkOfCode,"Error: Bulk of Code is mising,Valid Form is %%BulkOfCode%%",false);
}

void ParserGenerator::TopOfDefinition()
{
	if (token.Type == BulkOfCode)
		StrDefTop += token.Lexeme;
	match(BulkOfCode,"Error: Bulk of Code is mising,Valid Form is %%BulkOfCode%%",false);
}

void ParserGenerator::BottomOfDeclaration()
{
	if (token.Type == BulkOfCode)
		StrDecBottom += token.Lexeme;
	match(BulkOfCode,"Error: Bulk of Code is mising,Valid Form is %%BulkOfCode%%",false);
}

void ParserGenerator::BottomOfDefinition()
{
	if (token.Type == BulkOfCode)
		StrDefBottom += token.Lexeme;
	match(BulkOfCode,"Error: Bulk of Code is mising,Valid Form is %%BulkOfCode%%",false);
}



void ParserGenerator::match(TokenType expected,tstring ErrorMsg,bool Resume)
{
	if(expected==Dot)
	{
		Braces=0;
		Brackets=0;
		SquareBrakets=0;
	}
	else if(expected ==BulkOfCode&&token.Type!=BulkOfCode)
	{
		onError(ErrorMsg);
		while(!(token.Type==DefTop||token.Type==DefBottom||token.Type==DeclTop||token.Type==DeclBottom||token.Type==Grammar||token.Type==EndOfFile))
		{
			lasttoken = token;
			token = la1;
			la1 = la2;
			la2= scanner->GetToken();
		}
		return;
	}
	/*bool cond = true;
	while(cond)
	{*/
	//Token lastToken = token;
	switch(token.Type)
		{
			case OpenBracket:
				{
					Brackets++;
					//cond=false;
					break;
				}
			case OpenCurly: 
				{
					Braces++;
					//cond=false;
					break;

				}
			case OpenSquareBracket:
				{
					SquareBrakets++;
					//cond=false;
					break;
				}
			case CloseBracket:
				{
					if(Brackets<=0)
					{
						Resume= true;
						ErrorMsg = "Error: )";
						//token = scanner->GetToken();
						//expected = token.Type;
					}
					/*else 
						cond = false;*/
					
					Brackets--;
					break;
				}
			case CloseCurly:
				{
					if(Braces<=0)
					{
						Resume = true;
						ErrorMsg = "Error: }";
						//token = scanner->GetToken();
						//expected = token.Type;
					}
					/*else 
						cond=false;*/
                    Braces--;
					break;
				}
			case CloseSquareBracket:
				{
					if(SquareBrakets<=0)
					{
						Resume = true;
						ErrorMsg = "Error: ]";
						/*token = scanner->GetToken();
						expected = token.Type;*/
					}
					/*else 
						cond=false;*/
					SquareBrakets--;
					break;
				}
			/*default:
				{
					cond=false ;
					break;
				}
		}*/
	}
	if (token.Type == expected)
	{
		lasttoken = token;
 		token = la1;
		la1 = la2;
		la2= scanner->GetToken();
	}
	else
	{
		//perform error action
		//tstring msg; //construct the msg
		if(expected==Dot)
		{
			int i=0;
			if(lasttoken.Type==Attributes||lasttoken.Type==SemAction)
				i+=3;
			if(lasttoken.Type==String)
				i+=2;
			FileErrors[lasttoken.LineNo][lasttoken.ColNo+lasttoken.Lexeme.length()+i].push_back(ErrorMsg);
			NumberOfErrors++;	
		}
		else
        	onError(ErrorMsg);
		HasErrors = TRUE;
		if (Resume)
		{
			lasttoken = token;
			token = la1;
			la1 = la2;
			la2= scanner->GetToken();
		}
	}
	/*if (token.Type == expected)
	{
		lasttoken = token;
 		token = la1;
		la1 = la2;
		la2= scanner->GetToken();
	}
	else
	{
		
		//perform error action
		//tstring msg; //construct the msg
		if(expected==Dot)
		{
			int i=0;
			if(lasttoken.Type==Attributes||lasttoken.Type==SemAction)
				i+=3;
			if(lasttoken.Type==String)
				i+=2;
			FileErrors[lasttoken.LineNo][lasttoken.ColNo+lasttoken.Lexeme.length()+i].push_back(ErrorMsg);
			NumberOfErrors++;				
		}
		else
			onError(ErrorMsg);
		HasErrors = TRUE;
		if (Resume)
		{
			lasttoken = token;
			token = la1;
			la1 = la2;
			la2= scanner->GetToken();
		}
	}*/
}
void ParserGenerator::Productions()
{
	// hamdy, start from here
	/*ProductionRule * Rule = new ProductionRule();
	if(token.Type==Identifier)
	{
		if(NonTerminals.find(token.Lexeme)==NonTerminals.end())
		{	
			Rule->LHS = nextNonTerminalIndex;
			NonTerminalEntry NTE(nextNonTerminalIndex);
			NonTerminals[token.Lexeme]=NTE;
			nextNonTerminalIndex++;			
		}
		else
		{
			Rule->LHS = NonTerminals[token.Lexeme].NameIndex;
		}		
	}
	match(Identifier);
	match(Production);
	//Rule->RHS = new OredNode();
	Expression(&Rule->RHS);
	match(Dot);
	Grammer.push_back(Rule);*/
	// hamdy, start from here
	while(token.Type==Dot)
	{
		match(Dot);
		Braces=0;
		Brackets=0;
		SquareBrakets=0;
	}
	
	while(token.Type!=Identifier&&token.Type != Production&&token.Type!=Attributes)
	{
		match(Identifier,"Error: unexpected token "+token.Lexeme,true);
	}
	ProductionRule * Rule = new ProductionRule();
	map<tstring,NonTerminalEntry>::iterator NTiter = NULL;
	bool check1=false;
	bool check2=false;
	/*if(NonTerminals.size()==1)
	{
		(*NonTerminals.begin()).second.reachable = true;
	}*/
	bool reachable = false;
	location loc(token.LineNo,token.ColNo);
	if(token.Type==Identifier||token.Type==String)
	{
		if(Terminals.find(token.Lexeme)!=Terminals.end())
		{
			/*cout<<"input file contains errors\n";
			exit(1);*/
			onError("Error: Terminal "+token.Lexeme+" can not be used as LHS of a rule");
			//RHS_is_Terminal = true;
		}
		else if((NTiter=NonTerminals.find(token.Lexeme))==NonTerminals.end())
		{	
			if(!StartSymbolFound)
			{
				(*NTiter).second.reachable = true;
				StartSymbolFound = true;
				reachable=true;
			}
			else
				reachable=false;
			Rule->LHS = nextNonTerminalIndex;
			NonTerminalEntry NTE(nextNonTerminalIndex,true);
			NTE.Rule=Rule;
			NTE.reachable =reachable;
			NonTerminals[token.Lexeme]=NTE;
			nextNonTerminalIndex++;
			check1=true;

		}
		else
		{
			if(!StartSymbolFound)
			{
				(*NTiter).second.reachable = true;
				StartSymbolFound = true;
			}
			reachable=(*NTiter).second.reachable;
			(*NTiter).second.Defined = true;
			(*NTiter).second.locations.clear();
			ProductionRule* debugnode= (*NTiter).second.Rule;
			Rule->LHS = NonTerminals[token.Lexeme].NameIndex;
			if((*NTiter).second.Rule==NULL)
			{
				(*NTiter).second.Rule=Rule;
				check2= true;
			}	
		}	
	}
 	match(Identifier,"Error: Missing NonTerminal On the Left handSide",false);
	if(token.Type==Attributes)
	{
		Rule->Attributes= token.Lexeme;
		match(Attributes);
	}
	match(Production,"Error: missing production operator ->",false);
	//Rule->RHS = new OredNode();
	
	if(check1||check2)
	{
		Expression(&Rule->RHS,reachable);
		Rule->locations.push_back(loc);
		if(Rule->RHS!=NULL)
			Grammer.push_back(Rule);
	}
	else 
	{
		GenericCFGNode* RuleNode;
		Expression(&RuleNode,reachable);
		
		if(NTiter!=NULL)
		{
			//(*NTiter).second.Rule->locations.push_back(loc);
			if((*NTiter).second.Rule->RHS->NType ==_Ored&&RuleNode->NType==_Ored)
			{
				for(list<GenericCFGNode*>::iterator Iter= ((OredNode*)RuleNode)->Children.begin();Iter!=((OredNode*)RuleNode)->Children.end();Iter++)
				{
					((OredNode*)(*NTiter).second.Rule->RHS)->Children.push_back(*Iter);
				}
				//((OredNode*)(*NTiter).second.Rule->locations.push_back(loc);
				((OredNode*)RuleNode)->Children.clear();
			//	delete RuleNode;
			
			}
			else if((*NTiter).second.Rule->RHS->NType ==_Ored)
			{
				((OredNode*)(*NTiter).second.Rule->RHS)->Children.push_back(RuleNode);
				//((OredNode*)(*NTiter).second.Rule->locations.push_back(loc);
			}
			else if(RuleNode->NType==_Ored)
			{
				((OredNode*)RuleNode)->Children.push_back((*NTiter).second.Rule->RHS);
				(*NTiter).second.Rule->RHS=RuleNode;
			}
			else
			{
				GenericCFGNode* TempNode = (*NTiter).second.Rule->RHS;
				(*NTiter).second.Rule->RHS= new OredNode();
				((OredNode*)(*NTiter).second.Rule->RHS)->Children.push_back(TempNode);
				((OredNode*)(*NTiter).second.Rule->RHS)->Children.push_back(RuleNode);
			}
			(*NTiter).second.Rule->locations.push_back(loc);
		}
	}
		
	match(Dot,"Error: Missing Dot to terminate the prouction",false);
	
	
}
void ParserGenerator::Expression(GenericCFGNode** node,bool reachable)
{
	//GenericCFGNode*  node;
	//node1 = &node;
	if(token.Type==If||token.Type==Weak||token.Type==OpenBracket||token.Type==OpenCurly||token.Type==OpenSquareBracket||token.Type==Eps||token.Type==Any||token.Type==Sync||token.Type==SemAction||token.Type==Identifier||token.Type==String||token.Type==Character)
	{
		/*ExpLoc.lin_No = token.ColNo;
		ExpLoc.Col_No = token.LineNo;*/
		//AndedNode* ChildNode = new AndedNode();
		GenericCFGNode* ChildNode;
		Term(&ChildNode,reachable);
		if(token.Type!=Or)
		{
			*node = ChildNode;
			return;
		}
		*node = new OredNode();
		if(ChildNode->NType!=_Ored)
			((OredNode*)*node)->Children.push_back(ChildNode);
		else
		{
			OredNode* OrNode =(OredNode*)ChildNode; 
			for(list<GenericCFGNode*>::iterator chIter = OrNode->Children.begin();chIter!=OrNode->Children.end();chIter++)
			{
				((OredNode*)*node)->Children.push_back((*chIter)/*->Copy()*/);
			}
			/*ChildNode->Remove();
			delete ChildNode; */
		}
	}
	while(token.Type==Or)
	{
		match(Or);
		GenericCFGNode* Child ;//= new AndedNode();
		Term(&Child,reachable);
		//((OredNode*)*node)->Children.push_back(Child);
		if(Child->NType!=_Ored)
			((OredNode*)*node)->Children.push_back(Child);
		else
		{
			OredNode* OrNode =(OredNode*)Child; 
			for(list<GenericCFGNode*>::iterator chIter = OrNode->Children.begin();chIter!=OrNode->Children.end();chIter++)
			{
				((OredNode*)*node)->Children.push_back((*chIter)/*->Copy()*/);
			}
			/*Child->Remove();
			delete Child;*/ 
		}
	}
}
void ParserGenerator::Term(GenericCFGNode** node,bool reachable)
{
	*node=NULL;
	if(token.Type==Eps)
	{
		match(Eps);
		*node = new TerminalNode();
		((TerminalNode*)*node)->TerminalType = Eps;
		((TerminalNode*)*node)->NameIndex = 0;
	}
	else
	{
		bool ResolverExist=false;
		if(token.Type==If)
		{
			match(If);
			//match(OpenBracket);
			*node = new AndedNode();
			if(token.Type==Attributes)
				((AndedNode*)*node)->ResolverString=token.Lexeme;
			match(Attributes,"Error: Resolver attributes is missing,Valid Form is IF(.{ANY}.)",false);
			//match(BulkOfCode);
			//match(CloseBracket);
			ResolverExist= true;
		}
		if(token.Type==Weak||token.Type==OpenBracket||token.Type==OpenCurly||token.Type==OpenSquareBracket||token.Type==Any||token.Type==Sync||token.Type==SemAction||token.Type==Identifier||token.Type==String||token.Type==Character)
		{
			GenericCFGNode* child;
			Factor(&child,reachable);
			if(!ResolverExist)
			{
				if(!(token.Type==Weak||token.Type==OpenBracket||token.Type==OpenCurly||token.Type==OpenSquareBracket||token.Type==Any||token.Type==Sync||token.Type==SemAction||token.Type==Identifier||token.Type==String||token.Type==Character))
				{
					*node = child;
					return;
				}
				*node = new AndedNode();
			}
			if(child!=NULL)
			{
				//((AndedNode*)*node)->Children.push_back(child);
				if(child->NType!=_Anded)
					((AndedNode*)*node)->Children.push_back(child);
				else
				{
					AndedNode* AndNode =(AndedNode*)child; 
					for(list<GenericCFGNode*>::iterator chIter = AndNode->Children.begin();chIter!=AndNode->Children.end();chIter++)
					{
						((AndedNode*)*node)->Children.push_back(*chIter);
					}
					/*Child->Remove();
					delete Child;*/ 
				}
			}
		}
		while(token.Type==Weak||token.Type==OpenBracket||token.Type==OpenCurly||token.Type==OpenSquareBracket||token.Type==Any||token.Type==Sync||token.Type==SemAction||token.Type==Identifier||token.Type==String||token.Type==Character)
		{
			if((token.Type==String || token.Type==Identifier) &&(la1.Type == Production||(la1.Type==Attributes && la2.Type ==Production)))
			{
				/*onError("Missing Dot at the end of the sentence");
				HasErrors = TRUE;*/
				break;
			}
			GenericCFGNode* child;
			Factor(&child,reachable);
			if(child!=NULL)
			{
				//((AndedNode*)*node)->Children.push_back(child);
				if(child->NType!=_Anded)
					((AndedNode*)*node)->Children.push_back(child);
				else
				{
					AndedNode* AndNode =(AndedNode*)child; 
					for(list<GenericCFGNode*>::iterator chIter = AndNode->Children.begin();chIter!=AndNode->Children.end();chIter++)
					{
						((AndedNode*)*node)->Children.push_back(*chIter);
					}
					/*Child->Remove();
					delete Child;*/ 
				}
			}
		}
	}

}
void ParserGenerator::Factor(GenericCFGNode** node,bool reachable)
{
	//GenericCFGNode* node ;
	//node1=&node; 
	*node = NULL;
	if(token.Type==OpenBracket)
	{
		//*node = new BracketNode();
		match(OpenBracket);
		//GenericCFGNode* child;//=new OredNode();
		Expression(node,reachable);
		//((BracketNode*)node)->Children.push_back(child);
		if(*node==NULL&&token.Type==CloseBracket)
			onError("Warning: Contents of () is empty ,and so is ignored");
		match(CloseBracket,"Error: Missing )",false);
	}
	else if(token.Type==OpenSquareBracket)
	{
		match(OpenSquareBracket);
		GenericCFGNode* child=NULL;//=new OredNode();
		Expression(&child,reachable);
		if(child!=NULL&&token.Type==CloseSquareBracket)
		{
			*node = new OptionNode();
			((OptionNode*)*node)->Children.push_back(child);
		}
		else
		{
			onError("Warning: Contents of [] is empty ,and so is ignored");
		}
		match(CloseSquareBracket,"Error: Missing ]",false);
	}
	else if(token.Type==OpenCurly)
	{
		
		match(OpenCurly);
		GenericCFGNode* child=NULL;//OredNode* child=new OredNode();
		Expression(&child,reachable);
        if(child!=NULL)
		{
			*node = new ClosureNode();
			((ClosureNode*)*node)->Children.push_back(child);
		}
		else if(token.Type==CloseCurly)
		{
			onError("Warning: Contents of {} is empty ,and so is ignored");
		}
		match(CloseCurly,"Error: Missing }",false);
	}
	else if(token.Type==SemAction)
	{
		*node = new SemActionNode();
       // match(OpenActions);
		((SemActionNode*)*node)->SemAction = token.Lexeme;
		//match(BulkOfCode);
		//match(CloseActions);
		match(SemAction);
	}
	else if(token.Type==Any)
	{
		//is there an action for First and FollowSet
		*node = new AnyNode();
		match(Any);
	}
	else if(token.Type==Sync)
	{
		*node = new SyncNode();
		match(Sync);
	}
	else
	{	
		bool IsWeak = false;
		if(token.Type==Weak)
		{			
			IsWeak = true;
			match(Weak);
		}
		/*if(token.Type==String||token.Type==Character)
		{
			*node = new TerminalNode();
			((TerminalNode*)*node)->IsWeak = IsWeak;
			((TerminalNode*)*node)->TerminalType=token.Type; 
			if(Terminals.find(token.Lexeme)==Terminals.end())
			{
				((TerminalNode*)*node)->NameIndex = nextTerminalIndex;
				TerminalEntry TE(nextTerminalIndex,token.Type);
				Terminals[token.Lexeme] = TE;
				nextTerminalIndex++;
			}
			else
			{
				((TerminalNode*)*node)->NameIndex = Terminals[token.Lexeme].NameIndex;

			}
			if(token.Type==Character)
				match(Character);
			else
				match(String);		
		}
		else */
		if(token.Type==Identifier||token.Type==String)
		{
			map<tstring,TerminalEntry>::iterator Titer;
			if((Titer= Terminals.find(token.Lexeme))!=Terminals.end())
			{
				*node = new TerminalNode();
				((TerminalNode*)*node)->IsWeak=IsWeak;
				((TerminalNode*)*node)->TerminalType=token.Type;
				((TerminalNode*)*node)->NameIndex = Terminals[token.Lexeme].NameIndex;
				(*Titer).second.used = true;
				//(*Titer).second.locations.clear();
			}
			else
			{
				*node = new NonTerminalNode();
				((NonTerminalNode*)*node)->IsWeak=IsWeak;
				((TerminalNode*)*node)->TerminalType=token.Type;
				map<tstring,NonTerminalEntry>::iterator NTiter = NULL;
				if((NTiter=NonTerminals.find(token.Lexeme))==NonTerminals.end())
				{
					((NonTerminalNode*)*node)->NameIndex = nextNonTerminalIndex;
					NonTerminalEntry TE(nextNonTerminalIndex);
					location loc(token.LineNo,token.ColNo);
					TE.locations.push_back(loc);
					TE.reachable=reachable;
					TE.Rule=NULL;
					NonTerminals[token.Lexeme] = TE;
					nextNonTerminalIndex++;

				}
				else
				{
					if(reachable)
						(*NTiter).second.reachable=true;
					((NonTerminalNode*)*node)->NameIndex = NonTerminals[token.Lexeme].NameIndex;
					if((*NTiter).second.Defined ==false)
					{
						location loc(token.LineNo,token.ColNo);
						(*NTiter).second.locations.push_back(loc);
					}
				}

			}
			match(token.Type);
			if(token.Type==Attributes)
			{
				//match(OpenAttributes);
				switch((*node)->NType)
				{
				case _Terminal:
					((TerminalNode*)*node)->Attributes = token.Lexeme;
					break;
				case _NonTerminal:
					((NonTerminalNode*)*node)->Attributes = token.Lexeme;
		            break;
				}
				//match(CloseAttributes);
				match(Attributes);
		    }

		}
		
	}
	
}

tstring ParserGenerator::GetNonTerminalName( unsigned int NameIndex )
{
	for( map<tstring,NonTerminalEntry>::iterator iter = NonTerminals.begin(); iter != NonTerminals.end(); iter++)
	{
		if ( iter->second.NameIndex == NameIndex )
			return iter->first;
	}
	return _T("Undefined");
}

void ParserGenerator::onError(tstring errorMsg)
{
	int i=90;
	/*char* p= new char[20];
	itoa(i,p,3);*/
//	cout<<p<<endl;
	//write to file for example
	//cout<<"Line "<<token.LineNo<<" Column "<<token.ColNo<<" : "<<errorMsg.c_str()<<endl;
	
	FileErrors[token.LineNo][token.ColNo].push_back(errorMsg);
	NumberOfErrors++;
	if(NumberOfErrors>=1000)
	{
		PrintErrors();
		cout<<"Error: Too many errors"<<endl;;
		exit(1);
	}
	//if(FileErrors.max_size()
}
void ParserGenerator::CalculateFirstSetS()
{
	//Initialize FirstSet
	map<tstring,TerminalEntry>::iterator TIter;
	for(TIter=Terminals.begin();TIter!=Terminals.end();TIter++)
	{
		(*TIter).second.FirstSet = new BitSet_Min(nextTerminalIndex);
		(*TIter).second.FirstSet->Add((*TIter).second.NameIndex-TerminalBase);
	}
	map<tstring,NonTerminalEntry>::iterator NTIter;
	for(NTIter=NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++)
	{
		(*NTIter).second.FirstSet = new BitSet_Min(nextTerminalIndex);
		//(*NTIter).second.FollowSet = new BitSet_Min(nextTerminalIndex);
	}
	//FirstSet Implementation
	bool Change= true;
	while(Change)
	{
		Change = false;
		for(list<ProductionRule*>::iterator PIter= Grammer.begin();PIter!=Grammer.end();PIter++)
		{
			//for debugging
			GenericCFGNode* DebugNode = (*PIter)->RHS; 
			//not implemented yet
			bool Changed=false;
			BitSet_Min firstset;
			BitSet_Min fs = (*PIter)->RHS->FirstSet(this);
			for(map<tstring,NonTerminalEntry>::iterator iter=NonTerminals.begin();iter!=NonTerminals.end();iter++)
			{
				if((*iter).second.NameIndex==(*PIter)->LHS)
				{
					firstset = *(*iter).second.FirstSet;
					fs.SetUnion(firstset);
					//if(!(fs.IsSubsetOf(firstset)&&firstset.IsSubsetOf(fs)))//remember for optimization
					if(!fs.Equals(firstset))
						Changed= true;
					//delete (*iter).second.FirstSet;
					(*iter).second.FirstSet->SetUnion(fs);
					Change |= Changed;

					break;
				}
			}
			
			
			
		}
	}
}

void ParserGenerator::CalculateFollowSetS()
{
	//Initialize FollowSet
	for(map<tstring,NonTerminalEntry>::iterator NTIter=NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++)
	{
		if((*NTIter).second.NameIndex==NonTerminalBase)
		{
			(*NTIter).second.FollowSet = new BitSet_Min(nextTerminalIndex);
			(*NTIter).second.FollowSet->Add(nextTerminalIndex-1);
		}
		else
			(*NTIter).second.FollowSet = new BitSet_Min(nextTerminalIndex);
	}
	//FollowSet Calculation
	
	bool Change= true;
	while(Change)
	{
		Change = false;
		for(list<ProductionRule*>::iterator PIter= Grammer.begin();PIter!=Grammer.end();PIter++)
		{
			//for debug
			GenericCFGNode* debugnode= (*PIter)->RHS;
			///////////////
			Change|=(*PIter)->RHS->FollowSet(this);
			BitSet_Min Bs = (*PIter)->RHS->GetNonTerminals(this);
			int cursor=-1;


			if(!Bs .IsEmpty())
			{
				BitSet_Min followset;
				for(map<tstring,NonTerminalEntry>::iterator NTIter=NonTerminals.begin();NTIter!=NonTerminals.end();NTIter++)
				{
					if((*NTIter).second.NameIndex==(*PIter)->LHS)
					{
						followset = *((*NTIter).second.FollowSet);
					}
				}
				while((cursor=Bs.NextMemberFrom(cursor+1))!=-1)
				{
					for(map<tstring,NonTerminalEntry>::iterator NTiter=NonTerminals.begin();NTiter!=NonTerminals.end();NTiter++)
					{
						if((*NTiter).second.NameIndex==cursor+NonTerminalBase)
						{
							//Shakek feeha remember for debuging
							(*NTiter).second.FollowSet->SetUnion(followset);
							
						}
					}

				}
			}
		}
	}

	
	
}
void ParserGenerator::PrintFirstSet()
{
}
void ParserGenerator::PrintFollowSets()
{
}
void ParserGenerator::PrintRules()
{
}




