#include "stdafx.h"
#include "Scanner.h"
using namespace RDParserGenerator;

Scanner::Scanner(string fileName, unsigned int numberOfCharsPerRead)
{
	maxCharsToRead = numberOfCharsPerRead;
	currentLineNum      = 1;
	currentColNo        = 1;
	currentCharPosition = 0;
	TabSize             = 4;
	file                = new InputFile;
	file->open(fileName.c_str());
	buffer = new TCHAR[numberOfCharsPerRead];
	file->read(buffer, maxCharsToRead);
	bufferSize = file->gcount();
	initializeReservedKeywords();
}

Token Scanner::GetToken()
{
	Token token;
	tstring lexeme = _T("");
	States state = Start;
	TCHAR currentChar;	
	bool isFinished = false;
	TCHAR temp = _T('%');
	while ( state != Done && state != ErrorState)
	{
		currentChar = getNextCharachter();
		switch (state)
		{
		case Start:
			if ( _istalpha(currentChar) || currentChar == _T('_') )
			{
				lexeme += currentChar;
				state = InIdent;
			}
			else if ( currentChar == _T('"') )
				state = InString;
			else if ( currentChar == _T('\'') )
				state = InChar;
			else if ( currentChar == _T('-') )
			{
				lexeme += currentChar;
				state = InProduction;
			}
			else if ( currentChar == _T('.') )
			{
				lexeme += currentChar;
				token.Type = Dot;
				state = Done;
			}
			else if ( currentChar == _T('=') )
			{
				lexeme += currentChar;
				token.Type = Equal;
				state = Done;
			}
			else if ( currentChar == _T('(') )
			{
				lexeme += currentChar;
				state = InOpenBracket;
			}
			else if ( currentChar == _T(')') )
			{
				lexeme += currentChar;
				token.Type = CloseBracket;
				state = Done;
			}
			else if ( currentChar == _T('[') )
			{
				lexeme += currentChar;
				token.Type = OpenSquareBracket;
				state = Done;
			}
			else if ( currentChar == _T(']') )
			{
				lexeme += currentChar;
				token.Type = CloseSquareBracket;
				state = Done;
			}
			else if ( currentChar == _T('|') )
			{
				lexeme += currentChar;
				token.Type = Or;
				state = Done;
			}
			else if ( currentChar == _T('<') )
			{
				currentChar = getNextCharachter();
				if (currentChar == _T('.') )
					state = InSemAction;
				else
				{
					state = ErrorState;
					token.Type = Error;
				}
			}
			else if ( currentChar == _T('{') )
			{
				lexeme += currentChar;
				token.Type = OpenCurly;
				state = Done;
			}
			else if ( currentChar == _T('}') )
			{
				lexeme += currentChar;
				token.Type = CloseCurly;
				state = Done;
			}
			else if ( (currentChar == _T(' ')) || (currentChar == _T('\t')) || (currentChar == _T('\n')) || currentChar == _T('\r') )
				continue; //just skip
			else if ( currentChar == (TCHAR)0 )
			{
				token.Type = EndOfFile;
				state = Done;
			}
			else if ( currentChar == _T('/') )
			{
				currentChar = getNextCharachter();
				if (currentChar == _T('/') ) // one line comment
				{
					currentChar = getNextCharachter();
					while (currentChar != _T('\r') && currentChar != _T('\n') && currentChar != (TCHAR)0)
						currentChar = getNextCharachter();
				}
				else if (currentChar == _T('*') )
				{
					bool isClosing = 0;
					currentChar = getNextCharachter();
					while (!isClosing)
					{
						while (currentChar != _T('*') && currentChar != (TCHAR)0)
							currentChar = getNextCharachter();
						if ( currentChar == (TCHAR)0 )
						{
							token.Type = Error;
							state = ErrorState;
							isClosing = 1;
						}
						else //it is *
						{
							currentChar = getNextCharachter();
							if (currentChar == _T('/'))
								isClosing = 1;
						}
					}
				}
			}
			else if ( currentChar == _T('%') )
			{
				currentChar = getNextCharachter();
				if ( currentChar == _T('%') )
					state = InBulkCode;
				else
				{
					token.Type = Error;
					state = ErrorState;
				}
			}
			else
			{
				token.Type = Error;
				state = ErrorState;
			}
			token.LineNo = currentLineNum;
			token.ColNo = currentColNo - 1; // -1 because I have read the char actually and the token begins one before it
			break;
		case InIdent:
			while ( _istalpha(currentChar) || _istdigit(currentChar) || currentChar == _T('_'))
			{
				lexeme += currentChar;
				currentChar = getNextCharachter();
			}
			if ( currentChar != (TCHAR)0 && currentChar != _T('\n'))
				ungetCharachter();
			token.Type = Identifier;
			state = Done;

			break;
		case InString:
			while ( currentChar != _T('"') )
			{
				lexeme += currentChar;
				currentChar = getNextCharachter();
			}
			if ( currentChar != (TCHAR)0 )
			{
				//we didn't call ungetCharachter() because we want to eat "
				token.Type = String;
				state = Done;
			}
			else
			{
				token.Type = Error;
				state = ErrorState;
			}
			break;
		case InChar:
			if (currentChar == _T('\\') )
			{
				lexeme += currentChar;
				currentChar = getNextCharachter();
			}
			lexeme += currentChar;
			currentChar = getNextCharachter();
			
			if (currentChar == _T('\''))
			{
				token.Type = Character;
				state = Done;
			}
			else
			{
				token.Type = Error;
				state = ErrorState;
			}
			break;
		case InProduction:
			if (currentChar == _T('>'))
			{
				lexeme += currentChar;
				token.Type = Production;
				state = Done;
			}
			else
			{
				token.Type = Error;
				state = ErrorState;
			}
			break;
		case InOpenBracket:
			if (currentChar == _T('.'))
			{
				lexeme = _T("");
				token.Type = Attributes;
				state = AttributesState;
			}
			else 
			{
				if ( currentChar != (TCHAR)0 && currentChar != _T('\n'))
					ungetCharachter();
				token.Type = OpenBracket;
				state = Done;
			}
			break;
		case InSemAction:
			isFinished = false;
			temp = _T('.');
			token.Type = SemAction;
			while(!isFinished)
			{
				if (currentChar == _T('.'))
				{
					currentChar = getNextCharachter();
					if (currentChar == _T('>'))
					{
						isFinished = true;
						state = Done;
					}
					else if (currentChar == (TCHAR)0)
					{
						isFinished = true;
						state = ErrorState;
						token.Type = Error;
					}
					else
					{
						lexeme += temp;
						lexeme += currentChar;
					}
				}
				else if (currentChar == (TCHAR)0)
				{
					isFinished = true;
					state = ErrorState;
					token.Type = Error;
				}
				else
					lexeme += currentChar;
			if (!isFinished)
				currentChar = getNextCharachter();
			}
			break;
		case InBulkCode:
			isFinished = false;
			temp = _T('%');
			token.Type = BulkOfCode;
			while(!isFinished)
			{
				if (currentChar == _T('%'))
				{
					currentChar = getNextCharachter();
					if (currentChar == _T('%'))
					{
						isFinished = true;
						state = Done;
					}
					else if (currentChar == (TCHAR)0)
					{
						isFinished = true;
						state = ErrorState;
						token.Type = Error;
					}
					else
					{
						lexeme += temp;
						lexeme += currentChar;
					}
				}
				else if (currentChar == (TCHAR)0)
				{
					isFinished = true;
					state = ErrorState;
					token.Type = Error;
				}
				else
					lexeme += currentChar;
			if (!isFinished)
				currentChar = getNextCharachter();
			}
			break;
		case AttributesState:
			isFinished = false;
			temp = _T('.');
			while(!isFinished)
			{
				if (currentChar == _T('.'))
				{
					currentChar = getNextCharachter();
					if (currentChar == _T(')'))
					{
						isFinished = true;
						state = Done;
					}
					else if (currentChar == (TCHAR)0)
					{
						isFinished = true;
						state = ErrorState;
						token.Type = Error;
					}
					else
					{
						lexeme += temp;
						lexeme += currentChar;
					}
				}
				else if (currentChar == (TCHAR)0)
				{
					isFinished = true;
					state = ErrorState;
					token.Type = Error;
				}
				else
					lexeme += currentChar;
			if (!isFinished)
				currentChar = getNextCharachter();
			}
			break;
		}
	}
	if (state == Done && token.Type == Identifier)
		token.Type = reservedLookUp(lexeme);
	token.Lexeme = lexeme;
	return token;
}

Scanner::~Scanner()
{
	delete[] buffer;
	delete file;
	reservedWords.clear();
}

TCHAR Scanner::getNextCharachter()
{
	TCHAR currentChar = (TCHAR)0;
	if (currentCharPosition < bufferSize)
		currentChar = buffer[currentCharPosition++];
	else if ( !file->eof() )
	{
		file->read(buffer, maxCharsToRead);
		bufferSize = file->gcount();
		currentChar = buffer[0];
		currentCharPosition = 1;
	}

	if ( currentChar == _T('\n') )
	{
		currentLineNum++;
		currentColNo = 1; 
	}
	else if ( currentChar == _T('\t') )
		currentColNo += TabSize;
	else if ( currentChar != ((TCHAR)0) )
		currentColNo++;
	
	return currentChar;
}

void Scanner::ungetCharachter()
{
	if (currentCharPosition > 0)
	{
		currentCharPosition--;
		currentColNo--;
	}
}

void Scanner::initializeReservedKeywords()
{
	reservedWords.clear();
	reservedWords[_T("Tokens")] = Tokens;
	reservedWords[_T("Options")] = Options;
	reservedWords[_T("NameSpace")] = NameSpace;
	reservedWords[_T("ClassName")] = ClassName;
	reservedWords[_T("Language")] = Language;
	reservedWords[_T("TopOfDeclaration")] = DeclTop;
	reservedWords[_T("BottomOfDeclaration")] = DeclBottom;
	reservedWords[_T("TopOfDefinition")] = DefTop;
	reservedWords[_T("BottomOfDefinition")] = DefBottom;
	reservedWords[_T("CPlusPlus")] = CPlusPlus;
	reservedWords[_T("CSharp")] = CSharp;
	reservedWords[_T("JAVA")] = Java;
	reservedWords[_T("Grammar")] = Grammar;
	reservedWords[_T("WEAK")] = Weak;
	reservedWords[_T("ANY")] = Any;
	reservedWords[_T("SYNC")] = Sync;
	reservedWords[_T("IF")] = If;
	reservedWords[_T("Eps")] = Eps;
}

TokenType Scanner::reservedLookUp(tstring lexeme)
{
	map<tstring, TokenType>::const_iterator iter;
	for(iter = reservedWords.begin(); iter != reservedWords.end(); iter++)
		if ( (*iter).first == lexeme )
			return (*iter).second;
	return Identifier;
}