using System;
using System.IO;
using System.Xml;
using System.Data;
using System.Drawing;
using System.Collections;
using System.Drawing.Text;
using System.Windows.Forms;
using System.ComponentModel;
using System.Drawing.Drawing2D;
using System.Collections.Generic;

using Crownwood.DotNetMagic.Menus;
using Crownwood.DotNetMagic.Common;
using Crownwood.DotNetMagic.Docking;
using Crownwood.DotNetMagic.Controls;
using Crownwood.DotNetMagic.Controls.Command;

using FireEdit.Dialogs;
using System.Diagnostics;
using System.Text;

using GTG;
using System.Runtime.Serialization.Formatters.Binary;
using Errors;
using IDE.Forms.CCW;

namespace CCW
{
    /// <summary>
    /// The Main MDI Form.
    /// </summary>
    public class Interface : System.Windows.Forms.Form
    {
        [Serializable]
        class ProjectFile
        {
            public DateTime Created;
            public string Path;
            public string Name;
            public FileLanguage Language;

            public ProjectFile(DateTime created, string path, string name, FileLanguage language)
            {
                Language = language;
                Path = path;
                Name = name;
                Created = created;
            }
        }

        [Serializable]
        class ProjectFolder
        {
            public DateTime Created;
            public string Path;
            public string Name;
            public List<ProjectFile> Files;

            public ProjectFolder(DateTime created, string path, string name)
            {
                Path = path;
                Name = name;
                Created = created;

                Files = new List<ProjectFile>();
            }
        }

        [Serializable]
        class Project
        {
            public DateTime Created;
            public string Path;
            public string Name;
            public string FolderName;

            public List<ProjectFile> LexicalAnalyserSpecificationFiles;
            public List<ProjectFile> ParserSpecificationFiles;
            public List<ProjectFile> GraphicalRegularLanguageSpecificationFiles;
            public List<ProjectFile> LexicalAnalyserCodeFiles;
            public List<ProjectFile> ParserCodeFiles;
            public List<ProjectFile> MiscellaneousFiles;

            public List<ProjectFolder> RootFolders;

            public Project(DateTime created, string path, string name, string folderName)
            {
                Path = path;
                Created = created;
                Name = name;
                FolderName = folderName;

                LexicalAnalyserCodeFiles = new List<ProjectFile>();
                ParserCodeFiles = new List<ProjectFile>();
                MiscellaneousFiles = new List<ProjectFile>();
                LexicalAnalyserSpecificationFiles = new List<ProjectFile>();
                ParserSpecificationFiles = new List<ProjectFile>();
                GraphicalRegularLanguageSpecificationFiles = new List<ProjectFile>();

                RootFolders = new List<ProjectFolder>();

                if (!Directory.Exists(path + "\\" + folderName + "\\Specification Files"))
                    Directory.CreateDirectory(path + "\\" + folderName + "\\Specification Files");
                if (!Directory.Exists(path + "\\" + folderName + "\\Miscellaneous Files"))
                    Directory.CreateDirectory(path + "\\" + folderName + "\\Miscellaneous Files");
                if (!Directory.Exists(path + "\\" + folderName + "\\Specification Files\\Parsers"))
                    Directory.CreateDirectory(path + "\\" + folderName + "\\Specification Files\\Parsers");
                if (!Directory.Exists(path + "\\" + folderName + "\\Specification Files\\Lexical Analysers"))
                    Directory.CreateDirectory(path + "\\" + folderName + "\\Specification Files\\Lexical Analysers");
                if (!Directory.Exists(path + "\\" + folderName + "\\Specification Files\\Graphical Regular Language Specifications"))
                    Directory.CreateDirectory(path + "\\" + folderName + "\\Specification Files\\Graphical Regular Language Specifications");
                if (!Directory.Exists(path + "\\" + folderName + "\\Code Files"))
                    Directory.CreateDirectory(path + "\\" + folderName + "\\Code Files");
                if (!Directory.Exists(path + "\\" + folderName + "\\Code Files\\Lexical Analysers"))
                    Directory.CreateDirectory(path + "\\" + folderName + "\\Code Files\\Lexical Analysers");
                if (!Directory.Exists(path + "\\" + folderName + "\\Code Files\\Parsers"))
                    Directory.CreateDirectory(path + "\\" + folderName + "\\Code Files\\Parsers");
                if (!File.Exists(path + "\\" + folderName + "\\" + folderName + ".ccwproj"))
                    File.Create(path + "\\" + folderName + "\\" + folderName + ".ccwproj");
            }
        }

        enum FileLanguage
        {
            LexCSharp,
            LexCPlusPlus,
            LexJava,
            ParCSharp,
            ParCPlusPlus,
            ParJava,
			Gemix,
            CPlusPlus,
            CSharp,
            Java,
            None
        }

        #region Instance Fields
        private bool allowContextMenu, customContextMenu, captionBars, closeButtons;
        private int ignoreClose;
        private Panel drawingPanel;
        private Project currentProject;
        private Hashtable tabPageToFileInProject, tabPageToFileNotInProject, fileNotInProjectToTabPage, fileInProjectToTabPage;
        private TreeControl projectExplorer;
        private DockingManager dockingManager;
        private Content projectExplorerWindow, outputWindow, taskListWindow;
        private ImageList images;
        #endregion

        #region Designer Generated Fields
        public Crownwood.DotNetMagic.Controls.TabbedGroups tabbedgroups_Main;

        private Crownwood.DotNetMagic.Controls.ToolControl mainToolBar;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnNew;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnOpen;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnSave;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnSaveAs;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnPrint;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnPrintPreview;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnPageSetup;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnCut;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnCopy;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnPaste;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnUndo;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnRedo;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnFind;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnFindNext;
        private Crownwood.DotNetMagic.Controls.Command.ButtonCommand btnReplace;

        private Crownwood.DotNetMagic.Controls.StatusBarControl statusbarcontrol_Main;
        private Crownwood.DotNetMagic.Controls.StatusPanel statuspanel_Description;
        private Crownwood.DotNetMagic.Controls.StatusPanel statuspanel_Date;

        private Crownwood.DotNetMagic.Menus.MenuControl menuMainFunctionality;

        private Crownwood.DotNetMagic.Controls.Command.SeparatorCommand separatorCommand;

        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Separator;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_Project;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_LexicalAnalyserSpecificationFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_Java;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_LexicalAnalyserSpecificationFile_WizardAssistedFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_GraphicalRegularLanguageSpecification;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_ParserSpecificationFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_ParserSpecificationFile_BlankFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_ParserSpecificationFile_BlankFile_CSharp;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_ParserSpecificationFile_BlankFile_CPlusPlus;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_ParserSpecificationFile_BlankFile_Java;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_ParserSpecificationFile_WizardAssistedFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_New_File_Other;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Open;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Open_File;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Open_File_LexicalAnalyserSpecificationFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Open_File_LexicalAnalyserSpecificationFile_Java;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CPlusPlus;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CSharp;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Open_File_GraphicalRegularLanguageSpecificationFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Open_File_OtherFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Open_Project;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Open_File_ParserSpecificationFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Open_File_ParserSpecificationFile_Java;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Open_File_ParserSpecificationFile_CPlusPlus;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Open_File_ParserSpecificationFile_CSharp;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Close;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_CloseProject;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Save;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Save_File;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Save_Project;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_SaveAs;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_SaveAs_NewFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_SaveAs_NewProject;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_SaveAll;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_PrintPreview;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_PageSetup;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Print;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_RecentFiles;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_RecentProjects;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_File_Exit;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Edit;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Edit_Undo;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Edit_Redo;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Edit_Cut;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Edit_Copy;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Edit_Paste;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Edit_Delete;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Edit_SelectAll;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Edit_FindAndReplace;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Edit_GoTo;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Windows;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Windows_ProjectExplorer;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Windows_Output;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Windows_TaskList;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Windows_ShowAllWindows;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Windows_HideAllWindows;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_Folder;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_ParserSpecificationFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CSharp;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CPlusPlus;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_Java;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_ParserSpecificationFile_WizardAssistedFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_Java;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_WizardAssistedFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_GraphicalRegularLanguageSpecification;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddNewItem_File_OtherFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddExistingItem;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddExistingItem_LexicalAnalyserFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CSharp;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CPlusPlus;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_Java;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddExistingItem_GraphicalRegularLanguageSpecificationFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddExistingItem_ParserSpecificationFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddExistingItem_ParserSpecificationFile_CSharp;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddExistingItem_ParserSpecificationFile_CPlusPlus;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddExistingItem_ParserSpecificationFile_Java;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_AddExistingItem_OtherFile;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_ShowAllFiles;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_Refresh;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_IncludeInProject;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_ExcludeFromProject;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Project_Properties;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Tools;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Tools_SyntaxOptions;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Tools_LeftFactoring;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Tools_LeftFactoring_BNF;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Tools_LeftFactoring_EBNF;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Tools_LeftRecursionRemoval;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Tools_LEXcellent;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Tools_GTGEditor;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Tools_RegExManipulator;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Tools_ParSpring;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Tools_ParSpring_RDParser;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Tools_ParSpring_LL1Parser;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Help;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Help_Contents;
		private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Help_Samples;
		private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Help_Samples_ArabicCLanguage;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Help_AboutLEXcellent;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Help_AboutParSpring;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Help_AboutCCW;
        private Crownwood.DotNetMagic.Menus.MenuCommand menucommand_Help_Acknowledgements;

        private System.Windows.Forms.Timer timer;

        private System.ComponentModel.IContainer components;
        #endregion

        #region The Main Form
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Form mainForm = new Interface();
            Application.Run(mainForm);
        }

        public Interface()
        {
            // Reduce the amount of flicker that occurs when windows are redocked within
            // the container. As this prevents unsightly backcolors being drawn in the
            // WM_ERASEBACKGROUND that seems to occur.
            SetStyle(ControlStyles.DoubleBuffer, true);
            SetStyle(ControlStyles.AllPaintingInWmPaint, true);

            // Initialize the Hashtable
            tabPageToFileInProject = new Hashtable();
            tabPageToFileNotInProject = new Hashtable();
            fileInProjectToTabPage = new Hashtable();
            fileNotInProjectToTabPage = new Hashtable();

            // Initialize instance fields
            allowContextMenu = true;
            customContextMenu = false;
            captionBars = true;
            closeButtons = true;
            ignoreClose = 0;

            // Required for Windows Form Designer support
            InitializeComponent();

            // Update the date and time in status bar immediately
            timer_Tick(this, EventArgs.Empty);

            // Setup the docking manager instance
            dockingManager = new DockingManager(this, VisualStyle.Office2003);
            dockingManager.CaptionFont = new Font("Century Gothic", 10);

            // Hook into the events generated by the docking manager
            dockingManager.ContextMenu += new DockingManager.ContextMenuHandler(OnContextMenu);
            dockingManager.ContentShown += new DockingManager.ContentHandler(OnContentShown);
            dockingManager.ContentHidden += new DockingManager.ContentHandler(OnContentHidden);
            dockingManager.ContentHiding += new DockingManager.ContentHidingHandler(OnContentHiding);

            // Setup custom config handling
            dockingManager.SaveCustomConfig += new DockingManager.SaveCustomConfigHandler(OnSaveConfig);
            dockingManager.LoadCustomConfig += new DockingManager.LoadCustomConfigHandler(OnLoadConfig);

            // Create the initial starting docking windows			
            CreateProjectExplorerContent(this, EventArgs.Empty);
            CreateTaskListContent(this, EventArgs.Empty);
            CreateOutputContent(this, EventArgs.Empty);

            dockingManager.ToggleContentAutoHide(dockingManager.Contents["Output"]);
            dockingManager.ToggleContentAutoHide(dockingManager.Contents["Task List"]);
            dockingManager.ToggleContentAutoHide(dockingManager.Contents["Project Explorer"]);
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }
        #endregion

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(Interface));
            this.mainToolBar = new Crownwood.DotNetMagic.Controls.ToolControl();
            this.btnNew = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.btnOpen = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.btnSave = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.btnSaveAs = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.separatorCommand = new Crownwood.DotNetMagic.Controls.Command.SeparatorCommand();
            this.btnPrint = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.btnPrintPreview = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.btnPageSetup = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.btnCut = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.btnCopy = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.btnPaste = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.btnUndo = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.btnRedo = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.btnFind = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.btnFindNext = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.btnReplace = new Crownwood.DotNetMagic.Controls.Command.ButtonCommand();
            this.menuMainFunctionality = new Crownwood.DotNetMagic.Menus.MenuControl();
            this.menucommand_File = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_Project = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_Java = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_WizardAssistedFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_ParserSpecificationFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_CSharp = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_CPlusPlus = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_Java = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_ParserSpecificationFile_WizardAssistedFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_GraphicalRegularLanguageSpecification = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_New_File_Other = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Open = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Open_Project = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Open_File = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CSharp = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CPlusPlus = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_Java = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Open_File_ParserSpecificationFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Open_File_ParserSpecificationFile_CSharp = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Open_File_ParserSpecificationFile_CPlusPlus = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Open_File_ParserSpecificationFile_Java = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Open_File_GraphicalRegularLanguageSpecificationFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Open_File_OtherFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Separator = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Close = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_CloseProject = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Save = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Save_Project = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Save_File = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_SaveAs = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_SaveAs_NewProject = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_SaveAs_NewFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_SaveAll = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_PageSetup = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_PrintPreview = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Print = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_RecentFiles = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_RecentProjects = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_File_Exit = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Edit = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Edit_Undo = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Edit_Redo = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Edit_Cut = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Edit_Copy = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Edit_Paste = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Edit_Delete = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Edit_SelectAll = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Edit_FindAndReplace = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Edit_GoTo = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Windows = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Windows_ProjectExplorer = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Windows_TaskList = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Windows_Output = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Windows_ShowAllWindows = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Windows_HideAllWindows = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_Folder = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_Java = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_WizardAssistedFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CSharp = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CPlusPlus = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_Java = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_WizardAssistedFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_GraphicalRegularLanguageSpecification = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddNewItem_File_OtherFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddExistingItem = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CSharp = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CPlusPlus = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_Java = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_CSharp = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_CPlusPlus = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_Java = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddExistingItem_GraphicalRegularLanguageSpecificationFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_AddExistingItem_OtherFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_IncludeInProject = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_ExcludeFromProject = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_ShowAllFiles = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_Refresh = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Project_Properties = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Tools = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Tools_LEXcellent = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Tools_ParSpring = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Tools_ParSpring_RDParser = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Tools_ParSpring_LL1Parser = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Tools_LeftRecursionRemoval = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Tools_LeftFactoring = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Tools_LeftFactoring_BNF = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Tools_LeftFactoring_EBNF = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Tools_GTGEditor = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Tools_RegExManipulator = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Tools_SyntaxOptions = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Help = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Help_Contents = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Help_Samples = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Help_Samples_ArabicCLanguage = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Help_AboutLEXcellent = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Help_AboutParSpring = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Help_AboutCCW = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.menucommand_Help_Acknowledgements = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.images = new System.Windows.Forms.ImageList(this.components);
            this.tabbedgroups_Main = new Crownwood.DotNetMagic.Controls.TabbedGroups();
            this.menucommand_Project_AddExistingItem_LexicalAnalyserFile = new Crownwood.DotNetMagic.Menus.MenuCommand();
            this.statusbarcontrol_Main = new Crownwood.DotNetMagic.Controls.StatusBarControl();
            this.statuspanel_Description = new Crownwood.DotNetMagic.Controls.StatusPanel();
            this.statuspanel_Date = new Crownwood.DotNetMagic.Controls.StatusPanel();
            this.timer = new System.Windows.Forms.Timer(this.components);
            ((System.ComponentModel.ISupportInitialize)(this.mainToolBar)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.tabbedgroups_Main)).BeginInit();
            this.SuspendLayout();
            // 
            // mainToolBar
            // 
            this.mainToolBar.Commands.AddRange(new Crownwood.DotNetMagic.Controls.Command.CommandBase[] {
            this.btnNew,
            this.btnOpen,
            this.btnSave,
            this.btnSaveAs,
            this.separatorCommand,
            this.btnPrint,
            this.btnPrintPreview,
            this.btnPageSetup,
            this.separatorCommand,
            this.btnCut,
            this.btnCopy,
            this.btnPaste,
            this.separatorCommand,
            this.btnUndo,
            this.btnRedo,
            this.separatorCommand,
            this.btnFind,
            this.btnFindNext,
            this.btnReplace,
            this.separatorCommand});
            this.mainToolBar.EqualButtonHorz = true;
            this.mainToolBar.ImageAndText = Crownwood.DotNetMagic.Controls.Command.ImageAndText.ImageOnly;
            this.mainToolBar.Location = new System.Drawing.Point(0, 25);
            this.mainToolBar.Name = "mainToolBar";
            this.mainToolBar.Size = new System.Drawing.Size(492, 26);
            this.mainToolBar.TabIndex = 9;
            // 
            // btnNew
            // 
            this.btnNew.Image = ((System.Drawing.Image)(resources.GetObject("btnNew.Image")));
            this.btnNew.Tag = null;
            this.btnNew.Text = "";
            // 
            // btnOpen
            // 
            this.btnOpen.Image = ((System.Drawing.Image)(resources.GetObject("btnOpen.Image")));
            this.btnOpen.Tag = null;
            this.btnOpen.Text = "";
            // 
            // btnSave
            // 
            this.btnSave.Image = ((System.Drawing.Image)(resources.GetObject("btnSave.Image")));
            this.btnSave.Tag = null;
            this.btnSave.Text = "";
            this.btnSave.Tooltip = "Save";
            this.btnSave.Click += new System.EventHandler(this.menucommand_File_Save_File_Click);
            // 
            // btnSaveAs
            // 
            this.btnSaveAs.Image = ((System.Drawing.Image)(resources.GetObject("btnSaveAs.Image")));
            this.btnSaveAs.Tag = null;
            this.btnSaveAs.Text = "";
            this.btnSaveAs.Tooltip = "Save As";
            this.btnSaveAs.Click += new System.EventHandler(this.menucommand_File_SaveAs_NewFile_Click);
            // 
            // separatorCommand
            // 
            this.separatorCommand.Tag = null;
            // 
            // btnPrint
            // 
            this.btnPrint.Image = ((System.Drawing.Image)(resources.GetObject("btnPrint.Image")));
            this.btnPrint.Tag = null;
            this.btnPrint.Text = "";
            this.btnPrint.Tooltip = "Print";
            // 
            // btnPrintPreview
            // 
            this.btnPrintPreview.Image = ((System.Drawing.Image)(resources.GetObject("btnPrintPreview.Image")));
            this.btnPrintPreview.Tag = null;
            this.btnPrintPreview.Text = "";
            this.btnPrintPreview.Tooltip = "Print Preview";
            // 
            // btnPageSetup
            // 
            this.btnPageSetup.Image = ((System.Drawing.Image)(resources.GetObject("btnPageSetup.Image")));
            this.btnPageSetup.Tag = null;
            this.btnPageSetup.Text = "";
            this.btnPageSetup.Tooltip = "Page Setup";
            // 
            // btnCut
            // 
            this.btnCut.Image = ((System.Drawing.Image)(resources.GetObject("btnCut.Image")));
            this.btnCut.Tag = null;
            this.btnCut.Text = "";
            this.btnCut.Tooltip = "Cut";
            this.btnCut.Click += new System.EventHandler(this.menucommand_Edit_Cut_Click);
            // 
            // btnCopy
            // 
            this.btnCopy.Image = ((System.Drawing.Image)(resources.GetObject("btnCopy.Image")));
            this.btnCopy.Tag = null;
            this.btnCopy.Text = "";
            this.btnCopy.Tooltip = "Copy";
            this.btnCopy.Click += new System.EventHandler(this.menucommand_Edit_Copy_Click);
            // 
            // btnPaste
            // 
            this.btnPaste.Image = ((System.Drawing.Image)(resources.GetObject("btnPaste.Image")));
            this.btnPaste.Tag = null;
            this.btnPaste.Text = "";
            this.btnPaste.Tooltip = "Paste";
            this.btnPaste.Click += new System.EventHandler(this.menucommand_Edit_Paste_Click);
            // 
            // btnUndo
            // 
            this.btnUndo.Image = ((System.Drawing.Image)(resources.GetObject("btnUndo.Image")));
            this.btnUndo.Tag = null;
            this.btnUndo.Text = "";
            this.btnUndo.Tooltip = "Undo";
            this.btnUndo.Click += new System.EventHandler(this.menucommand_Edit_Undo_Click);
            // 
            // btnRedo
            // 
            this.btnRedo.Image = ((System.Drawing.Image)(resources.GetObject("btnRedo.Image")));
            this.btnRedo.Tag = null;
            this.btnRedo.Text = "";
            this.btnRedo.Tooltip = "Redo";
            this.btnRedo.Click += new System.EventHandler(this.menucommand_Edit_Redo_Click);
            // 
            // btnFind
            // 
            this.btnFind.Image = ((System.Drawing.Image)(resources.GetObject("btnFind.Image")));
            this.btnFind.Tag = null;
            this.btnFind.Text = "";
            this.btnFind.Tooltip = "Find";
            // 
            // btnFindNext
            // 
            this.btnFindNext.Image = ((System.Drawing.Image)(resources.GetObject("btnFindNext.Image")));
            this.btnFindNext.Tag = null;
            this.btnFindNext.Text = "";
            this.btnFindNext.Tooltip = "Find Next";
            // 
            // btnReplace
            // 
            this.btnReplace.Image = ((System.Drawing.Image)(resources.GetObject("btnReplace.Image")));
            this.btnReplace.Tag = null;
            this.btnReplace.Text = "";
            this.btnReplace.Tooltip = "Replace";
            // 
            // menuMainFunctionality
            // 
            this.menuMainFunctionality.AllowLayered = true;
            this.menuMainFunctionality.AnimateStyle = Crownwood.DotNetMagic.Menus.Animation.System;
            this.menuMainFunctionality.AnimateTime = 500;
            this.menuMainFunctionality.Cursor = System.Windows.Forms.Cursors.Arrow;
            this.menuMainFunctionality.Direction = Crownwood.DotNetMagic.Common.LayoutDirection.Horizontal;
            this.menuMainFunctionality.Dock = System.Windows.Forms.DockStyle.Top;
            this.menuMainFunctionality.Font = new System.Drawing.Font("Tahoma", 11F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.World);
            this.menuMainFunctionality.HighlightTextColor = System.Drawing.SystemColors.MenuText;
            this.menuMainFunctionality.IgnoreF10 = false;
            this.menuMainFunctionality.Location = new System.Drawing.Point(0, 0);
            this.menuMainFunctionality.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File,
            this.menucommand_Edit,
            this.menucommand_Windows,
            this.menucommand_Project,
            this.menucommand_Tools,
            this.menucommand_Help});
            this.menuMainFunctionality.Name = "menuMainFunctionality";
            this.menuMainFunctionality.Size = new System.Drawing.Size(492, 25);
            this.menuMainFunctionality.Style = Crownwood.DotNetMagic.Common.VisualStyle.Office2003;
            this.menuMainFunctionality.TabIndex = 0;
            this.menuMainFunctionality.TabStop = false;
            this.menuMainFunctionality.Text = "Main Functionality";
            this.menuMainFunctionality.Selected += new Crownwood.DotNetMagic.Menus.CommandHandler(this.menuMainFunctionality_Selected);
            this.menuMainFunctionality.Deselected += new Crownwood.DotNetMagic.Menus.CommandHandler(this.menuMainFunctionality_Deselected);
            // 
            // menucommand_File
            // 
            this.menucommand_File.Description = "Common file operations";
            this.menucommand_File.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File_New,
            this.menucommand_File_Open,
            this.menucommand_Separator,
            this.menucommand_File_Close,
            this.menucommand_File_CloseProject,
            this.menucommand_Separator,
            this.menucommand_File_Save,
            this.menucommand_File_SaveAs,
            this.menucommand_File_SaveAll,
            this.menucommand_Separator,
            this.menucommand_File_PageSetup,
            this.menucommand_File_PrintPreview,
            this.menucommand_File_Print,
            this.menucommand_Separator,
            this.menucommand_File_RecentFiles,
            this.menucommand_File_RecentProjects,
            this.menucommand_Separator,
            this.menucommand_File_Exit});
            this.menucommand_File.Text = "&File";
            // 
            // menucommand_File_New
            // 
            this.menucommand_File_New.Description = "Common operations for dealing with new items";
            this.menucommand_File_New.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File_New_Project,
            this.menucommand_File_New_File});
            this.menucommand_File_New.Text = "New";
            // 
            // menucommand_File_New_Project
            // 
            this.menucommand_File_New_Project.Description = "Create a new project";
            this.menucommand_File_New_Project.Text = "Project...";
            this.menucommand_File_New_Project.Click += new System.EventHandler(this.menucommand_File_New_Project_Click);
            // 
            // menucommand_File_New_File
            // 
            this.menucommand_File_New_File.Description = "Create a new file";
            this.menucommand_File_New_File.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile,
            this.menucommand_File_New_File_ParserSpecificationFile,
            this.menucommand_File_New_File_GraphicalRegularLanguageSpecification,
            this.menucommand_File_New_File_Other});
            this.menucommand_File_New_File.Text = "File";
            // 
            // menucommand_File_New_File_LexicalAnalyserSpecificationFile
            // 
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile.Description = "New lexical analyzer specification file";
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile,
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_WizardAssistedFile});
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile.Text = "Lexical Analyser Specification File";
            // 
            // menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile
            // 
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile.Description = "New blank lexical analyzer specification file";
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp,
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus,
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_Java});
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile.Text = "Blank File";
            // 
            // menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp
            // 
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp.Description = "New blank lexical analyzer specification file, C# used for coding";
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp.Text = "C#...";
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp.Click += new System.EventHandler(this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp_Click);
            // 
            // menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus
            // 
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus.Description = "New blank lexical analyzer specification file, C++ used for coding";
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus.Text = "C++...";
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus.Click += new System.EventHandler(this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus_Click);
            // 
            // menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_Java
            // 
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_Java.Description = "New blank lexical analyzer specification file, Java used for coding";
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_Java.Text = "Java...";
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_Java.Click += new System.EventHandler(this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_Java_Click);
            // 
            // menucommand_File_New_File_LexicalAnalyserSpecificationFile_WizardAssistedFile
            // 
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_WizardAssistedFile.Description = "Use the regular expression builder to create a set of regular expressions";
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_WizardAssistedFile.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_File_New_File_LexicalAnalyserSpecificationFile_WizardAssistedFile.Ima" +
                    "ge")));
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_WizardAssistedFile.Text = "Wizard-Assisted File...";
            this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_WizardAssistedFile.Click += new System.EventHandler(this.menucommand_File_New_File_LexicalAnalyserSpecificationFile_WizardAssistedFile_Click);
            // 
            // menucommand_File_New_File_ParserSpecificationFile
            // 
            this.menucommand_File_New_File_ParserSpecificationFile.Description = "New CFG file";
            this.menucommand_File_New_File_ParserSpecificationFile.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile,
            this.menucommand_File_New_File_ParserSpecificationFile_WizardAssistedFile});
            this.menucommand_File_New_File_ParserSpecificationFile.Text = "Parser Specification File";
            // 
            // menucommand_File_New_File_ParserSpecificationFile_BlankFile
            // 
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile.Description = "New blank CFG file";
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_CSharp,
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_CPlusPlus,
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_Java});
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile.Text = "Blank File";
            // 
            // menucommand_File_New_File_ParserSpecificationFile_BlankFile_CSharp
            // 
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_CSharp.Description = "New blank CFG file, C# used for coding";
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_CSharp.Text = "C#...";
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_CSharp.Click += new System.EventHandler(this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_CSharp_Click);
            // 
            // menucommand_File_New_File_ParserSpecificationFile_BlankFile_CPlusPlus
            // 
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_CPlusPlus.Description = "New blank CFG file, C++ used for coding";
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_CPlusPlus.Text = "C++...";
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_CPlusPlus.Click += new System.EventHandler(this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_CPlusPlus_Click);
            // 
            // menucommand_File_New_File_ParserSpecificationFile_BlankFile_Java
            // 
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_Java.Description = "New blank CFG file, Java used for coding";
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_Java.Text = "Java...";
            this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_Java.Click += new System.EventHandler(this.menucommand_File_New_File_ParserSpecificationFile_BlankFile_Java_Click);
            // 
            // menucommand_File_New_File_ParserSpecificationFile_WizardAssistedFile
            // 
            this.menucommand_File_New_File_ParserSpecificationFile_WizardAssistedFile.Description = "Use the CFG builder to create a context-free grammar";
            this.menucommand_File_New_File_ParserSpecificationFile_WizardAssistedFile.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_File_New_File_ParserSpecificationFile_WizardAssistedFile.Image")));
            this.menucommand_File_New_File_ParserSpecificationFile_WizardAssistedFile.Text = "Wizard-Assisted File...";
            // 
            // menucommand_File_New_File_GraphicalRegularLanguageSpecification
            // 
            this.menucommand_File_New_File_GraphicalRegularLanguageSpecification.Description = "New graphical regular language specification file";
            this.menucommand_File_New_File_GraphicalRegularLanguageSpecification.Text = "Graphical Regular Language Specification File...";
            this.menucommand_File_New_File_GraphicalRegularLanguageSpecification.Click += new System.EventHandler(this.menucommand_File_New_File_GraphicalRegularLanguageSpecification_Click);
            // 
            // menucommand_File_New_File_Other
            // 
            this.menucommand_File_New_File_Other.Description = "New file of any type";
            this.menucommand_File_New_File_Other.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_File_New_File_Other.Image")));
            this.menucommand_File_New_File_Other.Text = "Other File...";
            this.menucommand_File_New_File_Other.Click += new System.EventHandler(this.menucommand_File_New_File_Other_Click);
            // 
            // menucommand_File_Open
            // 
            this.menucommand_File_Open.Description = "Common operations for dealing with existing items";
            this.menucommand_File_Open.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File_Open_Project,
            this.menucommand_File_Open_File});
            this.menucommand_File_Open.Text = "Open";
            // 
            // menucommand_File_Open_Project
            // 
            this.menucommand_File_Open_Project.Description = "Open an existing project";
            this.menucommand_File_Open_Project.Text = "Project...";
            this.menucommand_File_Open_Project.Click += new System.EventHandler(this.menucommand_File_Open_Project_Click);
            // 
            // menucommand_File_Open_File
            // 
            this.menucommand_File_Open_File.Description = "Open an existing file";
            this.menucommand_File_Open_File.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile,
            this.menucommand_File_Open_File_ParserSpecificationFile,
            this.menucommand_File_Open_File_GraphicalRegularLanguageSpecificationFile,
            this.menucommand_File_Open_File_OtherFile});
            this.menucommand_File_Open_File.Text = "File";
            // 
            // menucommand_File_Open_File_LexicalAnalyserSpecificationFile
            // 
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile.Description = "Open an existing lexical analyzer specification file for editing";
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CSharp,
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CPlusPlus,
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_Java});
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile.Text = "Lexical Analyzer Specification File";
            // 
            // menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CSharp
            // 
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CSharp.Description = "Open an existing C# lexical analyzer specification file for editing";
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CSharp.Text = "C#...";
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CSharp.Click += new System.EventHandler(this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CSharp_Click);
            // 
            // menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CPlusPlus
            // 
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CPlusPlus.Description = "Open an existing C++ lexical analyzer specification file for editing";
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CPlusPlus.Text = "C++...";
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CPlusPlus.Click += new System.EventHandler(this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CPlusPlus_Click);
            // 
            // menucommand_File_Open_File_LexicalAnalyserSpecificationFile_Java
            // 
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_Java.Description = "Open an existing Java lexical analyzer specification file for editing";
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_Java.Text = "Java...";
            this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_Java.Click += new System.EventHandler(this.menucommand_File_Open_File_LexicalAnalyserSpecificationFile_Java_Click);
            // 
            // menucommand_File_Open_File_ParserSpecificationFile
            // 
            this.menucommand_File_Open_File_ParserSpecificationFile.Description = "Open an existing parser specification file for editing";
            this.menucommand_File_Open_File_ParserSpecificationFile.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File_Open_File_ParserSpecificationFile_CSharp,
            this.menucommand_File_Open_File_ParserSpecificationFile_CPlusPlus,
            this.menucommand_File_Open_File_ParserSpecificationFile_Java});
            this.menucommand_File_Open_File_ParserSpecificationFile.Text = "Parser Specification File";
            // 
            // menucommand_File_Open_File_ParserSpecificationFile_CSharp
            // 
            this.menucommand_File_Open_File_ParserSpecificationFile_CSharp.Description = "Open an existing C# parser specification file for editing";
            this.menucommand_File_Open_File_ParserSpecificationFile_CSharp.Text = "C#...";
            this.menucommand_File_Open_File_ParserSpecificationFile_CSharp.Click += new System.EventHandler(this.menucommand_File_Open_File_ParserSpecificationFile_CSharp_Click);
            // 
            // menucommand_File_Open_File_ParserSpecificationFile_CPlusPlus
            // 
            this.menucommand_File_Open_File_ParserSpecificationFile_CPlusPlus.Description = "Open an existing C++ parser specification file for editing";
            this.menucommand_File_Open_File_ParserSpecificationFile_CPlusPlus.Text = "C++...";
            this.menucommand_File_Open_File_ParserSpecificationFile_CPlusPlus.Click += new System.EventHandler(this.menucommand_File_Open_File_ParserSpecificationFile_CPlusPlus_Click);
            // 
            // menucommand_File_Open_File_ParserSpecificationFile_Java
            // 
            this.menucommand_File_Open_File_ParserSpecificationFile_Java.Description = "Open an existing Java parser specification file for editing";
            this.menucommand_File_Open_File_ParserSpecificationFile_Java.Text = "Java...";
            this.menucommand_File_Open_File_ParserSpecificationFile_Java.Click += new System.EventHandler(this.menucommand_File_Open_File_ParserSpecificationFile_Java_Click);
            // 
            // menucommand_File_Open_File_GraphicalRegularLanguageSpecificationFile
            // 
            this.menucommand_File_Open_File_GraphicalRegularLanguageSpecificationFile.Description = "Open an existing graphical regular language specification file for editing";
            this.menucommand_File_Open_File_GraphicalRegularLanguageSpecificationFile.Text = "Graphical Regular Language Specification File...";
            this.menucommand_File_Open_File_GraphicalRegularLanguageSpecificationFile.Click += new System.EventHandler(this.menucommand_File_Open_File_GraphicalRegularLanguageSpecificationFile_Click);
            // 
            // menucommand_File_Open_File_OtherFile
            // 
            this.menucommand_File_Open_File_OtherFile.Description = "Open an existing file of any type for editing";
            this.menucommand_File_Open_File_OtherFile.Text = "Other File...";
            this.menucommand_File_Open_File_OtherFile.Click += new System.EventHandler(this.menucommand_File_Open_File_OtherFile_Click);
            // 
            // menucommand_Separator
            // 
            this.menucommand_Separator.Description = "Separator";
            this.menucommand_Separator.Text = "-";
            // 
            // menucommand_File_Close
            // 
            this.menucommand_File_Close.Description = "Close the current file";
            this.menucommand_File_Close.Shortcut = System.Windows.Forms.Shortcut.F4;
            this.menucommand_File_Close.Text = "Close";
            this.menucommand_File_Close.Click += new System.EventHandler(this.menucommand_File_Close_Click);
            // 
            // menucommand_File_CloseProject
            // 
            this.menucommand_File_CloseProject.Description = "Close the current project";
            this.menucommand_File_CloseProject.Shortcut = System.Windows.Forms.Shortcut.CtrlF4;
            this.menucommand_File_CloseProject.Text = "Close Project";
            this.menucommand_File_CloseProject.Click += new System.EventHandler(this.menucommand_File_CloseProject_Click);
            // 
            // menucommand_File_Save
            // 
            this.menucommand_File_Save.Description = "Common operations for saving modified documents";
            this.menucommand_File_Save.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File_Save_Project,
            this.menucommand_File_Save_File});
            this.menucommand_File_Save.Text = "Save";
            // 
            // menucommand_File_Save_Project
            // 
            this.menucommand_File_Save_Project.Description = "Save changes in the current project";
            this.menucommand_File_Save_Project.Shortcut = System.Windows.Forms.Shortcut.CtrlShiftS;
            this.menucommand_File_Save_Project.Text = "Project";
            this.menucommand_File_Save_Project.Click += new System.EventHandler(this.menucommand_File_Save_Project_Click);
            // 
            // menucommand_File_Save_File
            // 
            this.menucommand_File_Save_File.Description = "Save changes in the current file";
            this.menucommand_File_Save_File.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_File_Save_File.Image")));
            this.menucommand_File_Save_File.Shortcut = System.Windows.Forms.Shortcut.CtrlS;
            this.menucommand_File_Save_File.Text = "File";
            this.menucommand_File_Save_File.Click += new System.EventHandler(this.menucommand_File_Save_File_Click);
            // 
            // menucommand_File_SaveAs
            // 
            this.menucommand_File_SaveAs.Description = "Save changes in the current project as a new project without altering the current" +
                " one";
            this.menucommand_File_SaveAs.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_File_SaveAs_NewProject,
            this.menucommand_File_SaveAs_NewFile});
            this.menucommand_File_SaveAs.Text = "Save As";
            // 
            // menucommand_File_SaveAs_NewProject
            // 
            this.menucommand_File_SaveAs_NewProject.Description = "Creates a new project as a copy of the current project";
            this.menucommand_File_SaveAs_NewProject.Text = "New Project...";
            this.menucommand_File_SaveAs_NewProject.Click += new System.EventHandler(this.menucommand_File_SaveAs_NewProject_Click);
            // 
            // menucommand_File_SaveAs_NewFile
            // 
            this.menucommand_File_SaveAs_NewFile.Description = "Creates a new file as a copy of the current file";
            this.menucommand_File_SaveAs_NewFile.Text = "New File...";
            this.menucommand_File_SaveAs_NewFile.Click += new System.EventHandler(this.menucommand_File_SaveAs_NewFile_Click);
            // 
            // menucommand_File_SaveAll
            // 
            this.menucommand_File_SaveAll.Description = "Save changes in all open files";
            this.menucommand_File_SaveAll.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_File_SaveAll.Image")));
            this.menucommand_File_SaveAll.Shortcut = System.Windows.Forms.Shortcut.F10;
            this.menucommand_File_SaveAll.Text = "Save All";
            this.menucommand_File_SaveAll.Click += new System.EventHandler(this.menucommand_File_SaveAll_Click);
            // 
            // menucommand_File_PageSetup
            // 
            this.menucommand_File_PageSetup.Description = "Adjustments before printing";
            this.menucommand_File_PageSetup.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_File_PageSetup.Image")));
            this.menucommand_File_PageSetup.Shortcut = System.Windows.Forms.Shortcut.CtrlShiftP;
            this.menucommand_File_PageSetup.Text = "Page Setup...";
            // 
            // menucommand_File_PrintPreview
            // 
            this.menucommand_File_PrintPreview.Description = "Preview for WYSIWYG printing";
            this.menucommand_File_PrintPreview.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_File_PrintPreview.Image")));
            this.menucommand_File_PrintPreview.Text = "Print Preview";
            // 
            // menucommand_File_Print
            // 
            this.menucommand_File_Print.Description = "Print the current document";
            this.menucommand_File_Print.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_File_Print.Image")));
            this.menucommand_File_Print.Shortcut = System.Windows.Forms.Shortcut.CtrlP;
            this.menucommand_File_Print.Text = "Print...";
            // 
            // menucommand_File_RecentFiles
            // 
            this.menucommand_File_RecentFiles.Description = "A list of the most recently used files";
            this.menucommand_File_RecentFiles.Text = "Recent Files";
            // 
            // menucommand_File_RecentProjects
            // 
            this.menucommand_File_RecentProjects.Description = "A list of the most recently used projects";
            this.menucommand_File_RecentProjects.Text = "Recent Projects";
            // 
            // menucommand_File_Exit
            // 
            this.menucommand_File_Exit.Description = "Exit the application";
            this.menucommand_File_Exit.Shortcut = System.Windows.Forms.Shortcut.AltF4;
            this.menucommand_File_Exit.Text = "Exit";
            this.menucommand_File_Exit.Click += new System.EventHandler(this.menucommand_File_Exit_Click);
            // 
            // menucommand_Edit
            // 
            this.menucommand_Edit.Description = "Common operations for editing in the current file";
            this.menucommand_Edit.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Edit_Undo,
            this.menucommand_Edit_Redo,
            this.menucommand_Separator,
            this.menucommand_Edit_Cut,
            this.menucommand_Edit_Copy,
            this.menucommand_Edit_Paste,
            this.menucommand_Edit_Delete,
            this.menucommand_Separator,
            this.menucommand_Edit_SelectAll,
            this.menucommand_Separator,
            this.menucommand_Edit_FindAndReplace,
            this.menucommand_Edit_GoTo});
            this.menucommand_Edit.Text = "&Edit";
            // 
            // menucommand_Edit_Undo
            // 
            this.menucommand_Edit_Undo.Description = "Undo the last operation";
            this.menucommand_Edit_Undo.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_Edit_Undo.Image")));
            this.menucommand_Edit_Undo.Text = "Undo";
            this.menucommand_Edit_Undo.Click += new System.EventHandler(this.menucommand_Edit_Undo_Click);
            // 
            // menucommand_Edit_Redo
            // 
            this.menucommand_Edit_Redo.Description = "Redo the last undone operation";
            this.menucommand_Edit_Redo.Image = global::IDE.Properties.Resources.Edit_RedoHS;
            this.menucommand_Edit_Redo.Text = "Redo";
            this.menucommand_Edit_Redo.Click += new System.EventHandler(this.menucommand_Edit_Redo_Click);
            // 
            // menucommand_Edit_Cut
            // 
            this.menucommand_Edit_Cut.Description = "Cut the current selection into the clipboard";
            this.menucommand_Edit_Cut.Text = "Cut";
            this.menucommand_Edit_Cut.Click += new System.EventHandler(this.menucommand_Edit_Cut_Click);
            // 
            // menucommand_Edit_Copy
            // 
            this.menucommand_Edit_Copy.Description = "Copy the current selection into the clipboard";
            this.menucommand_Edit_Copy.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_Edit_Copy.Image")));
            this.menucommand_Edit_Copy.Text = "Copy";
            this.menucommand_Edit_Copy.Click += new System.EventHandler(this.menucommand_Edit_Copy_Click);
            // 
            // menucommand_Edit_Paste
            // 
            this.menucommand_Edit_Paste.Description = "Paste the current contents of the clipboard into the current file, at the positio" +
                "n of the caret";
            this.menucommand_Edit_Paste.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_Edit_Paste.Image")));
            this.menucommand_Edit_Paste.Text = "Paste";
            this.menucommand_Edit_Paste.Click += new System.EventHandler(this.menucommand_Edit_Paste_Click);
            // 
            // menucommand_Edit_Delete
            // 
            this.menucommand_Edit_Delete.Description = "Delete the current selection";
            this.menucommand_Edit_Delete.Text = "Delete";
            this.menucommand_Edit_Delete.Click += new System.EventHandler(this.menucommand_Edit_Delete_Click);
            // 
            // menucommand_Edit_SelectAll
            // 
            this.menucommand_Edit_SelectAll.Description = "Select all the contents of the current file";
            this.menucommand_Edit_SelectAll.Shortcut = System.Windows.Forms.Shortcut.CtrlA;
            this.menucommand_Edit_SelectAll.Text = "Select All";
            this.menucommand_Edit_SelectAll.Click += new System.EventHandler(this.menucommand_Edit_SelectAll_Click);
            // 
            // menucommand_Edit_FindAndReplace
            // 
            this.menucommand_Edit_FindAndReplace.Description = "Find and replace strings in the current file";
            this.menucommand_Edit_FindAndReplace.Shortcut = System.Windows.Forms.Shortcut.CtrlF;
            this.menucommand_Edit_FindAndReplace.Text = "Find and Replace...";
            this.menucommand_Edit_FindAndReplace.Click += new System.EventHandler(this.menucommand_Edit_FindAndReplace_Click);
            // 
            // menucommand_Edit_GoTo
            // 
            this.menucommand_Edit_GoTo.Description = "Go to a certain line in the current file";
            this.menucommand_Edit_GoTo.Shortcut = System.Windows.Forms.Shortcut.CtrlG;
            this.menucommand_Edit_GoTo.Text = "Go To...";
            this.menucommand_Edit_GoTo.Click += new System.EventHandler(this.menucommand_Edit_GoTo_Click);
            // 
            // menucommand_Windows
            // 
            this.menucommand_Windows.Description = "Common windows";
            this.menucommand_Windows.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Windows_ProjectExplorer,
            this.menucommand_Windows_TaskList,
            this.menucommand_Windows_Output,
            this.menucommand_Separator,
            this.menucommand_Windows_ShowAllWindows,
            this.menucommand_Windows_HideAllWindows});
            this.menucommand_Windows.Text = "&Windows";
            // 
            // menucommand_Windows_ProjectExplorer
            // 
            this.menucommand_Windows_ProjectExplorer.Description = "View the files in the current project";
            this.menucommand_Windows_ProjectExplorer.Text = "Project Explorer";
            this.menucommand_Windows_ProjectExplorer.Click += new System.EventHandler(this.menucommand_Windows_ProjectExplorer_Click);
            // 
            // menucommand_Windows_TaskList
            // 
            this.menucommand_Windows_TaskList.Description = "View the tasks in the current build";
            this.menucommand_Windows_TaskList.Text = "Task List";
            this.menucommand_Windows_TaskList.Click += new System.EventHandler(this.menucommand_Windows_TaskList_Click);
            // 
            // menucommand_Windows_Output
            // 
            this.menucommand_Windows_Output.Description = "View the output of the current build";
            this.menucommand_Windows_Output.Text = "Output";
            this.menucommand_Windows_Output.Click += new System.EventHandler(this.menucommand_Windows_Output_Click);
            // 
            // menucommand_Windows_ShowAllWindows
            // 
            this.menucommand_Windows_ShowAllWindows.Description = "Show all docked windows";
            this.menucommand_Windows_ShowAllWindows.Image = global::IDE.Properties.Resources.WindowsHS;
            this.menucommand_Windows_ShowAllWindows.Shortcut = System.Windows.Forms.Shortcut.F8;
            this.menucommand_Windows_ShowAllWindows.Text = "Show All Windows";
            this.menucommand_Windows_ShowAllWindows.Click += new System.EventHandler(this.menucommand_Windows_ShowAllWindows_Click);
            // 
            // menucommand_Windows_HideAllWindows
            // 
            this.menucommand_Windows_HideAllWindows.Description = "Hide all docked windows";
            this.menucommand_Windows_HideAllWindows.Shortcut = System.Windows.Forms.Shortcut.ShiftF8;
            this.menucommand_Windows_HideAllWindows.Text = "Hide All Windows";
            this.menucommand_Windows_HideAllWindows.Click += new System.EventHandler(this.menucommand_Windows_HideAllWindows_Click);
            // 
            // menucommand_Project
            // 
            this.menucommand_Project.Description = "Project related tasks";
            this.menucommand_Project.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Project_AddNewItem,
            this.menucommand_Project_AddExistingItem,
            this.menucommand_Separator,
            this.menucommand_Project_IncludeInProject,
            this.menucommand_Project_ExcludeFromProject,
            this.menucommand_Separator,
            this.menucommand_Project_ShowAllFiles,
            this.menucommand_Project_Refresh,
            this.menucommand_Separator,
            this.menucommand_Project_Properties});
            this.menucommand_Project.Text = "&Project";
            // 
            // menucommand_Project_AddNewItem
            // 
            this.menucommand_Project_AddNewItem.Description = "Add a new item to the project";
            this.menucommand_Project_AddNewItem.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Project_AddNewItem_Folder,
            this.menucommand_Project_AddNewItem_File});
            this.menucommand_Project_AddNewItem.Text = "Add New Item";
            // 
            // menucommand_Project_AddNewItem_Folder
            // 
            this.menucommand_Project_AddNewItem_Folder.Description = "Add a new folder to your project";
            this.menucommand_Project_AddNewItem_Folder.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_Project_AddNewItem_Folder.Image")));
            this.menucommand_Project_AddNewItem_Folder.Text = "Folder";
            this.menucommand_Project_AddNewItem_Folder.Click += new System.EventHandler(this.menucommand_Project_AddNewItem_Folder_Click);
            // 
            // menucommand_Project_AddNewItem_File
            // 
            this.menucommand_Project_AddNewItem_File.Description = "Add a new file to your project";
            this.menucommand_Project_AddNewItem_File.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile,
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile,
            this.menucommand_Project_AddNewItem_File_GraphicalRegularLanguageSpecification,
            this.menucommand_Project_AddNewItem_File_OtherFile});
            this.menucommand_Project_AddNewItem_File.Text = "File";
            // 
            // menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile
            // 
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile.Description = "Add a new lexical analyser specification file to your project";
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile,
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_WizardAssistedFile});
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile.Text = "Lexical Analyser Specification File";
            // 
            // menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile
            // 
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile.Description = "Add a new blank lexical analyser specification file to your project";
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp,
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus,
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_Java});
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile.Text = "Blank File";
            // 
            // menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp
            // 
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp.Description = "Add a new lexical analyser specification file to your project, C# used for coding" +
                "";
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp.Text = "C#";
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp.Click += new System.EventHandler(this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp_Click);
            // 
            // menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus
            // 
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus.Description = "Add a new lexical analyser specification file to your project, C++ used for codin" +
                "g";
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus.Text = "C++";
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus.Click += new System.EventHandler(this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus_Click);
            // 
            // menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_Java
            // 
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_Java.Description = "Add a new lexical analyser specification file to your project, Java used for codi" +
                "ng";
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_Java.Text = "Java";
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_Java.Click += new System.EventHandler(this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_Java_Click);
            // 
            // menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_WizardAssistedFile
            // 
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_WizardAssistedFile.Description = "Use the regular expression builder to add a set of regular expressions to your pr" +
                "oject";
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_WizardAssistedFile.Image = global::IDE.Properties.Resources.ctxwiz_cls;
            this.menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_WizardAssistedFile.Text = "Wizard-Assisted File";
            // 
            // menucommand_Project_AddNewItem_File_ParserSpecificationFile
            // 
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile.Description = "Add a new parser specification file to your project";
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile,
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_WizardAssistedFile});
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile.Text = "Parser Specification File";
            // 
            // menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile
            // 
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile.Description = "Add a new blank parser specification file to your project";
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CSharp,
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CPlusPlus,
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_Java});
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile.Text = "Blank File";
            // 
            // menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CSharp
            // 
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CSharp.Description = "Add a new parser specification file to your project, C# used for coding";
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CSharp.Text = "C#";
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CSharp.Click += new System.EventHandler(this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CSharp_Click);
            // 
            // menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CPlusPlus
            // 
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CPlusPlus.Description = "Add a new parser specification file to your project, C++ used for coding";
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CPlusPlus.Text = "C++";
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CPlusPlus.Click += new System.EventHandler(this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CPlusPlus_Click);
            // 
            // menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_Java
            // 
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_Java.Description = "Add a new parser specification file to your project, Java used for coding";
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_Java.Text = "Java";
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_Java.Click += new System.EventHandler(this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_Java_Click);
            // 
            // menucommand_Project_AddNewItem_File_ParserSpecificationFile_WizardAssistedFile
            // 
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_WizardAssistedFile.Description = "Use the CFG builder to add a context-free grammar to your project";
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_WizardAssistedFile.Image = global::IDE.Properties.Resources.ctxwiz_cls;
            this.menucommand_Project_AddNewItem_File_ParserSpecificationFile_WizardAssistedFile.Text = "Wizard-Assisted File";
            // 
            // menucommand_Project_AddNewItem_File_GraphicalRegularLanguageSpecification
            // 
            this.menucommand_Project_AddNewItem_File_GraphicalRegularLanguageSpecification.Description = "Add a new graphical regular language specification file to your project";
            this.menucommand_Project_AddNewItem_File_GraphicalRegularLanguageSpecification.Text = "Graphical Regular Language Specification File";
            this.menucommand_Project_AddNewItem_File_GraphicalRegularLanguageSpecification.Click += new System.EventHandler(this.menucommand_Project_AddNewItem_File_GraphicalRegularLanguageSpecification_Click);
            // 
            // menucommand_Project_AddNewItem_File_OtherFile
            // 
            this.menucommand_Project_AddNewItem_File_OtherFile.Description = "Add a new file of any type you want to associate with your project";
            this.menucommand_Project_AddNewItem_File_OtherFile.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_Project_AddNewItem_File_OtherFile.Image")));
            this.menucommand_Project_AddNewItem_File_OtherFile.Text = "Other File";
            this.menucommand_Project_AddNewItem_File_OtherFile.Click += new System.EventHandler(this.menucommand_Project_AddNewItem_File_OtherFile_Click);
            // 
            // menucommand_Project_AddExistingItem
            // 
            this.menucommand_Project_AddExistingItem.Description = "Add an existing item to your project";
            this.menucommand_Project_AddExistingItem.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile,
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile,
            this.menucommand_Project_AddExistingItem_GraphicalRegularLanguageSpecificationFile,
            this.menucommand_Project_AddExistingItem_OtherFile});
            this.menucommand_Project_AddExistingItem.Text = "Add Existing Item";
            // 
            // menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile
            // 
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile.Description = "Add an existing lexical analyzer specification file to your project";
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CSharp,
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CPlusPlus,
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_Java});
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile.Text = "Lexical Analyzer Specification File";
            // 
            // menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CSharp
            // 
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CSharp.Description = "Add an existing lexical analyzer specification file to your project, C# used for " +
                "coding";
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CSharp.Text = "C#...";
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CSharp.Click += new System.EventHandler(this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CSharp_Click);
            // 
            // menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CPlusPlus
            // 
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CPlusPlus.Description = "Add an existing lexical analyzer specification file to your project, C++ used for" +
                " coding";
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CPlusPlus.Text = "C++...";
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CPlusPlus.Click += new System.EventHandler(this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CPlusPlus_Click);
            // 
            // menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_Java
            // 
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_Java.Description = "Add an existing lexical analyzer specification file to your project, Java used fo" +
                "r coding";
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_Java.Text = "Java...";
            this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_Java.Click += new System.EventHandler(this.menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_Java_Click);
            // 
            // menucommand_Project_AddExistingItem_ParserSpecificationFile
            // 
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile.Description = "Add an existing parser specification file to your project";
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_CSharp,
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_CPlusPlus,
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_Java});
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile.Text = "Parser Specification File";
            // 
            // menucommand_Project_AddExistingItem_ParserSpecificationFile_CSharp
            // 
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_CSharp.Description = "Add an existing lexical analyzer specification file to your project, C# used for " +
                "coding";
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_CSharp.Text = "C#...";
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_CSharp.Click += new System.EventHandler(this.menucommand_Project_AddExistingItem_ParserSpecificationFile_CSharp_Click);
            // 
            // menucommand_Project_AddExistingItem_ParserSpecificationFile_CPlusPlus
            // 
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_CPlusPlus.Description = "Add an existing lexical analyzer specification file to your project, C++ used for" +
                " coding";
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_CPlusPlus.Text = "C++...";
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_CPlusPlus.Click += new System.EventHandler(this.menucommand_Project_AddExistingItem_ParserSpecificationFile_CPlusPlus_Click);
            // 
            // menucommand_Project_AddExistingItem_ParserSpecificationFile_Java
            // 
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_Java.Description = "Add an existing lexical analyzer specification file to your project, Java used fo" +
                "r coding";
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_Java.Text = "Java...";
            this.menucommand_Project_AddExistingItem_ParserSpecificationFile_Java.Click += new System.EventHandler(this.menucommand_Project_AddExistingItem_ParserSpecificationFile_Java_Click);
            // 
            // menucommand_Project_AddExistingItem_GraphicalRegularLanguageSpecificationFile
            // 
            this.menucommand_Project_AddExistingItem_GraphicalRegularLanguageSpecificationFile.Description = "Add an existing graphical regular language specification file to your project";
            this.menucommand_Project_AddExistingItem_GraphicalRegularLanguageSpecificationFile.Text = "Graphical Regular Language Specification File...";
            this.menucommand_Project_AddExistingItem_GraphicalRegularLanguageSpecificationFile.Click += new System.EventHandler(this.menucommand_Project_AddExistingItem_GraphicalRegularLanguageSpecificationFile_Click);
            // 
            // menucommand_Project_AddExistingItem_OtherFile
            // 
            this.menucommand_Project_AddExistingItem_OtherFile.Description = "Add an existing file of any type to your project";
            this.menucommand_Project_AddExistingItem_OtherFile.Text = "Other File...";
            this.menucommand_Project_AddExistingItem_OtherFile.Click += new System.EventHandler(this.menucommand_Project_AddExistingItem_OtherFile_Click);
            // 
            // menucommand_Project_IncludeInProject
            // 
            this.menucommand_Project_IncludeInProject.Description = "Include the selected item in the project";
            this.menucommand_Project_IncludeInProject.Text = "Include In Project";
            // 
            // menucommand_Project_ExcludeFromProject
            // 
            this.menucommand_Project_ExcludeFromProject.Description = "Exclude the selected item from the project";
            this.menucommand_Project_ExcludeFromProject.Text = "Exclude From Project";
            // 
            // menucommand_Project_ShowAllFiles
            // 
            this.menucommand_Project_ShowAllFiles.Description = "Show all the available files in the selected folder. Some of these files may NOT " +
                "be related to the project";
            this.menucommand_Project_ShowAllFiles.Text = "Show All Files";
            // 
            // menucommand_Project_Refresh
            // 
            this.menucommand_Project_Refresh.Description = "Reload the file list";
            this.menucommand_Project_Refresh.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_Project_Refresh.Image")));
            this.menucommand_Project_Refresh.Text = "Refresh";
            // 
            // menucommand_Project_Properties
            // 
            this.menucommand_Project_Properties.Description = "View the properties of the current project";
            this.menucommand_Project_Properties.Image = ((System.Drawing.Image)(resources.GetObject("menucommand_Project_Properties.Image")));
            this.menucommand_Project_Properties.Text = "Properties...";
            // 
            // menucommand_Tools
            // 
            this.menucommand_Tools.Description = "Helper tools";
            this.menucommand_Tools.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Tools_LEXcellent,
            this.menucommand_Tools_ParSpring,
            this.menucommand_Separator,
            this.menucommand_Tools_LeftRecursionRemoval,
            this.menucommand_Tools_LeftFactoring,
            this.menucommand_Separator,
            this.menucommand_Tools_GTGEditor,
            this.menucommand_Tools_RegExManipulator,
            this.menucommand_Separator,
            this.menucommand_Tools_SyntaxOptions});
            this.menucommand_Tools.Text = "&Tools";
            // 
            // menucommand_Tools_LEXcellent
            // 
            this.menucommand_Tools_LEXcellent.Description = "Generate a lexical analyzer";
            this.menucommand_Tools_LEXcellent.Text = "LEXcellent";
            this.menucommand_Tools_LEXcellent.Click += new System.EventHandler(this.menucommand_Tools_LEXcellent_Click);
            // 
            // menucommand_Tools_ParSpring
            // 
            this.menucommand_Tools_ParSpring.Description = "Generate a parser";
            this.menucommand_Tools_ParSpring.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Tools_ParSpring_RDParser,
            this.menucommand_Tools_ParSpring_LL1Parser});
            this.menucommand_Tools_ParSpring.Text = "ParSpring";
            // 
            // menucommand_Tools_ParSpring_RDParser
            // 
            this.menucommand_Tools_ParSpring_RDParser.Description = "Generate an RD Parser";
            this.menucommand_Tools_ParSpring_RDParser.Text = "RD Parser Generator";
            this.menucommand_Tools_ParSpring_RDParser.Click += new System.EventHandler(this.menucommand_Tools_ParSpring_RDParser_Click);
            // 
            // menucommand_Tools_ParSpring_LL1Parser
            // 
            this.menucommand_Tools_ParSpring_LL1Parser.Description = "Generate an LL(1) Parser";
            this.menucommand_Tools_ParSpring_LL1Parser.Text = "LL(1) Parser Generator";
            this.menucommand_Tools_ParSpring_LL1Parser.Click += new System.EventHandler(this.menucommand_Tools_ParSpring_LL1Parser_Click);
            // 
            // menucommand_Tools_LeftRecursionRemoval
            // 
            this.menucommand_Tools_LeftRecursionRemoval.Description = "Remove the left recursion - if exists - from a context-free grammar";
            this.menucommand_Tools_LeftRecursionRemoval.Text = "Left Recursion Removal";
            this.menucommand_Tools_LeftRecursionRemoval.Click += new System.EventHandler(this.menucommand_Tools_LeftRecursionRemoval_Click);
            // 
            // menucommand_Tools_LeftFactoring
            // 
            this.menucommand_Tools_LeftFactoring.Description = "Left-factor a context-free grammar";
            this.menucommand_Tools_LeftFactoring.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Tools_LeftFactoring_BNF,
            this.menucommand_Tools_LeftFactoring_EBNF});
            this.menucommand_Tools_LeftFactoring.Text = "Left Factoring";
            // 
            // menucommand_Tools_LeftFactoring_BNF
            // 
            this.menucommand_Tools_LeftFactoring_BNF.Description = "Left-factor a context-free grammar in the BNF format";
            this.menucommand_Tools_LeftFactoring_BNF.Text = "BNF Format";
            this.menucommand_Tools_LeftFactoring_BNF.Click += new System.EventHandler(this.menucommand_Tools_LeftFactoring_BNF_Click);
            // 
            // menucommand_Tools_LeftFactoring_EBNF
            // 
            this.menucommand_Tools_LeftFactoring_EBNF.Description = "Left-factor a context-free grammar";
            this.menucommand_Tools_LeftFactoring_EBNF.Text = "EBNF Format";
            this.menucommand_Tools_LeftFactoring_EBNF.Click += new System.EventHandler(this.menucommand_Tools_LeftFactoring_EBNF_Click);
            // 
            // menucommand_Tools_GTGEditor
            // 
            this.menucommand_Tools_GTGEditor.Description = "Create a GTG and convert it to a regular expression";
            this.menucommand_Tools_GTGEditor.Text = "GTG Editor";
            this.menucommand_Tools_GTGEditor.Click += new System.EventHandler(this.menucommand_Tools_GTGEditor_Click);
            // 
            // menucommand_Tools_RegExManipulator
            // 
            this.menucommand_Tools_RegExManipulator.Description = "Manipulate regular expression";
            this.menucommand_Tools_RegExManipulator.Text = "RegEx Manipulator";
            this.menucommand_Tools_RegExManipulator.Click += new System.EventHandler(this.menucommand_Tools_RegExManipulator_Click);
            // 
            // menucommand_Tools_SyntaxOptions
            // 
            this.menucommand_Tools_SyntaxOptions.Description = "Options for customizing text appearances in files";
            this.menucommand_Tools_SyntaxOptions.Image = global::IDE.Properties.Resources.HighlightHS;
            this.menucommand_Tools_SyntaxOptions.Text = "Syntax Options...";
            this.menucommand_Tools_SyntaxOptions.Click += new System.EventHandler(this.menucommand_Tools_SyntaxOptions_Click);
            // 
            // menucommand_Help
            // 
            this.menucommand_Help.Description = "Help and support information";
            this.menucommand_Help.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Help_Contents,
            this.menucommand_Separator,
            this.menucommand_Help_Samples,
            this.menucommand_Separator,
            this.menucommand_Help_AboutLEXcellent,
            this.menucommand_Help_AboutParSpring,
            this.menucommand_Help_AboutCCW,
            this.menucommand_Separator,
            this.menucommand_Help_Acknowledgements});
            this.menucommand_Help.Text = "&Help";
            // 
            // menucommand_Help_Contents
            // 
            this.menucommand_Help_Contents.Description = "Help contents";
            this.menucommand_Help_Contents.Shortcut = System.Windows.Forms.Shortcut.F1;
            this.menucommand_Help_Contents.Text = "Contents...";
            // 
            // menucommand_Help_Samples
            // 
            this.menucommand_Help_Samples.Description = "Sample Videos";
            this.menucommand_Help_Samples.MenuCommands.AddRange(new Crownwood.DotNetMagic.Menus.MenuCommand[] {
            this.menucommand_Help_Samples_ArabicCLanguage});
            this.menucommand_Help_Samples.Text = "Samples";
            // 
            // menucommand_Help_Samples_ArabicCLanguage
            // 
            this.menucommand_Help_Samples_ArabicCLanguage.Description = "The Arabic C language";
            this.menucommand_Help_Samples_ArabicCLanguage.Text = "Arabic C Language";
            this.menucommand_Help_Samples_ArabicCLanguage.Click += new System.EventHandler(this.menucommand_Help_Samples_ArabicCLanguage_Click);
            // 
            // menucommand_Help_AboutLEXcellent
            // 
            this.menucommand_Help_AboutLEXcellent.Description = "Technical support for LEXcellent";
            this.menucommand_Help_AboutLEXcellent.Text = "About LEXcellent...";
            this.menucommand_Help_AboutLEXcellent.Click += new System.EventHandler(this.menucommand_Help_AboutLEXcellent_Click);
            // 
            // menucommand_Help_AboutParSpring
            // 
            this.menucommand_Help_AboutParSpring.Description = "Technical support for ParSpring";
            this.menucommand_Help_AboutParSpring.Text = "About ParSpring...";
            this.menucommand_Help_AboutParSpring.Click += new System.EventHandler(this.menucommand_Help_AboutParSpring_Click);
            // 
            // menucommand_Help_AboutCCW
            // 
            this.menucommand_Help_AboutCCW.Description = "Technical support for CCW 1.0";
            this.menucommand_Help_AboutCCW.Text = "About CCW 1.0...";
            this.menucommand_Help_AboutCCW.Click += new System.EventHandler(this.menucommand_Help_AboutCCW_Click);
            // 
            // menucommand_Help_Acknowledgements
            // 
            this.menucommand_Help_Acknowledgements.Description = "Acknowledgements";
            this.menucommand_Help_Acknowledgements.Text = "Acknowledgements...";
            this.menucommand_Help_Acknowledgements.Click += new System.EventHandler(this.menucommand_Help_Acknowledgements_Click);
            // 
            // images
            // 
            this.images.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("images.ImageStream")));
            this.images.TransparentColor = System.Drawing.Color.Magenta;
            this.images.Images.SetKeyName(0, "HLP.BMP");
            this.images.Images.SetKeyName(1, "Textbox.bmp");
            this.images.Images.SetKeyName(2, "List_Bullets.bmp");
            // 
            // tabbedgroups_Main
            // 
            this.tabbedgroups_Main.AllowDrop = true;
            this.tabbedgroups_Main.AtLeastOneLeaf = true;
            this.tabbedgroups_Main.Dock = System.Windows.Forms.DockStyle.Fill;
            this.tabbedgroups_Main.Location = new System.Drawing.Point(0, 51);
            this.tabbedgroups_Main.Name = "tabbedgroups_Main";
            this.tabbedgroups_Main.ProminentLeaf = null;
            this.tabbedgroups_Main.ResizeBarColor = System.Drawing.SystemColors.Control;
            this.tabbedgroups_Main.Size = new System.Drawing.Size(492, 291);
            this.tabbedgroups_Main.TabIndex = 1;
            this.tabbedgroups_Main.TabControlCreated += new Crownwood.DotNetMagic.Controls.TabbedGroups.TabControlCreatedHandler(this.OnTabControlCreated);
            this.tabbedgroups_Main.ExternalDrop += new Crownwood.DotNetMagic.Controls.TabbedGroups.ExternalDropHandler(this.OnExternalDrop);
            // 
            // menucommand_Project_AddExistingItem_LexicalAnalyserFile
            // 
            this.menucommand_Project_AddExistingItem_LexicalAnalyserFile.Description = "Add an existing lexical analyser specification file";
            this.menucommand_Project_AddExistingItem_LexicalAnalyserFile.Text = "Lexical Analyser File";
            // 
            // statusbarcontrol_Main
            // 
            this.statusbarcontrol_Main.IDE2005GradBack = false;
            this.statusbarcontrol_Main.Location = new System.Drawing.Point(0, 342);
            this.statusbarcontrol_Main.Name = "statusbarcontrol_Main";
            this.statusbarcontrol_Main.Office2003GradBack = true;
            this.statusbarcontrol_Main.PadTop = 3;
            this.statusbarcontrol_Main.Size = new System.Drawing.Size(492, 24);
            this.statusbarcontrol_Main.SizingGrip = false;
            this.statusbarcontrol_Main.StatusPanels.AddRange(new Crownwood.DotNetMagic.Controls.StatusPanel[] {
            this.statuspanel_Description,
            this.statuspanel_Date});
            this.statusbarcontrol_Main.TabIndex = 2;
            // 
            // statuspanel_Description
            // 
            this.statuspanel_Description.AutoScrollMargin = new System.Drawing.Size(0, 0);
            this.statuspanel_Description.AutoScrollMinSize = new System.Drawing.Size(0, 0);
            this.statuspanel_Description.AutoSizing = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
            this.statuspanel_Description.Location = new System.Drawing.Point(2, 2);
            this.statuspanel_Description.Name = "statuspanel_Description";
            this.statuspanel_Description.RequestedWidth = 150;
            this.statuspanel_Description.Size = new System.Drawing.Size(480, 16);
            this.statuspanel_Description.TabIndex = 0;
            // 
            // statuspanel_Date
            // 
            this.statuspanel_Date.Alignment = System.Drawing.StringAlignment.Center;
            this.statuspanel_Date.AutoScrollMargin = new System.Drawing.Size(0, 0);
            this.statuspanel_Date.AutoScrollMinSize = new System.Drawing.Size(0, 0);
            this.statuspanel_Date.AutoSizing = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
            this.statuspanel_Date.BackgroundImageLayout = System.Windows.Forms.ImageLayout.None;
            this.statuspanel_Date.Location = new System.Drawing.Point(2, 2);
            this.statuspanel_Date.Name = "statuspanel_Date";
            this.statuspanel_Date.RequestedWidth = 50;
            this.statuspanel_Date.Size = new System.Drawing.Size(0, 16);
            this.statuspanel_Date.TabIndex = 1;
            // 
            // timer
            // 
            this.timer.Enabled = true;
            this.timer.Interval = 900;
            this.timer.Tick += new System.EventHandler(this.timer_Tick);
            // 
            // Interface
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(492, 366);
            this.Controls.Add(this.tabbedgroups_Main);
            this.Controls.Add(this.mainToolBar);
            this.Controls.Add(this.statusbarcontrol_Main);
            this.Controls.Add(this.menuMainFunctionality);
            this.DoubleBuffered = true;
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.Name = "Interface";
            this.Text = "Compiler Construction Workbench 1.0";
            this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
            ((System.ComponentModel.ISupportInitialize)(this.mainToolBar)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.tabbedgroups_Main)).EndInit();
            this.ResumeLayout(false);

        }
        #endregion

        #region Menus' Event Handlers
        private void menucommand_File_New_Project_Click(object sender, EventArgs e)
        {
            IDE.Forms.CCW.NewProject dialog = new IDE.Forms.CCW.NewProject();

            if (dialog.ShowDialog() != DialogResult.Cancel)
            {
                currentProject = new Project(DateTime.Now, dialog.FilePath, dialog.FileName, dialog.FileName);
                fileInProjectToTabPage = new Hashtable();
                tabPageToFileInProject = new Hashtable();
            }

            CreateProjectExplorer();
        }

        private void menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "C++ Lexical Analyzer Specifications File (*.lexcpp)|*.lexcpp";
            dialog.AddExtension = true;
            dialog.DefaultExt = "lexcpp";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Save a new C++ lexical analyser file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(dialog.FileName);
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.LexCPlusPlus, filePath, fileName));
                if (!fileNotInProjectToTabPage.Contains(fileName))
                    this.fileNotInProjectToTabPage.Add(fileName, tabPage);
            }
        }

        private void menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "C# Lexical Analyzer Specifications File (*.lexcs)|*.lexcs";
            dialog.AddExtension = true;
            dialog.DefaultExt = "lexcs";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Save a new C# lexical analyser file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(dialog.FileName);
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.LexCSharp, filePath, fileName));
                if (!fileNotInProjectToTabPage.Contains(fileName))
                    this.fileNotInProjectToTabPage.Add(fileName, tabPage);
            }
        }

        private void menucommand_File_New_File_LexicalAnalyserSpecificationFile_BlankFile_Java_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "Java Lexical Analyzer Specifications File (*.lexjava)|*.lexjava";
            dialog.AddExtension = true;
            dialog.DefaultExt = "lexjava";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Save a new Java lexical analyser file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(dialog.FileName);
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.LexJava, filePath, fileName));
                if (!fileNotInProjectToTabPage.Contains(fileName))
                    this.fileNotInProjectToTabPage.Add(fileName, tabPage);
            }
        }

        private void menucommand_File_New_File_ParserSpecificationFile_BlankFile_CSharp_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "C# Parser Specifications File (*.parcs)|*.parcs";
            dialog.AddExtension = true;
            dialog.DefaultExt = "parcs";
            dialog.RestoreDirectory = true;
            dialog.Title = "Save a new C# parser file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(dialog.FileName);
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.ParCSharp, filePath, fileName));
                if (!fileNotInProjectToTabPage.Contains(fileName))
                    this.fileNotInProjectToTabPage.Add(fileName, tabPage);
            }
        }

        private void menucommand_File_New_File_ParserSpecificationFile_BlankFile_CPlusPlus_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "C++ Parser Specifications File (*.parcpp)|*.parcpp";
            dialog.AddExtension = true;
            dialog.DefaultExt = "parcpp";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Save a new C++ parser file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(dialog.FileName);
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.ParCPlusPlus, filePath, fileName));
                if (!fileNotInProjectToTabPage.Contains(fileName))
                    this.fileNotInProjectToTabPage.Add(fileName, tabPage);
            }
        }

        private void menucommand_File_New_File_ParserSpecificationFile_BlankFile_Java_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "Java Parser Specifications File (*.parjava)|*.parjava";
            dialog.AddExtension = true;
            dialog.DefaultExt = "parjava";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Save a new Java parser file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(dialog.FileName);
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.ParJava, filePath, fileName));
                if (!fileNotInProjectToTabPage.Contains(fileName))
                    this.fileNotInProjectToTabPage.Add(fileName, tabPage);
            }
        }

        private void menucommand_File_New_File_LexicalAnalyserSpecificationFile_WizardAssistedFile_Click(object sender, EventArgs e)
        {
            Wizards.LEXcellentWizard wizard = new Wizards.LEXcellentWizard();
            wizard.ShowDialog();
        }

        private void menucommand_File_New_File_GraphicalRegularLanguageSpecification_Click(object sender, EventArgs e)
		{
			menucommand_Tools_GTGEditor.PerformClick();
        }

        private void menucommand_File_New_File_Other_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Title = "Save a new file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(dialog.FileName);
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.None, filePath, fileName));
                if (!fileNotInProjectToTabPage.Contains(fileName))
                    this.fileNotInProjectToTabPage.Add(fileName, tabPage);
            }
        }

        private void menucommand_File_Open_Project_Click(object sender, EventArgs e)
        {
            OpenFileDialog projectFileDialog = new OpenFileDialog();
            projectFileDialog.CheckFileExists = true;
            projectFileDialog.CheckPathExists = true;
            projectFileDialog.InitialDirectory = "c:\\My Projects";
            projectFileDialog.Filter = "CCW Project Files (*.ccwproj)|*.ccwproj";

            if (projectFileDialog.ShowDialog() == DialogResult.OK)
            {
                FileStream formattedStream = new FileStream(projectFileDialog.FileName, FileMode.Open, FileAccess.Read, FileShare.Read);
                BinaryFormatter formatter = new BinaryFormatter();
                currentProject = (Project)(formatter.Deserialize(formattedStream));
                formattedStream.Close();

                fileInProjectToTabPage = new Hashtable();
                tabPageToFileInProject = new Hashtable();

                FileStream fileStream;
                StreamReader fileReader;

                foreach (ProjectFile file in currentProject.LexicalAnalyserCodeFiles)
                {
                    Crownwood.DotNetMagic.Controls.TabPage tabPage = NewTabPage(file.Language, file.Path, file.Name);
                    //fileInProjectToTabPage.Add(file.Name, tabPage);
                    //tabPageToFileInProject.Add(tabPage, file);
                    //tabbedgroups_Main.ActiveLeaf.TabPages.Add(tabPage);
                    Node node = new Node(file.Name);
                    ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[3].Nodes.Add(node);

                    fileStream = new FileStream(file.Path + "\\" + file.Name, FileMode.Open, FileAccess.Read, FileShare.Read);
                    fileReader = new StreamReader(fileStream);
                    ((Fireball.Windows.Forms.CodeEditorControl)(tabPage.Control)).Document.Text = fileReader.ReadToEnd();
                    fileReader.Close();
                    fileStream.Close();
                }
                foreach (ProjectFile file in currentProject.ParserCodeFiles)
                {
                    Crownwood.DotNetMagic.Controls.TabPage tabPage = NewTabPage(file.Language, file.Path, file.Name);
                    //fileInProjectToTabPage.Add(file.Name, tabPage);
                    //tabPageToFileInProject.Add(tabPage, file);
                    //tabbedgroups_Main.ActiveLeaf.TabPages.Add(tabPage);
                    Node node = new Node(file.Name);
                    ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[4].Nodes.Add(node);

                    fileStream = new FileStream(file.Path + "\\" + file.Name, FileMode.Open, FileAccess.Read, FileShare.Read);
                    fileReader = new StreamReader(fileStream);
                    ((Fireball.Windows.Forms.CodeEditorControl)(tabPage.Control)).Document.Text = fileReader.ReadToEnd();
                    fileReader.Close();
                    fileStream.Close();
                }
                foreach (ProjectFile file in currentProject.MiscellaneousFiles)
                {
                    Crownwood.DotNetMagic.Controls.TabPage tabPage = NewTabPage(file.Language, file.Path, file.Name);
                    //fileInProjectToTabPage.Add(file.Name, tabPage);
                    //tabPageToFileInProject.Add(tabPage, file);
                    //tabbedgroups_Main.ActiveLeaf.TabPages.Add(tabPage);
                    Node node = new Node(file.Name);
                    ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[5].Nodes.Add(node);

                    fileStream = new FileStream(file.Path + "\\" + file.Name, FileMode.Open, FileAccess.Read, FileShare.Read);
                    fileReader = new StreamReader(fileStream);
                    ((Fireball.Windows.Forms.CodeEditorControl)(tabPage.Control)).Document.Text = fileReader.ReadToEnd();
                    fileReader.Close();
                    fileStream.Close();
                }
                foreach (ProjectFile file in currentProject.LexicalAnalyserSpecificationFiles)
                {
                    Crownwood.DotNetMagic.Controls.TabPage tabPage = NewTabPage(file.Language, file.Path, file.Name);
                    //fileInProjectToTabPage.Add(file.Name, tabPage);
                    //tabPageToFileInProject.Add(tabPage, file);
                    //tabbedgroups_Main.ActiveLeaf.TabPages.Add(tabPage);
                    Node node = new Node(file.Name);
                    ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[0].Nodes.Add(node);

                    fileStream = new FileStream(file.Path + "\\" + file.Name, FileMode.Open, FileAccess.Read, FileShare.Read);
                    fileReader = new StreamReader(fileStream);
                    ((Fireball.Windows.Forms.CodeEditorControl)(tabPage.Control)).Document.Text = fileReader.ReadToEnd();
                    fileReader.Close();
                    fileStream.Close();
                }
                foreach (ProjectFile file in currentProject.ParserSpecificationFiles)
                {
                    Crownwood.DotNetMagic.Controls.TabPage tabPage = NewTabPage(file.Language, file.Path, file.Name);
                    //fileInProjectToTabPage.Add(file.Name, tabPage);
                    //tabPageToFileInProject.Add(tabPage, file);
                    //tabbedgroups_Main.ActiveLeaf.TabPages.Add(tabPage);
                    Node node = new Node(file.Name);
                    ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[1].Nodes.Add(node);

                    fileStream = new FileStream(file.Path + "\\" + file.Name, FileMode.Open, FileAccess.Read, FileShare.Read);
                    fileReader = new StreamReader(fileStream);
                    ((Fireball.Windows.Forms.CodeEditorControl)(tabPage.Control)).Document.Text = fileReader.ReadToEnd();
                    fileReader.Close();
                    fileStream.Close();
                }
                foreach (ProjectFile file in currentProject.GraphicalRegularLanguageSpecificationFiles)
                {
                    Crownwood.DotNetMagic.Controls.TabPage tabPage = NewTabPage(file.Language, file.Path, file.Name);
                    //fileInProjectToTabPage.Add(file.Name, tabPage);
                    //tabPageToFileInProject.Add(tabPage, file);
                    //tabbedgroups_Main.ActiveLeaf.TabPages.Add(tabPage);
                    Node node = new Node(file.Name);
                    ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[2].Nodes.Add(node);

                    fileStream = new FileStream(file.Path + "\\" + file.Name, FileMode.Open, FileAccess.Read, FileShare.Read);
                    fileReader = new StreamReader(fileStream);
                    ((Fireball.Windows.Forms.CodeEditorControl)(tabPage.Control)).Document.Text = fileReader.ReadToEnd();
                    fileReader.Close();
                    fileStream.Close();
				}
				foreach (ProjectFile file in currentProject.GraphicalRegularLanguageSpecificationFiles)
				{
					FileStream stream = new FileStream(file.Path + "\\" + file.Name, FileMode.Open, FileAccess.Read, FileShare.None);
					GTGForm frm = GTGForm.FromFile(stream, GetStatusString, SetStatusString, SetOutputText);
					stream.Close();

					Crownwood.DotNetMagic.Controls.TabPage tabPage =
						tabbedgroups_Main.ActiveLeaf.TabPages.Add(new Crownwood.DotNetMagic.Controls.TabPage(file.Name, frm));
					tabPage.ToolTip = file.Path + "\\" + file.Name;

					fileInProjectToTabPage.Add(file.Name, tabPage);
					tabPageToFileInProject.Add(tabPage, file);
					tabbedgroups_Main.ActiveLeaf.TabPages.Add(tabPage);
				}
            }
        }

        private void menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CPlusPlus_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "C++ Lexical Analyzer Specifications File (*.lexcpp)|*.lexcpp";
            dialog.AddExtension = true;
            dialog.DefaultExt = "lexcpp";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Open a new C++ lexical analyzer file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.LexCPlusPlus, filePath, fileName));
                this.fileNotInProjectToTabPage.Add(fileName, tabPage);

                StreamReader sr = new StreamReader(dialog.FileName);
                string str = sr.ReadToEnd();
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
                sr.Close();
            }
        }

        private void menucommand_File_Open_File_LexicalAnalyserSpecificationFile_CSharp_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "C# Lexical Analyzer Specifications File (*.lexcs)|*.lexcs";
            dialog.AddExtension = true;
            dialog.DefaultExt = "lexcs";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Open a new C# lexical analyzer file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.LexCSharp, filePath, fileName));
                this.fileNotInProjectToTabPage.Add(fileName, tabPage);

                StreamReader sr = new StreamReader(dialog.FileName);
                string str = sr.ReadToEnd();
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
                sr.Close();
            }
        }

        private void menucommand_File_Open_File_LexicalAnalyserSpecificationFile_Java_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "Java Parser Specifications File (*.lexjava)|*.lexjava";
            dialog.AddExtension = true;
            dialog.DefaultExt = "lexjava";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Open a new Java Lexical Analyzer file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.LexJava, filePath, fileName));
                this.fileNotInProjectToTabPage.Add(fileName, tabPage);

                StreamReader sr = new StreamReader(dialog.FileName);
                string str = sr.ReadToEnd();
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
                sr.Close();
            }
        }

        private void menucommand_File_Open_File_ParserSpecificationFile_CPlusPlus_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "C++ Parser Specifications File (*.parcpp)|*.parcpp";
            dialog.AddExtension = true;
            dialog.DefaultExt = "parcpp";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Open a new C++ parser file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.ParCPlusPlus, filePath, fileName));
                this.fileNotInProjectToTabPage.Add(fileName, tabPage);

                StreamReader sr = new StreamReader(dialog.FileName);
                string str = sr.ReadToEnd();
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
                sr.Close();
            }
        }

        private void menucommand_File_Open_File_ParserSpecificationFile_CSharp_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "C# Parser Specifications File (*.parcs)|*.parcs";
            dialog.AddExtension = true;
            dialog.DefaultExt = "parcs";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Open a new C# parser file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.ParCSharp, filePath, fileName));
                this.fileNotInProjectToTabPage.Add(fileName, tabPage);

                StreamReader sr = new StreamReader(dialog.FileName);
                string str = sr.ReadToEnd();
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
                sr.Close();
            }
        }

        private void menucommand_File_Open_File_ParserSpecificationFile_Java_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "Java Parser Specifications File (*.parjava)|*.parjava";
            dialog.AddExtension = true;
            dialog.DefaultExt = "parjava";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Open a new Java parser file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.ParJava, filePath, fileName));
                this.fileNotInProjectToTabPage.Add(fileName, tabPage);

                StreamReader sr = new StreamReader(dialog.FileName);
                string str = sr.ReadToEnd();
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
                sr.Close();
            }
        }

        private void menucommand_File_Open_File_OtherFile_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "All Files (*,*)|*.*";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Open a file";

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                int from = dialog.FileName.LastIndexOf('\\');
                string filePath = dialog.FileName.Substring(0, from);
                string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
                Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.None, filePath, fileName));
                this.fileNotInProjectToTabPage.Add(fileName, tabPage);

                StreamReader sr = new StreamReader(dialog.FileName);
                string str = sr.ReadToEnd();
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
                sr.Close();
            }
        }

		private void menucommand_File_Open_File_GraphicalRegularLanguageSpecificationFile_Click(object sender, EventArgs e)
		{
			OpenFileDialog dialog = new OpenFileDialog();
			dialog.RestoreDirectory = true;
			dialog.Filter = "GTG file (*.gtg)|*.gtg";
			dialog.AddExtension = true;
			dialog.DefaultExt = "gtg";
			dialog.InitialDirectory = "C:\\MyProjects";
			dialog.Title = "Open a new GTG file";
			if (dialog.ShowDialog() != DialogResult.OK) return;

			int from = dialog.FileName.LastIndexOf('\\');
			string filePath = dialog.FileName.Substring(0, from);
			string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
			FileStream stream = new FileStream(dialog.FileName, FileMode.Open, FileAccess.Read, FileShare.None);
			GTGForm frm = GTGForm.FromFile(stream, GetStatusString, SetStatusString, SetOutputText);
			stream.Close();

			Crownwood.DotNetMagic.Controls.TabPage tabPage =
			  tabbedgroups_Main.ActiveLeaf.TabPages.Add(new Crownwood.DotNetMagic.Controls.TabPage(fileName, frm));
			tabPage.ToolTip = dialog.FileName;
			tabbedgroups_Main.ActiveLeaf.TabControl.SelectedTab = tabPage;
		}

        private void menucommand_File_Close_Click(object sender, EventArgs e)
        {
            if (tabbedgroups_Main.ActiveTabPage == null)
                return;
        }

        private void menucommand_File_CloseProject_Click(object sender, EventArgs e)
        {
            currentProject = null;
            fileInProjectToTabPage = null;
            tabPageToFileInProject = null;
        }

        private void menucommand_File_Save_Project_Click(object sender, EventArgs e)
        {
            foreach (ProjectFile file in currentProject.LexicalAnalyserCodeFiles)
            {
                Crownwood.DotNetMagic.Controls.TabPage tempTabPage = (Crownwood.DotNetMagic.Controls.TabPage)(fileInProjectToTabPage[file.Name]);
                if (tempTabPage == null)
                    continue;

                FileStream stream = new FileStream(file.Path + "\\" + file.Name, FileMode.Truncate, FileAccess.Write, FileShare.None);
                StreamWriter writer = new StreamWriter(stream);
                writer.Write(((Fireball.Windows.Forms.CodeEditorControl)(tempTabPage.Control)).Document.Text);
                writer.Close();
                stream.Close();
            }
            foreach (ProjectFile file in currentProject.ParserCodeFiles)
            {
                Crownwood.DotNetMagic.Controls.TabPage tempTabPage = (Crownwood.DotNetMagic.Controls.TabPage)(fileInProjectToTabPage[file.Name]);
                if (tempTabPage == null)
                    continue;

                FileStream stream = new FileStream(file.Path + "\\" + file.Name, FileMode.Truncate, FileAccess.Write, FileShare.None);
                StreamWriter writer = new StreamWriter(stream);
                writer.Write(((Fireball.Windows.Forms.CodeEditorControl)(tempTabPage.Control)).Document.Text);
                writer.Close();
                stream.Close();
            }
            foreach (ProjectFile file in currentProject.ParserSpecificationFiles)
            {
                Crownwood.DotNetMagic.Controls.TabPage tempTabPage = (Crownwood.DotNetMagic.Controls.TabPage)(fileInProjectToTabPage[file.Name]);
                if (tempTabPage == null)
                    continue;

                FileStream stream = new FileStream(file.Path + "\\" + file.Name, FileMode.Truncate, FileAccess.Write, FileShare.None);
                StreamWriter writer = new StreamWriter(stream);
                writer.Write(((Fireball.Windows.Forms.CodeEditorControl)(tempTabPage.Control)).Document.Text);
                writer.Close();
                stream.Close();
            }
            foreach (ProjectFile file in currentProject.LexicalAnalyserSpecificationFiles)
            {
                Crownwood.DotNetMagic.Controls.TabPage tempTabPage = (Crownwood.DotNetMagic.Controls.TabPage)(fileInProjectToTabPage[file.Name]);
                if (tempTabPage == null)
                    continue;

                FileStream stream = new FileStream(file.Path + "\\" + file.Name, FileMode.Truncate, FileAccess.Write, FileShare.None);
                StreamWriter writer = new StreamWriter(stream);
                writer.Write(((Fireball.Windows.Forms.CodeEditorControl)(tempTabPage.Control)).Document.Text);
                writer.Close();
                stream.Close();
            }
            foreach (ProjectFile file in currentProject.MiscellaneousFiles)
            {
                Crownwood.DotNetMagic.Controls.TabPage tempTabPage = (Crownwood.DotNetMagic.Controls.TabPage)(fileInProjectToTabPage[file.Name]);
                if (tempTabPage == null)
                    continue;

                FileStream stream = new FileStream(file.Path + "\\" + file.Name, FileMode.Truncate, FileAccess.Write, FileShare.None);
                StreamWriter writer = new StreamWriter(stream);
                writer.Write(((Fireball.Windows.Forms.CodeEditorControl)(tempTabPage.Control)).Document.Text);
                writer.Close();
                stream.Close();
            }
			foreach (ProjectFile file in currentProject.GraphicalRegularLanguageSpecificationFiles)
			{
				Crownwood.DotNetMagic.Controls.TabPage tempTabPage = (Crownwood.DotNetMagic.Controls.TabPage)(fileInProjectToTabPage[file.Name]);
				if (tempTabPage == null)
					continue;

				FileStream stream = new FileStream(file.Path + "\\" + file.Name, FileMode.Truncate, FileAccess.Write, FileShare.None);
				BinaryFormatter formatter = new BinaryFormatter();
				formatter.Serialize(stream, ((GTGForm)tempTabPage.Control)._gtg);
				stream.Close();
			}

			{
				FileStream formattedStream = new FileStream(currentProject.Path + "\\" + currentProject.FolderName + "\\" + currentProject.Name + ".ccwproj", FileMode.Truncate, FileAccess.Write, FileShare.None);
				BinaryFormatter formatter = new BinaryFormatter();
				formatter.Serialize(formattedStream, currentProject);
				formattedStream.Close();
			}
        }

        private void menucommand_File_Save_File_Click(object sender, EventArgs e)
		{
			try
			{
				if (tabbedgroups_Main.ActiveTabPage == null)
					return;

				string filePath = tabbedgroups_Main.ActiveTabPage.ToolTip;

				if (tabbedgroups_Main.ActiveTabPage.Control is GTGForm)
				{
					BinaryFormatter formatter = new BinaryFormatter();
					FileStream stream = new FileStream(filePath, FileMode.Create, FileAccess.Write, FileShare.None);
					formatter.Serialize(stream, ((GTGForm)tabbedgroups_Main.ActiveTabPage.Control)._gtg);
					stream.Close();
				}
				else
				{
					FileStream stream = new FileStream(filePath, FileMode.Truncate, FileAccess.Write, FileShare.None);
					StreamWriter writer;
					if (filePath.EndsWith(".lexcs", StringComparison.CurrentCultureIgnoreCase) ||
					  filePath.EndsWith(".lexjava", StringComparison.CurrentCultureIgnoreCase) ||
					  filePath.EndsWith(".lexcpp", StringComparison.CurrentCultureIgnoreCase))
					{
						writer = new StreamWriter(stream, Encoding.UTF8);
					}
					else writer = new StreamWriter(stream);
					writer.Write(((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text);
					writer.Close();
					stream.Close();
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
			}
        }

        private void menucommand_File_SaveAs_NewFile_Click(object sender, EventArgs e)
        {

        }

        private void menucommand_File_SaveAs_NewProject_Click(object sender, EventArgs e)
        {

        }

        private void menucommand_File_SaveAll_Click(object sender, EventArgs e)
        {
            //if (tabbedgroups_Main.ActiveTabPage == null)
            //    return;

            //for (int i = 0; i < tabbedgroups_Main.RootSequence.Count; i++)
            //    for (int j = 0; j < tabbedgroups_Main.RootSequence[i].TabbedGroups; j++)
            //    {
            //        FileStream stream = new FileStream(page.ToolTip, FileMode.Truncate, FileAccess.Write, FileShare.None);
            //        StreamWriter writer = new StreamWriter(stream);
            //        writer.Write(((Fireball.Windows.Forms.CodeEditorControl)(page.Control)).Document.Text);
            //        writer.Close();//010 6847416 6841672
            //        stream.Close();
            //    };
        }

        private void menucommand_File_Exit_Click(object sender, System.EventArgs e)
        {
            Close();
        }

        private void menucommand_Edit_Copy_Click(object sender, EventArgs e)
        {
           if (tabbedgroups_Main.ActiveTabPage != null)
               ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Copy();
        }

        private void menucommand_Edit_Cut_Click(object sender, EventArgs e)
        {
            if (tabbedgroups_Main.ActiveTabPage != null)
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Cut();
        }

        private void menucommand_Edit_Paste_Click(object sender, EventArgs e)
        {
			if (tabbedgroups_Main.ActiveTabPage != null)
			    ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Paste();
        }

        private void menucommand_Edit_Delete_Click(object sender, EventArgs e)
        {
            if (tabbedgroups_Main.ActiveTabPage != null)
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Delete();
        }

        private void menucommand_Edit_SelectAll_Click(object sender, EventArgs e)
        {
            if (tabbedgroups_Main.ActiveTabPage != null)
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).SelectAll();
        }

        private void menucommand_Edit_Undo_Click(object sender, EventArgs e)
        {
            if (tabbedgroups_Main.ActiveTabPage != null)
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Undo();
        }

        private void menucommand_Edit_Redo_Click(object sender, EventArgs e)
        {
            if (tabbedgroups_Main.ActiveTabPage != null)
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Redo();
        }

        private void menucommand_Edit_FindAndReplace_Click(object sender, EventArgs e)
        {
            if (tabbedgroups_Main.ActiveTabPage != null)
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).ShowFind();
        }

        private void menucommand_Edit_GoTo_Click(object sender, EventArgs e)
        {
            if (tabbedgroups_Main.ActiveTabPage == null)
                return;

            IDE.Forms.CCW.GoToLineWindow window = new IDE.Forms.CCW.GoToLineWindow(((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Count);
            if ((window.ShowDialog() == DialogResult.OK))
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).GotoLine(window.LineNumber - 1);
        }

        private void menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_Java_Click(object sender, EventArgs e)
        {
            string caption = "Java Lexical Analyser Specification File";
            string fileExtension = "lexjava";
            IDE.Forms.CCW.NewItem newItemDialog = new IDE.Forms.CCW.NewItem(caption, fileExtension);

            if (newItemDialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Lexical Analysers\\" + newItemDialog.FileName);
				Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.LexJava, currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Lexical Analysers", newItemDialog.FileName));
                fileInProjectToTabPage.Add(newItemDialog.FileName, tabPage);
                ProjectFile file = new ProjectFile(DateTime.Now, currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Lexical Analysers", newItemDialog.FileName, FileLanguage.LexJava);
                currentProject.LexicalAnalyserSpecificationFiles.Add(file);
                this.tabPageToFileInProject.Add(tabPage, file);
                Node node = new Node(file.Name);
                ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[0].Nodes.Add(node);
            }
        }

        private void menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CPlusPlus_Click(object sender, EventArgs e)
        {
            string caption = "C++ Lexical Analyser Specification File";
            string fileExtension = "lexcpp";
            IDE.Forms.CCW.NewItem newItemDialog = new IDE.Forms.CCW.NewItem(caption, fileExtension);

            if (newItemDialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Lexical Analysers\\" + newItemDialog.FileName);
				Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.LexCPlusPlus, currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Lexical Analysers", newItemDialog.FileName));
                fileInProjectToTabPage.Add(newItemDialog.FileName, tabPage);
                ProjectFile file = new ProjectFile(DateTime.Now, currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Lexical Analysers", newItemDialog.FileName, FileLanguage.LexCPlusPlus);
                currentProject.LexicalAnalyserSpecificationFiles.Add(file);
                this.tabPageToFileInProject.Add(tabPage, file);
                Node node = new Node(file.Name);
                ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[0].Nodes.Add(node);
            }
        }

        private void menucommand_Project_AddNewItem_File_LexicalAnalyserSpecificationFile_BlankFile_CSharp_Click(object sender, EventArgs e)
        {
            string caption = "C# Lexical Analyser Specification File";
            string fileExtension = "lexcs";
            IDE.Forms.CCW.NewItem newItemDialog = new IDE.Forms.CCW.NewItem(caption, fileExtension);

            if (newItemDialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Lexical Analysers\\" + newItemDialog.FileName);
				Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.LexCSharp, currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Lexical Analysers", newItemDialog.FileName));
                fileInProjectToTabPage.Add(newItemDialog.FileName, tabPage);
                ProjectFile file = new ProjectFile(DateTime.Now, currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Lexical Analysers", newItemDialog.FileName, FileLanguage.LexCSharp);
                currentProject.LexicalAnalyserSpecificationFiles.Add(file);
                this.tabPageToFileInProject.Add(tabPage, file);
                Node node = new Node(file.Name);
                ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[0].Nodes.Add(node);
            }
        }

        private void menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_Java_Click(object sender, EventArgs e)
        {
            string caption = "Java Parser Specification File";
            string fileExtension = "parjava";
            IDE.Forms.CCW.NewItem newItemDialog = new IDE.Forms.CCW.NewItem(caption, fileExtension);

            if (newItemDialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Parsers\\" + newItemDialog.FileName);
				Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.ParJava, currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Parsers", newItemDialog.FileName));
                fileInProjectToTabPage.Add(newItemDialog.FileName, tabPage);
                ProjectFile file = new ProjectFile(DateTime.Now, currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Parsers", newItemDialog.FileName, FileLanguage.ParJava);
                currentProject.ParserSpecificationFiles.Add(file);
                this.tabPageToFileInProject.Add(tabPage, file);
                Node node = new Node(file.Name);
                ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[1].Nodes.Add(node);
            }
        }

        private void menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CPlusPlus_Click(object sender, EventArgs e)
        {
            string caption = "C++ Parser Specification File";
            string fileExtension = "parcpp";
            IDE.Forms.CCW.NewItem newItemDialog = new IDE.Forms.CCW.NewItem(caption, fileExtension);

            if (newItemDialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Parsers\\" + newItemDialog.FileName);
				Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.ParCPlusPlus, currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Parsers", newItemDialog.FileName));
                fileInProjectToTabPage.Add(newItemDialog.FileName, tabPage);
                ProjectFile file = new ProjectFile(DateTime.Now, currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Parsers", newItemDialog.FileName, FileLanguage.ParCPlusPlus);
                currentProject.ParserSpecificationFiles.Add(file);
                this.tabPageToFileInProject.Add(tabPage, file);
                Node node = new Node(file.Name);
                ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[1].Nodes.Add(node);
            }
        }

        private void menucommand_Project_AddNewItem_File_ParserSpecificationFile_BlankFile_CSharp_Click(object sender, EventArgs e)
        {
            string caption = "C# Parser Specification File";
            string fileExtension = "parcs";
            IDE.Forms.CCW.NewItem newItemDialog = new IDE.Forms.CCW.NewItem(caption, fileExtension);

            if (newItemDialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Parsers\\" + newItemDialog.FileName);
				Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.ParCSharp, currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Parsers", newItemDialog.FileName));
                fileInProjectToTabPage.Add(newItemDialog.FileName, tabPage);
                ProjectFile file = new ProjectFile(DateTime.Now, currentProject.Path + "\\" + currentProject.FolderName + "\\Specification Files\\Parsers", newItemDialog.FileName, FileLanguage.ParCSharp);
                currentProject.ParserSpecificationFiles.Add(file);
                this.tabPageToFileInProject.Add(tabPage, file);
                Node node = new Node(file.Name);
                ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[1].Nodes.Add(node);
            }
        }

        private void menucommand_Project_AddNewItem_File_GraphicalRegularLanguageSpecification_Click(object sender, EventArgs e)
		{
			SaveFileDialog dialog = new SaveFileDialog();
			dialog.RestoreDirectory = true;
			dialog.Filter = "GTG file (*.gtg)|*.gtg";
			dialog.AddExtension = true;
			dialog.DefaultExt = "gtg";
			dialog.InitialDirectory = "C:\\MyProjects";
			dialog.Title = "Create a new GTG file";
			if (dialog.ShowDialog() != DialogResult.OK) return;

			File.Create(dialog.FileName);
			int from = dialog.FileName.LastIndexOf('\\');
			string filePath = dialog.FileName.Substring(0, from);
			string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
			GTGForm frm = new GTGForm(GetStatusString, SetStatusString, SetOutputText);
			Crownwood.DotNetMagic.Controls.TabPage tabPage =
			  tabbedgroups_Main.ActiveLeaf.TabPages.Add(new Crownwood.DotNetMagic.Controls.TabPage(fileName, frm));
			tabPage.ToolTip = dialog.FileName;
			fileInProjectToTabPage.Add(fileName, tabPage);
			ProjectFile file = new ProjectFile(DateTime.Now, filePath, fileName, FileLanguage.None);
			currentProject.GraphicalRegularLanguageSpecificationFiles.Add(file);
			this.tabPageToFileInProject.Add(tabPage, file);
			tabbedgroups_Main.ActiveLeaf.TabControl.SelectedTab = tabPage;
        }

        private void menucommand_Project_AddNewItem_File_OtherFile_Click(object sender, EventArgs e)
        {
            string caption = "Miscellaneous File";
            string fileExtension = "";
            IDE.Forms.CCW.NewItem newItemDialog = new IDE.Forms.CCW.NewItem(caption, fileExtension);

            if (newItemDialog.ShowDialog() == DialogResult.OK)
            {
                File.Create(currentProject.Path + "\\" + currentProject.FolderName + "\\Miscellaneous Files\\" + newItemDialog.FileName);
				Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.None, currentProject.Path + "\\" + currentProject.FolderName + "\\Miscellaneous Files", newItemDialog.FileName));
                fileInProjectToTabPage.Add(newItemDialog.FileName, tabPage);
                ProjectFile file = new ProjectFile(DateTime.Now, currentProject.Path + "\\" + currentProject.FolderName + "\\Miscellaneous Files", newItemDialog.FileName, FileLanguage.None);
                currentProject.ParserSpecificationFiles.Add(file);
                this.tabPageToFileInProject.Add(tabPage, file);
                Node node = new Node(file.Name);
                ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[5].Nodes.Add(node);
            }
        }

        private void menucommand_Project_AddNewItem_Folder_Click(object sender, EventArgs e)
        {
            //Film Hindi

            //FolderBrowserDialog dialog = new FolderBrowserDialog();
            //dialog.RestoreDirectory = true;
            //dialog.Title = "Save a new file";

            //if (dialog.ShowDialog() == DialogResult.OK)
            //{
            //    File.Create(dialog.FileName);
            //    int from = dialog.FileName.LastIndexOf('\\');
            //    string filePath = dialog.FileName.Substring(0, from);
            //    string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
            //    currentProject.MiscellaneousFiles.Add(new ProjectFile(DateTime.Now, filePath, fileName, FileLanguage.None));
            //    Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.None, filePath, fileName));
            //    currentProject.FilesStatus.Add(fileName, tabPage);
            //    Node node = new Node(file.Name);
            //    ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[6].Nodes.Add(node);
            //}
        }

        private void menucommand_Project_AddExistingItem_ParserSpecificationFile_Java_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "Java Parser Code File (*.parjava)|*.parjava";
            dialog.AddExtension = true;
            dialog.DefaultExt = "parjava";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Open an existing Java parser file";
            if (dialog.ShowDialog() == DialogResult.Cancel) return;

            int from = dialog.FileName.LastIndexOf('\\');
            string filePath = dialog.FileName.Substring(0, from);
            string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
            currentProject.ParserSpecificationFiles.Add(new ProjectFile(DateTime.Now, filePath, fileName, FileLanguage.ParJava));
            Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.ParJava, filePath, fileName));
            fileInProjectToTabPage.Add(fileName, tabPage);
            ProjectFile file = new ProjectFile(DateTime.Now, filePath, fileName, FileLanguage.ParJava);
            currentProject.LexicalAnalyserSpecificationFiles.Add(file);
            this.tabPageToFileInProject.Add(tabPage, file);
            Node node = new Node(file.Name);
            ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[1].Nodes.Add(node);

            StreamReader sr = new StreamReader(dialog.FileName);
            string str = sr.ReadToEnd();
            ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
            sr.Close();
        }

        private void menucommand_Project_AddExistingItem_ParserSpecificationFile_CPlusPlus_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "C++ Parser Code File (*.parcpp)|*.parcpp";
            dialog.AddExtension = true;
            dialog.DefaultExt = "parcpp";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Open an existing C++ parser file";
            if (dialog.ShowDialog() == DialogResult.Cancel) return;

            int from = dialog.FileName.LastIndexOf('\\');
            string filePath = dialog.FileName.Substring(0, from);
            string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
            Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.ParCPlusPlus, filePath, fileName));
            fileInProjectToTabPage.Add(fileName, tabPage);
            ProjectFile file = new ProjectFile(DateTime.Now, filePath, fileName, FileLanguage.ParCPlusPlus);
            currentProject.LexicalAnalyserSpecificationFiles.Add(file);
            this.tabPageToFileInProject.Add(tabPage, file);
            Node node = new Node(file.Name);
            ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[1].Nodes.Add(node);

            StreamReader sr = new StreamReader(dialog.FileName);
            string str = sr.ReadToEnd();
            ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
            sr.Close();
        }

        private void menucommand_Project_AddExistingItem_ParserSpecificationFile_CSharp_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "C# Parser Code File (*.parcs)|*.parcs";
            dialog.AddExtension = true;
            dialog.DefaultExt = "parcs";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Open an existing C# parser file";
            if (dialog.ShowDialog() == DialogResult.Cancel) return;

            int from = dialog.FileName.LastIndexOf('\\');
            string filePath = dialog.FileName.Substring(0, from);
            string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
            Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.ParCSharp, filePath, fileName));
            fileInProjectToTabPage.Add(fileName, tabPage);
            ProjectFile file = new ProjectFile(DateTime.Now, filePath, fileName, FileLanguage.ParCSharp);
            currentProject.LexicalAnalyserSpecificationFiles.Add(file);
            this.tabPageToFileInProject.Add(tabPage, file);
            StreamReader sr = new StreamReader(dialog.FileName);
            Node node = new Node(file.Name);
            ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[1].Nodes.Add(node);

            string str = sr.ReadToEnd();
            ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
            sr.Close();
        }

        private void menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_Java_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "Java Scanner Code File (*.lexjava)|*.lexjava";
            dialog.AddExtension = true;
            dialog.DefaultExt = "lexjava";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Open an existing Java scanner file";
            if (dialog.ShowDialog() == DialogResult.Cancel) return;

            int from = dialog.FileName.LastIndexOf('\\');
            string filePath = dialog.FileName.Substring(0, from);
            string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
            Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.LexJava, filePath, fileName));
            fileInProjectToTabPage.Add(fileName, tabPage);
            ProjectFile file = new ProjectFile(DateTime.Now, filePath, fileName, FileLanguage.LexJava);
            currentProject.LexicalAnalyserSpecificationFiles.Add(file);
            this.tabPageToFileInProject.Add(tabPage, file);
            Node node = new Node(file.Name);
            ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[0].Nodes.Add(node);

            StreamReader sr = new StreamReader(dialog.FileName);
            string str = sr.ReadToEnd();
            ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
            sr.Close();
        }

        private void menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CPlusPlus_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "C++ Scanner Code File (*.lexcpp)|*.lexcpp";
            dialog.AddExtension = true;
            dialog.DefaultExt = "lexcpp";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Open an existing C++ scanner file";
            if (dialog.ShowDialog() == DialogResult.Cancel) return;

            int from = dialog.FileName.LastIndexOf('\\');
            string filePath = dialog.FileName.Substring(0, from);
            string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
            Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.LexCPlusPlus, filePath, fileName));
            fileInProjectToTabPage.Add(fileName, tabPage);
            ProjectFile file = new ProjectFile(DateTime.Now, filePath, fileName, FileLanguage.LexCPlusPlus);
            currentProject.LexicalAnalyserSpecificationFiles.Add(file);
            this.tabPageToFileInProject.Add(tabPage, file);
            Node node = new Node(file.Name);
            ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[0].Nodes.Add(node);

            StreamReader sr = new StreamReader(dialog.FileName);
            string str = sr.ReadToEnd();
            ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
            sr.Close();
        }

        private void menucommand_Project_AddExistingItem_LexicalAnalyserSpecificationFile_CSharp_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "C# Scanner Code File (*.lexcs)|*.lexcs";
            dialog.AddExtension = true;
            dialog.DefaultExt = "lexcs";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Open an existing C# scanner file";
            if (dialog.ShowDialog() == DialogResult.Cancel) return;

            int from = dialog.FileName.LastIndexOf('\\');
            string filePath = dialog.FileName.Substring(0, from);
            string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
            Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.LexCSharp, filePath, fileName));
            fileInProjectToTabPage.Add(fileName, tabPage);
            ProjectFile file = new ProjectFile(DateTime.Now, filePath, fileName, FileLanguage.LexCSharp);
            currentProject.LexicalAnalyserSpecificationFiles.Add(file);
            this.tabPageToFileInProject.Add(tabPage, file);
            Node node = new Node(file.Name);
            ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[0].Nodes.Add(node);

            StreamReader sr = new StreamReader(dialog.FileName);
            string str = sr.ReadToEnd();
            ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
            sr.Close();
        }

        private void menucommand_Project_AddExistingItem_OtherFile_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "All Files (*,*)|*.*";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Add an existing file to your project";
            if (dialog.ShowDialog() == DialogResult.Cancel) return;

            int from = dialog.FileName.LastIndexOf('\\');
            string filePath = dialog.FileName.Substring(0, from);
            string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
            Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.None, filePath, fileName));
            fileInProjectToTabPage.Add(fileName, tabPage);
            ProjectFile file = new ProjectFile(DateTime.Now, filePath, fileName, FileLanguage.None);
            currentProject.MiscellaneousFiles.Add(file);
            this.tabPageToFileInProject.Add(tabPage, file);
            Node node = new Node(file.Name);
            ((TreeControl)(dockingManager.Contents["Project Explorer"].Control)).Nodes[5].Nodes.Add(node);

            StreamReader sr = new StreamReader(dialog.FileName);
            string str = sr.ReadToEnd();
            ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
            sr.Close();
        }

        private void menucommand_Project_AddExistingItem_GraphicalRegularLanguageSpecificationFile_Click(object sender, EventArgs e)
		{
			OpenFileDialog dialog = new OpenFileDialog();
			dialog.RestoreDirectory = true;
			dialog.Filter = "GTG file (*.gtg)|*.gtg";
			dialog.AddExtension = true;
			dialog.DefaultExt = "gtg";
			dialog.InitialDirectory = "C:\\MyProjects";
			dialog.Title = "Open a new GTG file";
			if (dialog.ShowDialog() != DialogResult.OK) return;

			FileStream stream = new FileStream(dialog.FileName, FileMode.Open, FileAccess.Read, FileShare.None);
			GTGForm frm = GTGForm.FromFile(stream, GetStatusString, SetStatusString, SetOutputText);
			stream.Close();

			int from = dialog.FileName.LastIndexOf('\\');
			string filePath = dialog.FileName.Substring(0, from);
			string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
			Crownwood.DotNetMagic.Controls.TabPage tabPage =
			  tabbedgroups_Main.ActiveLeaf.TabPages.Add(new Crownwood.DotNetMagic.Controls.TabPage(fileName, frm));
			tabPage.ToolTip = dialog.FileName;
			fileInProjectToTabPage.Add(fileName, tabPage);
			ProjectFile file = new ProjectFile(DateTime.Now, filePath, fileName, FileLanguage.None);
			currentProject.GraphicalRegularLanguageSpecificationFiles.Add(file);
			this.tabPageToFileInProject.Add(tabPage, file);
			tabbedgroups_Main.ActiveLeaf.TabControl.SelectedTab = tabPage;
        }

        private void menucommand_Windows_ProjectExplorer_Click(object sender, EventArgs e)
        {
            RefreshProjectExplorer();
            if (dockingManager.Contents["Project Explorer"].Visible)
                dockingManager.ToggleContentAutoHide(dockingManager.Contents["Project Explorer"]);
            else
                dockingManager.ShowContent(dockingManager.Contents["Project Explorer"]);
        }

        private void menucommand_Windows_TaskList_Click(object sender, EventArgs e)
        {
            if (dockingManager.Contents["Task List"].Visible)
                dockingManager.ToggleContentAutoHide(dockingManager.Contents["Task List"]);
            else
                dockingManager.ShowContent(dockingManager.Contents["Task List"]);
        }

        private void menucommand_Windows_Output_Click(object sender, EventArgs e)
        {
            if (dockingManager.Contents["Output"].Visible)
                dockingManager.ToggleContentAutoHide(dockingManager.Contents["Output"]);
            else
                dockingManager.ShowContent(dockingManager.Contents["Output"]);
        }

        private void menucommand_Windows_ShowAllWindows_Click(object sender, EventArgs e)
        {
            if (dockingManager.Contents["Output"].Visible)
            {
                if (dockingManager.Contents["Output"].IsAutoHidden)
                    dockingManager.ToggleContentAutoHide(dockingManager.Contents["Output"]);
            }
            else
            {
                dockingManager.ShowContent(dockingManager.Contents["Output"]);
                if (dockingManager.Contents["Output"].IsAutoHidden)
                    dockingManager.ToggleContentAutoHide(dockingManager.Contents["Output"]);
            }

            if (dockingManager.Contents["Task List"].Visible)
            {
                if (dockingManager.Contents["Task List"].IsAutoHidden)
                    dockingManager.ToggleContentAutoHide(dockingManager.Contents["Task List"]);
            }
            else
            {
                dockingManager.ShowContent(dockingManager.Contents["Task List"]);
                if (dockingManager.Contents["Task List"].IsAutoHidden)
                    dockingManager.ToggleContentAutoHide(dockingManager.Contents["Task List"]);
            }

            if (dockingManager.Contents["Project Explorer"].Visible)
            {
                if (dockingManager.Contents["Project Explorer"].IsAutoHidden)
                    dockingManager.ToggleContentAutoHide(dockingManager.Contents["Project Explorer"]);
            }
            else
            {
                dockingManager.ShowContent(dockingManager.Contents["Project Explorer"]);
                if (dockingManager.Contents["Project Explorer"].IsAutoHidden)
                    dockingManager.ToggleContentAutoHide(dockingManager.Contents["Project Explorer"]);
            }
        }

        private void menucommand_Windows_HideAllWindows_Click(object sender, EventArgs e)
        {
            if (dockingManager.Contents["Output"].Visible)
                dockingManager.HideContent(dockingManager.Contents["Output"]);

            if (dockingManager.Contents["Task List"].Visible)
                dockingManager.HideContent(dockingManager.Contents["Task List"]);

            if (dockingManager.Contents["Project Explorer"].Visible)
                dockingManager.HideContent(dockingManager.Contents["Project Explorer"]);
        }

        private void menucommand_Tools_LEXcellent_Click(object sender, EventArgs e)
        {
            menucommand_File_Save_File.PerformClick();
            try
            {
                string path = tabbedgroups_Main.ActiveTabPage.ToolTip;
                int from = path.LastIndexOf('\\');
                string filePath = path.Substring(0, from);
                string fileName = path.Substring(from + 1, path.Length - from - 1).Split('.')[0];
                RunTool("Tools\\Lexcellent.exe", path, filePath + "\\" + fileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void menucommand_Tools_LeftFactoring_BNF_Click(object sender, EventArgs e)
        {
            menucommand_File_Save_File.PerformClick();
            string path = tabbedgroups_Main.ActiveTabPage.ToolTip;
            RunTool("Tools\\LF_LL1.exe", path, path);
        }

        private void menucommand_Tools_LeftFactoring_EBNF_Click(object sender, EventArgs e)
        {
            menucommand_File_Save_File.PerformClick();
            string path = tabbedgroups_Main.ActiveTabPage.ToolTip;
            RunTool("Tools\\LF_RD.exe", path, path);
        }

        private void menucommand_Tools_LeftRecursionRemoval_Click(object sender, EventArgs e)
        {
            //menucommand_File_Save_File.PerformClick();
            //try
            //{
            //    string path = tabbedgroups_Main.ActiveTabPage.ToolTip;
            //    int from = path.LastIndexOf('\\');
            //    string filePath = path.Substring(0, from);
            //    string fileName = path.Substring(from + 1, path.Length - from - 1).Split('.')[0];
            //    RunTool("Tools\\LRR.exe", path, filePath + "\\" + fileName);
            //}
            //catch (Exception ex)
            //{
            //    MessageBox.Show(ex.Message);
            //}

            tabbedgroups_Main.ActiveLeaf.TabPages.Add( NewTabPage(FileLanguage.ParCSharp, "", "test_result.parcs") );
         }

        private void menucommand_Tools_ParSpring_LL1Parser_Click(object sender, EventArgs e)
        {
            menucommand_File_Save_File.PerformClick();
            string path = tabbedgroups_Main.ActiveTabPage.ToolTip;
            int from = path.LastIndexOf('\\');
            string filePath = path.Substring(0, from);
            string fileName = path.Substring(from + 1, path.Length - from - 1).Split('.')[0];
            RunTool("Tools\\ParSpringLL1.exe", path, filePath + "\\" + fileName);
        }

        private void menucommand_Tools_ParSpring_RDParser_Click(object sender, EventArgs e)
        {
            menucommand_File_Save_File.PerformClick();
            string path = tabbedgroups_Main.ActiveTabPage.ToolTip;
            int from = path.LastIndexOf('\\');
            string filePath = path.Substring(0, from);
            string fileName = path.Substring(from + 1, path.Length - from - 1).Split('.')[0];
            RunTool("Tools\\ParSpringRD.exe", path, filePath + "\\" + fileName);
        }

        private void menucommand_Tools_SyntaxOptions_Click(object sender, EventArgs e)
        {
            FireEditLanguageOptions opt = new FireEditLanguageOptions();

            opt.ShowDialog();
        }

        void SetStatusString(string str)
        {
            statuspanel_Description.Text = str;
        }

        string GetStatusString()
        {
            return statuspanel_Description.Text;
        }

        private void menucommand_Tools_GTGEditor_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.RestoreDirectory = true;
            dialog.Filter = "GTG file (*.gtg)|*.gtg";
            dialog.AddExtension = true;
            dialog.DefaultExt = "gtg";
            dialog.InitialDirectory = "C:\\MyProjects";
            dialog.Title = "Create a new GTG file";
            if (dialog.ShowDialog() != DialogResult.OK) return;

            File.Create(dialog.FileName);
            int from = dialog.FileName.LastIndexOf('\\');
            string filePath = dialog.FileName.Substring(0, from);
            string fileName = dialog.FileName.Substring(from + 1, dialog.FileName.Length - from - 1);
            GTGForm frm = new GTGForm(GetStatusString, SetStatusString, SetOutputText);
            Crownwood.DotNetMagic.Controls.TabPage tabPage =
              tabbedgroups_Main.ActiveLeaf.TabPages.Add(new Crownwood.DotNetMagic.Controls.TabPage(fileName, frm));
            tabPage.ToolTip = dialog.FileName;
            tabbedgroups_Main.ActiveLeaf.TabControl.SelectedTab = tabPage;
        }

        private void menucommand_Tools_RegExManipulator_Click(object sender, EventArgs e)
        {
            RegexLogicManipulator frm = new RegexLogicManipulator();
            Crownwood.DotNetMagic.Controls.TabPage page =
              tabbedgroups_Main.ActiveLeaf.TabPages.Add(new Crownwood.DotNetMagic.Controls.TabPage("RegExManipulator", frm));
            tabbedgroups_Main.ActiveLeaf.TabControl.SelectedTab = page;
            //page.Select();
        }

        private void menucommand_Help_Samples_ArabicCLanguage_Click(object sender, System.EventArgs e)
		{
			Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(FileLanguage.Gemix, "", ""));
			this.fileNotInProjectToTabPage.Add("", tabPage);
			tabPage.RightToLeft = RightToLeft.Yes;
		}

        private void menucommand_Help_AboutLEXcellent_Click(object sender, System.EventArgs e)
        {

        }

        private void menucommand_Help_AboutParSpring_Click(object sender, System.EventArgs e)
        {

        }

        private void menucommand_Help_AboutCCW_Click(object sender, System.EventArgs e)
        {

        }

        private void menucommand_Help_Acknowledgements_Click(object sender, System.EventArgs e)
        {

		}

        #endregion

        #region Drawing Panel
        //State currentState = State.Normal; //Commented till the drawing project is included.

        enum State
        {
            Normal,
            MovingNode,
            NodeCreation,
            EdgeCreation,
            SeekingDestinationNode
        }

        #region Event Handlers
        private void drawingPanel_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
            //graphicalRegularExpression.Draw(e.Graphics);
        }

        private void drawingPanel_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            /*if (e.Button == MouseButtons.Left)
            {
                switch (currentState)
                {
                    case State.NodeCreation:
                        graphicalRegularExpression.AddNode(new Point2F(e.X, e.Y));
                        drawingPanel.Invalidate();
                        break;
                    case State.Normal:
                        nodeBeingMoved = graphicalRegularExpression.NearestNode(new Point2F(e.X, e.Y));
                        if (nodeBeingMoved != -1)
                            currentState = State.MovingNode;
                        break;
                    case State.EdgeCreation:
                        sourceNode = graphicalRegularExpression.NearestNode(new Point2F(e.X, e.Y));
                        if (sourceNode != -1)
                            currentState = State.SeekingDestinationNode;
                        break;
                }
            }*/
        }
        private void drawingPanel_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            /*if (e.Button != MouseButtons.Left) return;
            switch (currentState)
            {
                case State.MovingNode:
                    nodeBeingMoved = -1;
                    currentState = State.Normal;
                    break;
                case State.SeekingDestinationNode:
                    int target = graphicalRegularExpression.NearestNode(new Point2F(e.X, e.Y));
                    if (target != -1 && target != sourceNode && !graph.IsEdge(sourceNode, target))
                    {
                        WeightDialog dlg = new WeightDialog();
                        DialogResult dres = dlg.ShowDialog(this);
                        if (dres == DialogResult.OK)
                        {
                            graph.AddEdge(sourceNode, target, dlg.weight);
                            drawingPanel.Invalidate();
                        }
                    }
                    sourceNode = -1;
                    currentState = State.EdgeCreation;
                    break;
            }*/
        }
        private void drawingPanel_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            /*if (e.Button != MouseButtons.Left) return;
            switch (currentState)
            {
                case State.MovingNode:
                    graphicalRegularExpression.LocationFromNode(nodeBeingMoved).Set(e.X, e.Y);
                    drawingPanel.Invalidate();
                    break;
            }*/
        }
        #endregion
        #endregion

        #region Timer Handler
        private void timer_Tick(object sender, System.EventArgs e)
        {
            // Update the date and time section of the status bar
            statuspanel_Date.Text = DateTime.Now.ToLongDateString();
        }
        #endregion

        #region Contents Creation and Events
        #region Creation
        private Content CreateRichTextBoxContent()
        {
            // Create the TextBox for use in the new docking window
            TextBox tb = new TextBox();
            tb.BorderStyle = BorderStyle.None;
            tb.Multiline = true;

            // Create a new docking Content instance with our new TextBox
            return dockingManager.Contents.Add(tb, "TextBox");
        }

        private Content CreateProjectExplorer()
        {
            // Create the TreeControl for use in the new docking window
            projectExplorer = new TreeControl();
            projectExplorer.SetTreeControlStyle(TreeControlStyles.GroupOfficeLight);
            projectExplorer.BorderStyle = TreeBorderStyle.Solid;
            projectExplorer.Indicators = Indicators.AtGroup;
            projectExplorer.Name = "Project Explorer";
            projectExplorer.GroupColoring = GroupColoring.Office2003Light;
            projectExplorer.MouseDoubleClick += new MouseEventHandler(projectExplorer_MouseDoubleClick);

            // Create top level groups
            Node group = new Node("Lexical Analyser Specification Files");
            group.Expanded = true;
            projectExplorer.Nodes.Add(group);
            group = new Node("Parser Specification Files");
            group.Expanded = true;
            projectExplorer.Nodes.Add(group);
            group = new Node("Graphical Regular Language Specification Files");
            group.Expanded = true;
            projectExplorer.Nodes.Add(group);
            group = new Node("Lexical Analyser Code Files");
            group.Expanded = true;
            projectExplorer.Nodes.Add(group);
            group = new Node("Parser Code Files");
            group.Expanded = true;
            projectExplorer.Nodes.Add(group);
            group = new Node("Miscellaneous Files");
            group.Expanded = true;
            projectExplorer.Nodes.Add(group);
            group = new Node("Additional Folders");
            group.Expanded = true;
            projectExplorer.Nodes.Add(group);

            RefreshProjectExplorer();

            // Create a new docking Content instance with our new TreeControl
            projectExplorerWindow = dockingManager.Contents.Add(projectExplorer, "Project Explorer");
            return projectExplorerWindow;
        }

        private void RefreshProjectExplorer()
        {
            /* Original Code
             * // Create some dummy entries in the group
             * for (int j = 0; j < 3; j++)
             * {
             *    // Create a new child node
             *    Node child = new Node("");

             *    // Give an image and indicator different for each child node
             *    //child.Image = imageList.Images[];
             *    child.Indicator = (Indicator)(3);

             *    group.Nodes.Add(child);
             * }
             */
        }

        private Content CreateTaskListContent()
        {
            // Create the Task List for use in the new docking window
			Errors.LEXcellentErrors ef = new Errors.LEXcellentErrors();
			ef.Tag = this;

            // Create a new docking Content instance with our new Task List
            taskListWindow = dockingManager.Contents.Add(ef, "Task List", images, 2);
            return taskListWindow;
        }

        private Content CreateOutputContent()
        {
            // Create the Task List for use in the new docking window
            IDE.Forms.CCW.Output output = new IDE.Forms.CCW.Output(IDE.Forms.CCW.OutputType.Text);

            // Create a new docking Content instance with our new Task List
            outputWindow = dockingManager.Contents.Add(output, "Output", images, 1);
            return outputWindow;
        }

        private Crownwood.DotNetMagic.Controls.TabPage NewTabPage(FileLanguage language, string filePath, string fileName)
        {
            Fireball.Windows.Forms.CodeEditorControl editor = new Fireball.Windows.Forms.CodeEditorControl();
            string caption = "";

            switch (language)
            {
                case FileLanguage.LexCSharp:
                    Fireball.CodeEditor.SyntaxFiles.CodeEditorSyntaxLoader.SetSyntax(editor, Fireball.CodeEditor.SyntaxFiles.SyntaxLanguage.Template);
                    caption = fileName;
                    break;

                case FileLanguage.LexCPlusPlus:
                    Fireball.CodeEditor.SyntaxFiles.CodeEditorSyntaxLoader.SetSyntax(editor, Fireball.CodeEditor.SyntaxFiles.SyntaxLanguage.Text);
                    caption = fileName;
                    break;

                case FileLanguage.LexJava:
                    Fireball.CodeEditor.SyntaxFiles.CodeEditorSyntaxLoader.SetSyntax(editor, Fireball.CodeEditor.SyntaxFiles.SyntaxLanguage.TurboPascal);
                    caption = fileName;
                    break;

                case FileLanguage.ParCSharp:
                    Fireball.CodeEditor.SyntaxFiles.CodeEditorSyntaxLoader.SetSyntax(editor, Fireball.CodeEditor.SyntaxFiles.SyntaxLanguage.VB);
                    caption = fileName;
                    break;

                case FileLanguage.ParCPlusPlus:
                    Fireball.CodeEditor.SyntaxFiles.CodeEditorSyntaxLoader.SetSyntax(editor, Fireball.CodeEditor.SyntaxFiles.SyntaxLanguage.VBNET);
                    caption = fileName;
                    break;

                case FileLanguage.ParJava:
                    Fireball.CodeEditor.SyntaxFiles.CodeEditorSyntaxLoader.SetSyntax(editor, Fireball.CodeEditor.SyntaxFiles.SyntaxLanguage.VBScript);
                    caption = fileName;
                    break;

				case FileLanguage.Gemix:
					Fireball.CodeEditor.SyntaxFiles.CodeEditorSyntaxLoader.SetSyntax(editor, Fireball.CodeEditor.SyntaxFiles.SyntaxLanguage.Gemix);
					caption = fileName;
					break;

                case FileLanguage.CPlusPlus:
                    Fireball.CodeEditor.SyntaxFiles.CodeEditorSyntaxLoader.SetSyntax(editor, Fireball.CodeEditor.SyntaxFiles.SyntaxLanguage.CPP);
                    caption = fileName;
                    break;

                case FileLanguage.CSharp:
                    Fireball.CodeEditor.SyntaxFiles.CodeEditorSyntaxLoader.SetSyntax(editor, Fireball.CodeEditor.SyntaxFiles.SyntaxLanguage.CSharp);
                    caption = fileName;
                    break;

                case FileLanguage.Java:
                    Fireball.CodeEditor.SyntaxFiles.CodeEditorSyntaxLoader.SetSyntax(editor, Fireball.CodeEditor.SyntaxFiles.SyntaxLanguage.Java);
                    caption = fileName;
                    break;

                case FileLanguage.None:
                    Fireball.CodeEditor.SyntaxFiles.CodeEditorSyntaxLoader.SetSyntax(editor, Fireball.CodeEditor.SyntaxFiles.SyntaxLanguage.Rtf);
                    caption = fileName;
                    break;
            }

            //Fireball.Windows.Forms.CodeEditor.EditViewControl child1 = new Fireball.Windows.Forms.CodeEditor.EditViewControl(editor);
            //Fireball.Windows.Forms.CodeEditor.EditViewControl child2 = new Fireball.Windows.Forms.CodeEditor.EditViewControl(editor);

            //editor.ShowEOLMarker = true;
            //editor.EOLMarkerColor            
            //editor.ForeColor = Color.Green;
            //editor.BackColor = Color.Red;
            //editor.LineNumberBackColor = Color.Black;
            //editor.LineNumberBorderColor = Color.Red;
            //editor.LineNumberForeColor = Color.Black;

            Crownwood.DotNetMagic.Controls.TabPage newPage = new Crownwood.DotNetMagic.Controls.TabPage(caption, editor);
            newPage.ToolTip = filePath + "\\" + fileName;
            newPage.Selected = true;
            newPage.BackColor = Color.Green;

            return newPage;
        }

        private Crownwood.DotNetMagic.Controls.TabPage NewDrawingPage(string filePath, string fileName)
        {
            drawingPanel = new Panel();
            drawingPanel.BorderStyle = BorderStyle.None;
            drawingPanel.BackColor = Color.White;
            drawingPanel.MouseDown += new MouseEventHandler(drawingPanel_MouseDown);
            drawingPanel.MouseUp += new MouseEventHandler(drawingPanel_MouseUp);
            drawingPanel.MouseMove += new MouseEventHandler(drawingPanel_MouseMove);
            drawingPanel.Paint += new PaintEventHandler(drawingPanel_Paint);

            Crownwood.DotNetMagic.Controls.TabPage newPage = new Crownwood.DotNetMagic.Controls.TabPage(fileName, drawingPanel);
            newPage.ToolTip = filePath;
            newPage.Selected = true;

            return newPage;
        }
        #endregion

        #region Event Handlers
        protected void OnContentShown(Content c, EventArgs cea)
        {
            if (c.Title == "Project Explorer")
                menucommand_Windows_ProjectExplorer.Checked = true;

            if (c.Title == "Task List")
                menucommand_Windows_TaskList.Checked = true;

            if (c.Title == "Output")
                menucommand_Windows_Output.Checked = true;
        }

        protected void OnContentHidden(Content c, EventArgs cea)
        {
            if (c.Title == "Project Explorer")
                menucommand_Windows_ProjectExplorer.Checked = false;

            if (c.Title == "Task List")
                menucommand_Windows_TaskList.Checked = false;

            if (c.Title == "Output")
                menucommand_Windows_Output.Checked = false;
        }

        protected void OnContentHiding(Content c, CancelEventArgs cea)
        {
            switch (ignoreClose)
            {
                case 0:
                    // Allow all, do nothing
                    break;
                case 1:
                    // Ignore TreeControl
                    cea.Cancel = c.Control is TreeControl;
                    break;
                case 2:
                    // Ignore TextBox
                    cea.Cancel = c.Control is TextBox;
                    break;
                case 3:
                    // Ignore Task List
                    cea.Cancel = c.Control is Errors.LEXcellentErrors;
                    break;
                case 4:
                    // Ignore all, cancel
                    cea.Cancel = true;
                    break;
            }
        }

        private void OnTabControlCreated(Crownwood.DotNetMagic.Controls.TabbedGroups tg, Crownwood.DotNetMagic.Controls.TabControl projectExplorer)
        {
            // Place a thin border between edge of the tab control and inside contents
            projectExplorer.ControlTopOffset = 3;
            projectExplorer.ControlBottomOffset = 3;
            projectExplorer.ControlLeftOffset = 3;
            projectExplorer.ControlRightOffset = 3;

            tg.DisplayTabMode = DisplayTabModes.ShowAll;
            tg.LayoutLock = false;
            tg.ProminentOnly = false;

            //// Remove all existing content
            //tg.RootSequence.Clear();

            //// Switch to ordering horizontal
            //tg.RootDirection = LayoutDirection.Horizontal;

            //// Add a new sequence to end of root
            //TabGroupLeaf tgl = tg.RootSequence.AddNewLeaf();

            //// Access the default leaf group
            //TabGroupLeaf tgl1 = tg.RootSequence[0] as TabGroupLeaf;


            //// Add two leafs into the new sequence
            //TabGroupLeaf tgl2 = tgs.AddNewLeaf() as TabGroupLeaf;
            //TabGroupLeaf tgl3 = tgs.AddNewLeaf() as TabGroupLeaf;

            //// Add a page two each tab leaf
            //tgl1.TabPages.Add(NewTabPage(false));
            //tgl2.TabPages.Add(NewTabPage(false));
            //tgl3.TabPages.Add(NewTabPage(false));

            //// Setup spacing
            //tgl1.Space = 35;
            //tgs.Space = 65;
            //tgl2.Space = 30;
            //tgl3.Space = 70;

            //// Ask control to reposition children according to new spacing
            //tg.RootSequence.Reposition();

            //tg.ActiveLeaf = tg.RootSequence[0] as TabGroupLeaf;
        }

        private void OnExternalDrop(Crownwood.DotNetMagic.Controls.TabbedGroups tg,
                                    Crownwood.DotNetMagic.Controls.TabGroupLeaf tgl,
                                    Crownwood.DotNetMagic.Controls.TabControl projectExplorer,
                                    Crownwood.DotNetMagic.Controls.TabbedGroups.DragProvider dp)
        {
            //// Create a new tab page
            //Crownwood.DotNetMagic.Controls.TabPage tp = NewTabPage(false); //Dummy false, temporarily

            //// Define the text in this control
            //(tp.Control as RichTextBox).Text = "Dragged from node '" + (string)dp.Tag + "'";

            //// We want the new page to become selected
            //tp.Selected = true;

            //// Add new page into the destination tab control
            //tgl.TabPages.Add(tp);
        }
        #endregion

        //Common to all contents
        private void InitializeContent(Content c)
        {
            c.CaptionBar = captionBars;
            c.CloseButton = closeButtons;
        }
        #endregion

        #region Docking Windows Creation
        private void CreateProjectExplorerContent(object sender, System.EventArgs e)
        {
            // Create a TreeControl content instance
            projectExplorerWindow = CreateProjectExplorer();

            // Setup initial state to match menu selections
            InitializeContent(projectExplorerWindow);

            // Request a new docking window be created for the above Content on the left edge
            dockingManager.AddContentWithState(projectExplorerWindow, Crownwood.DotNetMagic.Docking.State.DockLeft);
        }

        private void CreateTaskListContent(object sender, System.EventArgs e)
        {
            // Create an Task List content instance
            Content c = CreateTaskListContent();

            // Setup initial state to match menu selections
            InitializeContent(c);

            // Request a new docking window be created for the above Content on the bottom edge
            dockingManager.AddContentWithState(c, Crownwood.DotNetMagic.Docking.State.DockBottom);
        }

        private void CreateOutputContent(object sender, System.EventArgs e)
        {
            // Create an Task List content instance
            Content c = CreateOutputContent();

            // Setup initial state to match menu selections
            InitializeContent(c);

            // Request a new docking window be created for the above Content on the bottom edge
            dockingManager.AddContentWithState(c, Crownwood.DotNetMagic.Docking.State.DockBottom);
        }
        #endregion

        #region Special Event Handlers
        private void OnSaveConfig(XmlTextWriter xmlOut)
        {
            // Add an extra node into the config to store some useless information
            xmlOut.WriteStartElement("MyCustomElement");
            xmlOut.WriteAttributeString("UselessData1", "Hello");
            xmlOut.WriteAttributeString("UselessData2", "World!");
            xmlOut.WriteEndElement();
        }

        private void OnLoadConfig(XmlTextReader xmlIn)
        {
            // We are expecting our custom element to be the current one
            if (xmlIn.Name == "MyCustomElement")
            {
                // Read in both the expected attributes
                string attr1 = xmlIn.GetAttribute(0);
                string attr2 = xmlIn.GetAttribute(1);

                // Must move past our element
                xmlIn.Read();
            }
        }

        private void OnContextMenu(PopupMenu pm, CancelEventArgs cea)
        {
            // Show the PopupMenu be cancelled and not shown?
            if (!allowContextMenu)
                cea.Cancel = true;
            else
            {
                if (customContextMenu)
                {
                    // Remove the Show All and Hide All commands
                    pm.MenuCommands.Remove(pm.MenuCommands["Show All"]);
                    pm.MenuCommands.Remove(pm.MenuCommands["Hide All"]);

                    // Add a custom item at the start
                    pm.MenuCommands.Insert(0, (new MenuCommand("Custom 1")));
                    pm.MenuCommands.Insert(1, (new MenuCommand("-")));

                    // Add a couple of custom commands at the end
                    pm.MenuCommands.Add(new MenuCommand("Custom 2"));
                    pm.MenuCommands.Add(new MenuCommand("Custom 3"));
                }
            }
        }

        private void TabPage_VisibleChanged(object sender, System.EventArgs e)
        {
            Crownwood.DotNetMagic.Controls.TabPage senderTabPage = sender as Crownwood.DotNetMagic.Controls.TabPage;

            if (!senderTabPage.Visible)
            {
                if (tabPageToFileInProject.Contains(sender))
                {
                    // Saving stuff
                    tabPageToFileInProject[sender] = null;

                    int from = senderTabPage.ToolTip.LastIndexOf('\\');
                    string filePath = senderTabPage.ToolTip.Substring(0, from);
                    string fileName = senderTabPage.ToolTip.Substring(from + 1, senderTabPage.ToolTip.Length - from - 1);

                    fileInProjectToTabPage[fileName] = null;
                }
                else if (tabPageToFileNotInProject.Contains(sender))
                {
                    // Saving stuff
                    tabPageToFileNotInProject[sender] = null;
                }
            }
        }

        private void projectExplorer_MouseDoubleClick(object sender, MouseEventArgs e) // Check if already open
        {
            Node node = projectExplorer.GetNodeAt(e.X, e.Y);

            if (node != null)
            {
                foreach (ProjectFile file in currentProject.LexicalAnalyserSpecificationFiles)
                    if (file.Name == node.Text)
                    {
                        Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(file.Language, file.Path, file.Name));
                        if (!fileInProjectToTabPage.Contains(file.Name))
                            this.fileInProjectToTabPage.Add(file.Name, tabPage);
                        if (!tabPageToFileInProject.Contains(tabPage))
                            this.tabPageToFileInProject.Add(tabPage, file);
                    }
                foreach (ProjectFile file in currentProject.ParserSpecificationFiles)
                    if (file.Name == node.Text)
                    {
                        Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(file.Language, file.Path, file.Name));
                        if (!fileInProjectToTabPage.Contains(file.Name))
                            this.fileInProjectToTabPage.Add(file.Name, tabPage);
                        if (!tabPageToFileInProject.Contains(tabPage))
                            this.tabPageToFileInProject.Add(tabPage, file);
                    }
                foreach (ProjectFile file in currentProject.GraphicalRegularLanguageSpecificationFiles)
                    if (file.Name == node.Text)
                    {
                        Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(file.Language, file.Path, file.Name));
                        if (!fileInProjectToTabPage.Contains(file.Name))
                            this.fileInProjectToTabPage.Add(file.Name, tabPage);
                        if (!tabPageToFileInProject.Contains(tabPage))
                            this.tabPageToFileInProject.Add(tabPage, file);
                    }
                foreach (ProjectFile file in currentProject.LexicalAnalyserCodeFiles)
                    if (file.Name == node.Text)
                    {
                        Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(file.Language, file.Path, file.Name));
                        if (!fileInProjectToTabPage.Contains(file.Name))
                            this.fileInProjectToTabPage.Add(file.Name, tabPage);
                        if (!tabPageToFileInProject.Contains(tabPage))
                            this.tabPageToFileInProject.Add(tabPage, file);
                    }
                foreach (ProjectFile file in currentProject.ParserCodeFiles)
                    if (file.Name == node.Text)
                    {
                        Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(file.Language, file.Path, file.Name));
                        if (!fileInProjectToTabPage.Contains(file.Name))
                            this.fileInProjectToTabPage.Add(file.Name, tabPage);
                        if (!tabPageToFileInProject.Contains(tabPage))
                            this.tabPageToFileInProject.Add(tabPage, file);
                    }
                foreach (ProjectFile file in currentProject.MiscellaneousFiles)
                    if (file.Name == node.Text)
                    {
                        Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(file.Language, file.Path, file.Name));
                        if (!fileInProjectToTabPage.Contains(file.Name))
                            this.fileInProjectToTabPage.Add(file.Name, tabPage);
                        if (!tabPageToFileInProject.Contains(tabPage))
                            this.tabPageToFileInProject.Add(tabPage, file);
                    }
            }
        }
        #endregion

        #region Menu Bar Descriptions
        private void menuMainFunctionality_Selected(MenuCommand item)
        {
            // Place description for selected menu item in the status bar
            statuspanel_Description.Text = item.Description;
        }

        private void menuMainFunctionality_Deselected(MenuCommand item)
        {
            // Clear down the status bar description
            statuspanel_Description.Text = "";
        }
        #endregion

		#region Process Handling Code
        Process _toolProcess = null;

        void RunTool(string processName, string fileName)
        {
            if (_toolProcess != null)
            {
                if (!_toolProcess.HasExited)
                    _toolProcess.Kill();
                _toolProcess.Dispose();
                _toolProcess = null;
            }
            ProcessStartInfo psi = new ProcessStartInfo(processName,
              "\"" + fileName + "\"");
            psi.WindowStyle = ProcessWindowStyle.Hidden;
            psi.CreateNoWindow = true;
            psi.WorkingDirectory = fileName.Substring(0, fileName.LastIndexOf("\\"));
            psi.RedirectStandardOutput = true;
            psi.RedirectStandardError = true;
            psi.UseShellExecute = false;
            _toolProcess = new Process();//Process.Start( psi );
            _toolProcess.EnableRaisingEvents = true;
            _toolProcess.StartInfo = psi;
            outputWindow.Control.Controls[0].Text = "";
            outputWindow.BringToFront();
            _toolProcess.OutputDataReceived += new DataReceivedEventHandler(ToolProcess_OutputDataReceived);
            _toolProcess.ErrorDataReceived += new DataReceivedEventHandler(ToolProcess_ErrorDataReceived);
            _toolProcess.Exited += new EventHandler(ToolProcess_Exited);
            _toolProcess.Start();
        }

        void RunTool(string processName, string inFileName, string outFileName)
        {
            if (_toolProcess != null)
            {
                if (!_toolProcess.HasExited)
                    _toolProcess.Kill();
                _toolProcess.Dispose();
                _toolProcess = null;
            }

            ProcessStartInfo psi = new ProcessStartInfo(processName,
              "\"" + inFileName + "\" \"" + outFileName + "\"");
            psi.WindowStyle = ProcessWindowStyle.Hidden;
            psi.CreateNoWindow = true;
            psi.WorkingDirectory = inFileName.Substring(0, inFileName.LastIndexOf("\\"));
            psi.RedirectStandardOutput = true;
            psi.RedirectStandardError = true;
            psi.UseShellExecute = false;
            _toolProcess = new Process();
            _toolProcess.EnableRaisingEvents = true;
            _toolProcess.StartInfo = psi;

            //_toolProcess.OutputDataReceived += new DataReceivedEventHandler(ToolProcess_OutputDataReceived);
            //_toolProcess.ErrorDataReceived += new DataReceivedEventHandler(ToolProcess_ErrorDataReceived);
            _toolProcess.Exited += new EventHandler(ToolProcess_Exited);
                     
            string ext = inFileName.Split('.')[1];
            if (ext == "parcpp" || ext == "lexcpp")
                outputLanguage = FileLanguage.CPlusPlus;
            else if (ext == "parcs" || ext == "lexcs")
                outputLanguage = FileLanguage.CSharp;
            else if (ext == "parjava" || ext == "lexjava")
                outputLanguage = FileLanguage.Java;

            outputPath = outFileName;
            outputText = "";
            errorList = "";
            errorCount = 0;
            warningCount = 0;

            _toolProcess.Start();
            //_toolProcess.BeginErrorReadLine();
            //_toolProcess.BeginOutputReadLine();
        }

        void ToolProcess_OutputDataReceived(object sender, DataReceivedEventArgs e)
        {
            //outputText += e.Data;
            //_toolProcess.BeginOutputReadLine();
        }

        void ToolProcess_ErrorDataReceived(object sender, DataReceivedEventArgs e)
        {
            //errorList += e.Data;
            //_toolProcess.BeginErrorReadLine();
        }

        FileLanguage outputLanguage;
        string outputPath = "";
        string outputText = "";
        string errorList = "";
        int errorCount = 0, warningCount = 0;

        void ToolProcess_Exited(object sender, EventArgs e)
        {
            //try
            //{
            Process sendingProcess = sender as Process;
            outputText += sendingProcess.StandardOutput.ReadToEnd();
            errorList += sendingProcess.StandardError.ReadToEnd();
            this.Invoke(new MethodInvoker(SetOutputText));
            this.Invoke(new MethodInvoker(SetErrorList));

            //if (errorList=="")
            if(errorCount == 0)
            {
                this.Invoke(new MethodInvoker(CreateTabPage));
            }

            _toolProcess.Dispose();
            _toolProcess = null;
            //}
            //catch (Exception ex)
            //{
            //    MessageBox.Show(ex.Message);
            //}

        }

        void SetOutputText(string text)
        {
            outputWindow.BringToFront();
            ((Output)outputWindow.Control).SetOutputText(text);
        }

        void SetErrorList()
        {
            string[] lines = errorList.Split('\n');
            ListView lstErrors = ((LEXcellentErrors)taskListWindow.Control).lstErrors;
            lstErrors.Items.Clear();
            string[] tokens;
            int index;
            for (int i = 0; i < lines.Length - 1; i++)
            {
                tokens = lines[i].Split('\t');
                index = (tokens[2][0] == 'E') ? 0 : 1;
                if (1 == index)
                    warningCount++;
                else
                    errorCount++;
                lstErrors.Items.Add(new ListViewItem(tokens, index));
            }
            ((LEXcellentErrors)taskListWindow.Control).btnWarnings.Text = warningCount.ToString() + " Warnings";
            ((LEXcellentErrors)taskListWindow.Control).btnErrors.Text = errorCount.ToString() + " Errors";
        }
        
        void SetOutputText()
        {
            outputWindow.BringToFront();
            ((Output)outputWindow.Control).SetOutputText(outputText);
        }

        void CreateTabPage()
        {
            int from = outputPath.LastIndexOf('\\');
            string filePath = outputPath.Substring(0, from);
            string fileName = outputPath.Substring(from + 1, outputPath.Length - from - 1);

            string ext = "";
            switch (outputLanguage)
            {
                case FileLanguage.CPlusPlus:
                    ext = ".cpp";
                    break;
                case FileLanguage.CSharp:
                    ext = ".cs";
                    break;
                case FileLanguage.Java:
                    ext = ".java";
                    break;
            }
            string barename = fileName.Split('.')[0];
            fileName = barename + ext;

            if (fileInProjectToTabPage.Contains(fileName))
            {
                StreamReader stream = new StreamReader(filePath + "\\" + fileName);
                ((Fireball.Windows.Forms.CodeEditorControl)((Crownwood.DotNetMagic.Controls.TabPage)fileInProjectToTabPage[fileName]).Control).Document.Text = stream.ReadToEnd();
                stream.Close();
                return;
            }


            Crownwood.DotNetMagic.Controls.TabPage tabPage = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(outputLanguage, filePath, fileName));

            this.fileInProjectToTabPage.Add(fileName, tabPage);
            StreamReader sr = new StreamReader(filePath + "\\" + fileName);
            string str = sr.ReadToEnd();
            ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str;
            sr.Close();

            if (ext == ".cpp")
            {
                fileName = barename + ".h";
                Crownwood.DotNetMagic.Controls.TabPage tabPage1 = tabbedgroups_Main.ActiveLeaf.TabPages.Add(NewTabPage(outputLanguage, filePath, fileName));

                this.fileInProjectToTabPage.Add(fileName, tabPage1);

                StreamReader sr1 = new StreamReader(filePath + "\\" + fileName);
                string str1 = sr1.ReadToEnd();
                ((Fireball.Windows.Forms.CodeEditorControl)(tabbedgroups_Main.ActiveTabPage.Control)).Document.Text = str1;
                sr1.Close();
            }
        }
        #endregion
    }
}